@@ -16,8 +16,6 @@ char Men3_mclose_c_sccsid[] = "@(#)mclose.c	1.7 26/4/92 (UKC)";
 #include "menu3.h"
 #include "menu_priv.h"
 
-static void freemen MPROTO((MENU *menu));
-
 /*  Remove the menu from the list of open menus and free the menu descriptor
  *  for re-use. If the menu is currently displayed it will be removed.
  */
@@ -37,21 +35,21 @@ int md;
 		if (Mremove(md) < 0)
 			return(-1);
 
-	freemen(_menu_[md].om_root);
+	mdelete(_menu_[md].om_root);
 	_menu_[md].om_root = NULL;
 	return(0);
 }
 
 /*  Free the memory occupied by the menu.
  */
-static void
-freemen(menu)
+void
+mdelete(menu)
 MENU * menu;
 {
 	if (menu == NULL)
 		return;
-	freemen(menu->me_topleft);
-	freemen(menu->me_botrite);
+	mdelete(menu->me_topleft);
+	mdelete(menu->me_botrite);
 	if ((menu->me_cap != NULL) && (menu->me_flags & ME_FREC)) {
 		/*  BUG: we cast "const char *" to "char *" here.
 		 *  me_cap is of type const char *, because menus are
