@@ -25,7 +25,7 @@ static void mredraw MPROTO((MENU * menu));
 static void setarea MPROTO((int md, int save, WOTYPE wn));
 static int mnonsel MPROTO((MENU *menu, const char *rvs));
 static int dragin MPROTO((MENU *menu));
-static int mbpress MPROTO((int x, int y, MENU *menu));
+static int mbpress MPROTO((int x, int y, MENU *menu, int wn));
 static void msadd MPROTO((MENU *menu));
 static MENU *mfind MPROTO((MENU *menu, int x, int y));
 static MENU *msfind MPROTO((int x, int y, WOTYPE wn, int ignore));
@@ -51,6 +51,41 @@ static int msnext = 0;		/* the number of menus in the stack */
 
 static int (*fpopsu)MPROTO((int wn)) = NULL, (*fpushsu)MPROTO((int wn)) = NULL;
 
+/*   Bitmaps for toggle items
+ */
+#define option_off_image_width 11
+#define option_off_image_height 11
+static unsigned char option_off_image_bits[] = {
+  0xff,0x07,0x01,0x04,0x01,0x04,0x01,0x04,0x01,0x04,0x01,0x04,
+  0x01,0x04,0x01,0x04,0x01,0x04,0x01,0x04,0xff,0x07};
+static bitmap_t option_off_image =
+	wn_make_static_X11bm(option_off_image_width, option_off_image_height, option_off_image_bits);
+
+#define option_on_image_width 11
+#define option_on_image_height 11
+static unsigned char option_on_image_bits[] = {
+  0xff,0x07,0xff,0x07,0xff,0x07,0xff,0x07,0xff,0x07,0xff,0x07,
+  0xff,0x07,0xff,0x07,0xff,0x07,0xff,0x07,0xff,0x07};
+static bitmap_t option_on_image =
+	wn_make_static_X11bm(option_on_image_width, option_on_image_height, option_on_image_bits);
+
+#define option_selected_image_width 11
+#define option_selected_image_height 11
+static unsigned char option_selected_image_bits[] = {
+  0x20,0x00,0x70,0x00,0xf8,0x00,0xfc,0x01,0xfe,0x03,0xff,0x07,
+  0xfe,0x03,0xfc,0x01,0xf8,0x00,0x70,0x00,0x20,0x00};
+static bitmap_t option_selected_image =
+	wn_make_static_X11bm(option_selected_image_width, option_selected_image_height, option_selected_image_bits);
+
+#define option_not_selected_image_width 11
+#define option_not_selected_image_height 11
+static unsigned char option_not_selected_image_bits[] = {
+  0x20,0x00,0x50,0x00,0x88,0x00,0x04,0x01,0x02,0x02,0x01,0x04,
+  0x02,0x02,0x04,0x01,0x88,0x00,0x50,0x00,0x20,0x00};
+static bitmap_t option_not_selected_image =
+	wn_make_static_X11bm(option_not_selected_image_width, option_not_selected_image_height, option_not_selected_image_bits);
+
+
 /*  Routines for turning on and off submenus RGA
  */
 void
@@ -414,7 +449,7 @@ int mode, ignore;
 			lastb = m;
 			return(0);
 		}
-		rv = mbpress(x,y,m);
+		rv = mbpress(x,y,m, wn);
 		lastb = lastpress;
 		/*  Try and cope with any popup menu which may now
 		 *  be under the coords.
@@ -432,7 +467,7 @@ int mode, ignore;
 			return(0);
 		lastb = m;
 		if (dragin(m))
-			return(mbpress(x,y,m));
+			return(mbpress(x,y,m, wn));
 
 		if (lastpress->me_flags & ME_ISUB)
 			/*  Don't try giving feedback on a submenu button
@@ -466,6 +501,19 @@ int mode, ignore;
 		lastpress->me_colour = om->om_fback[2];
 		mshow(lastpress,1);
 		lastpress = NULL;
+
+#if 0   /* This in incompatible with the ups replay feature. */
+		/* If it is a toggle item, reverse the toggle value
+		 * for an exclusive group item, set the select value */
+		if ( m->me_toggle)
+		{
+		   if ( m->me_select_val >= 0)
+		      *m->me_toggle = m->me_select_val;
+		   else
+		      *m->me_toggle = !*m->me_toggle;
+		}
+#endif
+
 		return(om->om_md << 16 | m->me_rv);
 	}
 	abort();		/* to make me happy */
@@ -535,9 +583,10 @@ MENU *menu;
 /*  Do the selection for with a button press, either real or assumed.
  */
 static int
-mbpress(x,y,menu)
+mbpress(x,y,menu, wn)
 int x,y;
 MENU *menu;
+int wn;
 {
 	MENU *m, *ms;
 	struct omenst *om;
@@ -593,7 +642,11 @@ MENU *menu;
 			 * to move it.
 			 */
 			mshift(ms,x + ms->me_xcurs,y + ms->me_ycurs,1);
+
 		}
+		/* Check that the menu is on the screen */
+		mposition_popup(ms, wn, &x, &y, 0);
+
 		src.x = ms->me_xstart;
 		size.w = ms->me_xend - ms->me_xstart + 1;
 		src.y = ms->me_ystart;
@@ -927,15 +980,15 @@ WOTYPE wn;
 		p1.x = menu->me_xstart;
 		p2.x = menu->me_xend;
 		p1.y = p2.y = menu->me_yend;
-		wn_draw_line(wn,p1.x,p1.y,p2.x,p2.y,BLACK);
+		wn_draw_line(wn,p1.x,p1.y,p2.x,p2.y,WN_FG(wn));
 		p1.y = p2.y = menu->me_ystart;
-		wn_draw_line(wn,p1.x,p1.y,p2.x,p2.y,BLACK);
+		wn_draw_line(wn,p1.x,p1.y,p2.x,p2.y,WN_FG(wn));
 		p1.y = menu->me_ystart;
 		p2.y = menu->me_yend;
 		p1.x = p2.x = menu->me_xend;
-		wn_draw_line(wn,p1.x,p1.y,p2.x,p2.y,BLACK);
+		wn_draw_line(wn,p1.x,p1.y,p2.x,p2.y,WN_FG(wn));
 		p1.x = p2.x = menu->me_xstart;
-		wn_draw_line(wn,p1.x,p1.y,p2.x,p2.y,BLACK);
+		wn_draw_line(wn,p1.x,p1.y,p2.x,p2.y,WN_FG(wn));
 	}
 	if (ISCLEAR(menu)) {
 		if (menu->me_flags & ME_HOR) {
@@ -951,7 +1004,7 @@ WOTYPE wn;
 			p2.y = p1.y;
 		}
 		if (!(menu->me_flags & ME_BGLINE))
-			wn_draw_line(wn,p1.x,p1.y,p2.x,p2.y,BLACK);
+			wn_draw_line(wn,p1.x,p1.y,p2.x,p2.y,WN_FG(wn));
 		rmshow(menu->me_topleft,wn);
 		rmshow(menu->me_botrite,wn);
 	}
@@ -969,6 +1022,9 @@ MENU *mp;
 int invert;
 int greying;
 {
+/* Parameters for toggle box: */
+#define BORDERLEFT 5
+#define BOXSIZE option_off_image_width
 	register int l, cx, cy, x, y;
 	int charheight, charbase;
 	struct DBCtl db;
@@ -978,6 +1034,9 @@ int greying;
 	struct omenst *om;
 	const char *str, *sp;
 
+	long bg = invert ? mp->me_fg_colour : mp->me_bg_colour;
+	long fg = invert ? mp->me_bg_colour : mp->me_fg_colour;
+
 	cl.pos.x = mp->me_xstart + 1;
 	cl.pos.y = mp->me_ystart + 1;
 	cl.size.w = mp->me_xend - cl.pos.x;
@@ -1001,7 +1060,7 @@ int greying;
 		if (*sp == '\n')
 			l++;
 	y = cy - (l * charheight) / 2 + charbase;
-	wblank(wn,&cl, invert ? mp->me_fg_colour : mp->me_bg_colour);
+	wblank(wn,&cl, bg);
 	while (*str) {
 		/* calculate the length of a caption line
 		 */
@@ -1010,15 +1069,17 @@ int greying;
 			if (*sp == '\n')
 				break;
 			l += fe[*sp & 0177];
-			if (l > cl.size.w - 5) {
+			if (l > cl.size.w - BORDERLEFT) {
 				l -= fe[*sp & 0177];
 				break;
 			}
 		}
 		if (mp->me_flags & ME_LEFT)
-			x = cl.pos.x + 3;
+			x = cl.pos.x + BORDERLEFT;
 		else
 			x = cx - l / 2;
+		if ( mp->me_toggle && x < (2*BORDERLEFT+BOXSIZE))
+		   x = 2*BORDERLEFT+BOXSIZE;
 		db.DBX = x;
 		db.DBY = y;
 		db.DBByteOffset = 0;
@@ -1028,10 +1089,7 @@ int greying;
 		db.DBSrcString = str;
 		db.DBScreen = 0;
 		db.DBFont = fnt;
-		if (invert)
-		  wn_wdbyte(wn,&db,&cl,mp->me_bg_colour,mp->me_fg_colour);
-		else
-		  wn_wdbyte(wn,&db,&cl,mp->me_fg_colour,mp->me_bg_colour);
+		wn_wdbyte(wn,&db,&cl,fg, bg);
 		str = sp;
 		while ((*str != '\n') && (*str != 0))
 			str++;
@@ -1039,6 +1097,23 @@ int greying;
 			str++;
 		y += charheight;
 	}
+	if ( mp->me_toggle)
+	{
+	    bitmap_t* image;
+	    if ( mp->me_select_val >= 0)
+		image = (*mp->me_toggle == mp->me_select_val)
+			? &option_selected_image
+			: &option_not_selected_image;
+	    else
+		image = (*mp->me_toggle)
+			? &option_on_image
+			: &option_off_image;
+	    wn_put_image(image,0,0,
+	        BOXSIZE, BOXSIZE, wn,
+	        mp->me_xstart+BORDERLEFT, (mp->me_ystart+mp->me_yend-BOXSIZE)/2, R_RPL,
+	        fg, bg);
+
+	}
 	if (!greying && mp->me_flags & ME_NSEL)
 	  wgrey(wn,&cl,0,mp);
 }
@@ -1086,8 +1161,8 @@ MENU* menu;
 
 	if (color_done == -1)
 	{
-	  color_fg.co_pixel = WN_FG;
-	  color_bg.co_pixel = WN_BG;
+	  color_fg.co_pixel = WN_FG(wn);
+	  color_bg.co_pixel = WN_BG(wn);
 	  if (wn_use_menu_colors())
 	  {
 	    const char *colorname;
@@ -1095,16 +1170,16 @@ MENU* menu;
 	    colorname = wn_get_default("DisabledTextColor");
 	    if (colorname == NULL)
 	      colorname = "gray";
-	    wn_parse_color(colorname, &color_fg);
-	    if (!wn_alloc_color(&color_fg, colorname))
-	      color_fg.co_pixel = WN_FG;
+	    wn_parse_color(wn, colorname, &color_fg);
+	    if (!wn_alloc_color(wn, &color_fg, colorname))
+	      color_fg.co_pixel = WN_FG(wn);
 
 	    colorname = wn_get_default("LastButtonTextColor");
 	    if (colorname == NULL)
 	      colorname = "white";
-	    wn_parse_color(colorname, &color_bg);
-	    if (!wn_alloc_color(&color_bg, colorname))
-	      color_bg.co_pixel = WN_BG;
+	    wn_parse_color(wn, colorname, &color_bg);
+	    if (!wn_alloc_color(wn, &color_bg, colorname))
+	      color_bg.co_pixel = WN_BG(wn);
 
 	    color_done = 1;	/* done and using colors */
 	  }
@@ -1119,7 +1194,7 @@ MENU* menu;
 	  if (colour == 0)
 	  {
 	    menu->me_fg_colour = color_fg.co_pixel;
-	    menu->me_bg_colour = WN_BG;
+	    menu->me_bg_colour = WN_BG(wn);
 	  }
 	  else
 	  {
@@ -1146,7 +1221,3 @@ MENU* menu;
 }
 
 
-
-
-
-
