@@ -37,6 +37,8 @@ typedef struct mnode {
 	struct mnode * me_botrite; /* pointer to b. or r. sub_menu */
 	struct mnode * me_parent;  /* pointer to parent */
 	struct omenst *me_omen;	   /* pointer to open menu structure */
+	int*   me_toggle;       /* For toggle items */
+	int    me_select_val;   /* if >= 0, select value for exclusive group */
 } MENU;
 
 #ifndef NULL
@@ -131,6 +133,9 @@ int Mfonts MPROTO((int md, font_t *font0, font_t *font1,
 #endif
 int Mnobox MPROTO((int md));
 int Mnonsel MPROTO((int md, const char *rvs, int display));
+int Mdelete_entry MPROTO((int md,int rv));
+int Mmaketoggle MPROTO((int md, int rv, int* p_value));
+int Mmakegroup MPROTO((int md, int* p_value, ...));
 int Mreset MPROTO((int md));
 int Mcansel MPROTO((int x, int y, int wout));
 int Mselect MPROTO((int x, int y, int mode, int wout, int ignore));
@@ -156,6 +161,7 @@ struct merrst *Mopenerr MPROTO((void));
 int Mperror MPROTO((const char *str));
 int Msize MPROTO((int md, int width, int height));
 int Mplace MPROTO((int md, int x, int y));
+int Mposition_popup MPROTO((int md, int wn, int* pcx, int* pcy, int may_warp_cursor));
 struct fontst *Mstdfont MPROTO((void));
 void Msetstdfont MPROTO((struct fontst *font));
 const char *Mversion MPROTO((void));
