@@ -29,11 +29,12 @@ int *fg_specified, *bg_specified;
   static color_checks_done = -1;
   color_t color;
   static int want_menu_colors = 0, menu_fg, menu_bg;
+  int wn = WN_STDWIN;
   
   if (color_checks_done == -1)
   {
-    menu_fg = WN_FG;
-    menu_bg = WN_BG;
+    menu_fg = WN_FG(wn);
+    menu_bg = WN_BG(wn);
     want_menu_colors = wn_use_menu_colors();
     if (want_menu_colors)
     {
@@ -43,21 +44,21 @@ int *fg_specified, *bg_specified;
       colorname = wn_get_default("MenuForeground");
       if (colorname != NULL)
       {
-	wn_parse_color(colorname, &color);
-	if (wn_alloc_color(&color, colorname))
+	wn_parse_color(wn, colorname, &color);
+	if (wn_alloc_color(wn, &color, colorname))
 	  menu_fg = color.co_pixel;
 	else
-	  menu_fg = WN_FG;
+	  menu_fg = WN_FG(wn);
       }
 
       colorname = wn_get_default("MenuBackground");
       if (colorname != NULL)
       {
-	wn_parse_color(colorname, &color);
-	if (wn_alloc_color(&color, colorname))
+	wn_parse_color(wn, colorname, &color);
+	if (wn_alloc_color(wn, &color, colorname))
 	  menu_bg = color.co_pixel;
 	else
-	  menu_bg = WN_BG;
+	  menu_bg = WN_BG(wn);
       }
     }
     color_checks_done = 0;	/* done */
@@ -82,9 +83,9 @@ int *fg_specified, *bg_specified;
     if (colorname != NULL)
     {
       *fg_specified = 1;
-      wn_parse_color(colorname, &color);
+      wn_parse_color(wn, colorname, &color);
       wn_use_alloc_color(1); /* set */
-      wn_alloc_color(&color, colorname);
+      wn_alloc_color(wn, &color, colorname);
     }
     if (!*p_fg)
       *p_fg = wn_use_alloc_color(0) ? color.co_pixel : menu_fg;
@@ -95,9 +96,9 @@ int *fg_specified, *bg_specified;
     if (colorname != NULL)
     {
       *bg_specified = 1;
-      wn_parse_color(colorname, &color);
+      wn_parse_color(wn, colorname, &color);
       wn_use_alloc_color(1); /* set */
-      wn_alloc_color(&color, colorname);
+      wn_alloc_color(wn, &color, colorname);
     }
     if (!*p_bg)
       *p_bg = wn_use_alloc_color(0) ? color.co_pixel : menu_bg;
