@@ -171,6 +171,7 @@ int argc;
 const char **argv;
 {
 	const char *prompt;
+	char *ptr;
 	char buf[200];
 	int i;
 	void (*func)PROTO((const char *pat));
@@ -215,10 +216,12 @@ const char **argv;
 	if (argc == 2) {
 		for (;;) {
 			(void) fprintf(stderr, "%s> ", prompt);
-			if (gets(buf) == NULL) {
+			if (fgets(buf, sizeof(buf), stdin) == NULL) {
 				(void) putchar('\n');
 				break;
 			}
+			if ((ptr = strchr (buf, '\n')) != 0)
+				*ptr = '\0';
 			(*func)(buf);
 		}
 	}
