@@ -9,20 +9,7 @@
 
 char utils_apply_to_files_in_sccsid[] = "@(#)appltofl.c	1.9 16 Sep 1994 (UKC)";
 
-#include "ifdefs.h"
-
-#include <sys/types.h>
-#include <errno.h>
-#include <string.h>
-
-#if defined(NO_DIRENT) || defined(clipper)
-#include <sys/dir.h>
-#undef dirent
-#define dirent direct
-#else
-#include <dirent.h>
-#endif
-
+#include <ifdefs.h>
 #include <local/ukcprog.h>
 
 #include "utils.h"
