@@ -11,25 +11,10 @@ char rm_rf_sccsid[] = "@(#)rm_rf.c	1.10 16 Sep 1994 (UKC)";
 
 #define UKC_WANT_COMMON_UNIX_EXTENSIONS 1	/* for lstat() */
 
-#include <sys/types.h>
+#include <ifdefs.h>
 #include <sys/stat.h>
 #include <sys/file.h>
 #include <sys/param.h>		/* for MAXPATHLEN */
-#include <errno.h>
-
-#include <string.h>
-
-#if defined(NO_DIRENT) || defined(clipper)
-#include <sys/dir.h>
-#undef dirent
-#define dirent direct
-#else
-#include <dirent.h>
-#endif
-
-#ifdef __STDC__
-#include <unistd.h>
-#endif
 
 #include <local/ukcprog.h>
 
