@@ -239,7 +239,7 @@ edesc_t *edesc;
 	const char *buf;
 	int i, nbytes;
 
-	wn_get_selection(&buf, &nbytes);
+	wn_get_selection(edesc->ed_wn, &buf, &nbytes);
 
 	for (i = 0; i < nbytes; ++i) {
 		edesc->ed_char = buf[i];
@@ -372,7 +372,7 @@ edesc_t *edesc;
 	if ((wn = edesc->ed_wn) == -1)
 		return;
 
-	wn_set_area(wn, 0, 0, wn_get_width(wn), wn_get_height(wn), WN_BG);
+	wn_set_area(wn, 0, 0, wn_get_width(wn), wn_get_height(wn), WN_BG(wn));
 	wn_text(wn, edesc->ed_font, edesc->ed_copy,
 				0, 0, edesc->ed_fg, edesc->ed_bg, WN_USE_TOP);
 	do_edit(edesc, EDM_SETCURSOR);
@@ -398,10 +398,10 @@ edesc_t *edesc;
 	oldwidth = wn_strwidth(edesc->ed_copy, font);
 	newwidth = wn_strwidth(edesc->ed_orig, font);
 	height = font->ft_height;
-	wn_set_area(wn, 0, 0, oldwidth, height, WN_BG);
+	wn_set_area(wn, 0, 0, oldwidth, height, WN_BG(wn));
 	if (edesc->ed_scrlen != 0)
 		wn_move_area(wn, oldwidth, 0, edesc->ed_scrlen, height,
-							newwidth, 0, WN_BG);
+							newwidth, 0, WN_BG(wn));
 	wn_text(wn, font, edesc->ed_orig, 0, 0, edesc->ed_fg, edesc->ed_bg, WN_USE_TOP);
 	(void) strcpy(edesc->ed_copy, edesc->ed_orig);
 	edesc->ed_curlen = strlen(edesc->ed_copy);
@@ -560,7 +560,7 @@ int cursor_on;
 		else	
 			wn_set_area(edesc->ed_wn,
 					wn_strnwidth(s, pos, font), 0,
-					2, font->ft_height, INVERT);
+					2, font->ft_height, WN_INVERT);
 		edesc->ed_flags ^= EDF_CURSOR_ON;
 	}
 }	
