@@ -233,7 +233,8 @@ obj_get_color_func_t getcolor;
 			*/
 			if ( *format == ':')
 			{
-			    const char* e_start = (char *)++format; /* remember this point */
+			    const char* e_start; /* remember this point */
+			    e_start = (char *)++format;
 			    resizes = TRUE;
 			    flags |= OB_RESIZES;
 			    min_width = uatoi(&format, DEF_WIDTH * Xscale, Xscale);
