@@ -154,6 +154,6 @@ extern int _ob_Default_fg_pixel, _ob_Default_bg_pixel;
 
 void _obj_error PROTO((const char *fmt, ...));
 
-void resize_format PROTO((struct odescst *p_odesc, short width));
+void resize_format PROTO((struct odescst *p_odesc, int width));
 
 /* LINTLIBRARY */
