@@ -37,19 +37,15 @@ extern const char noshare *sys_errlist[];
 
 #else /* !VMS */
 
-#ifndef __FreeBSD__
-#ifdef __linux__
-extern const char *const sys_errlist[];
-extern int sys_nerr;
-#else
-extern const char* sys_errlist[];
-extern int sys_nerr;
-#endif
-#endif
-
 #endif /* !VMS */
 #endif /* !MSDOS */
 
+#if !defined(__STDC__)
+extern char *sys_errlist[];
+extern int sys_nerr;
+#define strerror(n) \
+    (((n) >= 0 && (n) < sys_nerr) ? sys_errlist[n] : "unknown error")
+#endif
 
 /* forward declarations */
 static void new_buffer PROTO((char ** p_buf, int *p_lim, bool *p_from_malloc));
@@ -425,14 +421,7 @@ gencase:			if (modifier == 'h')
 					panic("null pointer for %s in formf");
 				break;
 			case 'm':
-				if (saved_errno < 0 || saved_errno > sys_nerr ||
-					    *sys_errlist[saved_errno] == '\0') {
-					sprintf(errno_buffer,
-						"errno %d out of range",
-								saved_errno);
-					s = errno_buffer;
-				} else
-					s = sys_errlist[saved_errno];
+				s = strerror (saved_errno);
 				break;
 #ifdef VMS
 			case 'M':
