@@ -969,7 +969,7 @@ int wn, x, y;
 	}
 	width = new_x - x;
 	height = new_y - y;
-	bm = wn_make_bitmap(width, height, wn_get_nplanes(), 
+	bm = wn_make_bitmap(width, height, wn_get_nplanes(wn), 
 				BM_NATURAL_BIT_ORDER, BM_NATURAL_PIXEL_FORMAT);
 	if (bm == NULL) {
 		fprintf(stderr,"wn_make_bitmap failed - aborting\n");
@@ -1161,7 +1161,7 @@ int wn;
 		{ 0, 0, 0, 0 },			/* BG */
 		{ 0, 0xffff, 0xffff, 0x0000 },	/* yellow */
 	};
-	int pixels[NPIXELS], planes;
+	unsigned long pixels[NPIXELS], planes;
 	font_t *font;
 
 	if (wn_get_pixels_and_planes(NPIXELS, 1, TRUE, pixels, &planes) != 0) {
@@ -1407,7 +1407,7 @@ const char **argv;
 redraw:
 	wn_get_window_size(wn,&w,&h);
 	wn_updating_off(wn);
-	if (wn_get_nplanes() == 1 || getenv("TESTCOLOR") == NULL)
+	if (wn_get_nplanes(wn) == 1 || getenv("TESTCOLOR") == NULL)
 		wn_set_area(wn,0,0,w,h,WHITE);
 	else {
 		wn_set_area(wn, 0, 0, w, h/3, Colors[FGREEN].co_pixel);
