@@ -12,6 +12,10 @@
 #ifndef WN_H_INCLUDED
 #define WN_H_INCLUDED
 
+/* We need to know options selected.
+ */
+#include <mtrprog/ifdefs.h>
+
 /* Define wn__PROTO as in gendefs.h.  We don't mandate gendefs.h for
  * wn.h, so we have to define wn__PROTO here.
  */
@@ -174,6 +178,13 @@ typedef struct bitmapst {
 	{ data, bit_o, byte_o, pixfmt, BM_CHOOSE_FORMAT, 0, \
 					w, h, np, xh, yh, lineinc, 0 }
 
+/*   Macro to create a bitmap from the output of the X11 bitmap program:
+ */
+#define wn_make_static_X11bm(_width, _height, _bits) \
+	wn_make_static_bm(_width, _height, 1, 0, 0, \
+		BM_BIT0_LEFT,BM_LSB_FIRST,BM_XY_PIXELS, \
+		(_width+7)/8, (unsigned short*)_bits)
+
 /*  Old interface to wn_make_static_bm() for backwards compatibility.
  */
 #define wn_static_bm(width, height, nplanes, xhot, yhot, bit_order, data) \
@@ -245,6 +256,10 @@ typedef struct bmcursorst {
 #define _WN_CAT(a,b) _WN_IDENT(a)b
 #endif /* !__STDC__ */
 
+/*  Macro to print NULL string.
+ */
+#define WN_PNULL(a)	((a) ? (a) : "<null>")
+
 /*  Convenience macro to construct an element in a bmcursor_t array from
  *  a cursor and mask bitmap.
  */
@@ -268,7 +283,9 @@ typedef struct bmcursorst {
 #define wn_rop_to_mem		wn_get_image
 #define wn_rop_mem_to_mem	wn_copy_image
 #define wn_rop_from_mem(bm, sx, sy, width, height, wn, dx, dy, ropfunc) \
-	wn_put_image(bm, sx, sy, width, height, wn, dx, dy, ropfunc, WN_FG, WN_BG)
+	wn_put_image(bm, sx, sy, width, height, wn, dx, dy, ropfunc, \
+	WN_FG(wn), WN_BG(wn))
+
 
 /*  Macros for drawing lines in bitmaps.
  */
@@ -278,18 +295,18 @@ typedef struct bmcursorst {
 #define wn_bm_vline(bm, x, y, height, rfunc) \
 	wn_copy_image((bm), (x), (y), 1, (height), (bm), (x), (y), rfunc)
 
-/*  The values of the background and foreground pixel values.
- *  Should be used only via the WN_FG and WN_BG macros.
- */
-extern int _wn_Fg_pixel, _wn_Bg_pixel;
 
 /*  Special color values meaning foreground and background.
- *
- *  The casts to int are to prevent these being accidentally used
- *  as lvalues.
  */
-#define WN_FG		((int)_wn_Fg_pixel)
-#define WN_BG		((int)_wn_Bg_pixel)
+#if WANT_MULTI_SCREEN
+#define WN_MY_PIXEL	0x80000000
+#define WN_FG(wn)	(0 | WN_MY_PIXEL)
+#define WN_BG(wn)	(1 | WN_MY_PIXEL)
+#else
+#define WN_FG(wn)	wn_fg_pixel(wn)
+#define WN_BG(wn)	wn_bg_pixel(wn)
+#endif
+
 
 /*  A colour value meaning invert the foreground and background colors.
  *  Using this has unpredictable effects on pixels of other colors, but
@@ -302,12 +319,6 @@ extern int _wn_Fg_pixel, _wn_Bg_pixel;
  */
 #define WN_TRANSPARENT	(-2)
 
-/*  Old names for WN_FG and WN_BG and WN_INVERT - kept for backwards compatibility.
- */
-#define WHITE	WN_BG
-#define BLACK	WN_FG
-#define INVERT	WN_INVERT
-
 /*  Rop function values.
  */
 #define R_RPL		0
@@ -357,7 +368,7 @@ typedef struct eventst {
 	short ev_x;		/* mouse x coord */
 	short ev_y;		/* mouse y coord */
 	short ev_buttons;	/* mouse button and modifier state */
-	long ev_time;		/* timestamp */
+	unsigned long ev_time;	/* timestamp */
 	int ev_wn;		/* window the event happened in */
 	int ev_fdmask;		/* input fds mask for EV_OTHER_INPUT events */
 } event_t;
@@ -505,7 +516,7 @@ typedef struct eventst {
 	wn_xtext(wn, font, s, x, y, WN_MAXWIDTH, R_RPL, fg_color, bg_color, use_which, 0)
 
 #define wn_mono_text(wn, font, s, x, y, ropfunc, use_which) \
-	wn_xtext(wn, font, s, x, y, WN_MAXWIDTH, ropfunc, WN_FG, WN_BG, use_which, 1)
+	wn_xtext(wn, font, s, x, y, WN_MAXWIDTH, ropfunc, WN_FG(wn), WN_BG(wn), use_which, 1)
 
 #define wn_wd_text(wn, font, s, x, y, width, fg_color, bg_color, use_which) \
 	wn_xtext(wn, font, s, x, y, width, R_RPL, fg_color, bg_color, use_which, 0)
@@ -516,7 +527,7 @@ typedef struct eventst {
 	wn_xwrite(wn, font, s,len, x,y, R_RPL, fg_color, bg_color, use_which, 0)
 
 #define wn_mono_write(wn, font, s, len, x, y, ropfunc, use_which) \
-	wn_xwrite(wn, font, s,len, x,y, ropfunc, WN_FG, WN_BG, use_which, 1)
+	wn_xwrite(wn, font, s,len, x,y, ropfunc, WN_FG(wn), WN_BG(wn), use_which, 1)
 
 
 /*  Simpler interfaces to wn_text.
@@ -538,10 +549,10 @@ typedef struct eventst {
 
 /* AMC enhancements */
 #define wn_xputs(wn, font, s, x, y, func, use_which) \
-	wn_xtext(wn, font, s, x, y, WN_MAXWIDTH, func, WN_FG, WN_BG, use_which, 0)
-#define wn_bputs(wn,s,x,y)	wn_btext(wn, s, x, y, WN_FG, WN_BG)
+	wn_xtext(wn, font, s, x, y, WN_MAXWIDTH, func, WN_FG(wn), WN_BG(wn), use_which, 0)
+#define wn_bputs(wn,s,x,y)	wn_btext(wn, s, x, y, WN_FG(wn), WN_BG(wn))
 
-#define wn_tputs(wn,s,x,y)	wn_ttext(wn, s, x, y, WN_FG, WN_BG)
+#define wn_tputs(wn,s,x,y)	wn_ttext(wn, s, x, y, WN_FG(wn), WN_BG(wn))
 
 /*  Old name for wn_bputs
  */
@@ -603,7 +614,7 @@ typedef void (*wn_abort_func_t)wn__PROTO((void));
 typedef void (*wn_deiconise_func_t)wn__PROTO((int wn));
 typedef void (*wn_draw_icon_func_t)wn__PROTO((int wn));
 typedef int (*wn_event_handler_func_t)wn__PROTO((event_t *ev));
-typedef void (*wn_quit_func_t)wn__PROTO((void));
+typedef void (*wn_close_func_t)wn__PROTO((int wn));
 typedef int (*wn_get_cur_posn_func_t)wn__PROTO((void));
 typedef void (*wn_overlay_insert_string_func_t)wn__PROTO((const char *text,
 							  int pos,
@@ -612,6 +623,9 @@ typedef void (*wn_overlay_insert_string_func_t)wn__PROTO((const char *text,
 							  int fg_color,
 							  int var_coloring_only));
 
+typedef void (*wn_clipboard_changed_func_t)wn__PROTO((void*));
+typedef void (*wn_iconify_notify_funct)wn__PROTO((int wn, int iconisize));
+
 /*  Old name for wn_get_window_handle(), kept for backwards compatibility.
  */
 #define wn_menu_id(wn)	wn_get_window_handle(wn)
@@ -623,13 +637,16 @@ typedef void (*wn_overlay_insert_string_func_t)wn__PROTO((const char *text,
 int wn_use_mono wn__PROTO((void));
 void wn_use_color_hint wn__PROTO((int use_color));
 void wn_npixels_hint wn__PROTO((int npixels));
-int wn_get_pixels_by_color wn__PROTO((color_t *colors, int ncolors));
-int wn_get_pixels wn__PROTO((color_t *colors, int ncolors));
-int wn_get_pixels_and_planes wn__PROTO((int npixels, int nplanes, int contig, int *pixels, int *p_planes));
-void wn_set_pixel_colors wn__PROTO((color_t *colors, int ncolors));
-void wn_get_pixel_colors wn__PROTO((color_t *colors, int ncolors));
-void wn_free_pixels wn__PROTO((color_t *colors, int ncolors));
-int wn_parse_color wn__PROTO((const char *name, color_t *color));
+int wn_get_pixels_by_color wn__PROTO((int wn, color_t *colors, int ncolors));
+int wn_get_pixels wn__PROTO((int wn, color_t *colors, int ncolors));
+int wn_get_pixels_and_planes wn__PROTO((int wn, int npixels, int nplanes, int contig, unsigned long *pixels, unsigned long *p_planes));
+void wn_set_pixel_colors wn__PROTO((int wn, color_t *colors, int ncolors));
+void wn_get_pixel_colors wn__PROTO((int wn, color_t *colors, int ncolors));
+void wn_free_pixels wn__PROTO((int wn, color_t *colors, int ncolors));
+int wn_parse_color wn__PROTO((int wn, const char *name, color_t *color));
+int wn_alloc_color wn__PROTO((int wn, color_t *color, const char *colorname));
+unsigned long wn_fg_pixel wn__PROTO((int wn));
+unsigned long wn_bg_pixel wn__PROTO((int wn));
 
 /* fonts */
 void wn_set_sysfont wn__PROTO((font_t *font));
@@ -700,9 +717,11 @@ void wn_suggest_resize_inc wn__PROTO((int base_width, int base_height,
 						int width_inc, int height_inc));
 void wn_set_classname wn__PROTO((const char *name));
 const char *wn_open_display wn__PROTO((void));
-void wn_set_quit_func wn__PROTO((wn_quit_func_t func));
+void wn_set_close_func wn__PROTO((wn_close_func_t func));
+void wn_set_icon_notify_func wn__PROTO((wn_iconify_notify_funct func));
 int wn_open_stdwin wn__PROTO((void));
-int wn_create_window wn__PROTO((const char *name));
+int wn_open_scr_stdwin wn__PROTO((const char* name, int xscreen));
+int wn_create_window wn__PROTO((const char *name, int xscreen));
 int wn_from_wid wn__PROTO((int wid));
 int wn_install_wid wn__PROTO((long wid));
 int wn_create_subwin wn__PROTO((int parwn, int x, int y, int width, int height, int type));
@@ -721,11 +740,13 @@ int wn_munge_args wn__PROTO((int argc, const char **argv));
 const char **wn_unmunge_args wn__PROTO((const char **args, int pos));
 int wn_get_wm_type wn__PROTO((void));
 const char *wn_get_default wn__PROTO((const char *name));
+void wn_set_default wn__PROTO((const char *name, const char* value));
 void wn_bell wn__PROTO((int wn));
 int wn_strnwidth wn__PROTO((const char *s, int nchars, font_t *font));
 int wn_strpos wn__PROTO((const char *s, int x, font_t *font, int halfshift));
 wn_abort_func_t wn_set_abort_func wn__PROTO((int wn, wn_abort_func_t func));
 const char *wn_version wn__PROTO((void));
+int wn_multi_screen wn__PROTO((void));
 
 /* recording and replaying of events */
 int wn_set_record_file wn__PROTO((const char *filename));
@@ -735,16 +756,28 @@ int wn_record_event wn__PROTO((const event_t *ev));
 int wn_get_recorded_event wn__PROTO((event_t *ev));
 
 /* selection handling */
-void wn_set_selection wn__PROTO((const char *buf, int nbytes));
-void wn_get_selection wn__PROTO((const char **p_buf, int *p_nbytes));
+void wn_set_selection wn__PROTO((int wn, const char *buf, int nbytes));
+void wn_get_selection wn__PROTO((int wn, const char **p_buf, int *p_nbytes));
+
+  /* Use the following two functions instead to arrange that the selection
+  ** stays highlighted until another selection is made.
+  **   funct_parm is called when the selection changes (RCB).
+  */
+void wn_set_xclipboard wn__PROTO((int wn, const char* buf, int nbytes,
+				long timestamp, wn_clipboard_changed_func_t,
+				void* funct_parm));
+void wn_clr_xclipboard wn__PROTO((void));
+
 
 /* getting window attributes */
 void wn_get_window_size wn__PROTO((int wn, int *p_width, int *p_height));
+int wn_get_root_window wn__PROTO((int wn));
+int wn_same_root_window wn__PROTO((int wn1, int wn2));
 const char *wn_get_window_name wn__PROTO((int wn));
 long wn_get_win_data wn__PROTO((int wn));
 int wn_get_width wn__PROTO((int wn));
 int wn_get_height wn__PROTO((int wn));
-int wn_get_nplanes wn__PROTO((void));
+int wn_get_nplanes wn__PROTO((int wn));
 int wn_use_alloc_color wn__PROTO((int set));
 int wn_use_alloc_color_for_highlight wn__PROTO((int set));
 int wn_use_menu_colors wn__PROTO((void));
@@ -758,6 +791,13 @@ void wn_change_win wn__PROTO((int wn, int parwn, int x, int y, int width, int he
 void wn_swap_wins wn__PROTO((int wn1, int wn2));
 void wn_adjust_win_size wn__PROTO((window_t wn, int x_delta, int y_delta, int w_delta, int h_delta));
 void wn_set_win_size wn__PROTO((window_t wn, int width, int height));
+void wn_set_window_hints wn__PROTO((int wn, const char* title, const char* icontitle, bitmap_t* icon));
+
+void wn_raise_window wn__PROTO((int wn));
+void wn_map_window wn__PROTO((int wn));
+void wn_unmap_window wn__PROTO((int wn));
+void wn_iconify_window wn__PROTO((int wn));
+void wn_lower_window wn__PROTO((int wn));
 
 /* coordinate transformation */
 int wn_trans_coords wn__PROTO((int oldwn, int x, int y, int newwn, int *p_x, int *p_y));
@@ -788,7 +828,7 @@ wn_event_handler_func_t wn_interpose_event_handler
 
 /* special purpose, unsupported (mostly for iclguide) */
 void wn_get_root_wpos wn__PROTO((int wn, int *p_x, int *p_y));
-int wn_open_rootwindow wn__PROTO((void));
+int wn_open_rootwindow wn__PROTO((int xscreen));
 int wn_windowfd wn__PROTO((int wn));
 void _wn_want_own_icon wn__PROTO((void));
 void _wn_dont_map_windows_on_create wn__PROTO((void));
@@ -796,6 +836,5 @@ void _wn_map_iclguide_window wn__PROTO((int wn));
 void set_window_error_handler wn__PROTO((void));
 int wn_expose_or_press_queued wn__PROTO((int wn, int md, int *p_x, int *p_y,
 					 int *exposed, int *resized));
-int wn_alloc_color wn__PROTO((color_t *color, const char *colorname));
 int wn_text_capture wn__PROTO((int set, int reset, int *y));
 #endif /* !WN_H_INCLUDED */
