@@ -19,8 +19,10 @@ char wn_wn_color_c_sccsid[] = "@(#)wn_color.c	1.16 1/3/93 (UKC)";
 #include "wn_misc.h"
 #include "wn_cu.h"
 
-static void set_mono_colors PROTO((color_t *color, int ncolors));
+static void set_mono_colors PROTO((int wn, color_t *color, int ncolors));
 
+/*  These flags are really specific to UPS and shoudl not be here.
+ */
 int wn__Use_mono = FALSE;
 int wn__Use_alloc_color = FALSE;
 int wn__Use_alloc_color_for_highlight = FALSE;
@@ -41,59 +43,93 @@ static int get_pixnum PROTO((char *s));
 #undef WHITE
 
 #include <sunwindow/cms_mono.h>
-#endif
+#endif /* SUNVIEW */
+
+unsigned long
+wn_fg_pixel (wn)
+int wn;
+{
+    swin_t *w;
+    W_CHECK(wn);
+    w = WN_TO_W(wn);
+    return W_FG_PIXEL(w);
+}
+
+unsigned long
+wn_bg_pixel (wn)
+int wn;
+{
+    swin_t *w;
+    W_CHECK(wn);
+    w = WN_TO_W(wn);
+    return W_BG_PIXEL(w);
+}
+
 
 int
 wn_use_alloc_color(set)
 int set;
 {
 #ifdef X11
-  if (set)
-    wn__Use_alloc_color = TRUE;
-  return wn__Use_alloc_color;
+    if (set)
+	wn__Use_alloc_color = TRUE;
+    return wn__Use_alloc_color;
 #endif
 #ifdef SUNVIEW
-	return 0; 
+    return 0; 
 #endif /* SUNVIEW */
 }
+
 int
 wn_use_alloc_color_for_highlight(set)
 int set;
 {
-  if (set)
-    wn__Use_alloc_color_for_highlight = TRUE;
-  return wn__Use_alloc_color_for_highlight;
+    if (set)
+	wn__Use_alloc_color_for_highlight = TRUE;
+    return wn__Use_alloc_color_for_highlight;
 }
 
 int
 wn_use_menu_colors()
 {
-  return wn__Use_menu_colors;
-}
-
-wn_use_extended_text_colors()
-{
-  return wn__Use_extended_text_colors;
+    return wn__Use_menu_colors;
 }
 
 int
-wn_get_nplanes()
+wn_use_extended_text_colors()
 {
+    return wn__Use_extended_text_colors;
+}
+
+int
+wn_get_nplanes(wn)
+int wn;
+{
+	register swin_t *w;
+
+	W_CHECK(wn);
+	w = WN_TO_W(wn);
 #ifdef X11
-	return DisplayPlanes(wn__Dpy, DefaultScreen(wn__Dpy));
+	return DisplayPlanes(wn__Dpy, W_SCREEN(w));
 #endif
 #ifdef SUNVIEW
-	return WN_TO_W(WN_STDWIN)->w_pw->pw_pixrect->pr_depth;
+	return w->w_pw->pw_pixrect->pr_depth;
 #endif /* SUNVIEW */
 }
 
 static void
-set_mono_colors(colors, ncolors)
+set_mono_colors(wn, colors, ncolors)
+int wn;
 color_t *colors;
 int ncolors;
 {
+	swin_t *w;
+
+	W_CHECK(wn);
+	w = WN_TO_W(wn);
+
 	while (ncolors > 0)
-		colors[--ncolors].co_pixel = WN_FG;
+		colors[--ncolors].co_pixel = W_FG_PIXEL(w);
 }
 
 #ifdef SUNVIEW
@@ -119,7 +155,8 @@ static int Initial_cms_size = 2;
 #endif /* SUNVIEW */
 
 void
-wn_free_pixels(colors, ncolors)
+wn_free_pixels(wn, colors, ncolors)
+int wn;
 color_t *colors;
 int ncolors;
 {
@@ -132,6 +169,10 @@ int ncolors;
 #ifdef X11
 	int i;
 	xpixel_t *pixels;
+	swin_t *w;
+
+	W_CHECK(wn);
+	w = WN_TO_W(wn);
 
 	if (DisplayPlanes(wn__Dpy, DefaultScreen(wn__Dpy)) == 1) {
 		return;
@@ -139,7 +180,7 @@ int ncolors;
 	pixels = (xpixel_t *) wn__e_malloc(ncolors * sizeof(int));
 	for (i = 0; i < ncolors; i++)
 		pixels[i] = colors[i].co_pixel;
-	XFreeColors(wn__Dpy, _wn_Cmap, pixels, ncolors, (xplanemask_t)0);
+	XFreeColors(wn__Dpy, W_CMAP(w), pixels, ncolors, (planemask_t)0);
 	free((char *)pixels);
 #endif /* X11 */
 }
@@ -339,9 +380,10 @@ int *pixels, *p_planes;
 #endif /* SUNVIEW */
 
 int
-wn_get_pixels_and_planes(npixels, nplanes, contig, pixels, p_planes)
+wn_get_pixels_and_planes(wn, npixels, nplanes, contig, pixels, p_planes)
+int wn;
 int npixels, nplanes, contig;
-int *pixels, *p_planes;
+unsigned long *pixels, *p_planes;
 {
 #ifdef SUNVIEW
 	int pixel, nfree, nwanted, new_cms_size;
@@ -386,25 +428,28 @@ int *pixels, *p_planes;
 	return 0;
 #endif /* SUNVIEW */
 #ifdef X11
-	unsigned long planes;
+	planemask_t plane_mask;
+	swin_t *w;
+
+	W_CHECK(wn);
+	w = WN_TO_W(wn);
 
 	if (wn__Use_mono)
 		return -1;
 
-	/*  BUG: we cast pixels to "unsigned long *" from "unsigned int *".
-	 *  Revolting int == long assumption.
-	 */
-	if (XAllocColorCells(wn__Dpy, _wn_Cmap, contig,
-			     &planes, (unsigned)nplanes,
-			     (unsigned long *)pixels, (unsigned)npixels) == 0)
+	if (_Xdebug) printf("ups: wn_get_pixels_and_planes(%d,%d)\n", npixels, nplanes);
+	if (XAllocColorCells(wn__Dpy, W_CMAP(w), contig,
+			     &plane_mask, (unsigned)nplanes,
+			     pixels, (unsigned)npixels) == 0)
 		return -1;
-	*p_planes = planes;
+	*p_planes = plane_mask;
 	return 0;
 #endif /* X11 */
 }
 
 int
-wn_get_pixels(colors, ncolors)
+wn_get_pixels(wn, colors, ncolors)
+int wn;
 color_t *colors;
 int ncolors;
 {
@@ -412,7 +457,7 @@ int ncolors;
 	int pixel, cno, nwanted, minsize, new_cms_size;
 
 	if (Cms_size == 0) {
-		set_mono_colors(colors, ncolors);
+		set_mono_colors(wn, colors, ncolors);
 		return 0;
 	}
 
@@ -447,20 +492,25 @@ int ncolors;
 #endif /* SUNVIEW */
 #ifdef X11
 	int i;
-	xplanemask_t junk_planemask;
+	planemask_t junk_planemask;
 	xpixel_t *pixels;
+	swin_t *w;
+
+	W_CHECK(wn);
+	w = WN_TO_W(wn);
 
 	if (wn__Use_mono)
 		return -1;
 
-	if (DisplayPlanes(wn__Dpy, DefaultScreen(wn__Dpy)) == 1) {
-		set_mono_colors(colors, ncolors);
+	if (DisplayPlanes(wn__Dpy, W_SCREEN(w)) == 1) {
+		set_mono_colors(wn, colors, ncolors);
 		return 0;
 	}
+	if (_Xdebug) printf("ups: wn_get_pixels(%d)\n", ncolors);
 
 	pixels = (xpixel_t *) wn__e_malloc(ncolors * sizeof(int));
-	if (XAllocColorCells(wn__Dpy, _wn_Cmap, FALSE, &junk_planemask,
-						0, pixels, (unsigned)ncolors) == 0) {
+	if (XAllocColorCells(wn__Dpy, W_CMAP(w), FALSE, &junk_planemask,
+					0, pixels, (unsigned)ncolors) == 0) {
 		free((char *)pixels);
 		return -1;
 	}
@@ -472,7 +522,8 @@ int ncolors;
 }
 
 void
-wn_get_pixel_colors(colors, ncolors)
+wn_get_pixel_colors(wn, colors, ncolors)
+int wn;
 color_t *colors;
 int ncolors;
 {
@@ -491,15 +542,19 @@ int ncolors;
 #ifdef X11
 	XColor cdef;
 	color_t *co;
+	swin_t *w;
 
-	if (DisplayPlanes(wn__Dpy, DefaultScreen(wn__Dpy)) == 1) {
+	W_CHECK(wn);
+	w = WN_TO_W(wn);
+
+	if (DisplayPlanes(wn__Dpy, W_SCREEN(w)) == 1) {
 		for (co = colors; co < colors + ncolors; co++)
 			co->co_red = co->co_green = co->co_blue = 0xffff;
 	}
 	else {
 		for (co = colors; co < colors + ncolors; co++) {
-			cdef.pixel = co->co_pixel;
-			XQueryColor(wn__Dpy, _wn_Cmap, &cdef);
+			cdef.pixel = PIX(w,co->co_pixel);
+			XQueryColor(wn__Dpy, W_CMAP(w), &cdef);
 			co->co_red = cdef.red;
 			co->co_green = cdef.green;
 			co->co_blue = cdef.blue;
@@ -509,17 +564,21 @@ int ncolors;
 }
 
 int
-wn_parse_color(name, color)
+wn_parse_color(wn, name, color)
+int wn;
 const char *name;
 color_t *color;
 {
 #ifdef X11
 	XColor xcolor;
+	swin_t *w;
 
-	if (XParseColor(wn__Dpy, _wn_Cmap, name, &xcolor) == 0)
-	{
-	  fprintf(stderr, "ups: Failed to find color `%s'\n", name);
-	  return -1;
+	W_CHECK(wn);
+	w = WN_TO_W(wn);
+
+	if (XParseColor(wn__Dpy, W_CMAP(w), name, &xcolor) == 0) {
+		fprintf(stderr, "ups: Failed to find color `%s'\n", name);
+		return -1;
 	}
 	color->co_red = xcolor.red;
 	color->co_blue = xcolor.blue;
@@ -532,7 +591,8 @@ color_t *color;
 }
 
 void
-wn_set_pixel_colors(colors, ncolors)
+wn_set_pixel_colors(wn, colors, ncolors)
+int wn;
 color_t *colors;
 int ncolors;
 {
@@ -563,8 +623,12 @@ int ncolors;
 	XColor *cdefs;
 	register XColor *cdef;
 	register color_t *co;
+	swin_t *w;
 
-	if (DisplayPlanes(wn__Dpy, DefaultScreen(wn__Dpy)) > 1) {
+	W_CHECK(wn);
+	w = WN_TO_W(wn);
+
+	if (DisplayPlanes(wn__Dpy, W_SCREEN(w)) > 1) {
 		cdefs = (XColor *) wn__e_malloc(ncolors * sizeof(XColor));
 		cdef = cdefs;
 		for (co = colors; co < colors + ncolors; co++, cdef++) {
@@ -572,27 +636,26 @@ int ncolors;
 			cdef->red = co->co_red;
 			cdef->green = co->co_green;
 			cdef->blue = co->co_blue;
-#ifdef X11
 			cdef->flags = DoRed | DoGreen | DoBlue;
-			if (cdef->pixel == WN_FG)
+			if (cdef->pixel == W_FG_PIXEL(w))
 				wn__set_x11_cursor_colors(cdef, (XColor *)NULL);
-			if (cdef->pixel == WN_BG)
+			if (cdef->pixel == W_BG_PIXEL(w))
 				wn__set_x11_cursor_colors((XColor *)NULL, cdef);
-#endif /* X11 */
 		}
-		XStoreColors(wn__Dpy, _wn_Cmap, cdefs, ncolors);
+		XStoreColors(wn__Dpy, W_CMAP(w), cdefs, ncolors);
 		free((char *)cdefs);
 	}
 #endif /* X11 */
 }
 
 int
-wn_get_pixels_by_color(colors, ncolors)
+wn_get_pixels_by_color(wn, colors, ncolors)
+int wn;
 color_t *colors;
 int ncolors;
 {
-	if (wn_get_pixels(colors, ncolors) != 0)
+	if (wn_get_pixels(wn, colors, ncolors) != 0)
 		return -1;
-	wn_set_pixel_colors(colors, ncolors);
+	wn_set_pixel_colors(wn, colors, ncolors);
 	return 0;
 }
