@@ -79,6 +79,7 @@ static Sigfunc safe_signal PROTO((int sig, Sigfunc func));
  *  See wn_pushback_event() and wn_next_event().
  */
 static event_t Pushedback_event;
+static wn_iconify_notify_funct icon_notify_funct;
 
 /*  Timeout when doing input in sample mode.  Set by wn_set_input_timeout().
  *  Default is no timeout (e.g. wait indefinitely).
@@ -407,21 +408,17 @@ int *p_xyset;
 		return;
 	}
 
-#ifdef SUNVIEW
 	if (wn__Lost_selection) {
 		ev->ev_type = EV_LOST_SELECTION;
 		wn__Lost_selection = FALSE;
 	} else
-#endif /* SUNVIEW */
 	{
 		ev->ev_type = select_on_mask(mask, sample, &resmask);
 	}
-#ifdef SUNVIEW
 	if (wn__Lost_selection) {
 		ev->ev_type = EV_LOST_SELECTION;
 		wn__Lost_selection = FALSE;
 	}
-#endif /* SUNVIEW */
 
 	if (w != NULL)
 		fd = w->w_pw->pw_windowfd;
@@ -515,10 +512,12 @@ int *p_xyset;
 	XKeyEvent *xk;
 	XButtonEvent *xb;
 	int sample;
+	/* UPS split windows. */
+	int split = (wn_NRootwin>1);
 	int old_width, old_height, xbuttons, nbytes;
 	char ch;
 	char *keystr;
-	register swin_t *w;
+	register swin_t *w, *w1, *w2;
 	extern int _wn_Autoraise;
 
 	if (wn == WN_ANY) {
@@ -550,15 +549,15 @@ int *p_xyset;
 		}
 	}
 	if (sample) {
-		if (!XCheckWindowEvent(wn__Dpy, w->w_win, REQUEST_MASK, &x_event)) {
-			ev->ev_type = EV_MOUSE_MOVED;
-			ev->ev_wn = w->w_wn;
-			ev->ev_buttons = w->w_bw->bw_buttons;
-			return;
-		}
+	    if (!XCheckWindowEvent(wn__Dpy, w->w_win, REQUEST_MASK, &x_event)) {
+	      ev->ev_type = EV_MOUSE_MOVED;
+	      ev->ev_wn = w->w_wn;
+	      ev->ev_buttons = w->w_bw->bw_buttons;
+	      return;
+	    }
 	}
 	else {
-		if (w != NULL) {
+		if (w != NULL && !split) {
 			Window win;
 
 			if (w->w_bw->bw_is_mapped)
@@ -617,8 +616,9 @@ int *p_xyset;
 	switch(x_event.type) {
 	case EnterNotify:
 		ev->ev_type = EV_WINDOW_SELECTED;
-		ev->ev_x = ((XEnterWindowEvent *)&x_event)->x;
-		ev->ev_y = ((XEnterWindowEvent *)&x_event)->y;
+		ev->ev_time = ((XEnterWindowEvent *)&x_event)->time;
+		ev->ev_x    = ((XEnterWindowEvent *)&x_event)->x;
+		ev->ev_y    = ((XEnterWindowEvent *)&x_event)->y;
 		*p_xyset = TRUE;
 		if (_wn_Autoraise) {
 			XRaiseWindow(wn__Dpy, w->w_win);
@@ -627,8 +627,9 @@ int *p_xyset;
 		break;
 	case LeaveNotify:
 		ev->ev_type = EV_WINDOW_DESELECTED;
-		ev->ev_x = ((XLeaveWindowEvent *)&x_event)->x;
-		ev->ev_y = ((XLeaveWindowEvent *)&x_event)->y;
+		ev->ev_time = ((XLeaveWindowEvent *)&x_event)->time;
+		ev->ev_x    = ((XLeaveWindowEvent *)&x_event)->x;
+		ev->ev_y    = ((XLeaveWindowEvent *)&x_event)->y;
 		*p_xyset = TRUE;
 		break;
 	case Expose:
@@ -659,10 +660,11 @@ int *p_xyset;
 				break;
 			XNextEvent(wn__Dpy, &x_event);
 		}
-		ev->ev_x = ((XMotionEvent *)&x_event)->x;
-		ev->ev_y = ((XMotionEvent *)&x_event)->y;
-		*p_xyset = TRUE;
 		ev->ev_type = EV_MOUSE_MOVED;
+		ev->ev_time = ((XMotionEvent *)&x_event)->time;
+		ev->ev_x    = ((XMotionEvent *)&x_event)->x;
+		ev->ev_y    = ((XMotionEvent *)&x_event)->y;
+		*p_xyset = TRUE;
 		break;
 	case ButtonPress:
 		/* Monitor multiple clicks on ButtonPress
@@ -677,7 +679,7 @@ int *p_xyset;
 			numClicks = 0;
 	case ButtonRelease:
 		xb = (XButtonPressedEvent *)&x_event;
-		lastButtonTime = xb->time;
+		lastButtonTime = ev->ev_time = xb->time;
 
 		if (xb->button == 0){
 			ev->ev_type = EV_MOUSE_MOVED;
@@ -731,6 +733,7 @@ int *p_xyset;
 		}
 				
 		ev->ev_type = (nbytes > 0) ? EV_KEY : 0;
+		ev->ev_time = xk->time;
 		
 		if (nbytes > 0) /* BUG: what if nbytes > 1 ? */
 			ev->ev_char = *keystr;
@@ -749,9 +752,17 @@ int *p_xyset;
 		break;
 
 	case ClientMessage:
-		process_client_message(&x_event, w->w_win);
+		process_client_message(&x_event, w->w_win, w->w_wn);
 		ev->ev_type = EV_MOUSE_MOVED;
 		break;
+	case MapNotify:
+		if (icon_notify_funct)
+		   (*icon_notify_funct)(w->w_wn,FALSE);
+		break;
+	case UnmapNotify:
+		if (icon_notify_funct)
+		   (*icon_notify_funct)(w->w_wn,TRUE);
+		break;
     
 	default:
 		ev->ev_type = 0;
@@ -975,6 +986,11 @@ event_t *ev;
 		}
 		bw->bw_buttons = ev->ev_buttons;
 
+		if (ev->ev_type == EV_LOST_SELECTION)
+		{
+			wn_clr_xclipboard();
+		}
+
 		if ((mask & ev->ev_type) == 0)
 			continue;
 
@@ -984,7 +1000,10 @@ event_t *ev;
 			 bw->bw_winch_event == 0)
 			bw->bw_winch_event = EV_WINDOW_EXPOSED;
 		else if (ev->ev_type == EV_LOST_SELECTION)
+		{
 			bw->bw_had_lost_selection_event = TRUE;
+			wn_clr_xclipboard();
+		}
 
 		if (Event_handler == NULL || (*Event_handler)(ev) == 0)
 			break;
@@ -1034,7 +1053,10 @@ event_t *ev;
 	else if (ev->ev_type == EV_WINDOW_EXPOSED && w->w_bw->bw_winch_event == 0)
 		w->w_bw->bw_winch_event = EV_WINDOW_EXPOSED;
 	else if (ev->ev_type == EV_LOST_SELECTION)
+	{
 		w->w_bw->bw_had_lost_selection_event = TRUE;
+		wn_clr_xclipboard();
+	}
 	ev->ev_x += w->w_x_offs;
 	ev->ev_y += w->w_y_offs;
 	ev->ev_wn = w->w_base->w_wn;
@@ -1224,7 +1246,7 @@ swin_t *w;
 {
 #ifdef X11
 	XSelectInput(wn__Dpy, w->w_win,
-		     ButtonPressMask | ButtonReleaseMask | ExposureMask);
+	 ButtonPressMask | ButtonReleaseMask | ExposureMask | StructureNotifyMask);
 	XSync(wn__Dpy, TRUE);
 #endif
 #ifdef SUNVIEW
@@ -1268,6 +1290,7 @@ wn_expose_or_press_queued(wn, md, p_x, p_y, exposed, resized)
   swin_t *w;
   event_t event;
   int x, y, xyset, ret = 0;
+  unsigned long win;
 
   if (_wn_Windowtab)
   {
@@ -1276,7 +1299,7 @@ wn_expose_or_press_queued(wn, md, p_x, p_y, exposed, resized)
     ev.type = KeyPress;
     ev.display = wn__Dpy;
     ev.window = WN_TO_W(WN_STDWIN)->w_win;
-    XSendEvent(wn__Dpy, ev.window, False, 0L, (XEvent *)&ev); 
+    XSendEvent(wn__Dpy, ev.window, False, 0L, (XEvent *)&ev);
     *exposed = *resized = 0;
 
     for(;;)
@@ -1301,7 +1324,6 @@ wn_expose_or_press_queued(wn, md, p_x, p_y, exposed, resized)
       else
       if (event.ev_type == EV_WINDOW_RESIZED)
 	*resized = 1;
-
       if (x_event.xany.send_event)
 	return ret;
     }
@@ -1309,3 +1331,9 @@ wn_expose_or_press_queued(wn, md, p_x, p_y, exposed, resized)
 #endif /* X11 */
   return 0;
 }
+void
+wn_set_icon_notify_func(func)
+wn_iconify_notify_funct func;
+{
+    icon_notify_funct = func;
+}
