@@ -621,14 +621,15 @@ int len, x, y, ropfunc, fg_color, bg_color, use_which, mono;
 		y += font->ft_baseline;
 
 	SC_UNDRAW(wn);
-	XSetState(wn__Dpy, _wn_Gc, (xpixel_t)fg_color, (xpixel_t)bg_color,
+	XSetState(wn__Dpy, W_GC(w),
+			PIX(w,fg_color), PIX(w,bg_color),
 			_wn_Roptab[ropfunc],
-			mono ? _wn_Planemask : (unsigned)AllPlanes);
-	XSetFont(wn__Dpy, _wn_Gc, ((XFontStruct *)font->ft_mdfont)->fid);
+			mono ? W_PLANEMASK(w) : (unsigned)AllPlanes);
+	XSetFont(wn__Dpy, W_GC(w), ((XFontStruct *)font->ft_mdfont)->fid);
 	if (bg_color == WN_TRANSPARENT)
-		XDrawString(wn__Dpy, w->w_win, _wn_Gc, x, y, s, len);
+		XDrawString(wn__Dpy, w->w_win, W_GC(w), x, y, s, len);
 	else
-		XDrawImageString(wn__Dpy, w->w_win, _wn_Gc, x, y, s, len);
+		XDrawImageString(wn__Dpy, w->w_win, W_GC(w), x, y, s, len);
 	SC_REDRAW(wn);
 	X_UPDATE(w);
 #endif /* X11 */
@@ -641,7 +642,7 @@ int len, x, y, ropfunc, fg_color, bg_color, use_which, mono;
 	pf = (machfont_t)font->ft_mdfont;
 
 	w->w_bw->bw_can_refresh = FALSE;
-	if (bg_color == WN_BG)
+	if (bg_color == W_BG_PIXEL(w))
 		pw_text(w->w_pw, x, y, op, pf, s);
 	else {
 		if (bg_color != WN_TRANSPARENT) {
