@@ -42,10 +42,10 @@ char wn_wn_init_c_sccsid[] = "@(#)wn_init.c	1.53 25 May 1995 (UKC)";
 #include "wn_color.h"
 #include "sccsdata.h"
 
-static void default_quit_func PROTO((void));
-static void default_quit_func() {};
+static void default_close_func PROTO((int wn));
+static void default_close_func(wn) {}
 
-static wn_quit_func_t Quit_func = default_quit_func;
+static wn_close_func_t Close_func = default_close_func;
 
 #if !defined(__STDC__) || (defined(WANT_STRERROR) && !defined(strerror))
 extern char *sys_errlist[];
@@ -55,15 +55,16 @@ extern int sys_nerr;
 #endif
 
 static int change_env_var PROTO((const char *s));
+static screen_t *new_screen PROTO((int s_screen));
 
 #ifdef X11
 static const char *get_xdef PROTO((const char *name, const char *def));
 static int create_x_window PROTO((swin_t *w, const char *name,
 						const char *geometry));
-static void create_gc PROTO((void));
-static void set_colors PROTO((void));
+static void set_x_gc PROTO((screen_t *scr));
+static void set_x_colors PROTO((screen_t *scr));
 static Display *open_display PROTO((const char *displayname, const char **p_errmsg));
-static void get_def_color PROTO((const char *defaultname, xpixel_t def_pixel, int def_rgb_val, const char *argspec, XColor *color));
+static void get_def_color PROTO((const char *defaultname, xpixel_t def_pixel, int def_rgb_val, const char *argspec, XColor *color, Colormap cmap));
 static int   /* RCB: Changed from type void to match X spec */
 non_fatal_x_error_handler PROTO((Display *displayname, XErrorEvent *err));
 static void initialize_atoms PROTO((Window p_win));
@@ -95,18 +96,6 @@ static int App_xpos, App_ypos, App_width, App_height;
 static int App_width_inc, App_height_inc, App_base_width, App_base_height;
 static bool App_suggests_resize_inc = FALSE;
 
-/*  Values for the foreground and background pixels.
- *  Initialised for mono displays - color ones overwrite them.
- */
-int _wn_Bg_pixel = 0;
-int _wn_Fg_pixel = 1;
-#ifdef X11
-xplanemask_t _wn_Planemask = 0x1;
-#endif /* X11 */
-#ifdef SUNVIEW
-int _wn_Planemask = 0x1;
-#endif /* SUNVIEW */
-
 int wn__Batching_enabled = TRUE;
 
 #ifdef X11
@@ -132,20 +121,18 @@ static int Want_to_start_as_icon = FALSE;
 
 #ifdef X11
 Display *wn__Dpy = NULL;
-#ifdef X11
-Colormap _wn_Cmap;
-extern int _Xdebug;
 #endif /* X11 */
 
-GC _wn_Gc;
-
 static const char *Default_display = NULL;
+static screen_t *Head_screen = NULL;
 
 static const char *Bg_color_name = NULL, *Fg_color_name = NULL;
 int _wn_Autoraise = FALSE;
 const char *_wn_Icon_geometry = NULL;
 
 static bool Want_reverse_video = FALSE;
+
+#ifdef X11
 static int   /* RCB: Changed from type void to match X spec */
 non_fatal_x_error_handler(d, err)
      Display *d;
@@ -160,11 +147,13 @@ non_fatal_x_error_handler(d, err)
 }
 
 /*
- * Process quit on frame menu
+ * Process close on frame menu
  */
-void process_client_message(event, frame)
+void
+process_client_message(event, frame, wn)
      XEvent *event;
      Window frame;
+     int wn;
 
 {
   XClientMessageEvent *xclient	= &event->xclient;
@@ -175,7 +164,7 @@ void process_client_message(event, frame)
   {
     protocol = event->xclient.data.l[0];
     if( protocol == atom_wm_delete_window)
-      (*Quit_func)();
+      (*Close_func)(wn);
   }
 }
 
@@ -202,7 +191,8 @@ static void initialize_atoms(frame)
 
 #endif /* X11 */
 
-void set_window_error_handler()
+void
+set_window_error_handler()
 {
 #ifdef X11
   XSetErrorHandler(non_fatal_x_error_handler);
@@ -221,6 +211,7 @@ const char *name;
 {
 	Appclass = name;
 }
+
 #ifdef X11
 XrmDatabase resourcesDB;
 static  void wn_merge_databases()
@@ -239,9 +230,13 @@ static  void wn_merge_databases()
     const char* xenvironment = getenv("XENVIRONMENT");
     XrmDatabase serverDB = 0, homeDB = 0;
 
-    /*  Get the defaults from the standard location*/
+    /* Get the defaults from the standard location. */
 
-    sprintf(path, "/usr/lib/X11/app-defaults/%s", Appclass);
+    sprintf(path, "/usr/X11R6/lib/X11/app-defaults/%s", Appclass);
+    if (access (path, F_OK) != 0)
+	sprintf(path, "/usr/lib/X11/app-defaults/%s", Appclass);
+    if (access (path, F_OK) != 0)
+	sprintf(path, "/usr/openwin/lib/app-defaults/%s", Appclass);
     applicationDB = XrmGetFileDatabase(path);
     XrmMergeDatabases(applicationDB,&resourcesDB);
 
@@ -333,12 +328,7 @@ const char *name;
 	if (wn__Dpy == NULL)
 		return NULL;
 
-	value = XGetDefault(wn__Dpy, Appname, name);
-
-	if (value == NULL && Appclass != NULL && strcmp(Appclass, Appname) != 0)
-		value = XGetDefault(wn__Dpy, Appclass, name);
-
-	if (value == NULL && resourcesDB != NULL) {
+	if (resourcesDB != NULL) {
 		char app_name[512], class_name[512];
 		XrmValue xrm_value;
 		char *str_type;
@@ -371,6 +361,18 @@ const char *name;
 #endif /* !X11 */
 }
 
+void
+wn_set_default(name, value)
+const char* name;
+const char* value;
+{
+#ifdef X11
+    char buff[100];
+    sprintf(buff,"%s.%s:%s", Appname, name,value);
+    XrmPutLineResource(&resourcesDB, buff);
+#endif /* X11 */
+}
+
 int
 wn_use_mono()
 {
@@ -504,10 +506,10 @@ const char **argv;
 
 	if (getenv("KSTMONO") != NULL)
 	{
-	  wn__Use_mono = TRUE;
-	  wn__Use_alloc_color = FALSE;
-	  wn__Use_menu_colors = FALSE;
-	  wn__Use_extended_text_colors = FALSE;
+		wn__Use_mono = TRUE;
+		wn__Use_alloc_color = FALSE;
+		wn__Use_menu_colors = FALSE;
+		wn__Use_extended_text_colors = FALSE;
 	}
 	if (getenv("KSTCOLOR") != NULL || getenv("KSTCOLOUR") != NULL)
 		wn__Use_mono = FALSE;
@@ -619,7 +621,15 @@ const char **argv;
 int
 wn_open_stdwin()
 {
-	return wn_create_window(Appname);
+	return wn_create_window(Appname, -1);
+}
+
+int
+wn_open_scr_stdwin(name,xscreen)
+const char* name;
+int xscreen;
+{
+	return wn_create_window(name, xscreen);
 }
 
 /*  return sys_errlist[errno] if in range
@@ -630,27 +640,91 @@ _wn_reason()
 	return (strerror (errno));
 } 
 
+
+screen_t *
+_wn_first_screen()
+{
+	return (Head_screen);
+}
+
+int
+wn_multi_screen()
+{
+	screen_t *ptr;
+
+	if (((ptr = _wn_first_screen()) == NULL) || (ptr->s_next == NULL))
+		return FALSE;
+	return TRUE;
+}
+
+static screen_t *
+new_screen(s_screen)
+int s_screen;
+{
+	screen_t *scr;
+	int i;
+
+	scr = (screen_t *)wn__e_malloc(sizeof(screen_t));
+	scr->s_screen = s_screen;
+	scr->s_planemask = 0x1;
+	scr->s_bg_pixel = 0;
+	scr->s_fg_pixel = 1;
+#ifdef X11
+	/* Must do colours first as used in setting up GC. */
+	set_x_colors (scr);
+	set_x_gc (scr);
+	scr->s_grey_pixmap = 0;
+#if WANT_MULTI_SCREEN
+	scr->s_npixel = 16;
+	scr->s_pixels = (xpixel_t *) wn__e_malloc(scr->s_npixel * sizeof(xpixel_t));
+	for (i = 0; i < scr->s_npixel; i++)
+		scr->s_pixels[i] = WN_MY_PIXEL;
+	scr->s_pixels[0] = scr->s_fg_pixel;
+	scr->s_pixels[1] = scr->s_bg_pixel;
+#endif
+#endif /* X11 */
+	scr->s_next = Head_screen;
+	Head_screen = scr;
+	return (scr);
+}
+
+screen_t *
+_wn_get_screen(s_screen)
+int s_screen;
+{
+	screen_t *ptr;
+
+	ptr = Head_screen;
+	while (ptr != NULL) {
+		if (ptr->s_screen == s_screen)
+			return (ptr);
+		ptr = ptr->s_next;
+	}
+	return (new_screen (s_screen));
+}
+
 #ifdef X11
 static void
-get_def_color(defaultname, def_pixel, def_rgb_val, argspec, color)
+get_def_color(defaultname, def_pixel, def_rgb_val, argspec, color, cmap)
 const char *defaultname;
 xpixel_t def_pixel;
 int def_rgb_val;
 const char *argspec;
 XColor *color;
+Colormap cmap;
 {
 	const char *colorspec;
 
 	colorspec = (argspec != NULL) ? argspec : wn_get_default(defaultname);
 	if (colorspec == NULL ||
-			XParseColor(wn__Dpy, _wn_Cmap, colorspec, color) == 0) {
+			XParseColor(wn__Dpy, cmap, colorspec, color) == 0) {
 		color->pixel = def_pixel;
 
 		/*  Set up a default color in case XQueryColor fails.
 		 */
 		color->red = color->green = color->blue = def_rgb_val;
 
-		(void) XQueryColor(wn__Dpy, _wn_Cmap, color);
+		(void) XQueryColor(wn__Dpy, cmap, color);
 	}
 }
 		
@@ -661,26 +735,25 @@ XColor *color;
  *  values, such that WN_FG = WN_BG | _wn_Planemask.
  */
 static void
-set_colors()
+set_x_colors(scr)
+screen_t *scr;
 {
 	XColor fg_color, bg_color;
 	Visual *v;
 	xpixel_t black_pixel, white_pixel;
-	xplanemask_t planes;
+	planemask_t plane_mask;
 	bool want_mono;
 	xpixel_t rw_pixel;
 
-#ifdef X11
-	_wn_Cmap = DefaultColormap(wn__Dpy, DefaultScreen(wn__Dpy));
-#endif /* X11 */
+	scr->s_cmap = DefaultColormap(wn__Dpy, scr->s_screen);
 
-	black_pixel = BlackPixel(wn__Dpy, DefaultScreen(wn__Dpy));
-	white_pixel = WhitePixel(wn__Dpy, DefaultScreen(wn__Dpy));
+	black_pixel = BlackPixel(wn__Dpy, scr->s_screen);
+	white_pixel = WhitePixel(wn__Dpy, scr->s_screen);
 
 	get_def_color("Foreground", black_pixel, 0x0000,
-						Fg_color_name, &fg_color);
+					Fg_color_name, &fg_color, scr->s_cmap);
 	get_def_color("Background", white_pixel, 0xffff,
-						Bg_color_name, &bg_color);
+					Bg_color_name, &bg_color, scr->s_cmap);
 
 	if (Want_reverse_video) {
 		XColor tmp;
@@ -695,13 +768,19 @@ set_colors()
 	 *  color allocation problem - you don't want ups to perturb the
 	 *  color map by allocating cells.
 	 */
-	v = DefaultVisual(wn__Dpy, DefaultScreen(wn__Dpy));
+	v = DefaultVisual(wn__Dpy, scr->s_screen);
+	if (v == NULL)
+	{
+	      /* RCB: Error handling */
+	      fprintf(stderr, "%s: Cannot access screen %d\n", Appname, scr->s_screen);
+	      exit(1);
+	}
 	want_mono = wn__Use_mono || (v->class != PseudoColor &&
 						    v->class != DirectColor);
 
 	if (!want_mono && wn__Use_alloc_color == FALSE) {
-	  if (XAllocColorCells(wn__Dpy, _wn_Cmap, TRUE,
-			       &planes, 1, &rw_pixel, 1) == 0)
+	  if (XAllocColorCells(wn__Dpy, scr->s_cmap, TRUE,
+			       &plane_mask, 1, &rw_pixel, 1) == 0)
 	  {
 	    if (v->class == PseudoColor)
 	    {
@@ -733,22 +812,22 @@ set_colors()
 		    v->class == TrueColor) {
 			int i, j;
 
-			j = DisplayPlanes(wn__Dpy, DefaultScreen(wn__Dpy));
-			for (i = 0, planes = 1; i < j; ++i)
-				planes <<= 1;
-			--planes;
+			j = DisplayPlanes(wn__Dpy, scr->s_screen);
+			for (i = 0, plane_mask = 1; i < j; ++i)
+				plane_mask <<= 1;
+			--plane_mask;
 		}
 		else {
-			planes = 1;
+			plane_mask = 1;
 		}
 
 		if (wn__Use_mono ||
-				XAllocColor(wn__Dpy, _wn_Cmap, &fg_color) == 0)
+			    XAllocColor(wn__Dpy, scr->s_cmap, &fg_color) == 0)
 			fg_color.pixel = Want_reverse_video ? white_pixel
 							    : black_pixel;
 
 		if (wn__Use_mono ||
-				XAllocColor(wn__Dpy, _wn_Cmap, &bg_color) == 0)
+			    XAllocColor(wn__Dpy, scr->s_cmap, &bg_color) == 0)
 			bg_color.pixel = Want_reverse_video ? black_pixel
 							    : white_pixel;
 
@@ -757,8 +836,8 @@ set_colors()
 		  wn__Use_alloc_color = TRUE;
 		  wn__Use_alloc_color_for_highlight = TRUE;
 		}
-		_wn_Fg_pixel = fg_color.pixel;
-		_wn_Bg_pixel = bg_color.pixel;
+		scr->s_fg_pixel = fg_color.pixel;
+		scr->s_bg_pixel = bg_color.pixel;
 
 		/*  Wn has the assumption that logical operations work
 		 *  on monochrome displays as if a set pixel was a one
@@ -768,24 +847,20 @@ set_colors()
 		 *  BUG: what if the fg and bg pixels differ by more than
 		 *       one bit?
 		 */
-		if (_wn_Fg_pixel < _wn_Bg_pixel)
+		if (fg_color.pixel < bg_color.pixel)
 			_wn_fix_ropfuncs();
 	}
 	else {
-		fg_color.pixel = _wn_Fg_pixel = rw_pixel | planes;
-		bg_color.pixel = _wn_Bg_pixel = rw_pixel;
-#ifdef X11
+		scr->s_fg_pixel = fg_color.pixel = rw_pixel | plane_mask;
+		scr->s_bg_pixel = bg_color.pixel = rw_pixel;
 		fg_color.flags = bg_color.flags = DoRed | DoGreen | DoBlue;
-#endif /* X11 */
-		XStoreColor(wn__Dpy, _wn_Cmap, &fg_color);
-		XStoreColor(wn__Dpy, _wn_Cmap, &bg_color);
+		XStoreColor(wn__Dpy, scr->s_cmap, &fg_color);
+		XStoreColor(wn__Dpy, scr->s_cmap, &bg_color);
 	}
 
-#ifdef X11
 	wn__set_x11_cursor_colors(&fg_color, &bg_color);
-#endif /* X11 */
 
-	_wn_Planemask = planes;
+	scr->s_planemask = plane_mask;
 }
 
 /*  Open the display, and set up an error handler which aborts with a core
@@ -817,12 +892,10 @@ const char **p_errmsg;
 				strcpy(errbuf, "Can't open display (DISPLAY environment variable unset)");
 		}
 		else {
-#ifdef X11
 			if (getenv("XSYNCHRONISE") != NULL)
 				(void) XSynchronize(display, TRUE);
 			real_displayname = DisplayString(display);
 			display_fd = ConnectionNumber(display);
-#endif /* X11 */
 
 			buf = wn__e_malloc(strlen(real_displayname) +
 					   sizeof(deq));
@@ -925,31 +998,28 @@ const char *s;
 
 #ifdef X11
 static void
-create_gc()
+set_x_gc(scr)
+screen_t *scr;
 {
 	XGCValues   gcv;
 	Window rootwin;
 #define GCMASK	(GCForeground | GCBackground)
 
-	rootwin = RootWindow(wn__Dpy, DefaultScreen(wn__Dpy));
+	rootwin = RootWindow(wn__Dpy, scr->s_screen);
 
-	gcv.foreground = _wn_Fg_pixel;
-	gcv.background = _wn_Bg_pixel;
-	_wn_Gc = XCreateGC(wn__Dpy, rootwin, (unsigned)GCMASK, &gcv);
+	gcv.foreground = scr->s_fg_pixel;
+	gcv.background = scr->s_fg_pixel;
+	scr->s_gc = XCreateGC(wn__Dpy, rootwin, (unsigned)GCMASK, &gcv);
 }
 
 void
-wn_get_X11_info(p_display, p_win, p_gc, p_colormap)
+wn_get_X11_info(p_display, p_win)
 Display **p_display;
 Window *p_win;
-GC *p_gc;
-Colormap *p_colormap;
 {
 	_wn_init();
 	*p_display = wn__Dpy;
-	*p_gc = _wn_Gc;
 	*p_win = wn_is_open(WN_STDWIN) ? WN_TO_W(WN_STDWIN)->w_win : 0;
-	*p_colormap = _wn_Cmap;
 }
 #endif /* X11 */
 
@@ -966,8 +1036,7 @@ wn_open_display()
 	if (getenv("WINDOW_PARENT") != NULL)
 		errmsg = NULL;
 	else
-		errmsg =
-		      "Can't open window (WINDOW_PARENT environment variable unset)";
+		errmsg = "Can't open window (WINDOW_PARENT environment variable unset)";
 #endif
 	return errmsg;
 }
@@ -993,14 +1062,13 @@ _wn_init()
 #ifdef X11
 	if ((wn__Dpy = open_display(Default_display, (const char **)NULL)) == NULL)
 		return -1;
-	set_colors();
-	create_gc();
+	new_screen (DefaultScreen(wn__Dpy));
 	if (getenv("XDEBUG") != NULL)
 		_Xdebug = TRUE;
 #endif /* X11 */
 #ifdef SUNVIEW
 	Old_sun_error_handler = win_errorhandler(my_error_handler);
-	_wn_Planemask = 1;
+	new_screen (0);
 	if ((wakeup_fd = wn__setup_sunview_wakeup_pipe()) != -1)
 		_wn_set_sunview_wakeup_fd(wakeup_fd);
 #endif /* SUNVIEW */
@@ -1232,7 +1300,7 @@ const char *name, *geometry;
 
 	if (flags & XValue) {
 		if (flags & XNegative) {
-			x = DisplayWidth(wn__Dpy, DefaultScreen(wn__Dpy)) + x -
+			x = DisplayWidth(wn__Dpy, W_SCREEN(w)) + x -
 					sh.width - 2 * border_width;
 		}
 		sh.x = x;
@@ -1240,7 +1308,7 @@ const char *name, *geometry;
 	}
 	if (flags & YValue) {
 		if (flags & YNegative) {
-			y = DisplayHeight(wn__Dpy, DefaultScreen(wn__Dpy)) + y -
+			y = DisplayHeight(wn__Dpy, W_SCREEN(w)) + y -
 						sh.height - 2 * border_width;
 		}
 		sh.y = y;
@@ -1257,12 +1325,17 @@ const char *name, *geometry;
 	}
 	sh.flags |= PWinGravity;
 
+	if (_Xdebug) printf ("ups: w             = %p\n", w);
+	if (_Xdebug) printf ("ups: W_SCREEN(w)   = %d\n", W_SCREEN(w));
+	if (_Xdebug) printf ("ups: RootWindow()  = %p\n", RootWindow(wn__Dpy, W_SCREEN(w)));
+	if (_Xdebug) printf ("ups: W_FG_PIXEL(w) = [%3d], W_BG_PIXEL(w) = [%3d]\n", W_FG_PIXEL(w), W_BG_PIXEL(w));
+
 	win = XCreateSimpleWindow(wn__Dpy,
-				  RootWindow(wn__Dpy, DefaultScreen(wn__Dpy)),
+				  RootWindow(wn__Dpy, W_SCREEN(w)),
 				  sh.x, sh.y,
 				  (unsigned)sh.width, (unsigned)sh.height,
 				  border_width,
-				  (xpixel_t)_wn_Fg_pixel, (xpixel_t)_wn_Bg_pixel);
+				  W_FG_PIXEL(w), W_BG_PIXEL(w));
 
 	xwmh.initial_state = Want_to_start_as_icon ? IconicState : NormalState;
 	xwmh.input = True;
@@ -1338,6 +1411,36 @@ swin_t *w, *iw;
 
 	wn__update_size(w);
 }
+
+void
+wn_set_window_hints(wn, title, icontitle, icon)
+int wn;
+const char* title;
+const char* icontitle;
+bitmap_t* icon;
+{
+	swin_t *w;
+	Pixmap icon_pixmap = None;
+	char title_buff[50];
+	char icontitle_buff[50];
+	W_CHECK(wn);
+	w = WN_TO_W(wn);
+	if ( title && !strncmp(title, "%a",2))
+	{
+	     sprintf(title_buff, "%s%s", Appname,title+2);
+	     title = title_buff;
+	}
+	if ( icontitle && !strncmp(icontitle, "%a",2))
+	{
+	     sprintf(icontitle_buff, "%s%s", Appname,icontitle+2);
+	     icontitle = icontitle_buff;
+	}
+        if ( icon)
+	{
+	    icon_pixmap = bm_to_pixmap(icon);
+	}
+	XSetStandardProperties(wn__Dpy, w->w_win,title, icontitle,icon_pixmap, NULL, 0, NULL);
+}
 #endif /* X11 */
 
 void
@@ -1404,58 +1507,151 @@ int wn;
 #endif
 }
 
+#if WANT_MULTI_SCREEN
+/*
+ *  Get pixel value.
+ */
+xpixel_t
+_wn_get_x_pixel (scr, pixel)
+screen_t *scr;
+xpixel_t pixel;
+{
+	if (pixel & WN_MY_PIXEL)
+		return (scr->s_pixels[pixel & (~WN_MY_PIXEL)]);
+	if (_Xdebug) printf("ups: pixel 0x%8lx not mine.\n", pixel);
+	return pixel;
+}
+
+/*
+ *  Return index of free entry in 's_pixels' array, mark "in use".
+ *  Grow array if no space.
+ */
+static int
+_wn_find_pix (scr)
+screen_t *scr;
+{
+	int i, rv;
+	size_t n;
+
+	for (i = 0; i < scr->s_npixel; i++) {
+		if (scr->s_pixels[i] == WN_MY_PIXEL) {
+		    scr->s_pixels[i] = 0;
+		    return i;
+		}
+	}
+
+	n = scr->s_npixel * 2;
+	scr->s_pixels = (xpixel_t *) realloc((char *)scr->s_pixels, n * sizeof(xpixel_t));
+	if (scr->s_pixels == NULL)
+		wn__panic("realloc failed in _wn_find_pix");
+	for (i = scr->s_npixel; i < n; i++)
+		scr->s_pixels[i] = WN_MY_PIXEL;
+	rv = scr->s_npixel;
+	scr->s_npixel = n;
+	scr->s_pixels[rv] = 0;
+	return rv;
+}
+#endif
+
 /* Function to determine if a TrueColor visual is being used */
 /* RGA: from Craig Amey <craiga@MetaSW.COM> */
 
-int wn_is_truecolor()
+int
+wn_is_truecolor(s_screen)
+int s_screen;
 {
-  Visual *v;
+	Visual *v;
 
-  v = DefaultVisual(wn__Dpy, DefaultScreen(wn__Dpy));
-  return (v->class == TrueColor);
+	v = DefaultVisual(wn__Dpy, s_screen);
+	return (v->class == TrueColor);
 }
 
-/* Function used to allocate shared colors in TrueColor mode */
+/* Function used to allocate shared colors. */
 /* RGA: from Craig Amey <craiga@MetaSW.COM> */
+/* (IDE) If we want multi-screen support we put the color into all of the
+ * colormaps.  This is because UPS code caches its colours without much
+ * regard to the window, and doing this gets better results.
+ */
 
-int wn_alloc_color(color, colorname)
+int
+_wn_alloc_x_color(wn, color, colorname)
+int wn;
+color_t *color;
+const char *colorname;
+{
+    XColor xcolor;
+    screen_t *scr;
+    swin_t *w;
+    int rv, pix;
+
+    W_CHECK(wn);
+    w = WN_TO_W(wn);
+
+    xcolor.red = color->co_red;
+    xcolor.green = color->co_green;
+    xcolor.blue = color->co_blue;
+
+    if (_Xdebug) printf("ups: [%2d] want %04x,%04x,%04x  ", wn, xcolor.red, xcolor.green, xcolor.blue);
+#if WANT_MULTI_SCREEN
+    pix = _wn_find_pix (_wn_first_screen());
+    for (scr = _wn_first_screen(); scr != NULL; scr = scr->s_next) {
+    if (XAllocColor(wn__Dpy, scr->s_cmap, &xcolor) == 0) {
+#else
+    if (XAllocColor(wn__Dpy, W_CMAP(w), &xcolor) == 0) {
+#endif
+	/* No R/W color cells left in colormap. */
+	fprintf(stderr, "ups: Failed to allocate color `%s'\n", colorname);
+	if (_Xdebug) printf("got [   ]     ,    ,      ");
+#if WANT_MULTI_SCREEN
+	scr->s_pixels[pix] = W_FG_PIXEL(w);
+	color->co_pixel = ((xpixel_t)pix | WN_MY_PIXEL);
+#else
+	color->co_pixel = W_FG_PIXEL(w);
+#endif
+
+	wn__Use_alloc_color = FALSE;
+	wn__Use_alloc_color_for_highlight = TRUE; /* note */
+	wn__Use_extended_text_colors = FALSE;
+	wn__Use_menu_colors = FALSE;
+	rv = 0;
+    } else {
+	if (_Xdebug) printf("got [%3ld] %04x,%04x,%04x  ", xcolor.pixel, xcolor.red, xcolor.green, xcolor.blue);
+#if WANT_MULTI_SCREEN
+	scr->s_pixels[pix] = xcolor.pixel;
+	color->co_pixel = ((xpixel_t)pix | WN_MY_PIXEL);
+#else
+	color->co_pixel = xcolor.pixel;
+#endif
+
+	wn_use_alloc_color(1);	/* set */
+	wn_use_alloc_color_for_highlight(1);	/* set */
+	rv = 1;
+    }
+#if WANT_MULTI_SCREEN
+    }
+#endif
+    if (_Xdebug) printf ("\n");
+    return rv;
+}
+
+int
+wn_alloc_color(wn, color, colorname)
+int wn;
 color_t *color;
 const char *colorname;
 {
 #ifdef X11
-  XColor xcolor;
-
-  xcolor.red = color->co_red;
-  xcolor.green = color->co_green;
-  xcolor.blue = color->co_blue;
-
-  if (XAllocColor(wn__Dpy, _wn_Cmap, &xcolor) == 0)
-  {
-    fprintf(stderr, "ups: Failed to allocate color `%s'\n", colorname);
-    color->co_pixel = WN_FG;
-    wn__Use_alloc_color = FALSE;
-    wn__Use_alloc_color_for_highlight = TRUE; /* note */
-    wn__Use_extended_text_colors = FALSE;
-    wn__Use_menu_colors = FALSE;
-    return 0;
-  }
-  else
-  {
-    color->co_pixel = xcolor.pixel;
-    wn_use_alloc_color(1);	/* set */
-    wn_use_alloc_color_for_highlight(1);	/* set */
-    return 1;
-  }
+    return (_wn_alloc_x_color(wn, color, colorname));
 #else
-  color->co_pixel = WN_FG;
-  return 1;
+    color->co_pixel = W_FG_PIXEL(w);
+    return 1;
 #endif
 }
 
 void
-wn_set_quit_func(func)
-wn_quit_func_t func;
+wn_set_close_func(func)
+wn_close_func_t func;
 {
-  Quit_func = func;
+	Close_func = func;
 }
 
