@@ -20,8 +20,8 @@ void _wn_make_retained_pixrect PROTO((struct pixwin *pw, int width, int height))
 #endif
 
 #ifdef X11
-void wn_get_X11_info PROTO((Display **p_display, Window *p_win, GC *p_gc, Colormap *p_colormap));
-void process_client_message PROTO((XEvent *ev, Window p_win));
+void wn_get_X11_info PROTO((Display **p_display, Window *p_win));
+void process_client_message PROTO((XEvent *ev, Window p_win, int wn));
 #endif
 
 extern const char *_wn_Sysfont_file;
