@@ -398,7 +398,7 @@ winblank()
 	int width, height;
 
 	wn_get_window_size(wn,&width,&height);
-	wn_set_area(wn, 0, 0, width, height,WN_BG);
+	wn_set_area(wn, 0, 0, width, height, WN_BG(wn));
 }
 
 /*  blank out the rectangle
@@ -407,7 +407,7 @@ void
 rblank(x, y, width, height)
 int x, y, width, height;
 {
-	wn_set_area(wn,x, y, width, height, WN_BG);
+	wn_set_area(wn,x, y, width, height, WN_BG(wn));
 }
 
 /*  Display a message in the message area
