@@ -98,11 +98,7 @@ char ups_ao_aout_c_sccsid[] = "@(#)ao_aout.c	1.2 24 May 1995 (UKC)";
 #endif
 #endif
 
-#undef USERCORE
-#if defined(OS_BSD44) || defined(OS_ULTRIX) || defined(ARCH_CLIPPER) || \
-    defined(ARCH_VAX) || defined(OS_RISCOS) || defined(OS_NEWSOS) || \
-    defined(OS_LINUX)
-#define USERCORE
+#ifdef USERCORE
 struct core {
 	union c_header {
 		struct user cu_u;
@@ -120,6 +116,7 @@ static bool get_rwindow PROTO((int fd, const char *corepath, struct core *c,
                                sunregs_t *sr));
 #endif
 
+
 #ifdef USERCORE
 #undef DATADDR
 
@@ -344,17 +341,24 @@ func_t **p_mainfunc;
 	return TRUE;
 }
 
+/*
+ *  Get a register value from a core file.
+ *  SunOS has the same structure (sunregs_t) in co->regs as when running
+ *  an executable.
+ *  The rest have to go through the u-area so have an array of ureg_t in
+ *  co->regs.  NOTE - used by systems that would not go through the u-area
+ *  to get at the registers when running an executable.
+ */
 int
 aout_get_core_reg(co, regno, p_val)
 Coredesc *co;
 int regno;
 taddr_t *p_val;
 {
-#ifdef OS_SUNOS
+#if defined(OS_SUNOS)
 	*p_val = get_sun_regval((sunregs_t *)core_getregs(co), 0, regno);
 	return 0;
-#else /* !OS_SUNOS */
-#ifdef USERCORE
+#elif defined(USERCORE)
 	ureg_t *uregs;
 	taddr_t addr;
 #ifdef ARCH_MIPS
@@ -382,9 +386,6 @@ taddr_t *p_val;
 	uregs = (ureg_t *)core_getregs(co);
 	addr = uregs[reg_to_uarea_index(regno)].ur_uaddr;
 #endif /* !ARCH_MIPS */
-	/* if (!core_dread(co, addr, (char *)p_val, 4)) return -1; */
-	/* Cannot use 'core_dread()' because "addr" IS the offset into
-	   the core file. */
 	if (!core_read(co, (off_t)addr, (char *)p_val, 4))
 		return -1;
 	return 0;
@@ -392,7 +393,6 @@ taddr_t *p_val;
 	panic("core_getreg NYI");
 	return 0;	/* to satisfy gcc */
 #endif /* !USERCORE*/
-#endif /* !OS_SUNOS */
 }
 
 #ifdef OS_SUNOS
@@ -462,9 +462,12 @@ char **p_regs;
 	size_t data_size, stack_size;
 	int signo;
 	char *regs;
-#ifdef OS_SUNOS_4
 	struct core core;
+#ifdef USERCORE
+	struct user *u;
+#endif
 	
+#if defined(OS_SUNOS_4)
 	if (read(fd, (char *)&core, sizeof(core)) != sizeof(core)) {
 		if (want_messages)
 			failmesg("Can't read", "core file", corepath);
@@ -484,53 +487,50 @@ char **p_regs;
 	data_size = core.c_dsize;
 	stack_size = core.c_ssize;
 	signo = core.c_signo;
-#endif /* OS_SUNOS_4 */
-#ifdef USERCORE
-	struct core core;
-	struct user *u;
-	
+#elif defined(USERCORE)
 	if (read(fd, (char *)&core, sizeof(core)) != sizeof(core)) {
 		if (want_messages)
 			errf("Can't read", "core file", corepath);
 		return -1;
 	}
 	u = &(core.c_un.cu_u);
+	data_size = ctob(u->u_dsize);
+	stack_size = ctob(u->u_ssize);
 	
+#if defined(OS_BSD44)
 	regs = e_malloc(sizeof(ureg_t) * N_UREGS);
-#ifdef OS_BSD44
-	cmdname = u->u_kproc.kp_proc.p_comm;
 	set_uarea_reg_offsets((ureg_t *)regs, get_uword, (int)&core);
+#elif defined(OS_LINUX)
+	regs = e_malloc(sizeof(pt_regs));
+	memcpy (regs, u->regs, sizeof(pt_regs));
+#else
+	regs = e_malloc(sizeof(ureg_t) * N_UREGS);
+	set_uarea_reg_offsets((ureg_t *)regs);
+#endif
+
+#if defined(OS_BSD44)
+	cmdname = u->u_kproc.kp_proc.p_comm;
 	signo = u->u_sig;
 	data_offset = sizeof(struct core);
 #else
 	cmdname = u->u_comm;
-	set_uarea_reg_offsets((ureg_t *)regs);
 	signo = 0;
 	data_offset = sizeof(struct user);
 #endif
-	data_size = ctob(u->u_dsize);
-	stack_size = ctob(u->u_ssize);
+
 #endif /* USERCORE */
 	
-#ifdef ARCH_CLIPPER
+#if defined(ARCH_CLIPPER)
 	dataseg.base = data_addr;
-#else
-#ifdef OS_SUNOS_4
+#elif defined(OS_SUNOS_4)
 	dataseg.base = (taddr_t)N_DATADDR(core.c_aouthdr);
-#else
-#ifdef USERCORE
-#ifdef ARCH_MIPS
+#elif defined(ARCH_MIPS)
 	dataseg.base = USRDATA;
-#else
-#if defined(ARCH_386) && defined(OS_BSD44)
+#elif defined(ARCH_386) && defined(OS_BSD44)
 	dataseg.base = (taddr_t)u->u_kproc.kp_eproc.e_vm.vm_daddr;
-#else
+#elif defined(USERCORE)
 	dataseg.base = ctob((u)->u_tsize);
-#endif /* !BSDI_386 */
-#endif /* !MIPS */
-#endif /* !USERCORE */
-#endif /* !SunOS */
-#endif /* !Clipper */
+#endif
 	dataseg.lim = dataseg.base + data_size;
 	dataseg.file_offset = data_offset;
 
@@ -575,6 +575,7 @@ int arg, offset;
 }
 #endif
 
+
 /*  Attempt to get the argv array of the target from the stack so
  *  we can put the arguments in the command line.  If we succeed,
  *  put as many arguments as will fit in command_line, which is
