@@ -48,8 +48,6 @@ char ups_ao_elflib_c_sccsid[] = "@(#)ao_elflib.c	1.1 24/5/95 (UKC)";
 #include "state.h"
 #include "obj_bpt.h"
 
-typedef struct Solib_addr Solib_addr;
-
 struct Solib_addr {
 	dev_t dev;
 	ino_t ino;
@@ -107,6 +105,14 @@ elf_get_dynamic_shlib_info PROTO((alloc_pool_t *ap, Libdep *ld, const char *text
 			   taddr_t debug_vaddr,
 			   Libdep **last_child));
 
+#ifdef AO_USE_PTRACE /* interp */
+static bool
+get_preload_shlib_list PROTO((alloc_pool_t *ap, const char *textpath,
+			Solib **p_solibs, Solib_addr **p_solib_addrs));
+static const char **
+add_to_env PROTO((const char *s));
+#endif
+
 static unsigned long Main_debug_vaddr = 0;
 
 static void
@@ -149,21 +155,17 @@ static Libdep *
 make_libdep(alloc_pool_t *ap, const char *name, Libdep *par, long mtime)
 {
 	Libdep *ld;
-	char *c;
-	int len;
+	const char *c;
 
 	ld = (Libdep *)alloc(ap, sizeof(Libdep));
 	ld->name = name;
 	if (name)
 	{
-	  len = strlen(ld->name);
-	  if (len > 0)
-	    len --;
-	  for(c = (char *)ld->name + len;
-	      c != (char *)ld->name && c && *c !='/'; c--);
-	  if (c && *c =='/')
+	  if ((c = strrchr (name, '/')) != NULL)
 	    c++;
-	  ld->name_only = (const char *)c;
+	  else
+	    c = name;
+	  ld->name_only = c;
 	}
 	else
 	  ld->name_only = name;
@@ -210,6 +212,11 @@ search_lib_path(const char *dirs, const char *seps, const char *name,
 
 /*  Apply the rules on page 5-20 of the System V ABI book to find the
  *  path (relative or absolute) of the shared library.
+ *
+ *  Latest ld.so man page (Linux) says
+ *  1) $LD_LIBRARY_PATH
+ *  2) /etc/ld/so.cache  (this is binary)
+ *  3) /usr/lib:/lib
  */
 static char *
 get_ld_path(Libdep *ld)
@@ -224,10 +231,9 @@ get_ld_path(Libdep *ld)
 	if ((ptr = getenv ("LD_LIBRARY_PATH")) != NULL) {
 		if (search_lib_path (ptr, ":;", ld->name, &path))
 			return path;
-	} else {
-		if (search_lib_path ("/lib:/usr/lib", ":", ld->name, &path))
-			return path;
 	}
+	if (search_lib_path ("/usr/lib:/lib", ":", ld->name, &path))
+		return path;
 
 	return strf("/usr/lib/%s", ld->name);
 }
@@ -330,14 +336,16 @@ load_elf_shlib(target_t *xp, Libdep *ld)
 		check_elf_file_type(fd, path, ET_DYN, "load symbols from"));
 	  if (ok)
 	  {
+	    int wn = get_message_wn();
+
 	    if (fstat(fd, &stbuf) == 0)
 	      mtime = stbuf.st_mtime;
-	    if (get_message_wn() != -1)
+	    if (wn != -1)
 	    {
-	      wn_updating_off(WN_STDWIN);
+	      wn_updating_off(wn);
 	      errf("\b%s symtab %s...", reload ? "Reloading" :
 		   "Loading", path);
-	      wn_updating_on(WN_STDWIN);
+	      wn_updating_on(wn);
 	    }
 	    if (verbose == 1)
 	      fprintf(stderr, "%s symtab %s...", reload ? "Reloading" :
@@ -346,12 +354,12 @@ load_elf_shlib(target_t *xp, Libdep *ld)
 				 (func_t **)NULL, &ip->ip_solibs);
 	    if (xp->xp_hit_solib_event == TRUE)
 	      xp->xp_new_dynamic_libs_loaded = TRUE;
-	    if (get_message_wn() != -1)
+	    if (wn != -1)
 	    {
-	      wn_updating_off(WN_STDWIN);
+	      wn_updating_off(wn);
 	      errf("\b%s symtab %s...done", reload ? "Reloading" :
 		     "Loading", path);
-	      wn_updating_on(WN_STDWIN);
+	      wn_updating_on(wn);
 	    }
 	    if (verbose == 1)
 		fputs("done\n", stderr);
@@ -420,37 +428,37 @@ target_t *xp;
 }
 
 bool
-elf_get_dynamic_solibs(alloc_pool_t *target_ap, const char *path, int fd,
+elf_get_dynamic_solibs(alloc_pool_t *target_ap, const char *path,
 		       Solib **p_solibs, bool check_only)
 {
-  Libdep *ld, first_child, *last_child, *ld1, *ld2;
-  int same;
+    Libdep *ld, first_child, *last_child, *ld1, *ld2;
+    int same;
 
-  if (Main_debug_vaddr != 0)
-  {
-    ld = make_libdep(target_ap, path, (Libdep *)NULL, 0);
-    last_child = &first_child;
-    elf_get_dynamic_shlib_info
-      (target_ap, ld, path, Main_debug_vaddr, &last_child);
-    last_child->next = NULL;
-    ld->deps = first_child.next;
-    for (same = 1, ld1 = (*p_solibs)->latest_libdep->deps, ld2 = ld->deps;
-	 ld1 && ld2; ld1 = ld1->next, ld2 = ld2->next)
-      if ((strcmp(ld1->name_only, ld2->name_only)  && ld2->wants_load == TRUE) ||
-	  (ld1->wants_load == FALSE && ld2->wants_load == TRUE))
-	/* RGA was noload before, now is not, after init file rescan */
-      {
-	same = 0;
-	break;
-      }
-    if (same == 0 || ld2) /* New list is longer */
+    if (Main_debug_vaddr != 0)
     {
-      if (!check_only)
-	(*p_solibs)->latest_libdep->deps = ld->deps; /* RGA is this right? */
-      return TRUE;
+	ld = make_libdep(target_ap, path, (Libdep *)NULL, 0);
+	last_child = &first_child;
+	elf_get_dynamic_shlib_info (target_ap, ld, path, Main_debug_vaddr,
+				    &last_child);
+	last_child->next = NULL;
+	ld->deps = first_child.next;
+	for (same = 1, ld1 = (*p_solibs)->latest_libdep->deps, ld2 = ld->deps;
+		 ld1 && ld2; ld1 = ld1->next, ld2 = ld2->next)
+	    if ((strcmp(ld1->name_only, ld2->name_only)  && ld2->wants_load == TRUE) ||
+		  (ld1->wants_load == FALSE && ld2->wants_load == TRUE))
+	    {
+		/* RGA was noload before, now is not, after init file rescan */
+		same = 0;
+		break;
+	    }
+	if (same == 0 || ld2) /* New list is longer */
+	{
+	    if (!check_only)
+		(*p_solibs)->latest_libdep->deps = ld->deps; /* RGA is this right? */
+	    return TRUE;
+	}
     }
-  }
-  return FALSE;
+    return FALSE;
 }
 
 bool
@@ -542,7 +550,7 @@ elf_get_core_shlib_info(alloc_pool_t *ap, iproc_t *ip)
 		 */
 		if (lmap.l_prev == NULL)
 			continue;
-#ifdef OS_LINUX
+#if HAVE_L_PHNUM_F
 		if (lmap.l_phnum == 0) {
 			errf("Ignoring lmap at address %lx in %s",
 			     lmaddr, path);
@@ -564,7 +572,24 @@ elf_get_core_shlib_info(alloc_pool_t *ap, iproc_t *ip)
 
 		elf_note_shlib_addr(ip, stbuf.st_dev, stbuf.st_ino,
 				    0, 0, 0,
-				    0, lmap.l_addr, 0);
+				    0, (taddr_t)lmap.l_addr, 0);
+		/* Here libpath is a full path, when we first got the
+		   shared library details we just got a name. */
+		/* Possibly not necessary as library should be findable
+		   by 'get_ld_path()' anyway. */
+		if (*libpath == '/') {
+			char *ptr;
+			Libdep *ld;
+			ptr = strrchr (libpath, '/') + 1;
+			for (ld = next_ld(so->latest_libdep);
+			     ld != NULL;
+			     ld = next_ld(ld)) {
+				if (strcmp (ld->name, ptr) == 0) {
+					ld->name = strsave(libpath);
+					break;
+				}
+			}
+		}
 	}
 
 	return TRUE;
@@ -601,7 +626,7 @@ elf_get_dynamic_shlib_info(alloc_pool_t *ap, Libdep *ld, const char *textpath,
 
 	if (debug.r_brk && dl_bp_installed == FALSE)
 	{
-	  bp = xp_add_breakpoint(xp, debug.r_brk);
+	  bp = xp_add_breakpoint(xp, (taddr_t)debug.r_brk);
 	  if (install_breakpoint(bp, xp) != 0)
 	    errf("can't install breakpoint in dynamic linker");
 	  else
@@ -614,7 +639,7 @@ elf_get_dynamic_shlib_info(alloc_pool_t *ap, Libdep *ld, const char *textpath,
 	for (lmaddr = (taddr_t)debug.r_map;
 	     lmaddr != TADDR_NULL;
 	     lmaddr = (taddr_t)lmap.l_next) {
-		char libpath[MAXPATHLEN], /*buf[MAXPATHLEN],*/ *path_ptr; 
+		char libpath[MAXPATHLEN], buf[MAXPATHLEN], *path_ptr; 
 		struct stat stbuf;
 
 		if (dread(xp, lmaddr, &lmap, sizeof(struct link_map)) == -1)
@@ -641,7 +666,7 @@ elf_get_dynamic_shlib_info(alloc_pool_t *ap, Libdep *ld, const char *textpath,
 		  return FALSE;
 		}
 		
-/*		if (libpath[0] == '.')
+		if (libpath[0] == '.')
 		{
 		  int i;
 		  char name[MAXPATHLEN];
@@ -659,7 +684,7 @@ elf_get_dynamic_shlib_info(alloc_pool_t *ap, Libdep *ld, const char *textpath,
 		  }
 		  path_ptr = buf;
 		}
-		else*/
+		else
 		  path_ptr = libpath;
 
 		if (stat(path_ptr, &stbuf) != 0) {
@@ -672,13 +697,21 @@ elf_get_dynamic_shlib_info(alloc_pool_t *ap, Libdep *ld, const char *textpath,
 		(*last_child)->next = make_libdep(ap, libpath_copy, ld,
 						  stbuf.st_mtime);
 		(*last_child) = (*last_child)->next;
+
+#ifdef AO_USE_PTRACE
+		elf_note_shlib_addr(ip,
+				    stbuf.st_dev, stbuf.st_ino,
+				    stbuf.st_size,
+				    0, 0,
+				    0, (taddr_t)lmap.l_addr, 0);
+#endif
 	      }
 	return TRUE;
 }
 
 bool
 scan_main_elf_symtab(alloc_pool_t *target_ap, const char *path, int fd,
-		     long modtime, Solib **p_solibs,
+		     long modtime, Solib **p_solibs, Solib_addr **p_solib_addrs,
 		     struct func_s **p_mainfunc, struct func_s **p_initfunc)
 {
 	Libdep *ld;
@@ -691,6 +724,10 @@ scan_main_elf_symtab(alloc_pool_t *target_ap, const char *path, int fd,
 	if (ld->so)
 	{
 	  ld->so->symtab->st_modtime = modtime;
+#ifdef AO_USE_PTRACE /* interp */
+	  if (!get_preload_shlib_list (target_ap, path, p_solibs, p_solib_addrs))
+	    return FALSE;
+#endif
 	  Main_debug_vaddr = ld->so->debug_vaddr;
 	}
 	return ok;
@@ -761,15 +798,18 @@ elf_next_symtab(target_t *xp, symtab_t *st, bool load_new, symtab_t **p_next_st)
 
 	so = AO_STDATA(st)->st_solib;
 
-/* RGA commented out to get load of dynamic libraries added into latest_libdep */
-/*	if (so->next != NULL) {	 
-		*p_next_st = so->next->symtab;
-		return TRUE;
-	}*/
-
-/* RGA commented out to get load of dynamic libraries added into latest_libdep */
-/*	if (!load_new || so->is_last_symtab) 
-		return FALSE;*/
+	/* RGA 07/18/00 add a check so the extensive lookup is done
+	   only when the dynamic linker breakpoint has been set */
+	if (xp->xp_hit_solib_event == FALSE)
+	{
+	  if (so->next != NULL) {	 
+	    *p_next_st = so->next->symtab;
+	    return TRUE;
+	  }
+	  
+	  if (!load_new || so->is_last_symtab) 
+	    return FALSE;
+	}
 
 	for (ld = next_ld(so->latest_libdep); ld != NULL; ld = next_ld(ld)) {
 		if (!load_elf_shlib(xp, ld))
@@ -994,4 +1034,154 @@ target_t *xp;
     /* Just to satisfy a link reference. */
 }
 
+void
+elf_rescan_dynamic_solibs(xp)
+target_t *xp;
+{
+#ifdef AO_ELF
+  iproc_t *ip;
+  
+  ip = GET_IPROC(xp);
+  if (ip->ip_pid)
+    xp->xp_new_dynamic_libs = 
+      elf_get_dynamic_solibs(xp->xp_apool, xp->xp_textpath,
+			     &ip->ip_solibs, FALSE);
+#endif
+}
+
+#ifdef AO_USE_PTRACE /* interp */
+/*  Get the list of shared libraries that the textpath will load
+ *  when it is run.  We do this in the same way as ldd(1), by
+ *  running the target with LD_TRACE_LOADED_OBJECTS set in its
+ *  environment and parsing the output.
+ *
+ *  (IDE) taken out of ao_shlib.c - FreeBSD/Linux sometimes need the
+ *  correct addresses of functions *before* the target has been started,
+ *  e.g. interpreted code at breakpoints calling library functions.
+ *  All we do here is create the 'Solib_addr' structures for later code
+ *  to pick up.
+ */
+static bool
+get_preload_shlib_list(ap, textpath, p_solibs, p_solib_addrs)
+alloc_pool_t *ap;
+const char *textpath;
+Solib **p_solibs;
+Solib_addr **p_solib_addrs;
+{
+    int pid, wpid, n;
+    int fds[2];
+    char buf[MAXPATHLEN+256], name[MAXPATHLEN];
+    char *pos, *libpath_copy;
+    const char **envp;
+    FILE *fp;
+    taddr_t addr;
+    struct stat stbuf;
+    Solib_addr *sa;
+/*
+    Libdep first_child, *last_child;
+    last_child = &first_child;
+*/
+
+    envp = add_to_env("LD_TRACE_LOADED_OBJECTS=1");
+    if (pipe(fds) != 0) {
+	    errf("Pipe failed: %s", get_errno_str());
+	    free((char *)envp);
+	    return FALSE;
+    }
+    if ((fp = fdopen(fds[0], "r")) == NULL) {
+	    errf("Fdopen failed: %s", get_errno_str());
+	    free((char *)envp);
+	    return FALSE;
+    }
+    if ((pid = vfork()) == -1) {
+	    errf("Vfork failed: %s", get_errno_str());
+	    fclose(fp);
+	    close(fds[1]);
+	    free((char *)envp);
+	    return FALSE;
+    }
+    if (pid == 0) {
+	    close(fds[0]);
+	    if (fds[1] != 1)
+		    dup2(fds[1], 1);
+	    execle(textpath, textpath, (char *)NULL, envp);
+	    failmesg("Can't exec", "", textpath);
+	    _exit(1);
+    }
+
+    free((char *)envp);
+    close(fds[1]);
+
+    while (fgets(buf, sizeof(buf), fp) != NULL)
+    {
+	/*  We seem to get carriage returns as well as newlines
+	 */
+	if ((pos = strchr(buf, '\r')) != NULL)
+		*pos = '\0';
+	if ((pos = strchr(buf, '\n')) != NULL)
+		*pos = '\0';
+	pos = buf;
+	addr = 0;
+	if ((sscanf(buf, "%*s => %s (%x)", name, &addr) == 2)
+		|| (sscanf(buf, "%*s => %s", name) == 1))
+	{
+	    if (stat(name, &stbuf) != 0)
+	    {
+		errf("Can't stat shared library %s: %s (ignored)",
+		     name, get_errno_str());
+		continue;
+	    }
+/*
+	    libpath_copy = alloc_strdup(ap, name);
+	    last_child->next = make_libdep(ap, libpath_copy, (Libdep *)NULL,
+					      stbuf.st_mtime);
+	    last_child = last_child->next;
+*/
+	    sa = e_malloc(sizeof(Solib_addr));
+	    sa->dev = stbuf.st_dev;
+	    sa->ino = stbuf.st_ino;
+	    sa->size = stbuf.st_size;
+#ifdef OS_SUNOS
+	    sa->mtv_sec = stbuf.mtv_sec;
+	    sa->mtv_nsec = stbuf.mtv_nsec;
+#endif
+	    sa->offset = 0;
+	    sa->vaddr = addr;
+	    sa->pagesize = 0;
+	    sa->next = *p_solib_addrs;
+	    *p_solib_addrs = sa;
+	}
+    }
+    fclose(fp);
+    while ((wpid = wait((int *)NULL)) != pid && wpid != -1)
+	    ;
+/*
+    last_child->next = NULL;
+*/
+    return TRUE;
+}
+
+/*  Return a pointer to an array of strings which consists of the
+ *  environment plus string s, which should be of the form
+ *  "name=value".
+ */
+static const char **
+add_to_env(s)
+const char *s;
+{
+	extern const char **environ;
+	const char **sptr, **envp, **src, **dst;
+
+	for (sptr = environ; *sptr != NULL; ++sptr)
+		;
+	envp = dst = (const char **)e_malloc(sizeof(char *) * (sptr - environ + 2));
+	src = environ;
+	while (*src != NULL)
+		*dst++ = *src++;
+	*dst++ = s;
+	*dst = NULL;
+	return envp;
+}
+#endif
+
 #endif /* AO_ELF */
