@@ -65,9 +65,16 @@ check_optsym_matches(alloc_pool_t *ap, stf_t *stf,
 	const char *global_prefix;
 	Compiler_type compiler_type = CT_UNKNOWN;
 	
-	elf_handle_optsym(ap, optstr, stf->stf_language,
+	elf_handle_optsym(ap, optstr, &stf->stf_language,
 			  &global_prefix, &compiler_type, p_has_debug_syms);
 
+        /* RCB: See if the compiler was found */
+        if ( compiler_type != CT_UNKNOWN)
+        {
+            stf->stf_fil->fi_flags |= FI_FOUND_COMPILER;
+            stf->stf_compiler_type = compiler_type;
+        }
+
 	/*  Can't be bothered to check for compiler type mismatch.
 	 */
 
@@ -191,10 +198,9 @@ scan_index_syms(symtab_t *st, Elfinfo *el, fil_t **p_sfiles, func_t **p_flist,
 			fil_t *fil;
 			bool has_debug_syms;
 			
-		case N_FLINE:
-		    break ;
-
-
+		case N_ILDPAD:
+			next_file_offset += nm.n_value;
+			break;
 		case N_UNDF:
 			file_offset = next_file_offset;
 			next_file_offset = file_offset + nm.n_value;
@@ -227,6 +233,7 @@ scan_index_syms(symtab_t *st, Elfinfo *el, fil_t **p_sfiles, func_t **p_flist,
 				stf = AO_FIDATA(fil);
 			}
 			else {
+
 				stf = make_stf(ap, alloc_strdup(ap, path),
 					       st, 0, srctype(path), 0);
 				
@@ -526,7 +533,7 @@ symio_t **p_symio;
 
 
 void
-elf_handle_optsym(alloc_pool_t *ap, const char *opts, language_t language,
+elf_handle_optsym(alloc_pool_t *ap, const char *opts, language_t* language,
 		  const char **p_global_prefix, Compiler_type *p_compiler_type,
 		  bool *p_has_debug_syms)
 {
@@ -543,7 +550,7 @@ elf_handle_optsym(alloc_pool_t *ap, const char *opts, language_t language,
 	else {
 		const char *s;
 
-		switch (language) {
+		switch (language?*language:LANG_UNKNOWN) {
 		case LANG_F77:
 			compiler_type = CT_F77;
 			break;
@@ -594,6 +601,38 @@ elf_handle_optsym(alloc_pool_t *ap, const char *opts, language_t language,
 			else if (*opt == 'g' && s == &opt[1]) {
 				has_debug_syms = TRUE;
 			}
+			else if (*opt == 'V' && opt[1] == '=') {
+			     double ver;
+			     if ( sscanf(opt+2,"%lf",&ver))
+				 set_compiler_version(ver);
+			}
+			else if (*opt == 'R' && compiler_type == CT_UNKNOWN)
+			{
+			     /* Attempt to identify both compiler and
+			     ** language from opt string
+			     */
+			     const char* nextopt = strchr(opt,';');
+			     const char* langString = strstr(opt,"C++");
+			     if ( langString )
+			     {
+				 if (langString < nextopt || nextopt==NULL)
+			         {
+				     compiler_type = CT_CC;
+				     if (language && *language == LANG_UNKNOWN)
+				         *language = LANG_CC;
+			         }
+			     } else
+			     {
+				 langString = strstr(opt," C ");
+			         if ( langString
+				     && (langString < nextopt || nextopt==NULL))
+			         {
+				     compiler_type = CT_CC;
+				     if (language && *language == LANG_UNKNOWN)
+				         *language = LANG_C;
+			         }
+			     }
+			}
 
 			while (isspace(*s))
 				++s;
