@@ -10,7 +10,7 @@
 /* @(#)ao_elfsym.h	1.2 04 Jun 1995 (UKC) */
 
 void elf_handle_optsym PROTO((alloc_pool_t *ap, const char *opts,
-			      language_t language,
+			      language_t* language,
 			      const char **p_global_prefix,
 			      Compiler_type *p_compiler_type,
 			      bool *p_has_debug_syms));
