@@ -75,6 +75,10 @@ struct Procfs_info {
 #define IS_INTEGER_REG(regno)	((regno) < 32)
 #define FLOAT_REGNO(regno)	((regno) - 32)
 #endif
+#ifdef ARCH_386
+#define IS_INTEGER_REG(regno)	((regno) < _NGREG)
+#define FLOAT_REGNO(regno)	((regno) - _NGREG)
+#endif
 
 int 
 procfs_get_last_attach_pid()
@@ -287,9 +291,14 @@ wait_for_stop(Procfs_info *pi)
 	unsigned long fdmask;
 	int wn_fd;
 	bool user_stopped;
+	int alarm_time;
 
 	user_stopped = FALSE;
 	fdmask = wn_get_wn_fds();
+	if ((alarm_time = get_run_alarm_time()) > 0)
+	    alarm_time /= 1000;	/* convert from micro-secs to milli-secs */
+	else
+	    alarm_time = INFTIM;
 
 	if (fdmask == 0)
 		return user_stopped;
@@ -307,15 +316,21 @@ wait_for_stop(Procfs_info *pi)
 
 	for (;;) {
 		int res;
-		
-		res = poll(fds, 2, INFTIM);
 
-		if (res <= 0) {
+		res = poll(fds, 2, alarm_time);
+
+		if (res < 0) {
 			errf("Warning: poll returned %d (%s)",
 			     res, get_errno_str());
 			return user_stopped;
 		}
 
+		if (res == 0) {
+			run_alarm_time_expired();
+			alarm_time = INFTIM;
+			continue;
+		}
+
 		if (fds[WN].revents != 0) {
 			if (user_wants_stop(FALSE)) {
 			  if (pi->at_pause == TRUE) 
@@ -711,15 +726,17 @@ stopres_t *p_stopres;
 	}
 
 	*p_stopres = xp_execto(xp, main_addr);
-	procfs_get_dynamic_solibs(xp);
+	rescan_dynamic_solibs(xp);
 	if (*p_stopres != SR_BPT) {
 		errf("Can't run process to start of main function");
 		if (*p_stopres != SR_DIED)
 			xp_kill(xp);
 		return -1;
 	}
-
-#if 0
+	/* RGA This block was commented out at some post 3.33 point,
+	   presumably to improve speed, but it is necessary in
+	   order to detect loading of dynamic libs. It has no
+	   effect unless we have hit the dynamic linker breakpoint */
 #ifdef AO_ELF
 	if (*p_stopres == SR_BPT)
 	{
@@ -727,13 +744,14 @@ stopres_t *p_stopres;
 	    elf_save_symtab_breakpoints(xp, FALSE);
 	  while (xp->xp_hit_solib_event == TRUE)
 	  {
-	    procfs_get_dynamic_solibs(xp);
+	    rescan_dynamic_solibs(xp);
 	    refresh_target_display(xp, *p_stopres, FALSE);
 	    if (xp->xp_new_dynamic_libs_loaded == TRUE &&
 		can_install_all_breakpoints(xp) == TRUE)
 	       {
 		 remove_all_breakpoints();
 		 procfs_load_mmap_info(xp, FALSE);
+		 redo_displayed_source_files();
 		 elf_restore_symtab_breakpoints();
 		 xp->xp_new_dynamic_libs_loaded = FALSE;
 		 xp->xp_hit_solib_event = FALSE;
@@ -748,7 +766,6 @@ stopres_t *p_stopres;
 	    }
 	  }
 	}
-#endif
 #endif
 	if (!procfs_load_mmap_info(xp, FALSE)) {
 		xp_kill(xp);
@@ -814,7 +831,6 @@ int pid;
 	  }
 	  xp->xp_new_dynamic_libs = 
 	    elf_get_dynamic_solibs(xp->xp_apool, xp->xp_textpath,
-				   ip->ip_procfs_info->procfd,
 				   &ip->ip_solibs, FALSE);
 	}
 #endif
@@ -990,8 +1006,13 @@ get_machine_regno(int upsreg)
 	case UPSREG_FP:
 	case UPSREG_AP:
 		return R_FP;
+#if (defined R_PSR)
 	case UPSREG_CONDITION_CODES:
 		return R_PSR;
+#elif (defined R_PS)
+	case UPSREG_CONDITION_CODES:
+		return R_PS;
+#endif
 	default:
 		if (upsreg < 0 || upsreg >= NPRGREG)
 			panic("bad reg in pr");
@@ -999,6 +1020,19 @@ get_machine_regno(int upsreg)
 	}
 }
 
+#ifdef ARCH_386
+
+static bool
+get_fpreg_addr(Procfs_info *pi, int regno, int num_bytes, void **p_addr)
+{
+	enum { N_FP_REGS = (sizeof(pi->fpregs.fp_reg_set.f_fpregs) / 
+			    sizeof(*pi->fpregs.fp_reg_set.f_fpregs)) };
+
+	panic ("get_fpreg_addr: NYI");
+}
+
+#else
+
 static bool
 get_fpreg_addr(Procfs_info *pi, int regno, int num_bytes, void **p_addr)
 {
@@ -1035,6 +1069,8 @@ get_fpreg_addr(Procfs_info *pi, int regno, int num_bytes, void **p_addr)
 	return TRUE;
 }
 
+#endif
+
 int
 procfs_readreg(ip, upsreg, p_val)
 iproc_t *ip;
@@ -1144,8 +1180,10 @@ procfs_get_reg_index(Procfs_regname regname)
 	switch (regname) {
 	case PROCFS_REG_SP:
 		return R_SP;
+#ifdef R_O0
 	case PROCFS_REG_O0:
 		return R_O0;
+#endif
 	case PROCFS_REG_NREGS:
 		return NPRGREG;
 	default:
@@ -1256,21 +1294,5 @@ const char *name;
 	return -1;
 }
 
-void
-procfs_get_dynamic_solibs(xp)
-target_t *xp;
-{
-#ifdef AO_ELF
-  iproc_t *ip;
-  
-  ip = GET_IPROC(xp);
-  if (ip->ip_pid)
-    xp->xp_new_dynamic_libs = 
-      elf_get_dynamic_solibs(xp->xp_apool, xp->xp_textpath,
-			     ip->ip_procfs_info->procfd,
-			     &ip->ip_solibs, FALSE);
-#endif
-}
-
-
 #endif /* AO_USE_PROCFS */
+
