@@ -179,14 +179,20 @@ static int
 wait_with_intr(p_status)
 wait_arg_t *p_status;
 {
+	int ret;
 	if (setjmp(Longjmp_env) == 1)
 		return 0;
 
 #ifdef OS_LINUX
-	return wait3(p_status, WNOHANG, NULL);
+	ret = wait3(p_status, WNOHANG, NULL);
 #else
-	return wait(p_status);
+	if (get_run_alarm_time() > 0)
+	    ret = wait3(p_status, WNOHANG, NULL);
+	else
+	    ret = wait(p_status);
 #endif
+
+	return ret;
 }
 
 
@@ -205,12 +211,24 @@ target_t *xp;
 	int pid, at_bpt;
 	bool user_stopped_target;
 	wait_arg_t status;
+	bool want_timer = FALSE;
+	struct timeval when, t;
 
 	ip = GET_IPROC(xp);
 
 	if (want_errors == -1)
 		want_errors = getenv("WANT_ERRORS") != NULL;
 
+	if (get_run_alarm_time() > 0) {
+	    want_timer = TRUE;
+	    gettimeofday (&when, NULL);
+	    when.tv_sec  += get_run_alarm_time() / (1000*1000);
+	    when.tv_usec += get_run_alarm_time() % (1000*1000);
+	    if (when.tv_usec > (1000*1000))
+		when.tv_sec++;
+	    when.tv_usec  = when.tv_usec % (1000*1000);
+	}
+
 	user_stopped_target = FALSE;
 	for (;;) {
 		abort_func_t oldfunc;
@@ -221,17 +239,25 @@ target_t *xp;
 			oldfunc = set_user_abort_func(stop_target);
 			pid = wait_with_intr(&status);
 			(void) set_user_abort_func(oldfunc);
-	}
+		}
 
 		if (pid == 0) {
+			/* Non-blocking wait() */
 			if (user_wants_stop(FALSE)) {
 				kill(ip->ip_pid, SIGTRAP);
 				user_stopped_target = TRUE;
 			}
+			if (want_timer > 0) {
+				gettimeofday (&t, NULL);
+				if ((t.tv_sec >= when.tv_sec) && (t.tv_usec > when.tv_usec))
+				{
+					run_alarm_time_expired();
+					want_timer = 0;
+				}
+			}
 			continue;
 		}
-		else
-		  if (user_wants_stop(FALSE))    /* RGA so Stop button breaks */
+		else if (user_wants_stop(FALSE)) /* RGA so Stop button breaks */
 		    user_stopped_target = TRUE;  /* out of a Step */
 		if (pid == ip->ip_pid)
 			break;
@@ -476,7 +502,7 @@ stopres_t *p_stopres;
 #ifdef AO_ELF
 		xp->xp_new_dynamic_libs = 
 		elf_get_dynamic_solibs(xp->xp_apool, xp->xp_textpath,
-				       NULL, &ip->ip_solibs, FALSE);
+				       &ip->ip_solibs, FALSE);
 #endif
 #endif
 	}
@@ -808,7 +834,6 @@ int data;
 	return res;
 }
 
-
 /*
  *  This routine *MUST* be the last in the file.
  */
@@ -867,6 +892,8 @@ int request, pid, addr, data;
 	return res;
 }
 
+
+
 #endif /* PTRACE_DEBUG */
 
 #endif /* AO_USE_PTRACE */
