@@ -26,6 +26,7 @@ char ups_ao_stack_c_sccsid[] = "@(#)ao_stack.c	1.3 24 May 1995 (UKC)";
 #		include <machine/vmparam.h>	/* for USRSTACK */
 #	else
 #	ifdef OS_SUNOS_5
+#		include <sys/reg.h>
 #		include <sys/frame.h>
 #		include <sys/ucontext.h>
 #	else 
@@ -84,7 +85,10 @@ static Stack *discard_junk_frames PROTO((target_t *xp, Stack *stk));
  *
  *  Note that the stk_outer links are not set by this function.
  */
-#ifdef ARCH_SUN4
+
+#if (defined ARCH_SUN4)
+
+/* ARCH_SUN4 */
 Stack *
 ao_get_stack_trace(xp)
 target_t *xp;
@@ -236,9 +240,11 @@ target_t *xp;
 
 	return discard_junk_frames(xp, stk);
 }
-#endif /* ARCH_SUN4 */
+/* ARCH_SUN4 */
 
-#ifdef ARCH_MIPS
+#elif (defined ARCH_MIPS)
+
+/* ARCH_MIPS */
 Stack *
 ao_get_stack_trace(xp)
 target_t *xp;
@@ -354,9 +360,124 @@ target_t *xp;
 
 	return stk;
 }
-#endif /* !ARCH_MIPS */
+/* ARCH_MIPS */
 
-#if !defined(ARCH_MIPS) && !defined(ARCH_SUN4)
+#elif (defined ARCH_386)
+
+/* ARCH_386 */
+Stack *
+ao_get_stack_trace(xp)
+target_t *xp;
+{
+	int lnum;
+	bool normal_frame;
+	taddr_t pc, fp, sp;
+	register Stack *stk;
+	func_t *f;
+	Stack *last;
+	struct frame fbuf;
+	
+	pc = xp_getreg(xp, UPSREG_PC);
+	fp = xp_getreg(xp, UPSREG_FP);
+	sp = xp_getreg(xp, UPSREG_SP);
+
+#ifdef DEBUG_STACK
+	if (Debug_flags & DBFLAG_STACK)
+		fprintf(stderr, "bst: pc=0x%x fp=0x%x sp=0x%x\n", pc, fp, sp);
+#endif
+
+	for (stk = last = NULL; fp != (taddr_t)0; last = stk) {
+		fil_t *fil;
+		int lnum;
+		normal_frame = TRUE;
+
+		if ((f = addr_to_func(pc)) == NULL) {
+			f = make_badfunc();
+		}
+#if defined(ARCH_BSDI386)
+		if (*f->fu_demangled_name == '[' && strcmp(f->fu_demangled_name, "[start]") == 0) {
+			fp = NULL;
+			break;
+		}
+#endif
+#ifdef ARCH_LINUX386
+		if (strcmp(f->fu_demangled_name, "_entry") == 0) {
+			fp = (taddr_t)0;
+			break;
+		}
+#endif
+
+		addr_to_fil_and_lnum(f, (last == NULL) ? pc : (pc - 1),
+				     &fil, &lnum, FALSE);
+
+#ifdef DEBUG_STACK
+		if (Debug_flags & DBFLAG_STACK)
+			fprintf(stderr, "bst: pc=0x%x fname=%s fp=%x\n",
+				pc, f->fu_demangled_name, fp);
+#endif
+
+		stk = make_stk(f, pc, fil, lnum, last);
+
+		/*  Get the next stack frame
+		 */
+
+		if (normal_frame) {
+			stk->stk_fp = stk->stk_ap = fp;
+			stk->stk_sp = sp;
+			if (dread(xp, fp, (char *)&fbuf, sizeof(fbuf)) != 0)
+				break;
+			sp = fp + sizeof(fbuf);
+
+#ifdef DEBUG_STACK
+			if (Debug_flags & DBFLAG_STACK) {
+				fprintf(stderr,
+					"bst: frame fp=0x%x: fp=0x%x pc=0x%x\n",
+					fp, fbuf.fr_savfp, fbuf.fr_savpc);
+			}
+#endif
+
+			fp = (taddr_t)fbuf.fr_savfp;
+			pc = fbuf.fr_savpc;
+		}
+
+#if defined(ARCH_FREEBSD386)
+#elif defined(ARCH_BSDI386)
+		/*  If the pc points at the signal trampoline code in the u
+		 *  area, we assume that the target has taken a signal.
+		 */
+#define u ((struct user *)USRSTACK)
+		if (pc >= (taddr_t)u->u_pcb.pcb_sigc &&
+		    pc < (taddr_t)u->u_pcb.pcb_sigc + sizeof u->u_pcb.pcb_sigc){
+#undef u
+			struct { int signo, code; taddr_t scp; } sigargs;
+			taddr_t scp;
+			struct sigcontext scbuf;
+
+			if (xp_read_data(xp, sp,
+				     (char *)&sigargs, sizeof(sigargs)) != 0)
+				break;
+			if (xp_read_data(xp, sigargs.scp,
+					    (char *)&scbuf, sizeof(scbuf)) != 0)
+				break;
+
+			stk->stk_siginfo = make_siginfo(sigargs.signo, 0);
+
+			sp = scbuf.sc_sp;
+			pc = scbuf.sc_pc;
+			fp = scbuf.sc_fp;
+		}
+#endif
+	}
+	if ((fp != (taddr_t)0) && (stk != NULL))
+		stk->stk_bad = TRUE;
+
+	return discard_junk_frames(xp, stk);
+}
+/* ARCH_386 */
+
+#else
+
+/* NOT SUN4/MIPS/386 */
 Stack *
 ao_get_stack_trace(xp)
 target_t *xp;
@@ -377,9 +498,6 @@ target_t *xp;
 	};
 	preamble_t *pr;
 	bool must_find_fp;
-#endif
-#ifndef OS_LINUX
-	struct sigcontext scbuf;
 #endif
 	int lnum;
 	bool normal_frame;
@@ -427,13 +545,6 @@ target_t *xp;
 			break;
 		}
 #endif
-#ifdef ARCH_LINUX386
-		if (strcmp(f->fu_demangled_name, "_entry") == 0) {
-			fp = (taddr_t)0;
-			break;
-		}
-#endif
-
 		addr_to_fil_and_lnum(f, (last == NULL) ? pc : (pc - 1),
 				     &fil, &lnum, FALSE);
 
@@ -441,6 +552,7 @@ target_t *xp;
 		if (strcmp(f->fu_demangled_name, "_sigtramp") == 0 && last != NULL) {
 			int signo;
 			taddr_t scp;
+			struct sigcontext scbuf;
 
 			if (dread(xp, last->stk_fp, (char *)&sigframe,
 							sizeof(sigframe)) != 0)
@@ -484,6 +596,7 @@ target_t *xp;
 				     (last == NULL || last->stk_siginfo != NULL)) {
 			taddr_t offset;
 			int res;
+			struct sigcontext scbuf;
 
 			if (last != NULL && last->stk_siginfo != NULL)
 				sp = (taddr_t)scbuf.sc_sp;
@@ -603,31 +716,6 @@ target_t *xp;
 			pc = fbuf.fr_savpc;
 		}
 
-#if defined(ARCH_BSDI386) && !defined(ARCH_FREEBSD386)
-		/*  If the pc points at the signal trampoline code in the u
-		 *  area, we assume that the target has taken a signal.
-		 */
-#define u ((struct user *)USRSTACK)
-		if (pc >= (taddr_t)u->u_pcb.pcb_sigc &&
-		    pc < (taddr_t)u->u_pcb.pcb_sigc + sizeof u->u_pcb.pcb_sigc){
-#undef u
-			struct { int signo, code; taddr_t scp; } sigargs;
-			taddr_t scp;
-
-			if (xp_read_data(xp, sp,
-				     (char *)&sigargs, sizeof(sigargs)) != 0)
-				break;
-			if (xp_read_data(xp, sigargs.scp,
-					    (char *)&scbuf, sizeof(scbuf)) != 0)
-				break;
-
-			stk->stk_siginfo = make_siginfo(sigargs.signo, 0);
-
-			sp = scbuf.sc_sp;
-			pc = scbuf.sc_pc;
-			fp = scbuf.sc_fp;
-		}
-#endif
 #ifdef ARCH_CLIPPER
 		/*  If the pc points into the u area, we assume that
 		 *  the target has taken a signal, and that the pc is
@@ -638,6 +726,7 @@ target_t *xp;
 			taddr_t scp;
 			Siginfo *si;
 			int signo;
+			struct sigcontext scbuf;
 
 			if (xp_read_data(xp, sp,
 					     (char *)&scp, sizeof(scp)) != 0)
@@ -672,6 +761,7 @@ target_t *xp;
 		if (!fbuf.fr_s) {
 			taddr_t scp;
 			int signo;
+			struct sigcontext scbuf;
 
 			/*  The hairiest bit of all: get the pc from
 			 *  the arguments of the signal catching routine.
@@ -713,7 +803,9 @@ target_t *xp;
 
 	return discard_junk_frames(xp, stk);
 }
-#endif /* !ARCH_SUN4 && !ARCH_MIPS */
+/* NOT SUN4/MIPS/386 */
+
+#endif
 
 #ifndef ARCH_MIPS
 /*  Discard any frames outside the main function.
