@@ -50,6 +50,8 @@ char ups_ao_symload_c_sccsid[] = "@(#)ao_symload.c	1.9 05 Jun 1995 (UKC)";
 #include "util.h"
 #include "cursors.h"
 
+static double compiler_version;
+
 static void get_fi_compiler PROTO((fil_t *fil));
 static stf_t *id_to_hdrstf PROTO((stf_t *stf, taddr_t id, int symno));
 static void get_fi_types PROTO((fil_t *fil, bool is_header));
@@ -406,7 +408,12 @@ bool is_header;
 				symrec.symno = hdrstf->stf_symlim - 1;
 		}
 #endif
-		if (nm.n_type != N_LSYM)
+		if (nm.n_type != N_LSYM
+#ifdef N_ISYM					/* RCB, for SC5 */
+                    && nm.n_type != N_ISYM
+                    && nm.n_type != N_ESYM
+#endif
+		)
 			continue;
 
 		name = symstring(symrec.symio, symrec.symno);
@@ -619,8 +626,6 @@ var_t *
 ao_get_fi_vars(fil)
 fil_t *fil;
 {
-	static int want_clcc_hack = -1;
-	char *def;
 	nlist_t nm;
 	snlist_t *sn;
 	class_t class;
@@ -634,6 +639,7 @@ fil_t *fil;
 	taddr_t addr;
 	errf_ofunc_t oldf;
         cursor_t old_cursor;
+	int rational_products_used = 0;
 
 	if (fil->fi_flags & FI_DONE_VARS)
 		return fil->fi_block->bl_vars;
@@ -656,37 +662,17 @@ fil_t *fil;
 
 	/*  Needed for typedefs etc, as with get_fu_blocks().
 	 */
-	if (want_clcc_hack == -1)
-	  want_clcc_hack =
-	    (def = (char *)wn_get_default("ClccWorkaround")) != NULL &&
-	      strcmp(def, "on") == 0;
-	if (want_clcc_hack)
-	{
-	  if (!(fil->fi_flags & FI_FOUND_COMPILER))
-	  {
-	    get_fi_compiler(fil);
-	    fil->fi_flags |= FI_FOUND_COMPILER;
-	  }
-	}
-	if (want_clcc_hack &&
-	    stf->stf_compiler_type == CT_CLCC && stf->stf_symtab &&
-	    /*stf->stf_symtab->st_type == STT_SHLIB &&
-	    stf->stf_symtab->st_dynamic == TRUE &&*/ stf->stf_fil)
-	{
-	  fil_t *tmp_fil = stf->stf_fil;
-	  int i = 1;
-	  
-	  while (i && tmp_fil->fi_next)
-	    tmp_fil = tmp_fil->fi_next;
-	  get_fi_types(tmp_fil, FALSE);
-	}
-	else
-	  get_fi_types(fil, FALSE);
+	get_fi_types(fil, FALSE);
 
 	fil->fi_block->bl_vars = last = NULL;
 	st = fil->fi_symtab;
 	ast = AO_STDATA(st);
 
+#ifdef AO_ELF
+	if (stf->stf_compiler_type == CT_CLCC)
+		rational_products_used = rational_products_running();
+#endif
+
 	for (sn = stf->stf_snlist; sn != NULL; sn = sn->sn_next) {
 		symrec.symno = sn->sn_symno;
 		getsym(symrec.symio, symrec.symno, &nm);
@@ -735,15 +721,18 @@ fil_t *fil;
 		   When compiled with SC4 as a C rather than C++ program, UPS
 		   gets a bogus value for static globals out of the symbol table.
 		   Is not a problem with other compilers.
-		   */
+		   SC5 does this for C++ also. (RCB)
+		*/
 
 		if (nm.n_value != 0
 #ifdef AO_ELF
-		    && !(stf->stf_compiler_type == CT_CC &&
-		      (class == CL_EXT || class == CL_STAT) &&
-		      fil->fi_language == LANG_C)
+		    && !((stf->stf_compiler_type == CT_CC &&
+			  (class == CL_EXT || class == CL_STAT)) ||
+			  (stf->stf_compiler_type == CT_CLCC &&
+			   rational_products_used &&
+			   (class == CL_EXT || class == CL_STAT)))
 #endif
-		)
+			  )
 			addr = (taddr_t)nm.n_value + ast->st_base_address;
 		else
 			addr = lookup_global_addr(st, sn->sn_symtab_name);
@@ -964,8 +953,6 @@ block_t *
 ao_get_fu_blocks(f)
 func_t *f;
 {
-	static int want_clcc_hack = -1;
-	char *def;
 	nlist_t nm, extra_nm;
 	class_t class;
 	type_t *type;
@@ -1006,32 +993,7 @@ func_t *f;
 	 *  enums (see ci_basetype_name), so we need any type information
 	 *  in the file.  Thus we load the file types.
 	 */
-	if (want_clcc_hack == -1)
-	  want_clcc_hack =
-	    (def = (char *)wn_get_default("ClccWorkaround")) != NULL &&
-	      strcmp(def, "on") == 0;
-	if (want_clcc_hack)
-	{
-	  if (!(f->fu_fil->fi_flags & FI_FOUND_COMPILER))
-	  {
-	    get_fi_compiler(f->fu_fil);
-	    f->fu_fil->fi_flags |= FI_FOUND_COMPILER;
-	  }
-	}
-	if (want_clcc_hack &&
-	    stf->stf_compiler_type == CT_CLCC && stf->stf_symtab &&
-	    /*stf->stf_symtab->st_type == STT_SHLIB &&
-	    stf->stf_symtab->st_dynamic == TRUE &&*/ stf->stf_fil)
-	{
-	  fil_t *tmp_fil = stf->stf_fil;
-	  int i = 1;
-	  
-	  while (i && tmp_fil->fi_next)
-	    tmp_fil = tmp_fil->fi_next;
-	  get_fi_types(tmp_fil, FALSE);
-	}
-	else
-	  get_fi_types(f->fu_fil, FALSE);
+	get_fi_types(f->fu_fil, FALSE);
 
 	ap = f->fu_symtab->st_apool;
 	
@@ -1274,6 +1236,10 @@ func_t *f;
 		case N_LCSYM:
 #ifdef N_ROSYM
 		case N_ROSYM:
+#endif
+#ifdef N_ISYM					/* RCB, for SC5 */
+		case N_ISYM:
+		case N_ESYM:
 #endif
 			s = symstring(symrec.symio, symrec.symno);
 
@@ -1828,30 +1794,36 @@ fil_t *fil;
 bool compiler_found;
 Compiler_type compiler;
 {
-  stf_t *stf;
-  static Compiler_type likely = CT_UNKNOWN;
-  static bool compiler_found_once = FALSE;
+    stf_t *stf;
+    static Compiler_type likely = CT_UNKNOWN;
 
-  if (compiler_found == TRUE)
-  {
-    compiler_found_once = TRUE;
-    if (likely == CT_UNKNOWN)
-      likely = compiler;
+    if (compiler_found == TRUE)
+    {
+	likely = compiler;
+	return likely;
+    }
+
+    if (fil)
+    {
+	if (!(fil->fi_flags & FI_FOUND_COMPILER))
+	{
+	    get_fi_compiler(fil);
+	    fil->fi_flags |= FI_FOUND_COMPILER;
+	}
+
+	stf = AO_FIDATA(fil);
+	if (stf->stf_compiler_type != CT_UNKNOWN)  /* RCB */
+	    likely = stf->stf_compiler_type;
+    }
     return likely;
-  }
+}
 
-  if (compiler_found_once == TRUE || fil == NULL)
-    return likely;
-
-  if (!(fil->fi_flags & FI_FOUND_COMPILER))
-  {
-    get_fi_compiler(fil);
-    fil->fi_flags |= FI_FOUND_COMPILER;
-    compiler_found_once = TRUE;
-  }
-
-  stf = AO_FIDATA(fil);
-  if (stf->stf_compiler_type != CT_UNKNOWN)  /* RCB */
-    likely = stf->stf_compiler_type;
-  return likely;
+void set_compiler_version(double ver)
+{
+    if ( ver > compiler_version)
+	compiler_version = ver;
+}
+double get_compiler_version()
+{
+    return compiler_version;
 }
