@@ -51,6 +51,9 @@ char ups_ao_symparse_c_sccsid[] = "@(#)ao_symparse.c	1.7 04 Jun 1995 (UKC)";
  *  page dbx(5) and information in the document "SPARCworks 3.0.x Debugger
  *  Interface" (on the SPARCworks CDROM).
  *
+ *  An updated version of the above document is "Stabs Interface Manual, Version
+ *  4.0 (Febuary, 1999).  Known as READMEs/stabs.ps on the SunPro5 distrubution.
+ *
  *  Information on stabs format for gcc was obtained from:
  *   THE "STABS" DEBUG FORMAT
  *      Julia Menapace, Jim Kingdon, David MacKenzie
@@ -104,14 +107,14 @@ static aggr_or_enum_def_t *Record PROTO((stf_t *stf, Symrec *sr, const char **p_
 static var_t *GnuBaseClass PROTO((stf_t *stf, Symrec *sr, const char **p_s,
 			   bool eval, Compiler_type compiler));
 static void GnuStructMethods PROTO((stf_t *stf, Symrec *sr, const char **p_s));
-static var_t *SC4BaseClass PROTO((stf_t *stf, Symrec *sr, const char **p_s,
+static var_t *SunProBaseClass PROTO((stf_t *stf, Symrec *sr, const char **p_s,
 			   bool eval, Compiler_type compiler));
-static void SC4StructMethods PROTO((stf_t *stf, Symrec *sr, const char **p_s));
+static void SunProStructMethods PROTO((stf_t *stf, Symrec *sr, const char **p_s));
 static void SkipPastSemi PROTO((stf_t *stf, Symrec *sr, const char **p_s));
 static void SkipToSplChar PROTO((stf_t *stf, Symrec *sr, const char **p_s, const char* chars));
-static var_t* SC4StaticMembers PROTO((stf_t *stf, Symrec *sr, const char **p_s,
+static var_t* SunProStaticMembers PROTO((stf_t *stf, Symrec *sr, const char **p_s,
 			   var_t* next, Compiler_type compiler));
-static var_t* SC4VectorTable PROTO((stf_t *stf, Symrec *sr, const char **p_s,
+static var_t* SunProVectorTable PROTO((stf_t *stf, Symrec *sr, const char **p_s,
 			   var_t* next, Compiler_type compiler));
 static var_t *Field PROTO((stf_t *stf, Symrec *sr, const char **p_s,
 			   int is_struct, var_t *next, bool eval,
@@ -157,6 +160,8 @@ static Ftype *add_ftype_to_list PROTO((stf_t *stf, int tnum, bool have_type,
 static void lookup_ftype PROTO((stf_t *stf, int tnum, Ftype **p_ft, 
                                 Ftype **p_prevft));
 
+static type_t* get_safe_type_ptr();
+
 /*  The address of this variable is used as a special value for ty_typedef
  *  to signal that it is a basic type not a user defined typedef.
  */
@@ -254,6 +259,10 @@ Ftype **p_ft;
 		case N_PSYM:
 #ifdef N_ROSYM
 		case N_ROSYM:
+#endif
+#ifdef N_ISYM					/* RCB, for SC5 */
+		case N_ISYM:
+		case N_ESYM:
 #endif
 		case N_RSYM:
 		case N_SSYM:
@@ -1009,6 +1018,21 @@ type_t *type;
 	errf("\bunknown size range");
 }
 
+/*  Return a safe pointer to an "undefined type"
+*/
+static type_t*
+get_safe_type_ptr()
+{
+    static type_t save_type;
+    if ( save_type.ty_size == 0 )
+    {
+	/* First time, set up the safe type object incase anybody looks at it */
+	save_type.ty_code = TY_INT_ASSUMED;
+	save_type.ty_size = -1;
+    }
+    return &save_type;
+}
+
 /*  TypeId:
  *	Typenum
  *	Typenum '=' TypeDef
@@ -1224,7 +1248,7 @@ type_t *type;
 {
 	const char *s;
 	bool is_signed;
-	typecode_t typecode;
+	typecode_t typecode = TY_NOTYPE;
 	int width, offset, nbits;
 	const char *typename;
 	size_t len;
@@ -1237,10 +1261,14 @@ type_t *type;
 		panic("unknown value for signed flag in gbi");
 	bump_str((Symrec*)sr, &s);
 
-	/*  Ignore the Display (char/bool) flag.
+	/*  Handle the Display (char/bool) flag.
 	 */
 	if (*s == 'c' || *s == 'b' || *s == 'v') /* RGA added 'v' */
+	{
+		if ( *s == 'b')         /* RCB */
+		    typecode = TY_BOOLEAN;
 	        bump_str((Symrec*)sr, &s);
+	}
 
 	width = parse_num(stf, (Symrec*)sr, &s);
 	scheck((Symrec*)sr, &s, ';');
@@ -1256,7 +1284,11 @@ type_t *type;
 	/*  Can't use a switch here because of problems with duplicate
 	 *  cases when two types are the same size.
 	 */
-	if (width == sizeof(char)) {
+	if ( typecode == TY_BOOLEAN)   /* RCB */
+	{
+	    ci_set_sizeof_bool(width);
+	}
+	else if (width == sizeof(char)) {
 		if (IS_FORTRAN(language))
 			typecode = TY_CHARACTER;
 		else
@@ -1772,7 +1804,7 @@ type_t *type;
 	var_t *members, *last_members;
 	Compiler_type compiler = CT_CC;
 	bool terminate = FALSE;
-	bool sc4 =FALSE;
+	bool sun_pro =FALSE;
 
 	if (stf->stf_compiler_type == CT_UNKNOWN)
 	  compiler = ao_compiler(NULL, FALSE, CT_UNKNOWN);
@@ -1788,9 +1820,9 @@ type_t *type;
 	/* Get Base class info */
 	if (is_struct == 2 || *(*p_s - 2) == 'Y') /* RGA SC3 class or struct */
 	{
-	  sc4 = TRUE;
+	  sun_pro = TRUE;
 	  field_scheck(sr, &s, ';');
-          members = SC4BaseClass(stf,sr,&s, eval, compiler);
+          members = SunProBaseClass(stf,sr,&s, eval, compiler);
 	} else if ( *s == '!')
 	{
 	  bump_str(sr, &s);
@@ -1823,12 +1855,12 @@ type_t *type;
 		  last_members = members;
 	}
 	bump_str(sr, &s);
-	if ( sc4)
+	if ( sun_pro)
 	{
-	    SC4StructMethods(stf, sr, &s);
-	    last_members = SC4StaticMembers(stf, sr, &s, last_members, compiler);
+	    SunProStructMethods(stf, sr, &s);
+	    last_members = SunProStaticMembers(stf, sr, &s, last_members, compiler);
 	    SkipPastSemi(stf, sr, &s); /* Skip list of friends */
-	    last_members = SC4VectorTable(stf, sr, &s, last_members, compiler);
+	    last_members = SunProVectorTable(stf, sr, &s, last_members, compiler);
 	}
 	ae->ae_aggr_members = last_members;
 
@@ -1930,7 +1962,7 @@ Compiler_type compiler;
 	    v->va_language = stf->stf_language;
 	    v->va_flags = VA_BASECLASS;
 	    v->va_class = CL_MOS;
-	    v->va_type = rtype;
+	    v->va_type = rtype?rtype:get_safe_type_ptr();
 	    v->va_next = next;
 	    ci_make_baseclass_name(v);
 	    next = v;
@@ -1953,7 +1985,7 @@ Compiler_type compiler;
 	    v->va_language = stf->stf_language;
 	    v->va_flags = VA_BASECLASS|VA_HIDE_PTR;
 	    v->va_class = CL_MOS;
-	    v->va_type = rtype;
+	    v->va_type = rtype?rtype:get_safe_type_ptr();
 	    v->va_next = next;
 	    ci_make_baseclass_name(v);
 	    next = v;
@@ -2066,11 +2098,11 @@ const char **p_s;
 }
 
 /*
-**   Special parse routines for SC4
+**   Special parse routines for SunPro
 **     Added by Russ Browne (russ@amc.com)
 */
 static var_t*
-SC4BaseClass(stf,sr, p_s, eval, compiler)
+SunProBaseClass(stf,sr, p_s, eval, compiler)
 stf_t *stf;
 Symrec *sr;
 const char **p_s;
@@ -2084,17 +2116,19 @@ Compiler_type compiler;
     **
     **   For example
     **
-    **     C0(0,24)C4(0,250;
+    **     C0(0,24)C4(0,250);
     **
     **   An initial A, B, or C represents a normal base class
     **   An initial I, J, or K represents a virtual base class
     **
     **   Ref: SPARCworks debugger Interface, Chapter 6, Page 60
+    **
     */
     var_t *next = NULL;
     var_t *v = NULL;
     type_t *rtype;
     const char* s = *p_s;
+    bool sc5_vbase = FALSE;
     int virtual;
     int offset;
     while  (*s != ';')
@@ -2111,24 +2145,47 @@ Compiler_type compiler;
 	bump_str(sr, &s);
 	offset = parse_num(stf, sr, &s);
 	rtype = TypeId(stf, sr, &s, eval, 0);
+
+	/* For the SC5 compiler, -offset is an index into the vtbl to
+	** find the offset of the virtual base class.  Discard
+	if (virtual &&  get_compiler_version() >= 5.0)
+	    continue;
+	** such base classes */
+
 	v = (var_t *) alloc(stf->stf_symtab->st_apool, sizeof(var_t));
 	v->va_addr = offset;
 	v->va_name = NULL;
 	v->va_language = stf->stf_language;
 	v->va_flags = VA_BASECLASS;
 	v->va_class = CL_MOS;
-	v->va_type = rtype;
+	v->va_type = rtype?rtype:get_safe_type_ptr();
 	v->va_next = next;
 	next = v;
 	if ( virtual )
 	{
-	     type_t *btype = (type_t*)alloc(stf->stf_symtab->st_apool, sizeof(type_t));
-	     btype->ty_code = DT_PTR_TO;
-	     btype->ty_qualifiers = 0;
-	     btype->ty_size = -1;
-	     btype->ty_base = rtype;
-	     v->va_type = btype;
-	     v->va_flags |= VA_HIDE_PTR;
+
+	     /* If it is the SC5 compiler, the offset for the first
+	     ** virtual base class will be 2, which is really offset
+	     ** -8 into the vtbl.
+	     ** For SC4, all offsets will be a multiple of 4. 
+	     */
+	     if ( offset&3)
+		sc5_vbase = TRUE;
+	     if (sc5_vbase)
+	     {
+		 v->va_flags |= VA_VTBLBASED;
+		 v->va_addr = -4*offset;
+	     }
+	     else
+	     {
+		 type_t *btype = (type_t*)alloc(stf->stf_symtab->st_apool, sizeof(type_t));
+		 btype->ty_code = DT_PTR_TO;
+		 btype->ty_qualifiers = 0;
+		 btype->ty_size = -1;
+		 btype->ty_base = rtype;
+		 v->va_type = btype;
+		 v->va_flags |= VA_HIDE_PTR;
+	     }
 	}
 	ci_make_baseclass_name(v);
     }
@@ -2136,7 +2193,7 @@ Compiler_type compiler;
     return v;
 }
 static void
-SC4StructMethods(stf,sr, p_s)
+SunProStructMethods(stf,sr, p_s)
 stf_t *stf;
 Symrec *sr;
 const char **p_s;
@@ -2164,6 +2221,7 @@ const char **p_s;
     /* Skip to the next ';' and consume it */
     const char* s = *p_s;
     const char* end = strchr(s, ';');
+
     while(*s && end == NULL)
     {
 	s = get_cont_symstring(sr);
@@ -2195,7 +2253,7 @@ const char* chars;
     *p_s = s;
 }
 static var_t*
-SC4StaticMembers(stf,sr, p_s, next, compiler)
+SunProStaticMembers(stf,sr, p_s, next, compiler)
 stf_t *stf;
 Symrec *sr;
 const char **p_s;
@@ -2212,15 +2270,8 @@ Compiler_type compiler;
     **   Ref: SPARCworks debugger Interface, Chapter 6, Page 61
     */
 
-    static type_t bogusType;
     const char* s = *p_s;
 
-    if ( bogusType.ty_size == 0 )
-    {
-	/* First time, set up the bogus type incase anybody looks at it */
-	bogusType.ty_code = TY_INT_ASSUMED;
-	bogusType.ty_size = -1;
-    }
 
     if (*s == '\\')
       s = get_cont_symstring(sr);
@@ -2238,7 +2289,7 @@ Compiler_type compiler;
 	v->va_language = stf->stf_language;
 	v->va_flags = VA_STATIC_MEMBER;
 	v->va_class = CL_MOS;
-	v->va_type = &bogusType; /* point to something safe */
+	v->va_type = get_safe_type_ptr(); /* point to something safe */
 	v->va_next = next;
 	next = v;
 	if (*s == '\\')
@@ -2252,7 +2303,7 @@ Compiler_type compiler;
 }
 
 static var_t*
-SC4VectorTable(stf,sr, p_s, next, compiler)
+SunProVectorTable(stf,sr, p_s, next, compiler)
 stf_t *stf;
 Symrec *sr;
 const char **p_s;
@@ -2414,11 +2465,20 @@ bool *terminate;
 	  s += 2;
 
 	type = TypeId(stf, sr, &s, eval, prior_chr);
-	if (type && type->ty_code == TY_NOTYPE)
-	{
-	  *p_s = s;
-	  return NULL;
-	}
+	/*
+	** RCB:  This can occur when the type is currently
+	**       under construction by one of the callers of
+	**       in the call stack.  Returning NULL causes early
+	**       termination of the list of fields.
+	**       Problem reported to user's list,
+	**          May 9, 2000 by J.W.Armstrong@swansea.ac.uk
+	**       
+	** if (type && type->ty_code == TY_NOTYPE)
+	** {
+	**   *p_s = s;
+	**   return NULL;
+	** }
+	*/
 	if(*s == '=')		/* RGA have seen this from clcc */
 	{
 	  prior_chr = *s == '\\' ? *(s-1) : *s;
@@ -2432,7 +2492,7 @@ bool *terminate;
 	    s++;
 	    v->va_addr = (taddr_t) parse_name(sr, &s, stf->stf_symtab->st_apool,
 				    0, compiler, FALSE);
-	    v->va_type = type;
+	    v->va_type = type?type:get_safe_type_ptr();
 	    (void)field_scheck(sr,&s, ';');
 	    *p_s = s;
 	    v->va_next = next;
@@ -2511,23 +2571,23 @@ bool *terminate;
 	v->va_addr = offset / 8;
 
 	if (!type || (type && !type->ty_size)) /* RGA for clcc */
-	  v->va_type = type;
+	  v->va_type = type?type:get_safe_type_ptr();
 	else
 	    /* added by Joseph M. Orost joe@echologic.com */
-	  if (type->ty_code == TY_ENUM && width != typesize(type) * 8) {
+	  if (type && type->ty_code == TY_ENUM && width != typesize(type) * 8) {
 	    v->va_type = ci_make_bitfield_type
 	      (stf->stf_symtab->st_apool,
 	       ci_code_to_type(TY_UINT), offset - v->va_addr * 8, width);
 	    v->va_type->ty_bitfield->bf_type = type;
 	  } else
-	if (IS_BASIC_TYPE(type->ty_code) && width != typesize(type) * 8) {
+	if (type && IS_BASIC_TYPE(type->ty_code) && width != typesize(type) * 8) {
 		v->va_type = ci_make_bitfield_type(stf->stf_symtab->st_apool,
 						   type,
 						   offset - (int)v->va_addr * 8,
 						   width);
 	}
 	else {
-		v->va_type = type;
+		v->va_type = type?type:get_safe_type_ptr();
 	}
 
 	v->va_next = next;
