@@ -42,7 +42,7 @@ char ups_ao_symread_c_sccsid[] = "@(#)ao_symread.c	1.4 04 Jun 1995 (UKC)";
 #ifdef ARCH_MIPS
 #include "mips_frame.h"
 #endif
-#if defined(OS_SUNOS_5) || defined(OS_LINUX)
+#ifdef AO_ELF
 #include "elfstab.h"
 #endif
 #include "target.h"
