@@ -43,7 +43,8 @@ bool symtab_sym_name_to_sn PROTO((snlist_t *snlist, const char *name,
                               snlist_t **p_sn, bool justone));
 
 const char *parse_name PROTO((Symrec *sr, const char **p_s,
-			      alloc_pool_t *ap, int func, Compiler_type compiler));
+			      alloc_pool_t *ap, int func,
+			      Compiler_type compiler, bool is_fortran));
 
 bool parse_number PROTO((stf_t *stf, Symrec *sr, const char **p_s, int *p_val));
 bool parse_typenum PROTO((stf_t *stf, Symrec *sr, bool assume_paren,
