@@ -243,7 +243,7 @@ const char **p_cmdline;
 	ip->ip_solib_addrs = NULL;
 	
 	if (!scan_main_elf_symtab(xp->xp_apool, xp->xp_textpath, textfd,
-				  xp->xp_modtime, &ip->ip_solibs,
+				  xp->xp_modtime, &ip->ip_solibs, &ip->ip_solib_addrs,
 				  &xp->xp_mainfunc, &xp->xp_initfunc))
 	  return -1;
 
