@@ -1106,7 +1106,13 @@ unsigned flags;
 	li->li_apool = tx->tx_apool;
 	li->li_undef_vars = NULL;
 	li->li_undef_funcs = tx->tx_undef_funcs;
+#if 0
 	li->li_text = tx->tx_text;
+#else
+	li->li_text = (textword_t *)alloc(li->li_apool, (size_t)(tx->tx_pclim * sizeof(textword_t)));
+	memcpy(li->li_text, tx->tx_text, (size_t)(tx->tx_pclim * sizeof(textword_t)));
+	free(tx->tx_text);
+#endif
 	li->li_text_size = tx->tx_pc;
 	li->li_data = data;
 	li->li_data_size = data_addr;
@@ -1140,6 +1146,11 @@ unsigned flags;
 	li->li_undef_vars = reverse_linkname_list(li->li_undef_vars);
 	li->li_vars = reverse_linkname_list(li->li_vars);
 
+#if 1
+	if (tx->tx_labels != NULL)
+		free(tx->tx_labels);
+#endif
+
 	return li;
 }
 
@@ -1162,7 +1173,11 @@ unsigned long flags;
 
 	ap = alloc_create_pool();
 	tx = (text_t *)alloc(ap, sizeof(text_t));
+#if 0
 	tx->tx_apool = alloc_create_pool();
+#else
+	tx->tx_apool = ap;
+#endif
 
 	if ((flags & CI_CP_DONT_PANIC) != 0) {
 		if (ci_Catching_panics)
@@ -1265,6 +1280,7 @@ void
 ci_free_linkinfo(li)
 linkinfo_t *li;
 {
+	/* free(li->li_text); */
 	alloc_free_pool(li->li_apool);
 }
 
@@ -1273,6 +1289,7 @@ ci_free_machine(ma)
 machine_t *ma;
 {
 	/* free(ma->ma_text); */
+	free(ma->ma_fdtab);
 	alloc_free_pool(ma->ma_apool);
 }
 
