@@ -7,7 +7,7 @@
  *  unaltered copy of this message (including the copyright).
  */
 
-/* @(#)ci_compile_expr.h	1.3 4/7/91 (UKC) */
+/* @(#)ci_compile_expr.h	1.4 18/10/92 (UKC) */
 
 typedef enum { EC_ADDRESS, EC_EFFECT, EC_VALUE } expr_context_t;
 
@@ -22,5 +22,4 @@ addrtype_t ci_compile_var_reference PROTO((text_t *tx, var_t *v,
 					lexinfo_t *lx, expr_context_t context));
 
 void ci_do_static_initialisations PROTO((text_t *tx, initlist_t *initlist,
-					 taddr_t data_addr, taddr_t bss_addr,
-					 char *data_copy));
+					 char *data, char *extdata));
