@@ -745,6 +745,18 @@ const char *opname;
 				diagf(ET_ERROR, id->id_lexinfo,
 				      "%s is not a member of %s", id->id_name,
 				      ci_basetype_name(aggr_expr->ex_type));
+			else if (v->va_flags & VA_VTBLBASED)
+			{
+				/*  If implementing this proves worth while, 
+				**  we want to simulate something like
+				**
+				**   *((type*) (((char*)xxx)+((int*)(xxx->_vtbl))[v->va_addr/4]))
+				**
+				**   in place of xxx->field.
+				*/
+				diagf(ET_ERROR, id->id_lexinfo,
+				      "Sorry, virtual base classes in expressions not implemented for this compiler");
+			}
 		}
 		else {
 			if (code == TY_U_STRUCT || code == TY_U_UNION) {
@@ -1634,9 +1646,13 @@ expr_t *expr;
 		ci_push_conversion(expr, typecode, ET_PROMOTION);
 		break;
 	/* ENUM Fix : Dibyendu 02/Jan/99 */
+/*         RGA 16 June 00 : Commenting this out as it breaks casting
+	   of ints to enum strings in `add expr', and the %v 
+           conversion character in $printf statements
 	case TY_ENUM:
-		ci_push_conversion(expr, TY_INT, ET_PROMOTION);
+	  	ci_push_conversion(expr, TY_INT, ET_PROMOTION);
   		break;
+*/
 	default:
 		break;
 	}
