@@ -106,7 +106,7 @@ static struct {
 
 lex_env_t *Lex_env;
 
-#ifndef OS_SUNOS_4	/* use clib version for SunOS */
+#if !HAVE_STRSTR
 /*  BUG: this should be in a library somewhere.
  */
 char *
@@ -122,7 +122,7 @@ const char *s, *pat;
 	    return (char *)s; /* UGH */
     return NULL;
 }
-#endif /* !OS_SUNOS_4 */
+#endif
 
 void
 yyerror(s)
@@ -636,8 +636,20 @@ yylex()
 	    int i;
 
 	    --line;
-	    for (s = line; isalnum(*s) || *s == '_' || *s == '$'; ++s)
-		;
+            for (s = line; isalnum(*s) || *s == '_' || *s == '$'; ++s)
+                ;
+            if (*s == ':' && s[1] == ':'             
+                && (isalnum(s[2]) || s[2] == '_' || s[2] == '$')) {
+                s += 3;               
+                while (isalnum(*s) || *s == '_' || *s == '$')
+                    s++;            
+            }
+	    /* RGA Hack to allow breakpoint code to call ostream::operator<<
+	      or istream::operator>> */
+            if ((*s == '<' && s[1] == '<' && s[2] == '(') || 
+		(*s == '>' && s[1] == '>' && s[2] == '('))
+	      s += 2;               
+
 	    len = s - line;
 
 	    for (i = 0; i < NKEYS; ++i)
