@@ -42,6 +42,12 @@ typedef enum {
 	OP_CVT_TO_LONG, OP_CVT_TO_ULONG,
 	OP_CVT_TO_DOUBLE, OP_CVT_TO_FLOAT,
 	OP_CVT_TO_PTR, OP_CVT_TO_VOID,
+#if WANT_LL
+	OP_CVT_TO_LONG_LONG, OP_CVT_TO_ULONG_LONG, 
+#endif
+#if WANT_LDBL
+	OP_CVT_TO_LONG_DOUBLE,
+#endif
 
 	OP_LAST
 } optype_t;
@@ -139,10 +145,17 @@ typedef struct initexprst {
  *  about types sizes etc here - see ci_constexpr.c.
  */
 typedef union {
+#if WANT_LDBL
+	long_double_t cv_long_double;
+#endif
 	double cv_double;
 	float cv_float;
 	int cv_int;
 	unsigned cv_unsigned;
+#if WANT_LL
+	long_long_t cv_long_long;
+	ulong_long_t cv_ulong_long;
+#endif
 } constval_t;
 
 typedef struct declaratorst {
@@ -206,16 +219,21 @@ typedef struct sizeof_exprst {
 	taddr_t sz_size;
 } sizeof_expr_t;
 
+#define WANT_UINT_CONST		1
+
 typedef struct {
 	lexinfo_t *co_lexinfo;
+	bool co_is_unsigned;
 	union {
 		long cou_integer_val;
+		unsigned long cou_unsigned_val;
 		double cou_floating_val;
 		string_const_t cou_string_val;
 	} co_u;
 } constant_t;
 
 #define co_integer_val	co_u.cou_integer_val
+#define co_unsigned_val co_u.cou_unsigned_val
 #define co_floating_val	co_u.cou_floating_val
 #define co_string_val	co_u.cou_string_val
 
