@@ -65,10 +65,12 @@ value_t vl;
 bool want_hex;
 int num_bytes;
 {
-#if HAVE_LONG_DOUBLE
+#if HAVE_LONG_DOUBLE && (SIZEOF_LONG_DOUBLE != SIZEOF_DOUBLE)
+#define LG_FMT "Lg"
 	static char buf[120];
 	long double d;
 #else
+#define LG_FMT "g"
 	static char buf[60];
 	double d;
 #endif
@@ -209,14 +211,14 @@ int num_bytes;
 
 		switch (num_bytes) {
 		    case sizeof(float):
-			sprintf(buf, "%.6Lg", d);
+			sprintf(buf, "%.6" LG_FMT, d);
 			break;
 		    case sizeof(double):
-			sprintf(buf, "%.12Lg", d);
+			sprintf(buf, "%.12" LG_FMT, d);
 			break;
 #if HAVE_LONG_DOUBLE && (SIZEOF_LONG_DOUBLE != SIZEOF_DOUBLE)
 		    case sizeof(long double):
-			sprintf(buf, "%.18Lg", d);
+			sprintf(buf, "%.18" LG_FMT, d);
 			break;
 #endif
 		}
