@@ -18,17 +18,17 @@ typedef union {
 	unsigned int vl_uint;
 	long vl_long;
 	unsigned long vl_ulong;
-#if defined(HAVE_LONG_LONG)
+#if HAVE_LONG_LONG
 	long long vl_longlong;
 	unsigned long long vl_ulonglong;
 #endif
 	float vl_float;
 	double vl_double;
-#if defined(HAVE_LONG_DOUBLE)
+#if HAVE_LONG_DOUBLE
 	long double vl_longdouble;
-	int vl_ints[4];	/* for illegal double values */
+	int vl_ints[sizeof(long double)/sizeof(int)];	/* for illegal double values */
 #else
-	int vl_ints[2];	/* for illegal double values */
+	int vl_ints[sizeof(double)/sizeof(int)];	/* for illegal double values */
 #endif
 	int vl_logical;
 	taddr_t vl_addr;
@@ -47,3 +47,6 @@ const char *get_real PROTO((bool words_big_endian,
 			    value_t vl, bool want_hex, int num_bytes));
 
 taddr_t adjust_saved_reg_addr PROTO((target_t *xp, taddr_t addr, size_t size));
+int dump_to_buffer PROTO((target_t *xp, taddr_t addr, int len, int grp,
+			  char *buffer));
+
