@@ -84,7 +84,7 @@ int command;
 {
 	target_t *xp;
 	bool want_refresh;
-	stopres_t stopres = 0;	
+	stopres_t stopres = SR_FAILED;	
         /* RGA as per Callum Gibson callum@bain.oz.au */
 	char *ptr;
 	long k = 0;
@@ -161,6 +161,7 @@ int command;
 #endif
 #endif
 		  }
+		stopres = SR_SIG;
 		break;
 	case MR_TGT_DETACH:
 		kill_or_detach_from_target(xp);
@@ -179,6 +180,9 @@ int command;
 
 	refresh_target_display(xp, stopres, command == MR_TGT_START ||
 			       command == MR_TGT_ATTACH);
+
+	/* suggest raise window if started anything */
+	run_done(stopres != SR_FAILED);
 }
 
 stopres_t
@@ -294,6 +298,7 @@ int lnum;
 	xp_enable_breakpoint(xp, (breakpoint_t *)NULL);
 
 	refresh_target_display(xp, stopres, just_started);
+	run_done(stopres != SR_FAILED);
 }
 
 void
@@ -371,6 +376,23 @@ int lnum;
 	refresh_target_display(xp, stopres, just_started);
 }
 
+void
+refresh_target_with_rescan(xp, stopres, just_started)
+target_t *xp;
+stopres_t stopres;
+bool just_started;
+{
+  /* force all libraries to be rescaned in elf_next_symtab()
+     Only used when the Rescan Init File is pressed */
+
+  int orig_state;
+
+  orig_state = xp->xp_hit_solib_event;
+  xp->xp_hit_solib_event = TRUE;
+  refresh_target_display(xp, stopres, just_started);
+  xp->xp_hit_solib_event = orig_state;
+}
+
 void
 refresh_target_display(xp, stopres, just_started)
 target_t *xp;
@@ -607,7 +629,6 @@ check_for_dynamic_libs(xp, p_stopres)
      stopres_t *p_stopres;
 {
 #ifdef AO_ELF
-#if AO_USE_PROCFS
   if (*p_stopres == SR_BPT)
   {
     if (xp->xp_hit_solib_event == TRUE)
@@ -616,13 +637,18 @@ check_for_dynamic_libs(xp, p_stopres)
       can_install_all_breakpoints(xp) == TRUE)
     {
       uninstall_all_breakpoints(xp);
-      procfs_get_dynamic_solibs(xp);
+      rescan_dynamic_solibs(xp);
       refresh_target_display(xp, *p_stopres, FALSE);
       if (xp->xp_new_dynamic_libs_loaded == TRUE)
       {
 	remove_all_breakpoints();
+#if AO_USE_PROCFS
 	procfs_load_mmap_info(xp, FALSE);
+#endif
+#if AO_USE_PTRACE
+#endif
 	elf_restore_symtab_breakpoints();
+	redo_displayed_source_files();
 	xp->xp_new_dynamic_libs_loaded = FALSE;
 	xp->xp_hit_solib_event = FALSE;
       }
@@ -637,7 +663,6 @@ check_for_dynamic_libs(xp, p_stopres)
     }
   }
 #endif
-#endif
 }
 void
 check_for_dynamic_libs_sstep(xp, p_stopres)
@@ -648,7 +673,7 @@ check_for_dynamic_libs_sstep(xp, p_stopres)
 #if AO_USE_PROCFS
   if (xp->xp_hit_solib_event == TRUE)
     elf_save_symtab_breakpoints(xp, FALSE);
-  procfs_get_dynamic_solibs(xp);
+  elf_rescan_dynamic_solibs(xp);
   refresh_target_display(xp, *p_stopres, FALSE);
   if (xp->xp_new_dynamic_libs_loaded == TRUE &&
       can_install_all_breakpoints(xp) == TRUE)
