@@ -39,6 +39,8 @@ void jump_to_lnum PROTO((func_t *f, fil_t *fil, int lnum));
 
 #ifdef TARGET_H_INCLUDED
 int get_restart_sig PROTO((target_t *xp));
+void refresh_target_with_rescan PROTO((target_t *xp, stopres_t stopres,
+				       bool just_started));
 void refresh_target_display PROTO((target_t *xp, stopres_t stopres,
 				   bool just_started));
 void check_for_dynamic_libs PROTO((target_t *xp, stopres_t *p_stopres));
