@@ -372,7 +372,7 @@ errf_ofunc_t out_f;
 	  errf_ofunc_t oldf = NULL;
 	  int i, method = 0;
 
-	  if (strstr(f->fu_demangled_name, "::"))
+	  if (f != NULL && strstr(f->fu_demangled_name, "::"))
 	    method = 1;
 	  if (method)
 	    for (i = 0; f->fu_demangled_name[i] != ':'; i++);
@@ -393,17 +393,30 @@ errf_ofunc_t out_f;
 	  }
 	  else
 	  {   /* may be a virtual or inherited method - try wilcard  */
-	    var_t *v, *v1;
+	    var_t *v = 0, *v1;
 	    common_block_t *cblock;
 	    fil_t *fil0, *fil1;
 
-	    sprintf(class_name, "*::%s", wholename);
-	    if (!find_global_by_name(class_name, fil, f,
-				     (common_block_t *)NULL, 0, &f0, &f1,
-				     &v, &v1, &cblock, &fil0, &fil1))
+	    if (method)
+	      have_var = show_global
+		(sw, fil, f, (common_block_t *)NULL,
+		 class_name, TRUE, TRUE, FALSE, (objid_t *)NULL, FALSE);
+	    if (!have_var)
 	    {
-	      show_func_source(sw, f);
-	      have_var = 1;
+	      sprintf(class_name, "*::%s", wholename);
+	      if (!find_global_by_name(class_name, fil, f,
+				       (common_block_t *)NULL, 0, &f0, &f1,
+				       &v, &v1, &cblock, &fil0, &fil1))
+	      {
+		if (f != NULL)
+		  show_func_source(sw, f);
+		else
+		  if (v)
+		    have_var = show_global
+		      (sw, fil, f, (common_block_t *)NULL,
+		       class_name, TRUE, TRUE, FALSE, (objid_t *)NULL, FALSE);
+		have_var = 1;
+	      }
 	    }
 	  }
 	  free(class_name);
@@ -1239,6 +1252,33 @@ objid_t *p_var_obj;
 	return TRUE;
 }
 
+void
+show_given_global(fil, v)
+fil_t *fil;
+var_t *v;
+{
+  objid_t var_obj;
+  objid_t obj;
+
+  if (fil != NULL) {
+    add_source_file_object_if_required(fil, FALSE, FALSE);
+    obj = (objid_t)fil;
+  }
+  else
+    obj = GLOBALS_OBJCODE;
+
+  if ((var_obj = find_var(obj, v)) == NULL) {
+    var_obj = add_var_object(obj, v, OBJ_LAST_CHILD, FALSE);
+    if (get_object_type(obj) == OT_SFILE) {
+      change_type(obj, OT_SFILE_EX);
+      sort_children(SRCHEAD_OBJCODE, src_cmp);
+    }
+  }
+
+  select_object(var_obj, TRUE, OBJ_SELF);
+  ensure_visible(var_obj);
+}
+
 bool show_static_member(type, v)
 type_t* type;
 var_t* v;
