@@ -23,15 +23,15 @@ static MENU MM1 = {
 };
 static MENU MM2 = {
 	0x40,
-	698,
+	0,
 	"cut and paste",
 	48,
 	55,
 	55,
 	350,
 	90,
-	0,
-	0,
+	24560,
+	3,
 	0,
 	0,
 	0,
@@ -63,7 +63,7 @@ static MENU MM3 = {
 };
 static MENU MM4 = {
 	0x40,
-	0,
+	629,
 	"saving state",
 	116,
 	55,
@@ -103,7 +103,7 @@ static MENU MM5 = {
 };
 static MENU MM6 = {
 	0x40,
-	912,
+	0,
 	"by name",
 	66,
 	316,
@@ -143,13 +143,13 @@ static MENU MM7 = {
 };
 static MENU MM8 = {
 	0x40,
-	839,
+	0,
 	"conditional",
 	100,
 	316,
 	194,
 	588,
-	229,
+	228,
 	0,
 	0,
 	0,
@@ -167,9 +167,9 @@ static MENU MM9 = {
 	"saving breakpoints",
 	117,
 	316,
-	229,
+	228,
 	588,
-	264,
+	263,
 	0,
 	0,
 	0,
@@ -187,9 +187,9 @@ static MENU MM10 = {
 	"C++ syntax",
 	67,
 	316,
-	264,
+	263,
 	588,
-	299,
+	298,
 	0,
 	0,
 	0,
@@ -203,15 +203,15 @@ static MENU MM10 = {
 };
 static MENU MM11 = {
 	0x4,
-	264,
+	263,
 	NULL,
 	117,
 	316,
-	229,
+	228,
 	588,
-	299,
-	0,
-	0,
+	298,
+	25672,
+	3,
 	0,
 	0,
 	0,
@@ -223,13 +223,13 @@ static MENU MM11 = {
 };
 static MENU MM12 = {
 	0x4,
-	229,
+	228,
 	NULL,
 	100,
 	316,
 	194,
 	588,
-	299,
+	298,
 	0,
 	0,
 	0,
@@ -249,7 +249,7 @@ static MENU MM13 = {
 	316,
 	125,
 	588,
-	299,
+	298,
 	0,
 	0,
 	0,
@@ -263,13 +263,13 @@ static MENU MM13 = {
 };
 static MENU MM14 = {
 	0x40,
-	802,
+	0,
 	"auto completion",
 	68,
 	316,
-	299,
+	298,
 	588,
-	333,
+	332,
 	0,
 	0,
 	0,
@@ -283,13 +283,13 @@ static MENU MM14 = {
 };
 static MENU MM15 = {
 	0x40,
-	767,
+	0,
 	"listing matches",
 	69,
 	316,
-	333,
+	332,
 	588,
-	368,
+	367,
 	0,
 	0,
 	0,
@@ -303,13 +303,13 @@ static MENU MM15 = {
 };
 static MENU MM16 = {
 	0x4,
-	333,
+	332,
 	NULL,
 	68,
 	316,
-	299,
+	298,
 	588,
-	368,
+	367,
 	0,
 	0,
 	0,
@@ -323,13 +323,13 @@ static MENU MM16 = {
 };
 static MENU MM17 = {
 	0x1040,
-	663,
+	351,
 	"activation",
 	70,
 	316,
-	368,
+	367,
 	588,
-	403,
+	401,
 	0,
 	0,
 	0,
@@ -347,11 +347,11 @@ static MENU MM18 = {
 	"execute",
 	71,
 	316,
-	403,
+	401,
 	588,
-	438,
-	-15992,
-	29285,
+	436,
+	0,
+	0,
 	0,
 	0,
 	0,
@@ -363,13 +363,13 @@ static MENU MM18 = {
 };
 static MENU MM19 = {
 	0x40,
-	0,
+	907,
 	"duplicates",
 	72,
 	316,
-	438,
+	436,
 	588,
-	473,
+	471,
 	0,
 	0,
 	0,
@@ -383,15 +383,15 @@ static MENU MM19 = {
 };
 static MENU MM20 = {
 	0x4,
-	438,
+	436,
 	NULL,
 	71,
 	316,
-	403,
+	401,
 	588,
-	473,
-	-21,
-	-11,
+	471,
+	0,
+	0,
 	0,
 	0,
 	0,
@@ -403,13 +403,13 @@ static MENU MM20 = {
 };
 static MENU MM21 = {
 	0x4,
-	403,
+	401,
 	NULL,
 	70,
 	316,
-	368,
+	367,
 	588,
-	473,
+	471,
 	0,
 	0,
 	0,
@@ -423,13 +423,13 @@ static MENU MM21 = {
 };
 static MENU MM22 = {
 	0x4,
-	368,
+	367,
 	NULL,
 	68,
 	316,
-	299,
+	298,
 	588,
-	473,
+	471,
 	0,
 	0,
 	0,
@@ -443,13 +443,13 @@ static MENU MM22 = {
 };
 static MENU MM23 = {
 	0x404,
-	299,
+	298,
 	NULL,
 	0,
 	316,
 	125,
 	588,
-	473,
+	471,
 	0,
 	0,
 	0,
@@ -463,7 +463,7 @@ static MENU MM23 = {
 };
 static MENU MM24 = {
 	0x260,
-	0,
+	594,
 	"breakpoints",
 	49,
 	55,
@@ -490,7 +490,7 @@ static MENU MM25 = {
 	90,
 	350,
 	159,
-	0,
+	72,
 	0,
 	0,
 	0,
@@ -543,15 +543,15 @@ static MENU MM27 = {
 };
 static MENU MM28 = {
 	0x40,
-	98,
+	0,
 	"typing shortcuts",
 	112,
 	55,
 	194,
 	350,
 	229,
-	-15992,
-	0,
+	25168,
+	3,
 	0,
 	0,
 	0,
@@ -590,8 +590,8 @@ static MENU MM30 = {
 	229,
 	589,
 	263,
-	0,
-	0,
+	-25496,
+	3,
 	0,
 	0,
 	0,
@@ -603,15 +603,15 @@ static MENU MM30 = {
 };
 static MENU MM31 = {
 	0x40,
-	274,
+	455,
 	"step button",
 	74,
 	319,
 	263,
 	589,
 	298,
-	-87,
-	-13,
+	0,
+	0,
 	0,
 	0,
 	0,
@@ -623,13 +623,13 @@ static MENU MM31 = {
 };
 static MENU MM32 = {
 	0x40,
-	0,
+	421,
 	"casting",
 	75,
 	319,
 	298,
 	589,
-	332,
+	333,
 	0,
 	0,
 	0,
@@ -649,7 +649,7 @@ static MENU MM33 = {
 	319,
 	263,
 	589,
-	332,
+	333,
 	0,
 	0,
 	0,
@@ -669,9 +669,9 @@ static MENU MM34 = {
 	319,
 	229,
 	589,
-	332,
-	0,
-	0,
+	333,
+	-25568,
+	3,
 	0,
 	0,
 	0,
@@ -683,11 +683,11 @@ static MENU MM34 = {
 };
 static MENU MM35 = {
 	0x40,
-	114,
+	767,
 	"tags",
 	76,
 	319,
-	332,
+	333,
 	589,
 	367,
 	0,
@@ -703,13 +703,13 @@ static MENU MM35 = {
 };
 static MENU MM36 = {
 	0x40,
-	0,
+	698,
 	"base classes",
 	110,
 	319,
 	367,
 	589,
-	401,
+	402,
 	0,
 	0,
 	0,
@@ -722,126 +722,86 @@ static MENU MM36 = {
 	NULL,
 };
 static MENU MM37 = {
+	0x40,
+	628,
+	"breakpoint code",
+	77,
+	319,
+	402,
+	589,
+	437,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+};
+static MENU MM38 = {
+	0x4,
+	402,
+	NULL,
+	110,
+	319,
+	367,
+	589,
+	437,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	&MM36,
+	&MM37,
+	NULL,
+	NULL,
+};
+static MENU MM39 = {
 	0x4,
 	367,
 	NULL,
 	76,
 	319,
-	332,
+	333,
 	589,
-	401,
-	0,
+	437,
 	0,
+	3,
 	0,
 	0,
 	0,
 	NULL,
 	&MM35,
-	&MM36,
-	NULL,
-	NULL,
-};
-static MENU MM38 = {
-	0x40,
-	0,
-	"breakpoint code",
-	77,
-	319,
-	401,
-	589,
-	436,
-	0,
-	0,
-	0,
-	0,
-	0,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-};
-static MENU MM39 = {
-	0x40,
-	49,
-	"RWC Strings",
-	78,
-	319,
-	436,
-	589,
-	471,
-	0,
-	-13,
-	0,
-	0,
-	0,
-	NULL,
-	NULL,
-	NULL,
+	&MM38,
 	NULL,
 	NULL,
 };
 static MENU MM40 = {
-	0x4,
-	436,
-	NULL,
-	77,
-	319,
-	401,
-	589,
-	471,
-	0,
-	0,
-	0,
-	0,
-	0,
-	NULL,
-	&MM38,
-	&MM39,
-	NULL,
-	NULL,
-};
-static MENU MM41 = {
-	0x4,
-	401,
-	NULL,
-	76,
-	319,
-	332,
-	589,
-	471,
-	0,
-	0,
-	0,
-	0,
-	0,
-	NULL,
-	&MM37,
-	&MM40,
-	NULL,
-	NULL,
-};
-static MENU MM42 = {
 	0x404,
-	332,
+	333,
 	NULL,
 	0,
 	319,
 	229,
 	589,
-	471,
-	0,
-	0,
+	437,
+	-25664,
+	3,
 	0,
 	0,
 	0,
 	NULL,
 	&MM34,
-	&MM41,
+	&MM39,
 	NULL,
 	NULL,
 };
-static MENU MM43 = {
+static MENU MM41 = {
 	0x240,
 	0,
 	"C++ support",
@@ -851,19 +811,19 @@ static MENU MM43 = {
 	350,
 	263,
 	0,
-	0,
+	3,
 	0,
 	0,
 	0,
 	NULL,
-	&MM42,
+	&MM40,
 	NULL,
 	NULL,
 	NULL,
 };
-static MENU MM44 = {
+static MENU MM42 = {
 	0x40,
-	0,
+	593,
 	"source files",
 	52,
 	55,
@@ -881,7 +841,7 @@ static MENU MM44 = {
 	NULL,
 	NULL,
 };
-static MENU MM45 = {
+static MENU MM43 = {
 	0x40,
 	0,
 	"general",
@@ -890,8 +850,8 @@ static MENU MM45 = {
 	298,
 	536,
 	332,
-	0,
-	0,
+	-22560,
+	3,
 	0,
 	0,
 	0,
@@ -901,9 +861,9 @@ static MENU MM45 = {
 	NULL,
 	NULL,
 };
-static MENU MM46 = {
+static MENU MM44 = {
 	0x40,
-	0,
+	21568,
 	"pointers",
 	84,
 	303,
@@ -921,7 +881,7 @@ static MENU MM46 = {
 	NULL,
 	NULL,
 };
-static MENU MM47 = {
+static MENU MM45 = {
 	0x4,
 	332,
 	NULL,
@@ -930,28 +890,28 @@ static MENU MM47 = {
 	298,
 	536,
 	367,
-	0,
-	0,
+	67,
+	285,
 	0,
 	0,
 	0,
 	NULL,
-	&MM45,
-	&MM46,
+	&MM43,
+	&MM44,
 	NULL,
 	NULL,
 };
-static MENU MM48 = {
+static MENU MM46 = {
 	0x40,
-	0,
+	72,
 	"arrays",
 	85,
 	303,
 	367,
 	536,
 	402,
-	0,
-	0,
+	325,
+	628,
 	0,
 	0,
 	0,
@@ -961,9 +921,9 @@ static MENU MM48 = {
 	NULL,
 	NULL,
 };
-static MENU MM49 = {
+static MENU MM47 = {
 	0x40,
-	0,
+	837,
 	"structures",
 	86,
 	303,
@@ -981,7 +941,7 @@ static MENU MM49 = {
 	NULL,
 	NULL,
 };
-static MENU MM50 = {
+static MENU MM48 = {
 	0x4,
 	402,
 	NULL,
@@ -990,18 +950,18 @@ static MENU MM50 = {
 	367,
 	536,
 	436,
-	0,
-	0,
+	325,
+	663,
 	0,
 	0,
 	0,
 	NULL,
-	&MM48,
-	&MM49,
+	&MM46,
+	&MM47,
 	NULL,
 	NULL,
 };
-static MENU MM51 = {
+static MENU MM49 = {
 	0x4,
 	367,
 	NULL,
@@ -1016,14 +976,14 @@ static MENU MM51 = {
 	0,
 	0,
 	NULL,
-	&MM47,
-	&MM50,
+	&MM45,
+	&MM48,
 	NULL,
 	NULL,
 };
-static MENU MM52 = {
+static MENU MM50 = {
 	0x40,
-	0,
+	767,
 	"linked data",
 	87,
 	303,
@@ -1041,9 +1001,9 @@ static MENU MM52 = {
 	NULL,
 	NULL,
 };
-static MENU MM53 = {
+static MENU MM51 = {
 	0x40,
-	0,
+	733,
 	"changing values",
 	88,
 	303,
@@ -1061,7 +1021,7 @@ static MENU MM53 = {
 	NULL,
 	NULL,
 };
-static MENU MM54 = {
+static MENU MM52 = {
 	0x4,
 	471,
 	NULL,
@@ -1076,12 +1036,12 @@ static MENU MM54 = {
 	0,
 	0,
 	NULL,
-	&MM52,
-	&MM53,
+	&MM50,
+	&MM51,
 	NULL,
 	NULL,
 };
-static MENU MM55 = {
+static MENU MM53 = {
 	0x40,
 	0,
 	"typedefs",
@@ -1090,8 +1050,8 @@ static MENU MM55 = {
 	506,
 	536,
 	540,
-	0,
-	0,
+	-22040,
+	3,
 	0,
 	0,
 	0,
@@ -1101,7 +1061,7 @@ static MENU MM55 = {
 	NULL,
 	NULL,
 };
-static MENU MM56 = {
+static MENU MM54 = {
 	0x40,
 	0,
 	"expressions",
@@ -1121,7 +1081,7 @@ static MENU MM56 = {
 	NULL,
 	NULL,
 };
-static MENU MM57 = {
+static MENU MM55 = {
 	0x40,
 	0,
 	"dump memory",
@@ -1141,7 +1101,7 @@ static MENU MM57 = {
 	NULL,
 	NULL,
 };
-static MENU MM58 = {
+static MENU MM56 = {
 	0x4,
 	575,
 	NULL,
@@ -1156,12 +1116,12 @@ static MENU MM58 = {
 	0,
 	0,
 	NULL,
-	&MM56,
-	&MM57,
+	&MM54,
+	&MM55,
 	NULL,
 	NULL,
 };
-static MENU MM59 = {
+static MENU MM57 = {
 	0x4,
 	540,
 	NULL,
@@ -1170,18 +1130,18 @@ static MENU MM59 = {
 	506,
 	536,
 	610,
-	0,
-	0,
+	-22112,
+	3,
 	0,
 	0,
 	0,
 	NULL,
-	&MM55,
-	&MM58,
+	&MM53,
+	&MM56,
 	NULL,
 	NULL,
 };
-static MENU MM60 = {
+static MENU MM58 = {
 	0x4,
 	506,
 	NULL,
@@ -1196,12 +1156,12 @@ static MENU MM60 = {
 	0,
 	0,
 	NULL,
-	&MM54,
-	&MM59,
+	&MM52,
+	&MM57,
 	NULL,
 	NULL,
 };
-static MENU MM61 = {
+static MENU MM59 = {
 	0x404,
 	436,
 	NULL,
@@ -1216,14 +1176,14 @@ static MENU MM61 = {
 	0,
 	0,
 	NULL,
-	&MM51,
-	&MM60,
+	&MM49,
+	&MM58,
 	NULL,
 	NULL,
 };
-static MENU MM62 = {
+static MENU MM60 = {
 	0x240,
-	0,
+	559,
 	"examing data",
 	53,
 	55,
@@ -1236,12 +1196,12 @@ static MENU MM62 = {
 	0,
 	0,
 	NULL,
-	&MM61,
+	&MM59,
 	NULL,
 	NULL,
 	NULL,
 };
-static MENU MM63 = {
+static MENU MM61 = {
 	0x4,
 	298,
 	NULL,
@@ -1256,12 +1216,12 @@ static MENU MM63 = {
 	0,
 	0,
 	NULL,
-	&MM44,
-	&MM62,
+	&MM42,
+	&MM60,
 	NULL,
 	NULL,
 };
-static MENU MM64 = {
+static MENU MM62 = {
 	0x4,
 	263,
 	NULL,
@@ -1270,18 +1230,18 @@ static MENU MM64 = {
 	229,
 	350,
 	333,
-	0,
+	872,
 	0,
 	0,
 	0,
 	0,
 	NULL,
-	&MM43,
-	&MM63,
+	&MM41,
+	&MM61,
 	NULL,
 	NULL,
 };
-static MENU MM65 = {
+static MENU MM63 = {
 	0x4,
 	229,
 	NULL,
@@ -1297,11 +1257,11 @@ static MENU MM65 = {
 	0,
 	NULL,
 	&MM29,
-	&MM64,
+	&MM62,
 	NULL,
 	NULL,
 };
-static MENU MM66 = {
+static MENU MM64 = {
 	0x4,
 	159,
 	NULL,
@@ -1317,18 +1277,18 @@ static MENU MM66 = {
 	0,
 	NULL,
 	&MM26,
-	&MM65,
+	&MM63,
 	NULL,
 	NULL,
 };
-static MENU MM67 = {
+static MENU MM65 = {
 	0x40,
 	0,
-	"editable fields",
+	"general",
 	54,
-	55,
+	305,
 	333,
-	350,
+	539,
 	367,
 	0,
 	0,
@@ -1341,7 +1301,107 @@ static MENU MM67 = {
 	NULL,
 	NULL,
 };
+static MENU MM66 = {
+	0x40,
+	0,
+	"special characters",
+	101,
+	305,
+	367,
+	539,
+	402,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+};
+static MENU MM67 = {
+	0x40,
+	0,
+	"edit history",
+	105,
+	305,
+	402,
+	539,
+	437,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+};
 static MENU MM68 = {
+	0x4,
+	402,
+	NULL,
+	101,
+	305,
+	367,
+	539,
+	437,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	&MM66,
+	&MM67,
+	NULL,
+	NULL,
+};
+static MENU MM69 = {
+	0x404,
+	367,
+	NULL,
+	0,
+	305,
+	333,
+	539,
+	437,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	&MM65,
+	&MM68,
+	NULL,
+	NULL,
+};
+static MENU MM70 = {
+	0x240,
+	0,
+	"editable fields",
+	0,
+	55,
+	333,
+	350,
+	367,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	&MM69,
+	NULL,
+	NULL,
+	NULL,
+};
+static MENU MM71 = {
 	0x40,
 	0,
 	"command arguments",
@@ -1361,11 +1421,11 @@ static MENU MM68 = {
 	NULL,
 	NULL,
 };
-static MENU MM69 = {
+static MENU MM72 = {
 	0x4,
 	367,
 	NULL,
-	54,
+	0,
 	55,
 	333,
 	350,
@@ -1376,12 +1436,12 @@ static MENU MM69 = {
 	0,
 	0,
 	NULL,
-	&MM67,
-	&MM68,
+	&MM70,
+	&MM71,
 	NULL,
 	NULL,
 };
-static MENU MM70 = {
+static MENU MM73 = {
 	0x40,
 	0,
 	"custom menu",
@@ -1401,7 +1461,7 @@ static MENU MM70 = {
 	NULL,
 	NULL,
 };
-static MENU MM71 = {
+static MENU MM74 = {
 	0x40,
 	0,
 	"init file",
@@ -1421,7 +1481,7 @@ static MENU MM71 = {
 	NULL,
 	NULL,
 };
-static MENU MM72 = {
+static MENU MM75 = {
 	0x4,
 	437,
 	NULL,
@@ -1436,16 +1496,16 @@ static MENU MM72 = {
 	0,
 	0,
 	NULL,
-	&MM70,
-	&MM71,
+	&MM73,
+	&MM74,
 	NULL,
 	NULL,
 };
-static MENU MM73 = {
+static MENU MM76 = {
 	0x4,
 	402,
 	NULL,
-	54,
+	0,
 	55,
 	333,
 	350,
@@ -1456,12 +1516,12 @@ static MENU MM73 = {
 	0,
 	0,
 	NULL,
-	&MM69,
 	&MM72,
+	&MM75,
 	NULL,
 	NULL,
 };
-static MENU MM74 = {
+static MENU MM77 = {
 	0x40,
 	0,
 	"attach / detach",
@@ -1481,11 +1541,11 @@ static MENU MM74 = {
 	NULL,
 	NULL,
 };
-static MENU MM75 = {
+static MENU MM78 = {
 	0x40,
 	0,
-	"resizing windows",
-	103,
+	"signals",
+	104,
 	55,
 	506,
 	350,
@@ -1501,7 +1561,7 @@ static MENU MM75 = {
 	NULL,
 	NULL,
 };
-static MENU MM76 = {
+static MENU MM79 = {
 	0x4,
 	506,
 	NULL,
@@ -1516,100 +1576,40 @@ static MENU MM76 = {
 	0,
 	0,
 	NULL,
-	&MM74,
-	&MM75,
-	NULL,
-	NULL,
-};
-static MENU MM77 = {
-	0x40,
-	0,
-	"signals",
-	104,
-	55,
-	541,
-	350,
-	575,
-	0,
-	0,
-	0,
-	0,
-	0,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-};
-static MENU MM78 = {
-	0x40,
-	0,
-	"ups version",
-	118,
-	312,
-	575,
-	558,
-	611,
-	0,
-	0,
-	0,
-	0,
-	0,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-};
-static MENU MM79 = {
-	0x40,
-	0,
-	"`step' action",
-	115,
-	312,
-	611,
-	558,
-	647,
-	0,
-	0,
-	0,
-	0,
-	0,
-	NULL,
-	NULL,
-	NULL,
+	&MM77,
+	&MM78,
 	NULL,
 	NULL,
 };
 static MENU MM80 = {
-	0x4,
-	611,
-	NULL,
-	118,
-	312,
+	0x40,
+	0,
+	"About X Resources",
+	355,
+	305,
+	541,
+	570,
 	575,
-	558,
-	647,
 	0,
 	0,
 	0,
 	0,
 	0,
 	NULL,
-	&MM78,
-	&MM79,
+	NULL,
+	NULL,
 	NULL,
 	NULL,
 };
 static MENU MM81 = {
 	0x40,
 	0,
-	"scrollbars",
-	79,
-	312,
-	647,
-	558,
-	684,
+	"Fonts and Colors",
+	356,
+	305,
+	575,
+	570,
+	610,
 	0,
 	0,
 	0,
@@ -1622,34 +1622,34 @@ static MENU MM81 = {
 	NULL,
 };
 static MENU MM82 = {
-	0x40,
-	0,
-	"assembler",
-	80,
-	312,
-	684,
-	558,
-	720,
+	0x4,
+	575,
+	NULL,
+	355,
+	305,
+	541,
+	570,
+	610,
 	0,
 	0,
 	0,
 	0,
 	0,
 	NULL,
-	NULL,
-	NULL,
+	&MM80,
+	&MM81,
 	NULL,
 	NULL,
 };
 static MENU MM83 = {
 	0x40,
 	0,
-	"authors",
-	81,
-	312,
-	720,
-	558,
-	757,
+	"Split Screens",
+	357,
+	305,
+	610,
+	570,
+	645,
 	0,
 	0,
 	0,
@@ -1662,14 +1662,54 @@ static MENU MM83 = {
 	NULL,
 };
 static MENU MM84 = {
-	0x4,
-	720,
+	0x40,
+	0,
+	"Raise/Lower",
+	358,
+	305,
+	645,
+	570,
+	680,
+	0,
+	0,
+	0,
+	0,
+	0,
 	NULL,
-	80,
-	312,
-	684,
-	558,
-	757,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+};
+static MENU MM85 = {
+	0x4,
+	645,
+	NULL,
+	357,
+	305,
+	610,
+	570,
+	680,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	&MM83,
+	&MM84,
+	NULL,
+	NULL,
+};
+static MENU MM86 = {
+	0x4,
+	610,
+	NULL,
+	355,
+	305,
+	541,
+	570,
+	680,
 	0,
 	0,
 	0,
@@ -1677,51 +1717,371 @@ static MENU MM84 = {
 	0,
 	NULL,
 	&MM82,
-	&MM83,
-	NULL,
-	NULL,
-};
-static MENU MM85 = {
-	0x4,
-	684,
-	NULL,
-	79,
-	312,
-	647,
-	558,
-	757,
-	0,
-	0,
-	0,
-	0,
-	0,
-	NULL,
-	&MM81,
-	&MM84,
-	NULL,
-	NULL,
-};
-static MENU MM86 = {
-	0x404,
-	647,
-	NULL,
-	0,
-	312,
-	575,
-	558,
-	757,
-	0,
-	0,
-	0,
-	0,
-	0,
-	NULL,
-	&MM80,
 	&MM85,
 	NULL,
 	NULL,
 };
 static MENU MM87 = {
+	0x40,
+	0,
+	"Breakpoint Menu",
+	359,
+	305,
+	680,
+	570,
+	714,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+};
+static MENU MM88 = {
+	0x40,
+	0,
+	"Scrolling",
+	360,
+	305,
+	714,
+	570,
+	749,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+};
+static MENU MM89 = {
+	0x4,
+	714,
+	NULL,
+	359,
+	305,
+	680,
+	570,
+	749,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	&MM87,
+	&MM88,
+	NULL,
+	NULL,
+};
+static MENU MM90 = {
+	0x40,
+	0,
+	"Resizing Windows",
+	103,
+	305,
+	749,
+	570,
+	784,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+};
+static MENU MM91 = {
+	0x40,
+	0,
+	"No Mousehole",
+	361,
+	305,
+	784,
+	570,
+	819,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+};
+static MENU MM92 = {
+	0x4,
+	784,
+	NULL,
+	103,
+	305,
+	749,
+	570,
+	819,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	&MM90,
+	&MM91,
+	NULL,
+	NULL,
+};
+static MENU MM93 = {
+	0x4,
+	749,
+	NULL,
+	359,
+	305,
+	680,
+	570,
+	819,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	&MM89,
+	&MM92,
+	NULL,
+	NULL,
+};
+static MENU MM94 = {
+	0x404,
+	680,
+	NULL,
+	0,
+	305,
+	541,
+	570,
+	819,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	&MM86,
+	&MM93,
+	NULL,
+	NULL,
+};
+static MENU MM95 = {
+	0x240,
+	0,
+	"preferences",
+	0,
+	55,
+	541,
+	350,
+	575,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	&MM94,
+	NULL,
+	NULL,
+	NULL,
+};
+static MENU MM96 = {
+	0x40,
+	0,
+	"ups version",
+	118,
+	312,
+	575,
+	558,
+	610,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+};
+static MENU MM97 = {
+	0x40,
+	0,
+	"`step' action",
+	115,
+	312,
+	610,
+	558,
+	645,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+};
+static MENU MM98 = {
+	0x4,
+	610,
+	NULL,
+	118,
+	312,
+	575,
+	558,
+	645,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	&MM96,
+	&MM97,
+	NULL,
+	NULL,
+};
+static MENU MM99 = {
+	0x40,
+	0,
+	"scrollbars",
+	79,
+	312,
+	645,
+	558,
+	680,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+};
+static MENU MM100 = {
+	0x40,
+	0,
+	"assembler",
+	80,
+	312,
+	680,
+	558,
+	715,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+};
+static MENU MM101 = {
+	0x40,
+	0,
+	"authors",
+	81,
+	312,
+	715,
+	558,
+	751,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+};
+static MENU MM102 = {
+	0x4,
+	715,
+	NULL,
+	80,
+	312,
+	680,
+	558,
+	751,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	&MM100,
+	&MM101,
+	NULL,
+	NULL,
+};
+static MENU MM103 = {
+	0x4,
+	680,
+	NULL,
+	79,
+	312,
+	645,
+	558,
+	751,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	&MM99,
+	&MM102,
+	NULL,
+	NULL,
+};
+static MENU MM104 = {
+	0x404,
+	645,
+	NULL,
+	0,
+	312,
+	575,
+	558,
+	751,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	&MM98,
+	&MM103,
+	NULL,
+	NULL,
+};
+static MENU MM105 = {
 	0x240,
 	0,
 	"miscellaneous",
@@ -1736,12 +2096,12 @@ static MENU MM87 = {
 	0,
 	0,
 	NULL,
-	&MM86,
+	&MM104,
 	NULL,
 	NULL,
 	NULL,
 };
-static MENU MM88 = {
+static MENU MM106 = {
 	0x40,
 	0,
 	"append text",
@@ -1761,7 +2121,7 @@ static MENU MM88 = {
 	NULL,
 	NULL,
 };
-static MENU MM89 = {
+static MENU MM107 = {
 	0x40,
 	0,
 	"always clear",
@@ -1781,7 +2141,7 @@ static MENU MM89 = {
 	NULL,
 	NULL,
 };
-static MENU MM90 = {
+static MENU MM108 = {
 	0x404,
 	645,
 	NULL,
@@ -1796,12 +2156,12 @@ static MENU MM90 = {
 	0,
 	0,
 	NULL,
-	&MM88,
-	&MM89,
+	&MM106,
+	&MM107,
 	NULL,
 	NULL,
 };
-static MENU MM91 = {
+static MENU MM109 = {
 	0x240,
 	0,
 	"help window",
@@ -1816,12 +2176,12 @@ static MENU MM91 = {
 	0,
 	0,
 	NULL,
-	&MM90,
+	&MM108,
 	NULL,
 	NULL,
 	NULL,
 };
-static MENU MM92 = {
+static MENU MM110 = {
 	0x4,
 	610,
 	NULL,
@@ -1836,16 +2196,16 @@ static MENU MM92 = {
 	0,
 	0,
 	NULL,
-	&MM87,
-	&MM91,
+	&MM105,
+	&MM109,
 	NULL,
 	NULL,
 };
-static MENU MM93 = {
+static MENU MM111 = {
 	0x4,
 	575,
 	NULL,
-	104,
+	0,
 	55,
 	541,
 	350,
@@ -1856,12 +2216,12 @@ static MENU MM93 = {
 	0,
 	0,
 	NULL,
-	&MM77,
-	&MM92,
+	&MM95,
+	&MM110,
 	NULL,
 	NULL,
 };
-static MENU MM94 = {
+static MENU MM112 = {
 	0x4,
 	541,
 	NULL,
@@ -1876,16 +2236,16 @@ static MENU MM94 = {
 	0,
 	0,
 	NULL,
-	&MM76,
-	&MM93,
+	&MM79,
+	&MM111,
 	NULL,
 	NULL,
 };
-static MENU MM95 = {
+static MENU MM113 = {
 	0x4,
 	471,
 	NULL,
-	54,
+	0,
 	55,
 	333,
 	350,
@@ -1896,12 +2256,12 @@ static MENU MM95 = {
 	0,
 	0,
 	NULL,
-	&MM73,
-	&MM94,
+	&MM76,
+	&MM112,
 	NULL,
 	NULL,
 };
-static MENU MM96 = {
+static MENU MM114 = {
 	0x404,
 	333,
 	NULL,
@@ -1916,32 +2276,32 @@ static MENU MM96 = {
 	0,
 	0,
 	NULL,
-	&MM66,
-	&MM95,
+	&MM64,
+	&MM113,
 	NULL,
 	NULL,
 };
-static MENU MM97 = {
+static MENU MM115 = {
 	0x2260,
-	0,
+	-30584,
 	"Help",
 	101,
 	-1,
 	-1,
 	76,
 	32,
-	0,
-	0,
+	-30584,
+	8738,
 	0,
 	0,
 	0,
 	NULL,
-	&MM96,
+	&MM114,
 	NULL,
 	NULL,
 	NULL,
 };
-static MENU MM98 = {
+static MENU MM116 = {
 	0x40,
 	0,
 	"Backwards",
@@ -1961,7 +2321,7 @@ static MENU MM98 = {
 	NULL,
 	NULL,
 };
-static MENU MM99 = {
+static MENU MM117 = {
 	0x40,
 	0,
 	"Forwards",
@@ -1981,7 +2341,7 @@ static MENU MM99 = {
 	NULL,
 	NULL,
 };
-static MENU MM100 = {
+static MENU MM118 = {
 	0x408,
 	134,
 	NULL,
@@ -1996,16 +2356,16 @@ static MENU MM100 = {
 	0,
 	0,
 	NULL,
-	&MM98,
-	&MM99,
+	&MM116,
+	&MM117,
 	NULL,
 	NULL,
 };
-static MENU MM101 = {
+static MENU MM119 = {
 	0x260,
 	0,
 	"Search",
-	122,
+	362,
 	76,
 	-1,
 	194,
@@ -2016,12 +2376,12 @@ static MENU MM101 = {
 	0,
 	0,
 	NULL,
-	&MM100,
+	&MM118,
 	NULL,
 	NULL,
 	NULL,
 };
-static MENU MM102 = {
+static MENU MM120 = {
 	0x8,
 	76,
 	NULL,
@@ -2030,26 +2390,26 @@ static MENU MM102 = {
 	-1,
 	194,
 	32,
-	0,
+	1,
 	0,
 	0,
 	0,
 	0,
 	NULL,
-	&MM97,
-	&MM101,
+	&MM115,
+	&MM119,
 	NULL,
 	NULL,
 };
-static MENU MM103 = {
+static MENU MM121 = {
 	0x40,
 	0,
-	"Selected object",
+	"Snapshot Selected Object",
 	99,
-	249,
-	19,
-	551,
-	50,
+	236,
+	10,
+	606,
+	45,
 	0,
 	0,
 	0,
@@ -2061,15 +2421,15 @@ static MENU MM103 = {
 	NULL,
 	NULL,
 };
-static MENU MM104 = {
+static MENU MM122 = {
 	0x40,
 	0,
-	"All objects",
+	"Snapshot All Objects",
 	119,
-	249,
-	50,
-	551,
-	82,
+	236,
+	45,
+	606,
+	80,
 	0,
 	0,
 	0,
@@ -2081,35 +2441,35 @@ static MENU MM104 = {
 	NULL,
 	NULL,
 };
-static MENU MM105 = {
+static MENU MM123 = {
 	0x4,
-	50,
+	45,
 	NULL,
 	99,
-	249,
-	19,
-	551,
-	82,
+	236,
+	10,
+	606,
+	80,
 	0,
 	0,
 	0,
 	0,
 	0,
 	NULL,
-	&MM103,
-	&MM104,
+	&MM121,
+	&MM122,
 	NULL,
 	NULL,
 };
-static MENU MM106 = {
+static MENU MM124 = {
 	0x40,
 	0,
-	"msg logging on",
-	120,
-	249,
+	"Raise Display",
 	82,
-	551,
-	114,
+	236,
+	80,
+	606,
+	115,
 	0,
 	0,
 	0,
@@ -2121,15 +2481,15 @@ static MENU MM106 = {
 	NULL,
 	NULL,
 };
-static MENU MM107 = {
+static MENU MM125 = {
 	0x40,
 	0,
-	"msg logging off",
-	121,
-	249,
+	"Raise Source",
 	114,
-	551,
-	147,
+	236,
+	115,
+	606,
+	150,
 	0,
 	0,
 	0,
@@ -2141,50 +2501,290 @@ static MENU MM107 = {
 	NULL,
 	NULL,
 };
-static MENU MM108 = {
+static MENU MM126 = {
+	0x40,
+	0,
+	"Raise Output",
+	122,
+	236,
+	150,
+	606,
+	185,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+};
+static MENU MM127 = {
 	0x4,
+	150,
+	NULL,
 	114,
+	236,
+	115,
+	606,
+	185,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	&MM125,
+	&MM126,
+	NULL,
+	NULL,
+};
+static MENU MM128 = {
+	0x4,
+	115,
+	NULL,
+	82,
+	236,
+	80,
+	606,
+	185,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	&MM124,
+	&MM127,
+	NULL,
+	NULL,
+};
+static MENU MM129 = {
+	0x4,
+	80,
+	NULL,
+	99,
+	236,
+	10,
+	606,
+	185,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	&MM123,
+	&MM128,
+	NULL,
+	NULL,
+};
+static MENU MM130 = {
+	0x40,
+	0,
+	"Message Logging",
+	120,
+	236,
+	185,
+	606,
+	221,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+};
+static MENU MM131 = {
+	0x40,
+	0,
+	"No Raise On Break",
+	353,
+	236,
+	221,
+	606,
+	255,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+};
+static MENU MM132 = {
+	0x4,
+	221,
 	NULL,
 	120,
-	249,
-	82,
-	551,
-	147,
+	236,
+	185,
+	606,
+	255,
 	0,
 	0,
 	0,
 	0,
 	0,
 	NULL,
-	&MM106,
-	&MM107,
+	&MM130,
+	&MM131,
 	NULL,
 	NULL,
 };
-static MENU MM109 = {
+static MENU MM133 = {
+	0x40,
+	0,
+	"Raise On Break",
+	106,
+	236,
+	255,
+	606,
+	290,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+};
+static MENU MM134 = {
+	0x40,
+	0,
+	"Raise on Break\nLower On Run",
+	121,
+	236,
+	290,
+	606,
+	360,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+};
+static MENU MM135 = {
+	0x40,
+	0,
+	"Raise on Break\nIconify On Run",
+	354,
+	236,
+	360,
+	606,
+	430,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+};
+static MENU MM136 = {
+	0x4,
+	360,
+	NULL,
+	121,
+	236,
+	290,
+	606,
+	430,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	&MM134,
+	&MM135,
+	NULL,
+	NULL,
+};
+static MENU MM137 = {
+	0x4,
+	290,
+	NULL,
+	106,
+	236,
+	255,
+	606,
+	430,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	&MM133,
+	&MM136,
+	NULL,
+	NULL,
+};
+static MENU MM138 = {
+	0x4,
+	255,
+	NULL,
+	120,
+	236,
+	185,
+	606,
+	430,
+	0,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	&MM132,
+	&MM137,
+	NULL,
+	NULL,
+};
+static MENU MM139 = {
 	0x404,
-	82,
+	185,
 	NULL,
 	0,
-	249,
-	19,
-	551,
-	147,
-	-87,
-	-13,
+	236,
+	10,
+	606,
+	430,
+	-100,
+	-22,
 	0,
 	0,
 	0,
 	NULL,
-	&MM105,
-	&MM108,
+	&MM129,
+	&MM138,
 	NULL,
 	NULL,
 };
-static MENU MM110 = {
+static MENU MM140 = {
 	0x2260,
 	0,
-	"Snapshot",
+	"Windows",
 	98,
 	194,
 	-1,
@@ -2196,12 +2796,12 @@ static MENU MM110 = {
 	0,
 	0,
 	NULL,
-	&MM109,
+	&MM139,
 	NULL,
 	NULL,
 	NULL,
 };
-static MENU MM111 = {
+static MENU MM141 = {
 	0x40,
 	0,
 	"cancel",
@@ -2221,7 +2821,7 @@ static MENU MM111 = {
 	NULL,
 	NULL,
 };
-static MENU MM112 = {
+static MENU MM142 = {
 	0x40,
 	0,
 	"confirm",
@@ -2241,7 +2841,7 @@ static MENU MM112 = {
 	NULL,
 	NULL,
 };
-static MENU MM113 = {
+static MENU MM143 = {
 	0x404,
 	49,
 	NULL,
@@ -2256,12 +2856,12 @@ static MENU MM113 = {
 	0,
 	0,
 	NULL,
-	&MM111,
-	&MM112,
+	&MM141,
+	&MM142,
 	NULL,
 	NULL,
 };
-static MENU MM114 = {
+static MENU MM144 = {
 	0x2260,
 	0,
 	"Quit",
@@ -2276,12 +2876,12 @@ static MENU MM114 = {
 	0,
 	0,
 	NULL,
-	&MM113,
+	&MM143,
 	NULL,
 	NULL,
 	NULL,
 };
-static MENU MM115 = {
+static MENU MM145 = {
 	0x8,
 	336,
 	NULL,
@@ -2296,8 +2896,8 @@ static MENU MM115 = {
 	0,
 	0,
 	NULL,
-	&MM110,
-	&MM114,
+	&MM140,
+	&MM144,
 	NULL,
 	NULL,
 };
@@ -2316,8 +2916,8 @@ MENU cmd_men = {
 	0,
 	0,
 	NULL,
-	&MM102,
-	&MM115,
+	&MM120,
+	&MM145,
 	NULL,
 	NULL,
 };
