@@ -23,7 +23,7 @@ static MENU MM1 = {
 };
 static MENU MM2 = {
 	0x40,
-	0,
+	184,
 	"Backwards",
 	98,
 	61,
@@ -83,7 +83,7 @@ static MENU MM4 = {
 };
 static MENU MM5 = {
 	0x260,
-	0,
+	506,
 	"Search",
 	120,
 	125,
@@ -123,15 +123,15 @@ static MENU MM6 = {
 };
 static MENU MM7 = {
 	0x40,
-	0,
+	1,
 	"Load",
 	108,
 	252,
 	-1,
 	379,
 	107,
-	0,
-	0,
+	160,
+	1024,
 	0,
 	0,
 	0,
@@ -162,13 +162,13 @@ static MENU MM8 = {
 	NULL,
 };
 static MENU MM9 = {
-	0x8,
-	379,
-	NULL,
-	117,
-	252,
-	-1,
+	0x40,
+	0,
+	"Close",
+	67,
 	506,
+	-1,
+	631,
 	107,
 	0,
 	0,
@@ -176,8 +176,48 @@ static MENU MM9 = {
 	0,
 	0,
 	NULL,
-	&MM7,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+};
+static MENU MM10 = {
+	0x8,
+	506,
+	NULL,
+	115,
+	379,
+	-1,
+	631,
+	107,
+	64,
+	0,
+	0,
+	0,
+	0,
+	NULL,
 	&MM8,
+	&MM9,
+	NULL,
+	NULL,
+};
+static MENU MM11 = {
+	0x8,
+	379,
+	NULL,
+	108,
+	252,
+	-1,
+	631,
+	107,
+	480,
+	0,
+	0,
+	0,
+	0,
+	NULL,
+	&MM7,
+	&MM10,
 	NULL,
 	NULL,
 };
@@ -188,16 +228,16 @@ MENU out_men = {
 	0,
 	-1,
 	-1,
-	506,
+	631,
 	107,
 	0,
-	0,
+	2,
 	0,
 	0,
 	0,
 	NULL,
 	&MM6,
-	&MM9,
+	&MM11,
 	NULL,
 	NULL,
 };
