@@ -4,7 +4,7 @@
 static MENU MM1 = {
 	0x40,
 	0,
-	"expand",
+	"Expand",
 	97,
 	21,
 	186,
@@ -13,6 +13,8 @@ static MENU MM1 = {
 	-30584,
 	-30584,
 	0,
+	0,
+	0,
 	NULL,
 	NULL,
 	NULL,
@@ -22,7 +24,7 @@ static MENU MM1 = {
 static MENU MM2 = {
 	0x40,
 	-30584,
-	"collapse",
+	"Collapse",
 	99,
 	322,
 	186,
@@ -31,6 +33,8 @@ static MENU MM2 = {
 	0,
 	0,
 	0,
+	0,
+	0,
 	NULL,
 	NULL,
 	NULL,
@@ -49,6 +53,8 @@ MENU sgh_men = {
 	0,
 	0,
 	0,
+	0,
+	0,
 	NULL,
 	&MM1,
 	&MM2,
