@@ -140,8 +140,10 @@ const char Bpt_format[] = "%8cs:%[-]:3c:61cb line:%[B]6cn%[-]9cq  \n";
  */
 #define BPT_FLIST_TERMINATOR	((fval_t)-1)
 
+Edit_history* bpt_history = NULL;
+
 fnamemap_t Bpt_fnamemap[] = {
-	{ FN_BPT_FNAME,	"function-name",	FALSE,	bpfunc_quitfunc	},
+	{ FN_BPT_FNAME,	"function-name",	FALSE,	bpfunc_quitfunc, &bpt_history	},
 	{ FN_BPT_LNUM,	"lnum",			FALSE,	bplnum_quitfunc	},
 	{ FN_BPT_STATE,	"state",		FALSE,	bpstate_quitfunc},
 	{ 0,		NULL,			FALSE,	NULL		},
@@ -619,64 +621,61 @@ bpfunc_draw(dets)
 register struct drawst *dets;
 {
 /* AMC enhancements */
-  if (dets->selected)
-    wn_wd_ttext(dets->dr_wn, (char *)dets->dr_fval,
-		dets->dr_x, dets->dr_y, dets->dr_width,
-		dets->dr_bg, dets->dr_fg);
-  else
-    wn_wd_ttext(dets->dr_wn, (char *)dets->dr_fval,
-		dets->dr_x, dets->dr_y, dets->dr_width,
-		dets->dr_fg, dets->dr_bg);
+	if (dets->selected)
+	    wn_wd_ttext(dets->dr_wn, (char *)dets->dr_fval,
+			dets->dr_x, dets->dr_y, dets->dr_width,
+			dets->dr_bg, dets->dr_fg);
+	else
+	    wn_wd_ttext(dets->dr_wn, (char *)dets->dr_fval,
+			dets->dr_x, dets->dr_y, dets->dr_width,
+			dets->dr_fg, dets->dr_bg);
 }
 
 static void
 bpstate_draw(dets)
-     register struct drawst *dets;
+register struct drawst *dets;
 {
-  long fg;
-  static color_done = -1;
-  static color_t color_fg;
-  int bp_enabled, use_alloc_color;
+	long fg;
+	static color_done = -1;
+	static color_t color_fg;
+	int bp_enabled, use_alloc_color;
 
-  if (color_done == -1)
-  {
-    const char *colorname;
+	if (color_done == -1)
+	{
+	    const char *colorname;
 
-    color_fg.co_pixel = WN_FG;
-    if (wn_use_extended_text_colors() == TRUE ||
-	wn_use_menu_colors() == TRUE)
-    {
-      colorname = wn_get_default("DisabledTextColor");
-      if (colorname == NULL)
-	colorname = "gray";
-      wn_parse_color(colorname, &color_fg);
-      if (!wn_alloc_color(&color_fg, colorname))
-	color_fg.co_pixel = WN_FG;
-      color_done = 1;	/* done and using colors */
-    }
-  }
+	    color_fg.co_pixel = WN_FG(dets->dr_wn);
+	    if (wn_use_extended_text_colors() || wn_use_menu_colors()) {
+		colorname = wn_get_default("DisabledTextColor");
+		if (colorname == NULL)
+			colorname = "gray";
+		wn_parse_color(dets->dr_wn, colorname, &color_fg);
+		if (!wn_alloc_color(dets->dr_wn, &color_fg, colorname))
+			color_fg.co_pixel = WN_FG(dets->dr_wn);
+		color_done = 1;	/* done and using colors */
+	    }
+	}
 
-  bp_enabled = global_bp_enabled(0,0);
-  use_alloc_color = wn_use_alloc_color(0);
+	bp_enabled = global_bp_enabled(0,0);
+	use_alloc_color = wn_use_alloc_color(0);
 
-  if (!bp_enabled && use_alloc_color)
-    fg = color_fg.co_pixel;
-  else
-    fg = dets->dr_fg;
+	if (!bp_enabled && use_alloc_color)
+	    fg = color_fg.co_pixel;
+	else
+	    fg = dets->dr_fg;
 
-    if (dets->selected)
-      wn_wd_ttext(dets->dr_wn, (char *)dets->dr_fval,
-		dets->dr_x, dets->dr_y, dets->dr_width,
-		dets->dr_bg, fg);
-  else
-    wn_wd_ttext(dets->dr_wn, (char *)dets->dr_fval,
-		dets->dr_x, dets->dr_y, dets->dr_width,
-		fg, dets->dr_bg);
+	if (dets->selected)
+	    wn_wd_ttext(dets->dr_wn, (char *)dets->dr_fval,
+			dets->dr_x, dets->dr_y, dets->dr_width,
+			dets->dr_bg, fg);
+	else
+	    wn_wd_ttext(dets->dr_wn, (char *)dets->dr_fval,
+			dets->dr_x, dets->dr_y, dets->dr_width,
+			fg, dets->dr_bg);
 	/*
 	**   If globally disabled, grey out the field
 	*/
-	if (!bp_enabled && !use_alloc_color)
-	{
+	if (!bp_enabled && !use_alloc_color) {
 		wn_shade_area(  dets->dr_wn
 			      , dets->dr_x, dets->dr_y
 			      , dets->dr_width, dets->dr_depth
@@ -1086,6 +1085,7 @@ int lnum;
 
 	bd = make_bd(fil, lnum, f);
 	
+	display_area_overlay_unselect_global_selection();
 	td_set_obj_updating(OBJ_UPDATING_OFF);
 	new_object((objid_t)bd, OT_BPT, BPHEAD_OBJCODE, OBJ_LAST_CHILD);
 	set_all_fields((objid_t)bd, fields, BPT_FLIST_TERMINATOR);
@@ -1789,6 +1789,7 @@ char *arg;
 
 	ba = (bptargs_t *)arg;
 
+	display_area_overlay_unselect_global_selection();
 	oldstate = td_set_obj_updating(OBJ_UPDATING_OFF);
 	
 	bd = (bpdesc_t *)obj;
@@ -2114,8 +2115,9 @@ func_t** pfirst;
 }
 
 void
-bpt_getcolor(obj, p_fg, p_bg)
+bpt_getcolor(obj, wn, p_fg, p_bg)
 objid_t obj;
+int wn;
 long *p_fg, *p_bg;
 {
   static color_t color = { -1, 0, 0, 0 };
@@ -2127,21 +2129,21 @@ long *p_fg, *p_bg;
   {
     const char *colorname;
 
-    color.co_pixel = WN_FG;
+    color.co_pixel = WN_FG(wn);
     if (wn_use_extended_text_colors() == TRUE)
     {
       colorname = wn_get_default("BreakpointTextColor");
       if (colorname != NULL)
       {
-	wn_parse_color(colorname, &color);
-	if (!wn_alloc_color(&color,colorname))
-	  color.co_pixel = WN_FG;
+	wn_parse_color(wn, colorname, &color);
+	if (!wn_alloc_color(wn, &color,colorname))
+	  color.co_pixel = WN_FG(wn);
       }
     }
   }
-  if (bd->bd_inactive)
-    *p_fg = WN_FG;
+  if (bd && bd->bd_inactive)
+    *p_fg = WN_FG(wn);
   else
     *p_fg = color.co_pixel;
-  *p_bg = WN_BG;
+  *p_bg = WN_BG(wn);
 }
