@@ -40,6 +40,14 @@ char ups_obj_buildf_c_sccsid[] = "@(#)obj_buildf.c	1.31 04 Jun 1995 (UKC)";
 #include "menudata.h"
 #include "tdr.h"
 
+typedef struct
+{
+  int row;
+  int user_row;
+  int type;
+  char *label;
+} Display_row;
+
 static const char *gen_getobjname PROTO((objid_t code));
 static void init_objtype PROTO((int objtype, const char *format, ot_t *ot,
 				bool have_window));
@@ -376,47 +384,223 @@ update_variable_values()
 	}
 }
 
+static int get_row_resource_number(resource, row_default, max_rows, check_only)
+char *resource; 
+int row_default;
+int max_rows;
+int check_only;
+{
+  static int rows[9], first_time = 1;
+  char *env_str, *ptr = 0;
+  char violation_msg[100];
+  int j, k, n, used, str_ok = 0;
+  long i = 0;
+
+  if (first_time)
+  {
+    first_time = 0;
+    for (i = 0; i < max_rows; i++)
+      rows[i] = 0;
+  }
+  i = row_default;
+  env_str = (char *)wn_get_default(resource);
+  if (env_str)
+  {
+    i = strtol(env_str, &ptr, 0);
+    str_ok = i && ptr;
+  }
+  else
+    if (check_only)
+      return 0;
+
+  if (env_str && str_ok)
+  {
+    str_ok = (i > 0 && i <= max_rows);
+  }
+  if (env_str && !str_ok)
+  {
+    sprintf(violation_msg,
+	    "WARNING: ups*%s resource must be an integer between 1 and %d\n",
+	    resource, max_rows);
+    fprintf(stderr, violation_msg);
+    i = row_default;
+  }
+
+  for (j = 0; j < max_rows; j++)
+  {
+    if (rows[j] == i)
+    {
+      if (env_str && str_ok)
+      {
+	sprintf(violation_msg,
+		"WARNING: ups*%s resource value of %d already used\n",
+		resource, i);
+	fprintf(stderr, violation_msg);
+      }
+      for (n = 1; n <= max_rows; n++)
+      {
+	for (used = 0, k = 0; k < max_rows; k++)
+	  if (rows[k] == n)
+	  {
+	    used = 1;
+	    break;
+	  }
+	if (!used)
+	{
+	  i = n;
+	  if (env_str && str_ok)
+	  {
+	    sprintf(violation_msg,
+		    "  - using %d for ups*%s instead.\n",
+		    i, resource);
+	    fprintf(stderr, violation_msg);
+	  }
+	  break;
+	}
+      }
+    }
+    if (rows[j] == 0)
+    {
+      rows[j] = i;
+      if (env_str && !str_ok)
+      {
+	sprintf(violation_msg,
+		"  - using %d for ups*%s instead.\n",
+		i, resource);
+	fprintf(stderr, violation_msg);
+      }
+      break;
+    }
+  }
+  return (i);
+}
+  
+static int row_compare(a, b)
+Display_row *a, *b;
+{
+  if (a->row < b->row)
+    return (1);
+  if (a->row > b->row)
+    return (-1);
+  return (0);  
+}
+
 void
 initialise_display_area(xp, args, use_full_path)
 target_t *xp;
 const char *args;
 bool use_full_path;
 {
-	symtab_t *st;
-	bool have_common_blocks, have_modules;
+  /* RGA 11/5/99 Add X resource support for controlling the order of
+     objects in the display window. Numbering starts at 1 at the top
+     The target row is a double height row, all others are single.
+     Defaults to historical ordering. If one wants to put the breakpoints
+     above the function stack (and one is not using Fortran), use:
 
-	have_modules = have_common_blocks = FALSE;
+     Ups*BreakpointsRow: 6
+     Ups*FunctionsRow: 7
 
-	st = NULL;
-	while (xp_next_symtab(xp, st, FALSE, &st)) {
-		if (st->st_cblist != NULL)
-			have_common_blocks = TRUE;
-		if (st->st_modules != NULL)
-			have_modules = TRUE;
-	}
+     Only rows of interest need to be specified  other rows are positioned 
+     according to their default values. The resources control the positions on 
+     tool invocation only. 
 
-	add_breakpoint_header((objid_t)NULL);
-	
-	new_object(FHEAD_OBJCODE, OT_FHEAD, (objid_t)NULL, OBJ_CHILD);
-	
-	if (have_common_blocks)
-		new_object(CBHEAD_OBJCODE, OT_CBHEAD, (objid_t)NULL, OBJ_CHILD);
+     */
 
-	if (have_modules)
-		new_object(MHEAD_OBJCODE, OT_MHEAD, (objid_t)NULL, OBJ_CHILD);
-		
-	new_object(SRCHEAD_OBJCODE, OT_SRCHEAD, (objid_t)NULL, OBJ_CHILD);
-	Src_header_header_exists = TRUE; 
+  static Display_row display_row_array[] = {
+    {1, 0, OT_COM, "TargetRow"}, 
+    {2, 0, OT_SGHEAD, "SignalsRow"}, 
+    {3, 0, OT_ENVHEAD, "EnvironmentRow"}, 
+    {4, 0, OT_GLOBALS, "UntypedVariablesRow"}, 
+    {5, 0, OT_SRCHEAD, "SourceFilesRow"}, 
+    {6, 0, OT_MHEAD, "ModulesRow"}, 
+    {7, 0, OT_CBHEAD, "CommonBlocksRow"}, 
+    {8, 0, OT_FHEAD, "FunctionsRow"}, 
+    {9, 0, OT_BPHEAD, "BreakpointsRow"}};
+
+#define MAX_ROW	(sizeof display_row_array / sizeof *display_row_array)
+ 
+   Display_row *dis;
+   int i, j, max_rows = MAX_ROW;
+   symtab_t *st;
+   bool have_common_blocks, have_modules;
+
+  have_modules = have_common_blocks = FALSE;
+  
+  st = NULL;
+  while (xp_next_symtab(xp, st, FALSE, &st)) {
+    if (st->st_cblist != NULL)
+      have_common_blocks = TRUE;
+    if (st->st_modules != NULL)
+      have_modules = TRUE;
+  }
+
+  if (have_modules == FALSE)
+  {
+    display_row_array[5].row = 0; /* disable OT_MHEAD */
+    max_rows--;
+  }
+  if (have_common_blocks == FALSE)
+  {
+    display_row_array[6].row = 0; /* disable OT_CBHEAD*/
+    max_rows--;
+  }
+  
+  for (i = j = 0, dis = display_row_array; j < MAX_ROW; j++, dis++)
+    if (dis->row > 0)
+    {
+      dis->user_row = get_row_resource_number(dis->label, i + 1, max_rows, 1);
+      i++;
+    }
+
+  for (i = j = 0, dis = display_row_array; j < MAX_ROW; j++, dis++)
+    if (dis->row > 0)
+    {
+      if (dis->user_row)
+	dis->row = dis->user_row;
+      else
+	dis->row = get_row_resource_number(dis->label, i + 1, max_rows, 0);
+      i++;
+    }
+  qsort((char *)display_row_array, MAX_ROW, sizeof(Display_row), row_compare);
+
+  for (j = 0, dis = display_row_array; j < MAX_ROW; j++, dis++)
+    switch (dis->type)
+    {
+    case OT_COM:
+      add_target_object((objid_t)NULL, xp->xp_textpath, args, use_full_path);
+      break;
+    case OT_SGHEAD:
+      add_signals_header((objid_t)NULL);
+      break;
+    case OT_ENVHEAD:
+      add_env_header((objid_t)NULL);
+      break;
+    case OT_GLOBALS:
+      add_globals_header((objid_t)NULL);
+      break;
+    case OT_SRCHEAD:
+      new_object(SRCHEAD_OBJCODE, OT_SRCHEAD, (objid_t)NULL, OBJ_CHILD);
 	/* RGA - ugh, but can't do collapse_files if this */
 	/* has not been done */
-	add_globals_header((objid_t)NULL);
-	add_env_header((objid_t)NULL);
-	add_signals_header((objid_t)NULL);
-
-	if (can_get_target_vars(xp))	
-		add_functions(xp, FHEAD_OBJCODE);
-
-	add_target_object((objid_t)NULL, xp->xp_textpath, args, use_full_path);
+      Src_header_header_exists = TRUE; 
+      break;
+    case OT_MHEAD:
+      if (have_modules)
+	new_object(MHEAD_OBJCODE, OT_MHEAD, (objid_t)NULL, OBJ_CHILD);
+      break;
+    case OT_CBHEAD:
+      if (have_common_blocks)
+	new_object(CBHEAD_OBJCODE, OT_CBHEAD, (objid_t)NULL, OBJ_CHILD);
+      break;
+    case OT_FHEAD:
+      new_object(FHEAD_OBJCODE, OT_FHEAD, (objid_t)NULL, OBJ_CHILD);
+      if (can_get_target_vars(xp))	
+	add_functions(xp, FHEAD_OBJCODE);
+      break;
+    case OT_BPHEAD:
+      add_breakpoint_header((objid_t)NULL);
+      break;
+    }
 }
 
 void
