@@ -9,6 +9,7 @@
 
 /* @(#)obj_env.h	1.7 09 Apr 1995 (UKC) */
 
+extern struct Edit_history* env_history;
 const char **get_environment PROTO((void));
 
 #ifdef OBJ_H_INCLUDED
@@ -19,7 +20,7 @@ void do_envhead PROTO((objid_t obj, int command, char *arg));
 void do_env PROTO((objid_t obj, int command, char *arg));
 void free_env PROTO((objid_t obj));
 void env_getsize PROTO((objid_t obj, objid_t unused_par, sz_t *sz));
-void env_getcolor PROTO((objid_t obj, long *p_fg, long *p_bg));
+void env_getcolor PROTO((objid_t obj, int wn, long *p_fg, long *p_bg));
 
 extern const char Envhead_format[];
 extern const char Env_format[];
