@@ -933,8 +933,9 @@ construct_source_path_used(fil, buff)
 }
 
 void
-srcfile_getcolor(obj, p_fg, p_bg)
+srcfile_getcolor(obj, wn, p_fg, p_bg)
 objid_t obj;
+int wn;
 long *p_fg, *p_bg;
 {
   static color_t color= { -1, 0, 0, 0 };
@@ -943,26 +944,27 @@ long *p_fg, *p_bg;
   {
     const char *colorname;
     
-    color.co_pixel = WN_FG;
+    color.co_pixel = WN_FG(wn);
     if (wn_use_extended_text_colors() == TRUE)
     {
       colorname = wn_get_default("SourceFileTextColor");
       if (colorname != NULL)
       {
-	wn_parse_color(colorname, &color);
-	if (!wn_alloc_color(&color,colorname))
-	  color.co_pixel = WN_FG;
+	wn_parse_color(wn, colorname, &color);
+	if (!wn_alloc_color(wn, &color,colorname))
+	  color.co_pixel = WN_FG(wn);
       }
     }
   }
   *p_fg = color.co_pixel;
-  *p_bg = WN_BG;
+  *p_bg = WN_BG(wn);
 }
 
 
 void
-func_getcolor(obj, p_fg, p_bg)
+func_getcolor(obj, wn, p_fg, p_bg)
 objid_t obj;
+int wn;
 long *p_fg, *p_bg;
 {
   static color_t color= { -1, 0, 0, 0 };
@@ -971,18 +973,18 @@ long *p_fg, *p_bg;
   {
     const char *colorname;
     
-    color.co_pixel = WN_FG;
+    color.co_pixel = WN_FG(wn);
     if (wn_use_extended_text_colors() == TRUE)
     {
       colorname = wn_get_default("FunctionTextColor");
       if (colorname != NULL)
       {
-	wn_parse_color(colorname, &color);
-	if (!wn_alloc_color(&color,colorname))
-	  color.co_pixel = WN_FG;
+	wn_parse_color(wn, colorname, &color);
+	if (!wn_alloc_color(wn, &color,colorname))
+	  color.co_pixel = WN_FG(wn);
       }
     }
   }
   *p_fg = color.co_pixel;
-  *p_bg = WN_BG;
+  *p_bg = WN_BG(wn);
 }
