@@ -239,14 +239,14 @@ register struct drawst *dets;
 		 dets->dr_y+dets->dr_depth/2,
 		 dets->dr_x + dets->dr_width,
 		 dets->dr_y+dets->dr_depth/2,
-		 WN_BG);
+		 WN_BG(dets->dr_wn));
   else
     wn_draw_line(dets->dr_wn,
 		 dets->dr_x,
 		 dets->dr_y+dets->dr_depth/2,
 		 dets->dr_x + dets->dr_width,
 		 dets->dr_y+dets->dr_depth/2,
-		 WN_FG);
+		 WN_FG(dets->dr_wn));
 }
 
 fdef_t Fsig_fdefs[] = {
@@ -765,23 +765,32 @@ char *arg;
 		
 	case MR_DISPLAY_SOURCE:
 		fil = stk->stk_fil;
-		
+
 		if (fil == NULL) {
 			errf("`%s' was not compiled with the -g flag",
 			     stk->stk_func->fu_demangled_name);
 			return;
 		}
-		
+
 		if (set_and_display_stopline(fil, stk->stk_lnum))
 			Current_stack = stk;
 		
 		break;
-		
+
 	case MR_ADD_EXPRESSION:
+		fil = stk->stk_fil;
+
+		if (fil == NULL) {
+			errf("`%s' was not compiled with the -g flag",
+			     stk->stk_func->fu_demangled_name);
+			return;
+		}
+
 		st_get_fi_vars(stk->stk_fil);	/* ensure globals loaded */
 		add_expr_object(par, FU_BLOCKS(stk->stk_func), "",
 				stk->stk_func->fu_language, OBJ_LAST_CHILD);
 		break;
+
 	case MR_SHOW_PATH:
 		fil = stk->stk_fil;
 		if (!show_source_path_assumed(fil, FALSE))
@@ -1325,6 +1334,8 @@ int **obj_var;
     return found;
 
   par = (objid_t) Current_stack;
+  if (!par)
+    return found;
   f = get_stack_func(par, &junk, &junk);
   
 #ifdef ARCH_MIPS
