@@ -20,6 +20,8 @@ char ups_reg_c_sccsid[] = "@(#)reg.c	1.18 16 Sep 1994 (UKC)";
 #include <local/ukcprog.h>
 #include <mtrprog/alloc.h>
 #include <local/wn.h>
+#include <local/wn.h>
+#include <local/menu3.h>
 #include <local/obj/mhole.h>
 
 #include "reg.h"
@@ -59,6 +61,7 @@ struct Region {
 	re_destroy_proc_t destroy_proc;
 	re_accept_focus_proc_t accept_focus_proc;
 	re_release_focus_proc_t release_focus_proc;
+	re_get_display_proc_t get_display_proc;
 
 	char *focus_data;
 	bool force_quit;
@@ -97,14 +100,27 @@ static void grey_out PROTO((Region *region, int wn,
 			    int width, int height,
 			    re_draw_reason_t draw_reason));
 #endif /* HH */
+static void set_current_typing_region PROTO((int wn));
+static void icon_notify PROTO(( int wn, int iconify));
+static void close_window PROTO((int wn));
 
 /*  BUG: these should be packaged up with the root of a region tree in
  *       a regtree_t structure
  */
 static bool Quit_event_loop;
+static Region  *Root_regions[MAXROOTS];
+static bool    IsLeadRegion[MAXROOTS];
+static int numRegions = 0;
 static Region *Keyboard_re = NULL;
 static Region *Default_keyboard_re = NULL;
-static Region *Root_region;
+static int iconic_state = FALSE;
+
+
+/* structure used to arbitrate between multible typing line regions:
+*/
+static struct { int root_win; Region* typing_line, *extended; }
+      typing_lines[MAXROOTS] = { { -1, 0, 0}, {-1, 0, 0}, {-1, 0, 0}};
+
 
 void
 re_set_data(re, data)
@@ -150,6 +166,20 @@ Region *re;
 {
 	return re->wn;
 }
+int
+re_get_root_wn(re)
+Region *re;
+{
+    return wn_get_root_window(re->wn);
+}
+
+bool
+re_same_root_wn(re1, re2)
+Region* re1;
+Region* re2;
+{
+    return wn_same_root_window(re1->wn, re2->wn);
+}
 
 void
 re_set_minsize(re, width, height)
@@ -259,6 +289,23 @@ re_release_focus_proc_t release_proc;
 	re->release_focus_proc = release_proc;
 }
 
+void
+re_set_display_callback(re, proc)
+Region* re;
+re_get_display_proc_t proc;
+{
+    re->get_display_proc = proc;
+}
+
+struct Edit_display*
+re_get_display(re)
+Region* re;
+{
+    if ( re &&  re->get_display_proc)
+	return (*re->get_display_proc)(re);
+    return NULL;
+}
+
 Region *
 re_make_region(wn)
 int wn;
@@ -348,6 +395,7 @@ int fixed_size;
 	new_re->destroy_proc = NULL;
 	new_re->accept_focus_proc = NULL;
 	new_re->release_focus_proc = NULL;
+	new_re->get_display_proc = NULL;
 
 	new_re->data = NULL;
 	new_re->force_quit = FALSE;
@@ -682,11 +730,16 @@ char *focus_data;
 	Default_keyboard_re = re;
 }
 
+
 void
-re_set_rootwin(region)
+re_add_rootwin(region, lead_region)
 Region *region;
+bool lead_region;  /* True if iconifying region iconifys all */
 {
-	Root_region = region;
+   
+        Root_regions[numRegions] = region;
+        IsLeadRegion[numRegions]  = lead_region;
+        numRegions++;
 }
 
 void
@@ -694,12 +747,22 @@ re_redraw_root(event_type, clear_first)
 unsigned long event_type;
 bool clear_first;
 {
+	int regNum;
+	Region* root;
 	if (event_type == EV_WINDOW_RESIZED) {
-		await_big_enough_window(Root_region);
-		resize(Root_region);
+	    for ( regNum = 0; regNum < numRegions; regNum++)
+		{
+		root = Root_regions[regNum];
+		await_big_enough_window(root);
+		resize(root);
+		}
+	}
+	for ( regNum = 0; regNum < numRegions; regNum++)
+	{
+		root = Root_regions[regNum];
+		re_expose(root, clear_first);
 	}
 
-	re_expose(Root_region, clear_first);
 }
 
 static void
@@ -737,42 +800,53 @@ Region *re;
 		mesg_width = wn_strwidth(mesg, font);
 		mesg_height = font->ft_height;
 
-		wn_set_area(re->wn, 0, 0, win_width, win_height, WN_BG);
+		wn_set_area(re->wn, 0, 0, win_width, win_height, WN_BG(re->wn));
 		wn_tputs(re->wn, mesg, (win_width - mesg_width) / 2,
-				          (win_height - mesg_height) / 2);
+					(win_height - mesg_height) / 2);
 		wn_next_event(re->wn, EV_WINDOW_RESIZED | EV_WINDOW_EXPOSED,
-									&event);
+								    &event);
 	}
 }
 
 void
-re_event_loop(region)
-Region *region;
+re_event_loop()
 {
 	const int evmask = EV_BUTTON_UP | EV_BUTTON_DOWN | EV_MOUSE_MOVED | EV_KEY |
 			   EV_INTERRUPT | EV_WINDOW_EXPOSED | EV_WINDOW_RESIZED;
 	Region *root;
 	int last_cursor;
+	int last_wn = -1;
 	static const char **last_mhcaps;
+	wn_set_icon_notify_func(icon_notify);
+	wn_set_close_func(close_window);
 
-	root = region;
 	last_cursor = -1;
 	
 	for (Quit_event_loop = FALSE; !Quit_event_loop; ) {
-		unsigned long resize_event;
+		bool resized = FALSE;
 		int cursor;
 		Region *re;
 		event_t event;
+		int regNum;
 
+		for ( regNum = 0; regNum < numRegions; regNum++)
+		{
+		    unsigned long resize_event;
+		    root = Root_regions[regNum];
 		resize_event = wn_get_resize_event(root->wn);
 		if (resize_event != 0) {
 			if (display_area_overlay_control(0, 0))
 			  display_area_overlay_control(0, 2);
 			re_redraw_root(resize_event, FALSE);
 			td_record_refresh();
-			/* reset, no paint */
-			continue;
+			resized = TRUE;
+		    }
 		}
+		if ( resized)
+		{
+		    /* reset, no paint */
+		    continue;
+		  }
 
 		wn_next_event(WN_ANY, evmask, &event);
 
@@ -784,22 +858,28 @@ Region *region;
 		/*  We want key presses anywhere in the window to go to the
 		 *  typing line.
 		 */
+		set_current_typing_region(event.ev_wn);
 		if (Keyboard_re != NULL && event.ev_type == EV_KEY) {
 			re = Keyboard_re;
 		}
-		else {
+		else
+		{
 			re = (Region *)wn_get_win_data(event.ev_wn);
+			if ( event.ev_wn != last_wn)
+			{
 
-			cursor = (re != NULL) ? re->cursor : CU_DEAD;
-			if (cursor != last_cursor) {
+			    if (re)
+			    {
+			        cursor = re->cursor;
 				set_bm_cursor(re->wn, cursor);
-				last_cursor = cursor;
-			}
 			if (re->mhcaps != last_mhcaps) {
 				mhdraw(re->mhcaps);
 				last_mhcaps = re->mhcaps;
 			}
 		}
+			    last_wn= event.ev_wn;
+			}
+		}
 
 		if (re != NULL) {
 			const int swmask = B_SHIFT_MASK | B_CONTROL_MASK;
@@ -821,6 +901,13 @@ Region *region;
 	}
 }
 
+void re_handle_event(re, ev)
+Region* re;
+event_t* ev;
+{
+    (*re->input_proc)(re, ev);
+}
+
 void
 re_expose(re, clear_first)
 Region *re;
@@ -829,7 +916,7 @@ bool clear_first;
 
 	if (clear_first)
 		wn_set_area(re->wn, 0, 0,
-				   re->size[WIDTH], re->size[HEIGHT], WN_BG);
+			    re->size[WIDTH], re->size[HEIGHT], WN_BG(re->wn));
 	expose(re, RE_EXPOSED);
 }
 
@@ -1194,3 +1281,147 @@ Region *region;
 {
 	return region->parent;
 }
+
+/*  Set up a typing line region, and the "extended" region containing it
+**  The "extended" region is a possibly larger region that should be
+**  used to prompt for input.
+*/
+void
+re_set_typing_regions(typing_line, extended)
+Region* typing_line;
+Region* extended;
+{
+    int index =0;
+    while (typing_lines[index].root_win >= 0)
+	index++;
+
+    typing_lines[index].root_win = wn_get_root_window(typing_line->wn);
+    typing_lines[index].typing_line = typing_line;
+    typing_lines[index].extended = extended;
+}
+
+/* See that the current typing line is in the same main window as the cursor */
+static void
+set_current_typing_region(wn)
+int wn;
+{
+    static int last_wn = -1;
+    int root;
+
+    if (last_wn >= 0 && wn_same_root_window(wn,  last_wn))
+	return;
+
+    root = wn_get_root_window(wn);
+    if ( root>=0)
+    {
+	bool saw_Keyboard_re = FALSE;
+	int index = -1;
+	int i;
+	for ( i = 0; i < numRegions; i++)
+	{
+	    if (typing_lines[i].root_win == root)
+		index = i;
+	    if ( Keyboard_re == typing_lines[i].typing_line)
+		saw_Keyboard_re = TRUE;
+	}
+
+	if ( index >= 0)
+	{
+	    Default_keyboard_re = typing_lines[index].typing_line;
+	    set_typing_line_region(Default_keyboard_re);
+	    set_extended_typing_line_region(typing_lines[index].extended);
+
+	    if (saw_Keyboard_re)
+		re_set_keyboard_focus(Default_keyboard_re, NULL);
+	}
+    }
+    last_wn = wn;
+}
+
+/*  If a lead window is iconified, iconify all the others.
+**  If a lead window is de-iconified, de-iconify all others.
+*/
+static void icon_notify( wn, iconify)
+int wn;
+int iconify;
+{
+   static int icon_state[MAXROOTS];
+   static int was_icon_state[MAXROOTS];
+   int i,j;
+   int do_raise = FALSE;
+   wn = wn_get_root_window(wn);
+   for (i = 0; i < numRegions; i++)
+      if ( wn==Root_regions[i]->wn)
+          break;
+
+   if ( i >= numRegions)
+       return;  /* should never happen */
+
+   icon_state[i] = iconify;
+
+   /* If it is a leader window, iconify/deiconify all the others */
+   if (   iconic_state != iconify
+       && IsLeadRegion[i])
+   {
+        iconic_state = iconify;
+        for (j = 0; j < numRegions; j++)
+        {
+            if ( j == i)
+                continue;
+            if( iconify)
+            {
+                was_icon_state[j] = icon_state[j];
+                wn_iconify_window(Root_regions[j]->wn);
+            } else if ( !was_icon_state[j])
+            {
+                wn_raise_window(Root_regions[j]->wn);
+                do_raise = TRUE;
+            }
+        }
+        /* If we raised anything, put the main window on top */
+        if (do_raise)
+            wn_raise_window(wn);
+    }
+}
+
+/* Close the window, if it is not a lead window, otherwise quit ups
+*/
+static void
+close_window(wn)
+int wn;
+{
+  extern MENU tgt_stop_men;
+  int i;
+  wn = wn_get_root_window(wn);
+  for (i = 0; i < numRegions; i++)
+      if ( wn==Root_regions[i]->wn)
+          break;
+
+  if ( i >= numRegions)
+       return;  /* should never happen */
+
+   if ( IsLeadRegion[i] )
+   {
+#if WANT_ICONIFY_WM_DELETE
+        /* Iconify the lead window, thus the entire app */
+        wn_iconify_window(wn);
+#else
+	/* Quit - could cause unpleasant surprises */
+        if (Mselectable(&tgt_stop_men))
+            target_stop();
+        re_set_exit_event_loop_flag();
+#endif
+   } else
+   {
+        /* "close" the window
+        **  we just unmap it, so it doesn't get displayed
+        */
+        wn_unmap_window(wn);
+   }
+}
+
+int
+re_in_icon_state()
+{
+   return iconic_state;
+}
