@@ -146,7 +146,8 @@ size_t *p_start, *p_lim;
 		if (ebstart > start)
 			break;
 
-		start += edit_get_propchange_point(eb->limpc) - ebstart;
+		/* RCB: Add one for newline at end of edit block. */
+		start += edit_get_propchange_point(eb->limpc) - ebstart+1;
 	}
 
 	*p_start = start;
@@ -175,7 +176,8 @@ int *p_lnum;
 		if (lim > point)
 			break;
 
-		delta += lim - edit_get_propchange_point(eb->startpc);
+		/* RCB: Add one for newline at end of edit block. */
+		delta += lim - edit_get_propchange_point(eb->startpc)+1;
 	}
 
 	point -= delta;
@@ -192,6 +194,27 @@ int *p_lnum;
 	return FALSE;
 }
 
+Edit_propchange*
+srcbuf_get_propchange_at_point(sb, point)
+Srcbuf *sb;
+size_t point;
+{
+        size_t pos = point;
+        Editblock *eb;
+        
+        for (eb = sb->eblist; eb != NULL; eb = eb->next) {
+                size_t ebstart;
+                
+                ebstart = edit_get_propchange_point(eb->startpc);
+                if (    ebstart <= point
+                     && point <= edit_get_propchange_point(eb->limpc))
+                    return eb->startpc;
+        }
+        
+        return NULL;
+}
+
+
 Edit_buffer *
 srcbuf_get_buffer(sb)
 Srcbuf *sb;
@@ -418,6 +441,12 @@ char *user_data;
 
 	edit_insert(sb->buffer, start, text, strlen(text), TRUE);
 
+	/* RCB: Back up over the newline at the end of the text
+	** This is to prevent consecutive property changes when
+	** there is more than one breakpoint on a single source line.
+	*/
+	edit_move_propchange_point(limpc,-1);
+
 	eb->srcbuf = sb;
 	eb->startpc = startpc;
 	eb->limpc = limpc;
@@ -571,7 +600,7 @@ bool call_callback;
 	sb = eb->srcbuf;
 	start = edit_get_propchange_point(eb->startpc);
 	lim = edit_get_propchange_point(eb->limpc);
-	
+
 	oldval = edit_set_propchange_updates(sb->buffer, FALSE);
 	
 	edit_delete_propchange(sb->buffer, eb->startpc);
@@ -579,7 +608,7 @@ bool call_callback;
 
 	edit_set_propchange_updates(sb->buffer, oldval);
 
-	edit_delete(sb->buffer, start, lim - start);
+	edit_delete(sb->buffer, start, lim - start + 1);
 
 	eb->removing = TRUE;
 	
