@@ -39,6 +39,12 @@ char ups_srcwin_c_sccsid[] = "@(#)srcwin.c	1.7 24 May 1995 (UKC)";
 #include "ui.h"
 #include "state.h"
 
+void bpt_getcolor PROTO((objid_t obj, int wn, long *p_fg, long *p_bg));
+void env_getcolor PROTO((objid_t obj, int wn, long *p_fg, long *p_bg));
+void srcfile_getcolor PROTO((objid_t obj, int wn, long *p_fg, long *p_bg));
+void func_getcolor PROTO((objid_t obj, int wn, long *p_fg, long *p_bg));
+void sig_getcolor PROTO((objid_t obj, int wn, long *p_fg, long *p_bg));
+
 #define TO_1BASE(lnum)	((lnum) + 1)
 #define TO_0BASE(lnum)	((lnum) - 1)
 
@@ -46,6 +52,7 @@ char ups_srcwin_c_sccsid[] = "@(#)srcwin.c	1.7 24 May 1995 (UKC)";
 #define INT_MAX	((int)~((unsigned)0 >> 1))
 #endif
 
+Edit_history* breakpoint_history = NULL;
 typedef struct {
 	Edit_fontinfo *fontinfo;
 	Edit_flags flagmask;
@@ -110,8 +117,13 @@ static void set_highlighting PROTO((Highlighting *hl, Edit_buffer *buffer,
 static void set_coloring PROTO((Highlighting *hl, Edit_buffer *buffer,
 				size_t start, size_t lim, Edit_flags flagmask,
 				Edit_flags flags));
+static void set_coloring_on PROTO((Highlighting *hl, Edit_buffer *buffer,
+				size_t start, size_t lim, Edit_flags flagmask,
+				Edit_flags flags));
+static void set_coloring_off PROTO((Highlighting *hl, Edit_buffer *buffer,
+				size_t start, size_t lim, Edit_flags flagmask));
 static void init_outwin PROTO((Outwin *ow, int wn, tbar_id_t tbar,
-			       Edit_buffer *buffer, int fold_lines));
+			       Edit_buffer *buffer, int fold_lines, Edit_history*));
 static bool set_point PROTO((Outwin *ow, size_t point, Region *region, 
                              char *data));
 static size_t update_ow_tbar PROTO((Outwin *ow, bool set_pos));
@@ -134,6 +146,7 @@ static int Srcfont_height, Editfont_height, Sysfont_width;
 
 static bool Want_display_checks = FALSE;
 static int Tabspaces = 8;
+static int overlay_last_flag = 0, overlay_last_buff_len, overlay_last_len;
 
 void
 display_overlay_set_font(font)
@@ -193,7 +206,7 @@ tbar_id_t tbar;
 	Edit_keymap *keymap;
 
 	sw = (Srcwin *)e_malloc(sizeof(Srcwin));
-	init_outwin(&sw->outwin, wn, tbar, (Edit_buffer *)NULL, TRUE); /* folded lines */
+	init_outwin(&sw->outwin, wn, tbar, (Edit_buffer *)NULL, TRUE, breakpoint_history); /* folded lines, history */
 	sw->fil = NULL;
 	sw->hist = NULL;
 	
@@ -284,7 +297,7 @@ tbar_id_t tbar;
 	buffer = edit_create_buffer(tb, Srcfont_info, 0);
 	
 	ow = (Outwin *)e_malloc(sizeof(Outwin));
-	init_outwin(ow, wn, tbar, buffer, TRUE); /* Folded lines */
+	init_outwin(ow, wn, tbar, buffer, TRUE, NULL); /* Folded lines, no history */
 
 	ow->next = All_windows;
 	All_windows = ow;
@@ -293,12 +306,13 @@ tbar_id_t tbar;
 }
 
 static void
-init_outwin(ow, wn, tbar, buffer, fold_lines)
+init_outwin(ow, wn, tbar, buffer, fold_lines, history)
 Outwin *ow;
 int wn;
 tbar_id_t tbar;
 Edit_buffer *buffer;
 int fold_lines;
+Edit_history* history;
 {
 	Edit_display *display;
 
@@ -306,7 +320,7 @@ int fold_lines;
 		display = NULL;
 	}
 	else {
-		display = edit_create_wn_display(wn, WN_FG, WN_BG,
+		display = edit_create_wn_display(wn, WN_FG(wn), WN_BG(wn),
 						 FALSE, FALSE);
 
 		/*
@@ -335,14 +349,15 @@ int fold_lines;
 
 		if (buffer != NULL)
 			edit_set_buffer(display, buffer);
+		edit_set_history(display, history);
 	}
 	
 	ow->tbar = tbar;
 	ow->display = display;
 	ow->buffer = buffer;
 	ow->wn = wn;
-	ow->fg = WN_FG;
-	ow->bg = WN_BG;
+	ow->fg = WN_FG(wn);
+	ow->bg = WN_BG(wn);
 	ow->data = NULL;
 	set_highlighting_style(&ow->search_hl, (font_t *)NULL, FLAG_HIGHLIGHT2, FLAG_HIGHLIGHT2);
 }
@@ -472,6 +487,44 @@ Outwin *ow;
 	tb_set_visible_units(ow->tbar, height);
 }
 
+/* De-iconize and raise the root window containing the region */
+void outwin_raise(ow)
+Outwin* ow;
+{
+    if ( ow && ow->wn >= 0)
+	wn_raise_window(ow->wn);
+}
+
+void outwin_lower(ow)
+Outwin* ow;
+{
+    if ( ow && ow->wn >= 0)
+        wn_lower_window(ow->wn);
+}
+
+void outwin_iconify(ow)
+Outwin* ow;
+{
+    if ( ow && ow->wn >= 0)
+        wn_iconify_window(ow->wn);
+}
+
+
+void outwin_map(ow)
+Outwin* ow;
+{
+    if ( ow && ow->wn >= 0)
+        wn_map_window(ow->wn);
+}
+
+void outwin_unmap(ow)
+Outwin* ow;
+{
+    if ( ow && ow->wn >= 0)
+        wn_unmap_window(ow->wn);
+}
+
+
 int
 srcwin_get_window_height(sw)
 Srcwin *sw;
@@ -664,12 +717,12 @@ Edit_buffer *buffer;
 	last_fg = ow->fg;
 	
 	if (mtime == 0) {
-		ow->fg = WN_BG;
-		ow->bg = WN_FG;
+		ow->fg = WN_BG(ow->wn);
+		ow->bg = WN_FG(ow->wn);
 	}
 	else {
-		ow->fg = WN_FG;
-		ow->bg = WN_BG;
+		ow->fg = WN_FG(ow->wn);
+		ow->bg = WN_BG(ow->wn);
 	}
 
 	if (ow->fg != last_fg) {
@@ -920,8 +973,10 @@ Highlighting *hl;
 		bool oldval;
 		
 		oldval = edit_set_propchange_updates(hl->buffer, FALSE);
-		edit_delete_propchange(hl->buffer, hl->start);
-		edit_delete_propchange(hl->buffer, hl->lim);
+		if (hl->start)
+		    edit_delete_propchange(hl->buffer, hl->start);
+		if (hl->lim)
+		    edit_delete_propchange(hl->buffer, hl->lim);
 		edit_set_propchange_updates(hl->buffer, oldval);
 
 		hl->buffer = NULL;
@@ -956,6 +1011,35 @@ bool clear;
 	edit_set_propchange_updates(buffer, oldval);
 }
 
+static void
+set_coloring_on(hl, buffer, start, lim, flagmask, flags)
+Highlighting *hl;
+Edit_buffer *buffer;
+size_t start, lim;
+Edit_flags flagmask;
+Edit_flags flags;
+{
+	hl->buffer = buffer;
+	
+	edit_add_propchange(buffer, start, (char *)NULL,
+			    hl->fontinfo, flagmask,
+			    flags, FALSE);
+}
+
+static void
+set_coloring_off(hl, buffer, start, lim, flagmask)
+Highlighting *hl;
+Edit_buffer *buffer;
+size_t start, lim;
+Edit_flags flagmask;
+{
+	hl->buffer = buffer;
+	
+	edit_add_propchange(buffer, lim, (char *)NULL,
+			    hl->fontinfo, flagmask,
+			    0, TRUE);
+}
+
 static void
 set_coloring(hl, buffer, start, lim, flagmask, flags)
 Highlighting *hl;
@@ -1094,7 +1178,7 @@ const char *text;
 {
   Outwin *ow;
 
-  ow = get_or_create_outwin();
+  ow = get_or_create_outwin_unmapped();
   edit_insert(ow->buffer, edit_get_buffer_length(ow->buffer), text,
 	      (size_t)strlen(text), TRUE);
   
@@ -1241,24 +1325,22 @@ Srcwin *sw;
 Region *region;
 event_t *ev;
 {
-	Editblock *eb;
+	Editblock *eb = NULL;
 	Edit_propchange *pc;
 	size_t point;
-	Edit_buffer *buffer;
-	
+
 	if (!edit_pixel_to_point(sw->outwin.display, ev->ev_x, ev->ev_y,
 				 &point))
 		return;
 
-	get_buffer(sw->fil, &buffer);
-
 	/*  Need to do this here so we don't get the wrong propchange
 	 *  below.
 	 */
 	clear_highlighting(&sw->outwin.search_hl);
 	
-	pc = edit_get_propchange_at_point(buffer, point);
-	eb = (Editblock *)edit_get_propchange_data(pc);
+	pc = srcbuf_get_propchange_at_point(sw->fil->fi_srcbuf, point);
+	if ( pc)
+	    eb = (Editblock *)edit_get_propchange_data(pc);
 
 	if (eb == NULL) {
 		errf("Not in an editable section");
@@ -1378,7 +1460,10 @@ bool force;
 	edit_set_want_cursor(sw->outwin.display, FALSE);
 
 	if (srcbuf_change_editblock(eb, force, &error_point))
+	{
+		edit_save_in_history(sw->outwin.display);
 		return TRUE;
+	}
 	
 	edit_set_mark(sw->outwin.display, edit_get_point(sw->outwin.display));
 	edit_move_point(sw->outwin.display, error_point);
@@ -1440,24 +1525,22 @@ Srcwin *sw;
 Region *region;
 event_t *ev;
 {
-	Editblock *eb;
+	Editblock *eb = NULL;
 	Edit_propchange *pc;
 	size_t point;
-	Edit_buffer *buffer;
-	
+
 	if (!edit_pixel_to_point(sw->outwin.display, ev->ev_x, ev->ev_y,
 				 &point))
 		return FALSE;
 
-	get_buffer(sw->fil, &buffer);
-
 	/*  Need to do this here so we don't get the wrong propchange
 	 *  below.
 	 */
 	clear_highlighting(&sw->outwin.search_hl);
 	
-	pc = edit_get_propchange_at_point(buffer, point);
-	eb = (Editblock *)edit_get_propchange_data(pc);
+	pc = srcbuf_get_propchange_at_point(sw->fil->fi_srcbuf, point);
+	if ( pc)
+	    eb = (Editblock *)edit_get_propchange_data(pc);
 	if (eb == NULL) 
 	  return FALSE;
  	else 
@@ -1485,7 +1568,7 @@ Edit_buffer *buffer;
 		display = NULL;
 	}
 	else {
-		display = edit_create_wn_display(wn, WN_FG, WN_BG,
+		display = edit_create_wn_display(wn, WN_FG(wn), WN_BG(wn),
 						 FALSE, FALSE);
 
 		edit_use_folded_lines(display, Tabspaces, Want_display_checks,
@@ -1499,8 +1582,8 @@ Edit_buffer *buffer;
 	dw->display = display;
 	dw->buffer = buffer;
 	dw->wn = wn;
-	dw->fg = WN_FG;
-	dw->bg = WN_BG;
+	dw->fg = WN_FG(wn);
+	dw->bg = WN_BG(wn);
 	dw->data = NULL;
 	set_highlighting_style(&dw->search_hl, (font_t *)NULL, FLAG_HIGHLIGHT2, FLAG_HIGHLIGHT2);
 }
@@ -1535,12 +1618,23 @@ int var_coloring_only;
 {
   Outwin *dw;
   size_t buff_len, sol = 0;
-  int width, height, i, j = 0, max_chars, sol_int = 0, buff_len_int;
+  int width, height, i, j = 0, max_chars, sol_int = 0, buff_len_int,
+    flag = 0, blank_text_len;
   char blank_text[256];
-  long val_fg, val_bg, var_fg, var_bg;
+  static long bg, val_fg, env_fg, bpt_fg, src_fg, func_fg, sig_fg;
+  static int got_colors = 0;
 	
-  get_value_colors(TRUE, &val_fg, &val_bg);
-  get_variable_colors(&var_bg, &var_fg);
+  dw = get_display_area_overlay();
+  if (!got_colors)
+  {
+    got_colors = 1;
+    get_value_colors(dw->wn, TRUE, &val_fg, &bg);
+    bpt_getcolor(0, dw->wn, &bpt_fg, &bg);
+    env_getcolor(0, dw->wn, &env_fg, &bg);
+    srcfile_getcolor(0, dw->wn, &src_fg, &bg);
+    func_getcolor(0, dw->wn, &func_fg, &bg);
+    sig_getcolor(0, dw->wn, &sig_fg, &bg);
+  }
   if (!prepend_newline)
   {
     for (i = 0; i < len; i++)
@@ -1550,7 +1644,6 @@ int var_coloring_only;
       return;			/* skip trailing blank strings */
   }
  
-  dw = get_display_area_overlay();
   wn_get_window_size(dw->wn, &width, &height);
   max_chars = (int)width/Sysfont_width;
   buff_len = edit_get_buffer_length(dw->buffer);
@@ -1577,11 +1670,12 @@ int var_coloring_only;
     for (i = 0; i < pos + sol_int - buff_len_int + j; i++)
       blank_text[i + j] = ' ';
     blank_text[i + j] = 0;
-    if (strlen(blank_text) + len >= 255)
+    blank_text_len = strlen(blank_text);
+    if (blank_text_len + len >= 255)
     {
       edit_insert(dw->buffer, buff_len, (const char *)blank_text,
-		  strlen(blank_text), FALSE);
-      edit_insert(dw->buffer, buff_len + strlen(blank_text), text, len, FALSE);
+		  blank_text_len, FALSE);
+      edit_insert(dw->buffer, buff_len + blank_text_len, text, len, FALSE);
     }
     else
     {
@@ -1592,12 +1686,52 @@ int var_coloring_only;
       for (i = 0; i > pos + sol_int - buff_len_int + j; i--, buff_len--)
 	edit_delete(dw->buffer, buff_len - 1, 1);
 
+      blank_text_len = strlen(blank_text);
       edit_insert(dw->buffer, buff_len, (const char *)blank_text,
-		  strlen(blank_text), FALSE);
+		  blank_text_len, FALSE);
     }
     if (fg_color == val_fg)
+      flag = FLAG_VALUE;
+    else
+      if (fg_color == bpt_fg)
+	flag = FLAG_BPT;
+      else
+	if (fg_color == env_fg)
+	  flag = FLAG_ENV;
+	else
+	  if (fg_color == src_fg)
+	    flag = FLAG_SRC;
+	  else
+	    if (fg_color == func_fg)
+	      flag = FLAG_FUNC;
+	    else
+	      if (fg_color == sig_fg)
+		flag = FLAG_SIG;
+    /* RGA Optimize the coloring by turning it on and off only when
+       necessary. Long source file lists will all be one color
+       for instance. This techinque can reduce the setup time for a 
+       search from minutes, for targets with thousands of files,
+       to millisecs */
+
+    /*    if (flag)
       set_coloring(&dw->search_hl, dw->buffer,
-		   buff_len, buff_len + strlen(blank_text), FLAG_VALUE, FLAG_VALUE);
+		   buff_len, buff_len + strlen(blank_text),
+		   flag, flag);*/
+
+    if (overlay_last_flag != flag)
+    {
+      if (overlay_last_flag > 0)
+	set_coloring_off(&dw->search_hl, dw->buffer,
+			 overlay_last_buff_len, overlay_last_len,
+			 overlay_last_flag);
+      if (flag)
+	set_coloring_on(&dw->search_hl, dw->buffer,
+			buff_len, buff_len + blank_text_len,
+			flag, flag);
+    }
+    overlay_last_flag = flag;
+    overlay_last_buff_len = buff_len;
+    overlay_last_len = buff_len + blank_text_len;
   }
 }
 
@@ -1646,7 +1780,12 @@ bool forwards;
 	  wn_text_capture(1, 0, &y); /* set */
 	  display_area_overlay_delete_all_text(dw);
 	  oldval = edit_set_propchange_updates(dw->buffer, FALSE);
+	  overlay_last_flag = -1; /* initialize */
 	  display_all_lines(0, 0, 1);
+	  if (overlay_last_flag > 0)
+	    set_coloring_off(&dw->search_hl, dw->buffer,
+			     overlay_last_buff_len, overlay_last_len,
+			     overlay_last_flag);
 	  edit_set_propchange_updates(dw->buffer, oldval);
 	  outwin_redraw(dw);
 	  dstart = 0;
@@ -1758,3 +1897,8 @@ bool val;
   return edit_set_propchange_updates(dw->buffer, val);
 }
 
+void display_area_overlay_unselect_global_selection()
+{
+    Outwin *dw = (Outwin *)re_get_data(get_display_area_region());
+    edit_unselect_global_selection(dw->display);
+}
