@@ -11,6 +11,7 @@
 
 #define SRCWIN_H_INCLUDED
 
+extern struct Edit_history* breakpoint_history;
 typedef struct Srcwin Srcwin;
 typedef struct Outwin Outwin;
 
@@ -22,6 +23,7 @@ void display_overlay_set_font PROTO((font_t *font));
 
 void srcwin_set_tabwidth PROTO((int tabwidth));
 void srcwin_set_want_checks_flag PROTO((void));
+void display_area_overlay_unselect_global_selection PROTO((void));
 
 #ifdef TB_H_INCLUDED
 Srcwin *srcwin_create PROTO((int wn, tbar_id_t tbar_id));
@@ -29,6 +31,9 @@ Outwin *outwin_create PROTO((int wn, tbar_id_t tbar_id));
 Outwin * display_area_overlay_create PROTO((int wn, tbar_id_t tbar));
 #endif
 
+void outwin_lower PROTO((Outwin* ow));
+void outwin_raise PROTO((Outwin* ow));
+void outwin_iconify PROTO((Outwin* ow));
 void outwin_insert PROTO((Outwin *ow, const char *text, size_t len));
 void outwin_insert_string PROTO((const char *text));
 void outwin_putc PROTO((Outwin *ow, int ch));
@@ -43,6 +48,13 @@ size_t srcwin_get_pos PROTO((Srcwin *sw));
 void srcwin_redraw PROTO((Srcwin *sw));
 void outwin_redraw PROTO((Outwin *sw));
 
+/* De-iconize and raise the root window containing the window */
+void outwin_raise PROTO((Outwin *ow));
+
+/* Map or unmap the window. */
+void outwin_map PROTO((Outwin *ow));
+void outwin_unmap PROTO((Outwin *ow));
+
 void srcwin_clear_history PROTO((Srcwin *sw));
 void srcwin_push_current_pos PROTO((Srcwin *sw));
 void srcwin_pop PROTO((Srcwin *sw));
