@@ -19,6 +19,7 @@ char ups_st_lookup_c_sccsid[] = "@(#)st_lookup.c	1.8 04 Jun 1995 (UKC)";
 #include <local/wn.h>
 #include <local/ukcprog.h>
 #include <local/arg.h>
+#include <local/obj/obj.h>	/* for OBJ_UPDATING_OFF */
 #include <mtrprog/strcache.h>
 #include <mtrprog/alloc.h>
 #include <mtrprog/genmergesort.h>
@@ -32,6 +33,8 @@ char ups_st_lookup_c_sccsid[] = "@(#)st_lookup.c	1.8 04 Jun 1995 (UKC)";
 #include "state.h"
 #include "srcwin.h"
 #include "ui.h"
+#include "expr.h"
+#include "tdr.h"
 
 #define CMP_STRCMP 0
 #define CMP_STRNCMP 1
@@ -663,6 +666,7 @@ func_t **p_f1;
 /*  We search all the symbol tables including those in the cache.
  *  This is so we can match names in shared libraries (like printf)
  *  before the target has been started.
+ *  Returns -1 if item not found
  */
 int
 find_global_by_name(name, fil, f, cblock, exact, p_f, p_f1, p_v, p_v1, p_cblock, p_fil, p_fil1)
@@ -759,7 +763,7 @@ fil_t **p_fil, **p_fil1;
 		}
 	}
 	
-	if (glhead == NULL && fil != NULL && filename == NULL) {
+	if ((glhead == NULL) && (fil != NULL) && (filename == NULL)) {
 		scan_symtabs(fil, (const char *)NULL, 0, funcpat, funcname,
 					p_v != NULL, &glhead, &found_file, 1);
 	}
@@ -774,12 +778,7 @@ fil_t **p_fil, **p_fil1;
 /*	}*/
 
 
-	/* RGA - comment out. Stops "file.c:glob" in typing line working */
-
-/*	if (filename != NULL && !found_file) {
-		errf("No source file name `%.*s'", (int)filename_len, filename);
-		return -1;
-	}*/
+	/* If nothing found, give up. */
 
 	if (glhead == NULL) {
 		const char *orvar;
@@ -796,10 +795,6 @@ fil_t **p_fil, **p_fil1;
 		return -1;
 	}
 
-	if (glhead == NULL && fil != NULL && filename == NULL)
-		scan_symtabs(fil, (const char *)NULL, 0, funcpat, funcname,
-					p_v != NULL, &glhead, &found_file, 1);
-
 
 	matches = exact_matches = exact_extern_matches = partial_matches = 0;
 	exact_extern_gl = NULL;		/* to satisfy gcc */
@@ -868,7 +863,8 @@ fil_t **p_fil, **p_fil1;
 	  else
 	    if (partial_gls[0]->gl_fil && !partial_gls[1]->gl_fil)
 	      gl->gl_fil = partial_gls[0]->gl_fil;*/
-	
+
+
 	/* RGA when duplicate function names exist, and the files are */
 	/* the same, just use the first one found. This can happen when */
 	/* a function is linked statically ahead of a dynamic library */
@@ -889,7 +885,7 @@ fil_t **p_fil, **p_fil1;
 	    gl = exact_gls[0];
 	    gl1 = exact_gls[1];
 	}
-	
+
 	if (exact_matches > 0 && partial_matches > 1 &&
 	    partial_gls[0]->gl_func != NULL &&
 	    partial_gls[1]->gl_func != NULL &&
@@ -906,15 +902,15 @@ fil_t **p_fil, **p_fil1;
 
 	if (exact_matches > 1)
 	{
-	  if (exact)
-	    dupf_mesg(funcname, exact_gls[0], exact_gls[1]);
-	  else
-	    errf("\bUsing first one of multiple/overloaded symbols...");
-	  for (gl = glhead; gl != NULL; gl = gl->gl_next) 
-	    if (gl->gl_func && !(gl->gl_func->fu_flags & FU_STATIC))
-	      break;
-	  gl = gl ? gl : exact_gls[0];
-	  gl1 = exact_gls[1];
+	    if (exact)
+		dupf_mesg(funcname, exact_gls[0], exact_gls[1]);
+	    else
+		errf("\bUsing first one of multiple/overloaded symbols...");
+	    for (gl = glhead; gl != NULL; gl = gl->gl_next) 
+		if (gl->gl_func && !(gl->gl_func->fu_flags & FU_STATIC))
+		    break;
+	    gl = gl ? gl : exact_gls[0];
+	    gl1 = exact_gls[1];
 	}
 
 	if (gl != NULL) {
@@ -1315,3 +1311,42 @@ do_free_func_bp_list_list(funchead)
 {
   free_func_bp_list_list(funchead);
 }
+
+int
+find_and_show_matching_globals(fil, fi_name, fi_name_len, name)
+     fil_t *fil;
+     const char *fi_name;
+     int fi_name_len;
+     char *name;
+{
+  global_list_t *glhead = 0, *gl;
+  int found, oldstate;
+  const char *funcpat;
+  char *funcname;
+  static const char globchars[] = "*?";
+  const char *globname;
+  
+  funcname = (char *)name;
+  globname = funcname;
+  funcpat = arg_do_quoting(&globname, globchars, TRUE);
+
+  find_matching_globals(get_current_target(), fil, fi_name, fi_name_len,
+			funcpat, funcname, &glhead);
+  found = !!glhead;
+  if (found)
+  {
+    oldstate = td_set_obj_updating(OBJ_UPDATING_OFF);
+    clear_selection();
+    for (gl = glhead; gl != NULL; gl = gl->gl_next) 
+    {
+      if (gl->gl_var)
+      {
+	show_given_global(gl->gl_fil, gl->gl_var);
+      }
+    }
+    td_set_obj_updating(oldstate);
+  }
+  if (glhead)
+    free_global_list_list(glhead);
+  return found;
+}
