@@ -200,6 +200,7 @@ const char *name;
 	for (al = addrlist; al != NULL; al = al->al_next)
 		if (strcmp(al->al_name, name) == 0)
 			return al->al_addr;
+
 	return 0;
 }
 
@@ -242,6 +243,41 @@ const char *name;
 	return find_addr_in_addrlist(st->st_addrlist, name);
 }
 
+/*  Find the name of a global from the address
+**  ( Returns the un-demangled symtab name)
+*/
+
+const char* lookup_global_name(addr, offset)
+taddr_t addr;
+size_t* offset;
+{
+    taddr_t best_addr = 0;
+    const char* best_name = NULL;
+    target_t *xp = get_current_target();
+    symtab_t *st = NULL;
+    while ( xp_next_symtab(xp, st, FALSE, &st))
+    {
+	addrlist_t *al;
+
+	for (al = st->st_addrlist; al != NULL; al = al->al_next)
+	{
+	    if ( al->al_addr == addr)
+	    {
+		*offset = 0;
+		return al->al_name;
+	    }
+	    if ( al->al_addr > best_addr
+		&& al->al_addr < addr )
+	    {
+		best_addr = al->al_addr;
+		best_name = al->al_name;
+	    }
+	}
+    }
+    *offset = addr-best_addr;
+    return best_name;
+}
+
 symtab_t *
 make_symtab(ap, path, sfiles, funclist, ops, data)
 alloc_pool_t *ap;
@@ -504,8 +540,12 @@ int lnum;
 	      && ln->ln_num == lnum) /* RGA */
 	    return ln->ln_addr;
 	  else
-	    if (ln->ln_fil == fil && ln->ln_num == lnum)
-			return ln->ln_addr;
+	    if (ln->ln_num == lnum &&
+		(ln->ln_fil == fil || (ln->ln_fil->fi_data == fil->fi_data)))
+	      /* RGA after a reattach, fils are different, but fi_data 
+		 point to the same data.
+		 */
+	      return ln->ln_addr;
 	}
 	
 	return 1;
@@ -585,7 +625,7 @@ bool rematch, reload;
   if (fil->fi_srcbuf == NULL) {
     fil->fi_srcbuf = srcpath_visit_file(fil->fi_symtab->st_apool,
 					fil->fi_path_hint,
-					fil->fi_name_only);
+					fil->fi_name);
   }
 
   /* RGA start */
@@ -605,8 +645,9 @@ bool rematch, reload;
 
     if (rematch || reload)
     {
-      if ((fp = fopen("/tmp/temp-ups.bpts", "w")) == NULL)
-	errf("Can't create temp saved breakpoints file `/tmp/temp-ups.bpts'");
+      if ((fp = fopen(get_temp_bpt_filename(), "w")) == NULL)
+	errf("Can't create temp saved breakpoints file `%s'",
+	     get_temp_bpt_filename());
       else
       {
 	save_matching_breakpoints(NULL, fil, (char *)fp);
@@ -651,7 +692,7 @@ bool rematch, reload;
 	{
 	  if (!strcmp(fil->fi_source_path, pl->path))
 	    old_path_found = 1;
-	  if (rematch || reload && !old_path_found)
+	  if (rematch || (reload && !old_path_found))
 	    continue;
 	}
 	(void) sprintf(nbuf, "%s%s", pl->path, fil->fi_name);
@@ -756,7 +797,6 @@ symtab_t *fu_symtab;
   time_t file_time;
   struct tm *tm_ptr;
   char time_str[32];
-  errf_ofunc_t old_ofunc = NULL;
   Outwin *ow;
   char buff[256];
   target_t *xp;
@@ -855,3 +895,12 @@ fil_t *fil;
 #endif /* !AO_ELF */
 }
 
+char *
+get_temp_bpt_filename()
+{
+  static char filename[32];
+
+  if (!*filename)
+    sprintf(filename, "/tmp/ups-bpts.%d", (int)getpid());
+  return filename;
+}
