@@ -454,7 +454,7 @@ tstate_t tstate;
 void
 target_stop()
 {
-  ps_stop(get_current_target());
+	ps_stop(get_current_target());
 }
 
 static void
@@ -475,24 +475,7 @@ user_wants_non_frame_functions()
       (def = (char *)wn_get_default("NonFrameFunctions")) != NULL &&
 	strcmp(def, "on") == 0;
     if (def == NULL)
-      /* see if Purify/Quantify/Purecov are used */
-    {
-      func_t *f, *f1;
-      int res;
-      errf_ofunc_t oldf, oldf1;
-
-      res = find_func_by_name("main", &f, &f1);
-      oldf = errf_set_ofunc(null_ofunc);
-      oldf1 = errf_set_ofunc1(null_ofunc);
-      res = find_func_by_name("purify_is_running", &f, &f1);
-      if (res != 0)
-	res = find_func_by_name("quantify_is_running", &f, &f1);
-      if (res != 0)
-	res = find_func_by_name("purecov_is_running", &f, &f1);
-      non_frame_functions = res == 0;
-      errf_set_ofunc(oldf);
-      errf_set_ofunc1(oldf1);
-    }
+      non_frame_functions = (rational_products_running() != 0);
   }
   return non_frame_functions;
 }
@@ -538,8 +521,8 @@ void
 rescan_dynamic_solibs(xp)
 target_t *xp;
 {
-#if AO_USE_PROCFS
-  procfs_get_dynamic_solibs(xp);
+#ifdef AO_ELF
+    elf_rescan_dynamic_solibs(xp);
 #endif
 }
 
