@@ -7,7 +7,7 @@
  *  unaltered copy of this message (including the copyright).
  */
 
-/* @(#)trun.h	1.7 4/7/91 (UKC) */
+/* @(#)trun.h	1.9 16 Apr 1994 (UKC) */
 
 typedef enum rtypeen {
 	RT_STEP,
@@ -28,6 +28,7 @@ typedef enum rtypeen {
 #undef KNOW_ASM
 #endif
 
-#ifdef PROC_H_INCLUDED
-proc_t run_target PROTO((proc_t proc, breakpoint_t stop_bp, rtype_t rtype));
+#ifdef TARGET_H_INCLUDED
+stopres_t dx_run_target PROTO((target_t *xp, rtype_t rtype));
+stopres_t dx_run_target_ss PROTO((target_t *xp, rtype_t rtype));
 #endif
