@@ -56,6 +56,12 @@ char ups_ui_iploop_c_sccsid[] = "@(#)ui_iploop.c	1.41 04 Jun 1995 (UKC)";
 #include "va.h"
 #include "st_debug.h"
 
+#ifndef __STDC__
+double strtod();
+#endif
+
+Edit_history* prompt_history = NULL;
+
 static void my_cu_set PROTO((int wn, int cursor));
 static void draw_message PROTO((int wn, const char *mesg));
 static int prompt_for_input PROTO((const char *what, const char *prompt,
@@ -134,9 +140,24 @@ int normal;
 	int orig_y, old_inmode;
 	int wn, height, dist;
 	bool done_scroll;
+	static unsigned int sleep_for = 0;
+	static bool delay_found = FALSE;
 
 	done_scroll = FALSE;
-	
+	if (!delay_found)
+	{
+	  const char *defstr = wn_get_default("ScrollDelay");
+	    char *ptr;
+	    long k = 0;
+
+	    if (defstr != NULL)
+	    {
+		k = strtol (defstr, &ptr, 0);
+	    if (k && ptr)
+	      sleep_for = 1000.0 * k;
+	  }
+	    delay_found = TRUE;
+	}
 	wn = ev->ev_wn;
 	height = wn_get_height(wn);
 	
@@ -148,10 +169,10 @@ int normal;
 		
 		if ((ev->ev_buttons & B_LEFT) == 0)
 			break;
-		
+
 		if (ev->ev_y < 0 || ev->ev_y >= height)
 			continue;
-		
+
 		if (orig_y == ev->ev_y) {
 			my_cu_set(wn, CU_SC_PRESSED);
 		}
@@ -159,25 +180,27 @@ int normal;
 			my_cu_set(wn, (orig_y > ev->ev_y) ? CU_SCROLL_UP
 				  			  : CU_SCROLL_DOWN);
 		}
-		
+
 		dist = (orig_y - ev->ev_y) / 3;
 
 		if (!normal)
-		  dist = -dist;
+			dist = -dist;
 
 		if (dist >= 0 && dist < sizeof(scrolldist))
 			dist = scrolldist[dist];
 		else if (dist < 0 && -dist < sizeof(scrolldist))
 			dist = -scrolldist[-dist];
-		
+
 		if (dist != 0) {
 			wn_updating_off(wn);
 			tb_scroll(tbar_id, dist, TRUE);
 			wn_updating_on(wn);
 			done_scroll = TRUE;
+			if (sleep_for > 0)
+				usleep (sleep_for/(abs(dist)*abs(dist)));
 		}
 	}
-	
+
 	wn_inmode(wn, old_inmode);
 	my_cu_set(wn, CU_SBAR);
 
@@ -256,7 +279,7 @@ const char *mesg;
 	size_t maxlen;
 
 	wn_get_window_size(wn, &w, &h);
-	wn_set_area(wn, 1, 1, w - 1, h - 1, WN_BG);
+	wn_set_area(wn, 1, 1, w - 1, h - 1, WN_BG(wn));
 
 	maxlen = wn_strpos(mesg, w - 3, (font_t *)NULL, FALSE);
 	if (maxlen < strlen(mesg)) {
@@ -360,7 +383,33 @@ const char *text;
 	    dump_memory(text);
 	}
 	else if (*text == '/' || *text == '?') {
-	    srcwin_search(get_current_srcwin(), text+1, *text == '/');
+	    /* For split screens, search something on the current window */
+	    Region* typing_region =  get_typing_line_region();
+	    int    typing_wn = re_get_wn(typing_region);
+	    int    ourRoot = wn_get_root_window(typing_wn);
+	    Region* src_region = get_current_srcwin_region();
+	    int	   src_wn = re_get_wn(src_region);
+	    int    srcRoot = wn_get_root_window(src_wn);
+	    if ( srcRoot == ourRoot)
+	    {
+	        srcwin_search(get_current_srcwin(), text+1, *text == '/');
+	    } else
+	    {
+	        Region* displ_region = get_display_area_region();
+	        int    displ_wn = re_get_wn(displ_region);
+	        int    displRoot = wn_get_root_window(displ_wn);
+		if ( displRoot == ourRoot)
+		{
+    		    Outwin* dw = (Outwin *)re_get_data(get_display_area_region());
+    		    display_area_overlay_search
+      		       ((Outwin*)displ_region, text+1, *text == '/');
+		} else
+		{
+	  	    Outwin* ow = get_current_outwin();
+		    if  (ow)
+		        outwin_search(ow, text+1, *text == '/');
+		}
+	    }
 	}
 	else {
 		cursor_t old_cursor;
@@ -587,21 +636,32 @@ const char *what, *prompt, *default_string, *options;
 char **p_string;
 {
 	const char *preprompt;
+	Region *tx_region;
 	Region *ta_region;
+	int tx_wn, tx_width, tx_height, tx_margin, tx_base_margin;
 	int ta_wn, ta_width, ta_height;
-	int wn, pwidth, maxlen;
+	int wn, orig_wn, pwidth, maxlen;
 	char *text;
 	bool force;
 	font_t *font;
 	Edit_buffer *buffer;
 	Edit_display *display;
 	
+	/* Because the history and enter buttons are not functional
+	** dueing the local event loop, we cover them up
+	*/
+	tx_region = get_extended_typing_line_region();
+	tx_wn = re_get_wn(tx_region);
+	wn_get_window_size(tx_wn, &tx_width, &tx_height);
+
 	ta_region = get_typing_line_region();
-	ta_wn = re_get_wn(ta_region);
+	ta_wn = orig_wn = re_get_wn(ta_region);
 	wn_get_window_size(ta_wn, &ta_width, &ta_height);
+	tx_margin = (tx_height-ta_height)/2;
+	tx_base_margin = (tx_height-ta_height)/4;
 
 	font = wn_get_sysfont();
-	pwidth = wn_strwidth(prompt, font);
+	pwidth = 2+wn_strwidth(prompt, font);
 	
 	maxlen = (ta_width - pwidth) / font->ft_width;
 
@@ -625,16 +685,18 @@ char **p_string;
 		preprompt = "";
 	}
 	
-	wn_set_area(ta_wn, -2, -2, ta_width + 4, ta_height + 4, WN_FG);
+	wn_set_area(tx_wn, 0, 0, tx_width, tx_height, WN_FG(orig_wn));
 
-	wn = wn_create_subwin(ta_wn, pwidth, 0, ta_width - pwidth, ta_height,
+	wn = wn_create_subwin(tx_wn, pwidth, tx_margin, tx_width - pwidth, tx_height-tx_margin,
 			      WN_INPUT_OUTPUT);
 
-	wn_ttext(ta_wn, preprompt, 0, 0, WN_BG, WN_FG);
-	wn_ttext(ta_wn, prompt, wn_strwidth(preprompt, font), 0, WN_BG, WN_FG);
+	wn_ttext(ta_wn, preprompt, 2, tx_base_margin, WN_BG(wn), WN_FG(wn));
+	wn_ttext(ta_wn, prompt, 2+wn_strwidth(preprompt, font),
+		 tx_base_margin, WN_BG(wn), WN_FG(wn));
+
+	display = make_text_display(wn, WN_BG(wn), WN_FG(wn), default_string);
+	edit_set_history(display, prompt_history);
 
-	display = make_text_display(wn, WN_BG, WN_FG, default_string);
-	
 	buffer = edit_get_buffer(display);
 	force = FALSE;
 
@@ -654,11 +716,12 @@ char **p_string;
 				break;
 			}
 
-			wn_set_area(ta_wn, -2, -2, ta_width + 4, ta_height + 4,
-				    WN_FG);
-			wn_ttext(ta_wn, preprompt, 0, 0, WN_BG, WN_FG);
-			wn_ttext(ta_wn, prompt, wn_strwidth(preprompt, font),
-				 0, WN_BG, WN_FG);
+			wn_set_area(tx_wn, 0, 0, tx_width, tx_height,
+				    WN_FG(wn));
+			wn_ttext(ta_wn, preprompt, 2, tx_base_margin, WN_BG(wn),
+				    WN_FG(wn));
+			wn_ttext(ta_wn, prompt, 2+wn_strwidth(preprompt, font),
+				    tx_base_margin, WN_BG(wn), WN_FG(wn));
 			edit_redraw_display(display);
 			continue;
 		}
@@ -704,13 +767,14 @@ char **p_string;
 		
 	}
 
+	edit_save_in_history(display);
 	edit_close_display(display);
 	edit_destroy_buffer(buffer);
 	wn_close_window(wn);
 
-	wn_set_area(ta_wn, -2, -2, ta_width + 4, ta_height + 4, WN_BG);
+	wn_set_area(tx_wn, 0, 0, tx_width, tx_height, WN_BG(orig_wn));
 	
-	edit_redraw_display((Edit_display *)re_get_data(ta_region));
+	re_expose(tx_region,FALSE);
 	
 	if (text == NULL)
 		return -1;
@@ -731,109 +795,111 @@ char **p_string;
 **
 */
 void dump_memory (text)
-const char* text;
+     const char* text;
 {
-    static short default_grp = 0x0201;  /* Actually, a char[2] */
-    char buffer[100];
-    taddr_t addr;
-    int length = 16;
-    int grp;
-    int parse_error = FALSE;
-    int read_error = FALSE;
-    const char* t = text;
-    char* tt;
-    target_t * xp = get_current_target();
-    if ( xp == NULL || !can_get_target_vars(xp))
-    {
-	errf("Target not running");
-	return;
-    }
+#if IS_BIG_ENDIAN
+#define DEFAULT_GROUPING 2
+#else
+#define DEFAULT_GROUPING 1
+#endif
+  char buffer[100];
+  taddr_t addr;
+  int length = 16;
+  int grp = 0;
+  int parse_error = FALSE;
+  int read_error = FALSE;
+  const char* t = text;
+  target_t * xp = get_current_target();
+  if ( xp == NULL || !can_get_target_vars(xp))
+  {
+    errf("Target not running");
+    return;
+  }
 
-    /*  Parse the command line */
-    t += 2;  /* skip %d */
-    switch (*t++)
+  /*  Parse the command line */
+  t += 2;  /* skip %d */
+  switch (*t++)
+  {
+  case 'b':
+    grp = 1; break;
+  case 's':
+    grp = 2; break;
+  case 'l':
+    grp = 4; break;
+  case ' ':
+    grp=DEFAULT_GROUPING; break;
+  default:
+    parse_error = TRUE; break;
+  }
+  while (*t == ' ')
+    t++;
+  if ( !parse_error)
+  {
+    if ( t[0] == '0' && (t[1] == 'x' || t[1] == 'X'))
+      t+=2;
+    parse_error = (0==sscanf(t,"%lx",&addr));
+    while (isxdigit(*t))
+      t++;
+  }
+  while (*t == ' ')
+    t++;
+  if (*t && !parse_error)
+  {
+    if ( t[0] == '.' && t[1] == '.')
     {
-    case 'b':
-       grp = 1; break;
-    case 's':
-       grp = 2; break;
-    case 'l':
-       grp = 4; break;
-    case ' ':
-       grp = *(char*)&default_grp; break;
-    default:
-       parse_error = TRUE; break;
-    }
-    while (*t == ' ')
+      taddr_t end_addr;
+      t+= 2;
+      while (*t == ' ')
 	t++;
-    if ( !parse_error)
+      if ( t[0] == '0' && (t[1] == 'x' || t[1] == 'X'))
+	t+=2;
+      parse_error = (0==sscanf(t,"%lx",&end_addr));
+      while (isxdigit(*t))
+	t++;
+      if ( end_addr <= addr)
+	parse_error = TRUE;
+      length = end_addr - addr;
+    } else if (isxdigit(*t))
     {
-	if ( t[0] == '0' && (t[1] == 'x' || t[1] == 'X'))
-	    t+=2;
-	parse_error = (0==sscanf(t,"%lx",&addr));
+      while (*t == ' ')
+	t++;
+      if ( t[0] == '0' && (t[1] == 'x' || t[1] == 'X'))
+      {
+	t+=2;
+	parse_error = (0==sscanf(t,"%x",&length));
 	while (isxdigit(*t))
-	   t++;
-    }
-    while (*t == ' ')
-	t++;
-    if (*t && !parse_error)
-    {
-	if ( t[0] == '.' && t[1] == '.')
-	{
-	    taddr_t end_addr;
-	    t+= 2;
-	    while (*t == ' ')
-		t++;
-	    if ( t[0] == '0' && (t[1] == 'x' || t[1] == 'X'))
-		t+=2;
-	    parse_error = (0==sscanf(t,"%lx",&end_addr));
-	    while (isxdigit(*t))
-		t++;
-	    if ( t == tt || end_addr <= addr)
-		parse_error = TRUE;
-	    length = end_addr - addr;
-	} else if (isxdigit(*t))
-	{
-	    while (*t == ' ')
-		t++;
-	    if ( t[0] == '0' && (t[1] == 'x' || t[1] == 'X'))
-	    {
-		t+=2;
-	        parse_error = (0==sscanf(t,"%x",&length));
-		while (isxdigit(*t))
-		    t++;
-	    }else
-	    {
-	        parse_error = (0==sscanf(t,"%u",&length));
-		while (isdigit(*t))
-		    t++;
-	    }
-	} else
-	    parse_error = TRUE;
-    }
-    while (*t == ' ')
-	t++;
+	  t++;
+      }else
+      {
+	parse_error = (0==sscanf(t,"%u",&length));
+	while (isdigit(*t))
+	  t++;
+      }
+    } else
+      parse_error = TRUE;
+  }
+  while (*t == ' ')
+    t++;
 
 
-    /*  Write the command to the output */
-    outwin_insert_string(text);
-    outwin_insert_string("\n");
+  /*  Write the command to the output */
+  (void)get_current_outwin(); /* ensure window displayed */
+  outwin_insert_string(text);
+  outwin_insert_string("\n");
 
-    if (parse_error)
-    {
-	outwin_insert_string("Invalid %d command\n");
-	return;
-    }
-    while(length && !read_error)
-    {
-	int len = length;
-	if ( len > 16)
-	    len = 16;
-	read_error = dump_to_buffer (xp, addr, len, grp, buffer);
-	outwin_insert_string(buffer);
-	length -= len;
-	addr += len;
-    }
+  if (parse_error)
+  {
+    outwin_insert_string("Invalid %d command\n");
+    return;
+  }
+  while(length && !read_error)
+  {
+    int len = length;
+    if ( len > 16)
+      len = 16;
+    read_error = dump_to_buffer (xp, addr, len, grp, buffer);
+    outwin_insert_string(buffer);
+    length -= len;
+    addr += len;
+  }
 }
-
-
