@@ -54,6 +54,8 @@ char ups_ui_layout_c_sccsid[] = "@(#)ui_layout.c	1.32 20 Jun 1995 (UKC)";
 #include "sccsdata.h"
 #include "util.h"
 
+Edit_history* typing_line_history = NULL;
+
 typedef struct {
 	int ma_md;
 	const char *ma_menu_name;
@@ -70,6 +72,15 @@ static void dragbox_draw PROTO((Region *region, int wn,
 				    int width, int height,
 				    re_draw_reason_t draw_reason));
 
+static void historybutton_input PROTO((Region *region, event_t *ev));
+static void historybutton_draw PROTO((Region *region, int wn,
+				    int width, int height,
+				    re_draw_reason_t draw_reason));
+static void enterbutton_input PROTO((Region *region, event_t *ev));
+static void enterbutton_draw PROTO((Region *region, int wn,
+				    int width, int height,
+				    re_draw_reason_t draw_reason));
+
 static void line_input PROTO((Region *region, event_t *ev));
 static void line_draw PROTO((Region *region, int wn,
 				    int width, int height,
@@ -111,11 +122,12 @@ static void mousehole_draw PROTO((Region *region, int wn,
 				  int width, int height,
 				  re_draw_reason_t draw_reason));
 
-void initialise_menu PROTO((Region *region, int md,
+static void initialise_menu PROTO((Region *region, int md,
 			    const char *menu_name,
 			    void (*menu_func)(char *data, int mfmd, int rv),
 			    char *menu_func_data));
-static void create_srcwin PROTO((Region *region, double srcwin_fraction));
+static int make_permanent_menu PROTO((Region* region));
+static void create_srcwin PROTO((Region *region, double srcwin_fraction, int split));
 static tbar_id_t make_tbar PROTO((Region *region));
 
 static int tbar_srcwin_callback PROTO((char *data, tb_action_t action, int val));
@@ -130,6 +142,9 @@ static void output_window_menu_func PROTO((char *data, int md, int rv));
 static void init_line_region PROTO((Region *line));
 static bool typing_line_quit PROTO((Region *region, char *data, bool force));
 static bool typing_line_accept_focus PROTO((Region *region, char *data));
+static Edit_display* typing_line_get_display PROTO((Region *region));
+static Edit_display* source_window_get_display PROTO((Region* region));
+static Edit_display* display_area_get_display PROTO((Region *region));
 static bool display_area_release_focus PROTO((Region *region, char *data, 
                                               bool force));
 static void show_version PROTO((Edit_display *display));
@@ -173,6 +188,7 @@ int default_value, min_value;
 }
 
 /*  Do the initial division of the window into regions.
+ *  The default configuration, not called if -split is used on the command line.
  */
 Region *
 divide_window_into_regions(wn, want_outwin)
@@ -183,7 +199,9 @@ bool want_outwin;
 		TOP, DISP_AREA_AND_TB, TOPLEFT,
 		TYPING_LINE_AND_PMENU, TARGET_MENU_AND_DRAGBOX,
 		TYPING_LINE, DMENU, MESSAGE_AREA, DRAGBOX,
-		PMENU, MOUSEHOLE, DISPLAY_AREA_TB, DISPLAY_AREA,
+		PMENU, MOUSEHOLE,
+		TYPING_LINE_ENTER_AREA, HISTORY_BUTTON, ENTER_BUTTON,
+		DISPLAY_AREA_TB, DISPLAY_AREA,
 		TARGET_MENU,
 		START_TARGET_MENU, NEXT_TARGET_MENU, STEP_TARGET_MENU,
 		CONT_TARGET_MENU, STOP_TARGET_MENU, ATTACH_TARGET_MENU,
@@ -196,14 +214,16 @@ bool want_outwin;
 	Outwin *dw;
 
 	/*  Dimensions and percentages */
-	int mousehole_width, menu_height;
-	int pmenu_width, tbar_width, srcwin_name_width;
+	int disable_mousehole;
+	int mousehole_width = 0, menu_height;
+	int historybutton_width, enterbutton_width;
+	int pmenu_width, tbar_width;
 	int display_area_percent, srcwin_percent, outwin_percent;
 	float display_area_fraction, srcwin_fraction, outwin_fraction;
 
-	int i, md;
+	int i;
+	int md;
 	tbar_id_t obj_tbar_id;
-	extern MENU cmd_men;
 	extern MENU tgt_start_men, tgt_next_men, tgt_step_men;
 	extern MENU
 	  tgt_cont_men, tgt_stop_men, tgt_kill_men,
@@ -220,15 +240,17 @@ bool want_outwin;
 	menufont = Mstdfont();
 	menu_height = menufont->ft_height + 8;
 
-	/*  BUG: get rid of these explicit pixel sizes.
-	 */
-/*	mousehole_width = get_default("MouseholeWidth", 160, 5);*/
-	mousehole_width = get_default("MouseholeWidth", 18*sysfont->ft_width, 5);
-/*	pmenu_width = get_default("PermanentMenuWidth", 220, 5);*/
-	pmenu_width = get_default("PermanentMenuWidth", 28*sysfont->ft_width, 5);
+	/* RCB: Allow user to suppress the mousehole.  Useful for large fonts
+	*/
+	disable_mousehole =  ( wn_get_default("WantMousehole")
+	     && !strcmp(wn_get_default("WantMousehole"), "no"));
+	if (!disable_mousehole)
+	    mousehole_width = get_default("MouseholeWidth", 18*sysfont->ft_width, 32);
+
+	historybutton_width = get_default("HistoryButtonWidth", 15, 12);
+	enterbutton_width = get_default("EnterButtonWidth", 15, 12);
+	pmenu_width = get_default("PermanentMenuWidth", 28*menufont->ft_width, 5);
 	tbar_width = get_default("ScrollbarWidth", 20, 5);
-/*	srcwin_name_width = get_default("SrcwinNameWidth", 150, 20);*/
-	srcwin_name_width = get_default("SrcwinNameWidth", 20*sysfont->ft_width, 20);
 
 	if (wn_get_default("SrcwinPercent") != NULL) {
 		srcwin_percent = get_default("SrcwinPercent", 50, 10);
@@ -264,9 +286,12 @@ bool want_outwin;
 		  &regs[TARGET_MENU_AND_DRAGBOX],menu_height,FIXED,
 		  RE_END);
 	dragbox_line = line;
-	create_srcwin(root, srcwin_fraction);
+	create_srcwin(root, srcwin_fraction, FALSE);
 
-	re_divide(regs[TOP], RE_VERTICAL,
+	if ( disable_mousehole )
+	    regs[TOPLEFT] = regs[TOP];
+	else
+	   re_divide(regs[TOP], RE_VERTICAL,
 		  &regs[TOPLEFT],		0,			1.0,
 		  &line,			1,			FIXED,
 		  &regs[MOUSEHOLE],	mousehole_width,	FIXED,
@@ -281,10 +306,18 @@ bool want_outwin;
 		  RE_END);
 	
 	re_divide(regs[TYPING_LINE_AND_PMENU], RE_VERTICAL,
-		  &regs[TYPING_LINE],		0,		REST,
+		  &regs[TYPING_LINE_ENTER_AREA],		0,		REST,
 		  &line,				1,		FIXED,
 		  &regs[PMENU],			pmenu_width,	FIXED,
 		  RE_END);
+
+	re_divide(regs[TYPING_LINE_ENTER_AREA], RE_VERTICAL,
+		  &regs[TYPING_LINE],		0,		REST,
+		  &line,			1,		FIXED,
+		  &regs[HISTORY_BUTTON],	historybutton_width,	FIXED,
+		  &line,			1,		FIXED,
+		  &regs[ENTER_BUTTON],		enterbutton_width,	FIXED,
+		  RE_END);
 	
 	re_divide(regs[TARGET_MENU_AND_DRAGBOX], RE_VERTICAL,
 		  &regs[TARGET_MENU],		0,		REST,
@@ -316,12 +349,14 @@ bool want_outwin;
 		  &regs[DISPLAY_AREA],		0,		1.0,
 		  RE_END);
 	
-	md = Minsert(&cmd_men);
+	md = make_permanent_menu(regs[PMENU]);
+	Mdelete_entry(md,MR_RAISE_DISPLAY_WIN);
+	Mdelete_entry(md,MR_RAISE_SRC_WIN);
+	if (!want_split_output(0))
+	{
+	    Mdelete_entry(md,MR_RAISE_OUT_WIN);
+	}
 
-	initialise_menu(regs[PMENU], md, "pmenu",
-					permanent_menu_func, (char *)NULL);
-
-	set_dynamic_cmd_menu_state(md, 0);	/* RGA reset */
 	initialise_menu(regs[DMENU], -1, NULL,
 					dynamic_menu_func, (char *)NULL);
 	set_dynamic_menu_region(regs[DMENU]);
@@ -330,9 +365,21 @@ bool want_outwin;
 				(re_input_proc_t)NULL, (re_destroy_proc_t)NULL);
 	set_message_wn(re_get_wn(regs[MESSAGE_AREA]));
 
-	re_set_callbacks(regs[MOUSEHOLE], mousehole_draw,
+	if ( !disable_mousehole)
+	    re_set_callbacks(regs[MOUSEHOLE], mousehole_draw,
 				(re_input_proc_t)NULL, (re_destroy_proc_t)NULL);
 
+	re_set_callbacks(regs[HISTORY_BUTTON], historybutton_draw,
+					historybutton_input, (re_destroy_proc_t)NULL);
+	re_set_margins(regs[HISTORY_BUTTON], 1, 1, 1, 1);
+	re_set_mhcaps(regs[HISTORY_BUTTON], "menu", "paste", "(menu)");
+	re_set_cursor(regs[HISTORY_BUTTON], CU_MENU);
+	re_set_callbacks(regs[ENTER_BUTTON], enterbutton_draw,
+					enterbutton_input, (re_destroy_proc_t)NULL);
+	re_set_margins(regs[ENTER_BUTTON], 1, 1, 1, 1);
+	re_set_mhcaps(regs[ENTER_BUTTON], "enter", "paste", "(menu)");
+	re_set_cursor(regs[ENTER_BUTTON], CU_MENU);
+
 	re_set_callbacks(regs[DRAGBOX], dragbox_draw,
 					dragbox_input, (re_destroy_proc_t)NULL);
 	re_set_margins(regs[DRAGBOX], 1, 1, 1, 1);
@@ -350,9 +397,10 @@ bool want_outwin;
 					(re_accept_focus_proc_t)NULL,
 					display_area_release_focus);
 	re_set_minsize(regs[DISPLAY_AREA], 1, 2 * sysfont->ft_height + 3);
+	re_set_display_callback(regs[DISPLAY_AREA], display_area_get_display);
 	set_display_area_region(regs[DISPLAY_AREA]);
 	re_set_cursor(regs[DISPLAY_AREA], CU_DISPLAY);
-	re_set_mhcaps(regs[DISPLAY_AREA], "select", "edit", "toggle");
+	re_set_mhcaps(regs[DISPLAY_AREA], "select", "edit", "(menu)");
 	
 	for (i = 0; i < TM_NTAGS; ++i) {
 		tmbuf.tm_mdtab[i].md = Minsert(tgtmenus[i]);
@@ -367,7 +415,7 @@ bool want_outwin;
 	initialise_typing_line(regs[TYPING_LINE]);
 	re_set_keyboard_focus(regs[TYPING_LINE], (char *)NULL);
 	re_set_default_keyboard_focus(regs[TYPING_LINE], (char *)NULL);
-	set_typing_line_region(regs[TYPING_LINE]);
+	re_set_typing_regions(regs[TYPING_LINE],regs[TYPING_LINE_ENTER_AREA]);
 
 	dw = create_display_area_overlay
 	  (re_get_wn(regs[DISPLAY_AREA]), obj_tbar_id);
@@ -377,7 +425,318 @@ bool want_outwin;
 	if (want_outwin)
 		create_outwin(root, outwin_fraction);
 
-	re_set_rootwin(root);
+	re_add_rootwin(root, TRUE);
+	re_redraw_root(EV_WINDOW_RESIZED, TRUE);
+
+	return root;
+}
+
+/*  Do the initial division of the display/pmenu/cmdmenu window into regions.
+ *  Only called if -split is used on the command line.
+ */
+Region *
+divide_window_into_regions1(wn, want_outwin)
+int wn;
+bool want_outwin;
+{
+	enum {
+		TOP, DISP_AREA_AND_TB, TOPLEFT,
+		TYPING_LINE_AND_PMENU, 
+		TYPING_LINE, DMENU, MESSAGE_AREA,
+		PMENU, MOUSEHOLE,
+		TYPING_LINE_ENTER_AREA, HISTORY_BUTTON, ENTER_BUTTON,
+		DISPLAY_AREA_TB, DISPLAY_AREA,
+		MAX_REGNUM
+	};
+	Region *root, *line, *regs[MAX_REGNUM];
+	font_t *sysfont, *menufont;
+	Outwin *dw;
+
+	/*  Dimensions and percentages */
+	int disable_mousehole;
+	int mousehole_width = 0, menu_height;
+	int historybutton_width, enterbutton_width;
+	int pmenu_width, tbar_width;
+
+	int i;
+	int md;
+	tbar_id_t obj_tbar_id;
+
+	/*  Set up fixed dimensions.  Some of these vary with font heights.
+	 */
+	sysfont = wn_get_sysfont();
+	menufont = Mstdfont();
+	menu_height = menufont->ft_height + 8;
+
+	/* RCB: Allow user to suppress the mousehole.  Useful for large fonts
+	*/
+	disable_mousehole =  ( wn_get_default("WantMousehole")
+	     && !strcmp(wn_get_default("WantMousehole"), "no"));
+	if (!disable_mousehole)
+	    mousehole_width = get_default("MouseholeWidth", 18*sysfont->ft_width, 5);
+
+
+	historybutton_width = get_default("HistoryButtonWidth", 15, 12);
+	enterbutton_width = get_default("EnterButtonMenuWidth", 15, 12);
+	pmenu_width = get_default("PermanentMenuWidth", 28*menufont->ft_width, 5);
+	tbar_width = get_default("ScrollbarWidth", 20, 5);
+
+	for (i = 0; i < MAX_REGNUM; ++i)
+		regs[i] = NULL;
+	line = NULL;
+
+	root = re_make_region(wn);
+
+	re_divide(root, RE_HORIZONTAL,
+		  &regs[TOP],	        	0,		0.0,
+		  &line,			1,		FIXED,
+		  RE_END);
+	init_line_region(line);
+	re_divide(root, RE_HORIZONTAL,
+		  &regs[DISP_AREA_AND_TB],	0,	        1.0,
+		  &line,			1,		FIXED,
+		  RE_END);
+
+	if ( disable_mousehole )
+	    regs[TOPLEFT] = regs[TOP];
+	else
+	   re_divide(regs[TOP], RE_VERTICAL,
+		  &regs[TOPLEFT],		0,			1.0,
+		  &line,			1,			FIXED,
+		  &regs[MOUSEHOLE],	mousehole_width,	FIXED,
+		  RE_END);
+	
+	re_divide(regs[TOPLEFT], RE_HORIZONTAL,
+		  &regs[TYPING_LINE_AND_PMENU],	menu_height,	FIXED,
+		  &line,				1,		FIXED,
+		  &regs[DMENU],			menu_height,	FIXED,
+		  &line,				1,		FIXED,
+		  &regs[MESSAGE_AREA],		menu_height,	FIXED,
+		  RE_END);
+	
+	re_divide(regs[TYPING_LINE_AND_PMENU], RE_VERTICAL,
+		  &regs[TYPING_LINE_ENTER_AREA],		0,		REST,
+		  &line,				1,		FIXED,
+		  &regs[PMENU],			pmenu_width,	FIXED,
+		  RE_END);
+
+	re_divide(regs[TYPING_LINE_ENTER_AREA], RE_VERTICAL,
+		  &regs[TYPING_LINE],		0,		REST,
+		  &line,			1,		FIXED,
+		  &regs[HISTORY_BUTTON],	historybutton_width,	FIXED,
+		  &line,			1,		FIXED,
+		  &regs[ENTER_BUTTON],		enterbutton_width,	FIXED,
+		  RE_END);
+	
+	re_divide(regs[DISP_AREA_AND_TB], RE_VERTICAL,
+		  &regs[DISPLAY_AREA_TB],		tbar_width,	FIXED,
+		  &line,				1,		FIXED,
+		  &regs[DISPLAY_AREA],		0,		1.0,
+		  RE_END);
+	
+	md = make_permanent_menu(regs[PMENU]);
+	Mdelete_entry(md,MR_RAISE_DISPLAY_WIN);
+	if (!want_split_output(0))
+	{
+	    Mdelete_entry(md,MR_RAISE_OUT_WIN);
+	}
+
+	initialise_menu(regs[DMENU], -1, NULL,
+					dynamic_menu_func, (char *)NULL);
+	set_dynamic_menu_region(regs[DMENU]);
+
+	re_set_callbacks(regs[MESSAGE_AREA], message_area_draw,
+				(re_input_proc_t)NULL, (re_destroy_proc_t)NULL);
+	set_message_wn(re_get_wn(regs[MESSAGE_AREA]));
+
+	if ( !disable_mousehole)
+	    re_set_callbacks(regs[MOUSEHOLE], mousehole_draw,
+				(re_input_proc_t)NULL, (re_destroy_proc_t)NULL);
+
+	re_set_callbacks(regs[HISTORY_BUTTON], historybutton_draw,
+					historybutton_input, (re_destroy_proc_t)NULL);
+	re_set_margins(regs[HISTORY_BUTTON], 1, 1, 1, 1);
+	re_set_mhcaps(regs[HISTORY_BUTTON], "menu", "paste", "(menu)");
+	re_set_cursor(regs[HISTORY_BUTTON], CU_MENU);
+	re_set_callbacks(regs[ENTER_BUTTON], enterbutton_draw,
+					enterbutton_input, (re_destroy_proc_t)NULL);
+	re_set_margins(regs[ENTER_BUTTON], 1, 1, 1, 1);
+	re_set_mhcaps(regs[ENTER_BUTTON], "enter", "paste", "(menu)");
+	re_set_cursor(regs[ENTER_BUTTON], CU_MENU);
+
+	obj_tbar_id = make_tbar(regs[DISPLAY_AREA_TB]);
+	tb_set_callback_and_data(obj_tbar_id, tbar_obj_callback, (char *)NULL);
+	obj_set_callback_and_data(display_area_tbar_callback, (char *)obj_tbar_id);
+
+	re_set_callbacks(regs[DISPLAY_AREA], display_area_draw,
+				display_area_input, (re_destroy_proc_t)NULL);
+	re_set_keyboard_focus_callbacks(regs[DISPLAY_AREA],
+					(re_accept_focus_proc_t)NULL,
+					display_area_release_focus);
+	re_set_minsize(regs[DISPLAY_AREA], 1, 2 * sysfont->ft_height + 3);
+	re_set_display_callback(regs[DISPLAY_AREA], display_area_get_display);
+	set_display_area_region(regs[DISPLAY_AREA]);
+	re_set_cursor(regs[DISPLAY_AREA], CU_DISPLAY);
+	re_set_mhcaps(regs[DISPLAY_AREA], "select", "edit", "(menu)");
+	
+	initialise_typing_line(regs[TYPING_LINE]);
+	re_set_typing_regions(regs[TYPING_LINE],regs[TYPING_LINE_ENTER_AREA]);
+
+	dw = create_display_area_overlay
+	  (re_get_wn(regs[DISPLAY_AREA]), obj_tbar_id);
+	re_set_data(regs[DISPLAY_AREA], (char *)dw);
+	set_display_area_overlay(dw);
+
+	re_add_rootwin(root, TRUE);
+
+	return root;
+}
+
+/*  Do the initial division of the source window and tmenu into regions.
+ *  Only called if -split is used on the command line.
+ */
+Region *
+divide_window_into_regions2(wn, want_outwin)
+int wn;
+bool want_outwin;
+{
+	enum {
+		TOP,
+		TYPING_LINE_AND_PMENU,
+		TYPING_LINE,
+		PMENU,
+		TYPING_LINE_ENTER_AREA, HISTORY_BUTTON, ENTER_BUTTON,
+		TARGET_MENU,
+		START_TARGET_MENU, NEXT_TARGET_MENU, STEP_TARGET_MENU,
+		CONT_TARGET_MENU, STOP_TARGET_MENU, ATTACH_TARGET_MENU,
+		DETACH_TARGET_MENU, EXIT_TARGET_MENU,
+		OUTPUT, MAX_REGNUM
+	};
+	Region *root, *line, *regs[MAX_REGNUM];
+	target_menu_info_t tmbuf;
+	font_t *sysfont, *menufont;
+
+	/*  Dimensions and percentages */
+	int menu_height;
+	int historybutton_width, enterbutton_width;
+	int pmenu_width;
+	int outwin_percent;
+	float srcwin_fraction, outwin_fraction;
+
+	int i;
+	int md;
+	extern MENU tgt_start_men, tgt_next_men, tgt_step_men;
+	extern MENU
+	  tgt_cont_men, tgt_stop_men, tgt_kill_men,
+	  tgt_attach_men, tgt_detach_men;
+	static MENU *tgtmenus[TM_NTAGS] = {
+		&tgt_start_men, &tgt_next_men, &tgt_step_men,
+		&tgt_cont_men, &tgt_stop_men, &tgt_attach_men,
+		&tgt_detach_men, &tgt_kill_men
+	};
+
+	/*  Set up fixed dimensions.  Some of these vary with font heights.
+	 */
+	sysfont = wn_get_sysfont();
+	menufont = Mstdfont();
+	menu_height = menufont->ft_height + 8;
+	outwin_percent = get_default("OutwinPercent", 10, 1);
+	outwin_fraction = (double)outwin_percent / 100;
+	srcwin_fraction = 1.0;
+
+	for (i = 0; i < MAX_REGNUM; ++i)
+		regs[i] = NULL;
+	line = NULL;
+
+	root = re_make_region(wn);
+
+	re_divide(root, RE_HORIZONTAL,
+		  &regs[TOP],	        	0,		0.0,
+		  &line,			1,		FIXED,
+		  RE_END);
+	init_line_region(line);
+
+	historybutton_width = get_default("HistoryButtonWidth", 15, 12);
+	enterbutton_width = get_default("EnterButtonMenuWidth", 15, 12);
+	pmenu_width = get_default("PermanentMenuWidth", 28*menufont->ft_width, 5);
+
+	re_divide(regs[TOP], RE_HORIZONTAL,
+		  &regs[TYPING_LINE_AND_PMENU],	menu_height,	FIXED,
+		  &line,				1,		FIXED,
+		  &regs[TARGET_MENU],			menu_height,	FIXED,
+		  RE_END);
+
+	re_divide(regs[TYPING_LINE_AND_PMENU], RE_VERTICAL,
+		  &regs[TYPING_LINE_ENTER_AREA],		0,		REST,
+		  &line,				1,		FIXED,
+		  &regs[PMENU],			pmenu_width,	FIXED,
+		  RE_END);
+
+	re_divide(regs[TYPING_LINE_ENTER_AREA], RE_VERTICAL,
+		  &regs[TYPING_LINE],		0,		REST,
+		  &line,			1,		FIXED,
+		  &regs[HISTORY_BUTTON],	historybutton_width,	FIXED,
+		  &line,			1,		FIXED,
+		  &regs[ENTER_BUTTON],		enterbutton_width,	FIXED,
+		  RE_END);
+	create_srcwin(root, srcwin_fraction, TRUE);
+
+	re_divide(regs[TARGET_MENU], RE_VERTICAL,
+		  &regs[START_TARGET_MENU],		0,		1.0,
+		  &line,				1,		FIXED,
+		  &regs[NEXT_TARGET_MENU],		0,		1.0,
+		  &line,				1,		FIXED,
+		  &regs[STEP_TARGET_MENU],		0,		1.0,
+		  &line,				1,		FIXED,
+		  &regs[CONT_TARGET_MENU],		0,		1.0,
+		  &line,				1,		FIXED,
+		  &regs[STOP_TARGET_MENU],		0,		1.0,
+		  &line,				1,		FIXED,
+		  &regs[ATTACH_TARGET_MENU],		0,		1.0,
+		  &line,				1,		FIXED,
+		  &regs[DETACH_TARGET_MENU],		0,		1.0,
+		  &line,				1,		FIXED,
+		  &regs[EXIT_TARGET_MENU],		0,		1.0,
+		  RE_END);
+	
+	md = make_permanent_menu(regs[PMENU]);
+	Mdelete_entry(md,MR_RAISE_SRC_WIN);
+	if (!want_split_output(0))
+	{
+	    Mdelete_entry(md,MR_RAISE_OUT_WIN);
+	} 
+	initialise_typing_line(regs[TYPING_LINE]);
+	re_set_keyboard_focus(regs[TYPING_LINE], (char *)NULL);
+	re_set_default_keyboard_focus(regs[TYPING_LINE], (char *)NULL);
+	re_set_typing_regions(regs[TYPING_LINE],regs[TYPING_LINE_ENTER_AREA]);
+
+	re_set_callbacks(regs[HISTORY_BUTTON], historybutton_draw,
+					historybutton_input, (re_destroy_proc_t)NULL);
+	re_set_margins(regs[HISTORY_BUTTON], 1, 1, 1, 1);
+	re_set_mhcaps(regs[HISTORY_BUTTON], "menu", "paste", "(menu)");
+	re_set_cursor(regs[HISTORY_BUTTON], CU_MENU);
+	re_set_callbacks(regs[ENTER_BUTTON], enterbutton_draw,
+					enterbutton_input, (re_destroy_proc_t)NULL);
+	re_set_margins(regs[ENTER_BUTTON], 1, 1, 1, 1);
+	re_set_mhcaps(regs[ENTER_BUTTON], "enter", "paste", "(menu)");
+	re_set_cursor(regs[ENTER_BUTTON], CU_MENU);
+
+
+
+	for (i = 0; i < TM_NTAGS; ++i) {
+		tmbuf.tm_mdtab[i].md = Minsert(tgtmenus[i]);
+		tmbuf.tm_mdtab[i].wn = re_get_wn(regs[START_TARGET_MENU + i]);
+		initialise_menu(regs[START_TARGET_MENU + i],
+				tmbuf.tm_mdtab[i].md, "tmenu", target_menu_func,
+				(char *)NULL);
+	}
+	tmbuf.tm_current_md = -1;
+	set_target_menu_info(&tmbuf);
+
+	if (want_outwin)
+		create_outwin(root, outwin_fraction);
+
+	re_add_rootwin(root, TRUE);
 	re_redraw_root(EV_WINDOW_RESIZED, TRUE);
 
 	return root;
@@ -511,6 +870,252 @@ re_draw_reason_t draw_reason;
 			wn, win_x, win_y, R_RPL);
 }
 
+static void
+historybutton_input(region, ev)
+Region *region;
+event_t *ev;
+{
+/* We pick a larger max entries than will probably fit on the screen.
+** Normally, the max is set by the individual EditHistory objects.
+** Unless the user uses the Xresources to change them, the real max
+** is 20.
+*/
+#define MAXHISTMENU 50
+
+#define MAXMENULEN 60
+    popup_t historybuttonpop;
+    char buff[(MAXMENULEN+1)*MAXHISTMENU];
+    char* cp_cap = buff;
+    const char *historybutton[MAXHISTMENU+1];
+    int buttons = ev->ev_buttons;
+    Region* focus_re = re_get_keyboard_focus_region();
+    Edit_display *display = re_get_display(focus_re);
+
+
+    if (buttons == B_LEFT && display)
+    {
+	int res;
+	int count = MAXHISTMENU;
+	edit_get_recent_history(display, &count, historybutton);
+	if ( count)
+	{
+	    int width, height;
+	    historybutton[count] = NULL;
+	    historybuttonpop.po_mdesc = -1;
+	    historybuttonpop.po_save_last = FALSE;
+	    historybuttonpop.po_last = 0;
+	    historybuttonpop.po_caps = historybutton;
+	    historybuttonpop.po_left = TRUE;
+
+	    /* strip any leading tabs or spaces. */
+	    /* if there are newlines, or if the line is very long,
+	    ** copy the caption to the buffer */
+	    while (count--)
+	    {
+		const char* cap = historybutton[count];
+		while (*cap <= ' ' )
+		    cap++;
+		if (strchr(cap, '\n') || (strlen(cap) >= MAXHISTMENU))
+		{
+		    int len = 0;
+		    int index = 0;
+		    while ( len < MAXHISTMENU)
+		    {
+			char ch = cap[index++];
+			if (ch == 0)
+			    break;
+			if ( ch == '\n')
+			{
+			    cp_cap[len++] = '\\';
+			    if ( len < MAXHISTMENU)
+			        cp_cap[len++] = 'n';
+			}else if ( ch == '\t')
+			    cp_cap[len++] = ' ';
+			else
+			    cp_cap[len++] = ch;
+		    }
+		    cp_cap[len++] = 0;
+		    cap = cp_cap;
+		    cp_cap+= len;
+		}
+		historybutton[count] = cap;
+	    }
+	    wn_get_window_size(ev->ev_wn, &width, &height);
+	    res = select_from_popup(ev->ev_wn, "HistoryMenu", buttons, &historybuttonpop,
+			    ev->ev_x, ev->ev_y);
+	    if ( res >= 0 )
+		edit_set_history_pointer(display, res);
+
+	    Mclose(historybuttonpop.po_mdesc);
+
+	}
+
+    } else if (   focus_re && buttons == B_RIGHT)
+    {
+	/* Pass the right button event to the focus region */
+	re_handle_event(focus_re, ev);
+    } else if (   focus_re && buttons == B_MIDDLE)
+    {
+	/* Paste from the clipboard */
+
+	/* invert the region */
+	int wn = ev->ev_wn;
+	int width, height;
+	wn_get_window_size(wn, &width, &height);
+	wn_invert_area(wn, 0,0, width, height);
+
+	/* wait for button up */
+	wn_next_event(WN_ANY, EV_BUTTON_UP, ev);
+	if ( wn == ev->ev_wn)
+	{
+	    event_t key_event;
+	    key_event = *ev;
+	    key_event.ev_type = EV_KEY;
+	    key_event.ev_flags = 0;
+	    key_event.ev_char = 'y'&0x1f;  /* Control-Y */
+	    key_event.ev_buttons = 0;
+	    re_handle_event(focus_re, &key_event);
+	}
+	wn_invert_area(wn, 0,0, width, height);
+    }
+}
+
+static void
+historybutton_draw(region, wn, win_width, win_height, draw_reason)
+Region *region;
+int wn;
+int win_width, win_height;
+re_draw_reason_t draw_reason;
+{
+#define historybutton_image_width 12
+#define historybutton_image_height 5
+static unsigned char historybutton_image_bits[] = {
+	0xfc,0x07,0xf8,0x03,0xf0,0x01,0xe0,0x00,0x40,0x00};
+	static bitmap_t historybutton_bm =
+	   wn_make_static_X11bm (historybutton_image_width,
+	      historybutton_image_height, historybutton_image_bits);
+	int bm_x, bm_y, win_x, win_y;
+	int width, height;
+
+	if (draw_reason != RE_EXPOSED && draw_reason != RE_RESIZED)
+		return;
+
+	if (historybutton_bm.bm_width < win_width) {
+		bm_x = 0;
+		width = historybutton_bm.bm_width;
+		win_x = (win_width - historybutton_bm.bm_width) / 2;
+	}
+	else {
+		bm_x = (historybutton_bm.bm_width - win_width) / 2;
+		width = win_width;
+		win_x = 0;
+	}
+
+	if (historybutton_bm.bm_height < win_height) {
+		bm_y = 0;
+		height = historybutton_bm.bm_height;
+		win_y = (win_height - historybutton_bm.bm_height) / 2;
+	}
+	else {
+		bm_y = (historybutton_bm.bm_height - win_height) / 2;
+		height = win_height;
+		win_y = 0;
+	}
+
+	wn_rop_from_mem(&historybutton_bm, bm_x, bm_y, width, height,
+			wn, win_x, win_y, R_RPL);
+}
+
+static void
+enterbutton_input(region, ev)
+Region *region;
+event_t *ev;
+{
+    int buttons = ev->ev_buttons;
+    Region* focus_re = re_get_keyboard_focus_region();
+
+
+    if (   focus_re
+	&& (buttons == B_LEFT || buttons == B_MIDDLE))
+    {
+	/* invert the region */
+	int wn = ev->ev_wn;
+	int width, height;
+	wn_get_window_size(wn, &width, &height);
+	wn_invert_area(wn, 0,0, width, height);
+
+	/* wait for button up */
+	wn_next_event(WN_ANY, EV_BUTTON_UP, ev);
+	wn_invert_area(wn, 0,0, width, height);
+	if ( wn == ev->ev_wn)
+	{
+	    event_t key_event;
+	    key_event = *ev;
+	    key_event.ev_type = EV_KEY;
+	    key_event.ev_flags = 0;
+	    if ( buttons == B_LEFT)
+		key_event.ev_char = '\033';  /* Escape */
+	    else
+		key_event.ev_char = 'y'&0x1f;  /* Control-Y */
+	    key_event.ev_buttons = 0;
+	    re_handle_event(focus_re, &key_event);
+	}
+    } else if (   focus_re && buttons == B_RIGHT)
+    {
+	/* Pass the right button event to the focus region */
+	re_handle_event(focus_re, ev);
+    }
+}
+
+static void
+enterbutton_draw(region, wn, win_width, win_height, draw_reason)
+Region *region;
+int wn;
+int win_width, win_height;
+re_draw_reason_t draw_reason;
+{
+#define enter_image_width 15
+#define enter_image_height 15
+static unsigned char enter_image_bits[] = {
+	0x00,0x00,0x00,0x00,0x00,0x00,0xc0,0x30,0x60,0x18,0x30,0x0c,
+	0x18,0x06,0x0c,0x03,0x18,0x06,0x30,0x0c,0x60,0x18,0xc0,0x30,
+	0x00,0x00,0x00,0x00,0x00,0x00};
+
+	static bitmap_t enter_bm =
+	   wn_make_static_X11bm(enter_image_width,
+	      enter_image_height, enter_image_bits);
+	int bm_x, bm_y, win_x, win_y;
+	int width, height;
+
+	if (draw_reason != RE_EXPOSED && draw_reason != RE_RESIZED)
+		return;
+
+	if (enter_bm.bm_width < win_width) {
+		bm_x = 0;
+		width = enter_bm.bm_width;
+		win_x = (win_width - enter_bm.bm_width) / 2;
+	}
+	else {
+		bm_x = (enter_bm.bm_width - win_width) / 2;
+		width = win_width;
+		win_x = 0;
+	}
+
+	if (enter_bm.bm_height < win_height) {
+		bm_y = 0;
+		height = enter_bm.bm_height;
+		win_y = (win_height - enter_bm.bm_height) / 2;
+	}
+	else {
+		bm_y = (enter_bm.bm_height - win_height) / 2;
+		height = win_height;
+		win_y = 0;
+	}
+
+	wn_rop_from_mem(&enter_bm, bm_x, bm_y, width, height,
+			wn, win_x, win_y, R_RPL);
+}
+
 static void
 line_draw(region, wn, width, height, draw_reason)
 Region *region;
@@ -521,7 +1126,7 @@ re_draw_reason_t draw_reason;
 	switch (draw_reason) {
 	case RE_EXPOSED:
 	case RE_RESIZED:
-		wn_set_area(wn, 0, 0, width, height, WN_FG);
+		wn_set_area(wn, 0, 0, width, height, WN_FG(wn));
 		break;
 	case RE_RESIZING:
 	case RE_UNDRAW:
@@ -542,6 +1147,7 @@ Region *region;
 	re_set_keyboard_focus_callbacks(region,
 					typing_line_accept_focus,
 					typing_line_quit);
+	re_set_display_callback(region, typing_line_get_display);
 	
 	re_set_margins(region, 5, 5, 4, 4);
 	re_set_cursor(region, CU_DISPLAY);
@@ -557,7 +1163,7 @@ int width, height;
 re_draw_reason_t draw_reason;
 {
 	if (draw_reason == RE_EXPOSED || draw_reason == RE_RESIZED)
-		wn_set_area(wn, 0, 0, width, height, WN_BG);
+		wn_set_area(wn, 0, 0, width, height, WN_BG(wn));
 }
 
 /*  Set up the typing area
@@ -576,12 +1182,13 @@ re_draw_reason_t draw_reason;
 	if (display == NULL) {
 		Edit_keymap *keymap;
 		
-		display = make_text_display(wn, WN_FG, WN_BG, "");
+		display = make_text_display(wn, WN_FG(wn), WN_BG(wn), "");
 		re_set_data(region, (char *)display);
 
 		keymap = edit_create_keymap(edit_get_keymap(display));
 		edit_add_keymap_control_entry(keymap, 'v', show_version);
 		edit_set_keymap(display, keymap);
+		edit_set_history(display, typing_line_history);
 	}
 	else {
 		switch (draw_reason) {
@@ -630,6 +1237,7 @@ event_t *ev;
 		if (edit_quit_requested(display)) {
 			char *text;
 			
+			edit_save_in_history(display);
 			text = get_typing_line_string();
 			process_typing_line_text(text);
 			free(text);
@@ -663,6 +1271,7 @@ event_t *ev;
 			if (edit_quit_requested(display)) {
 			    char *text;
 			
+			    edit_save_in_history(display);
 			    text = get_typing_line_string();
 			    process_typing_line_text(text);
 			    free(text);
@@ -707,6 +1316,12 @@ bool force;
 	return TRUE;
 }
 
+static Edit_display* typing_line_get_display(region)
+Region *region;
+{
+    return (Edit_display *)re_get_data(region);
+}
+
 static void
 init_line_region(line)
 Region *line;
@@ -759,7 +1374,7 @@ add_outwin()
 }
 
 Outwin *
-get_or_create_outwin()
+get_or_create_outwin_unmapped()
 {
 	Outwin *ow;
 
@@ -772,72 +1387,156 @@ get_or_create_outwin()
 
 	return ow;
 }
+Outwin *
+get_or_create_outwin()
+{
+	Outwin *ow = get_or_create_outwin_unmapped();
+	outwin_map(ow);
+	return ow;
+}
 
 static void
 create_outwin(root, outwin_fraction)
 Region *root;
 double outwin_fraction;
 {
+	int out_wn = get_split_out_win();
 	extern MENU out_men;
+	enum {
+		TOP, TOPLEFT,
+		TYPING_LINE,
+		PMENU, MOUSEHOLE,
+		TYPING_LINE_ENTER_AREA, HISTORY_BUTTON, ENTER_BUTTON,
+		MENU_AND_DRAGBOX,DRAGBOX,MENU,
+		TEXT_AND_TB,
+		OUTWIN,TBAR,
+		MAX_REGNUM
+	};
+	Region *regs[MAX_REGNUM], *line, *dragbox_line;
 	static int md = -1;
-	Region *line, *dragbox_line;
-	Region *text_and_tb_region;
-	Region *outwin_region, *tbar_region;
-	Region *menu_and_dragbox_region, *dragbox_region, *menu_region;
 	int header_height, tbar_width;
 	Outwin *ow;
 	tbar_id_t tbar_id;
+	int i;
+	int pmd;
+
+	for (i = 0; i < MAX_REGNUM; ++i)
+		regs[i] = NULL;
+	line = NULL;
+
+
+	if ( out_wn != -1)
+	{
+	    font_t *sysfont = wn_get_sysfont();
+	    font_t *menufont = Mstdfont();
+	    int menu_height = menufont->ft_height + 8;
+	    int historybutton_width = get_default("HistoryButtonWidth", 15, 12);
+	    int enterbutton_width = get_default("EnterButtonMenuWidth", 15, 12);
+	    int pmenu_width = get_default("PermanentMenuWidth", 28*menufont->ft_width, 5);
+	    root = re_make_region(out_wn);
+	    outwin_fraction = REST;
+	    re_divide(root, RE_HORIZONTAL,
+		      &regs[TOP],	        menu_height,	FIXED,
+		      &line,			1,		FIXED,
+		      RE_END);
+	    init_line_region(line);
+
+	    re_divide(regs[TOP], RE_VERTICAL,
+		  &regs[TYPING_LINE_ENTER_AREA],		0,		REST,
+		  &line,				1,		FIXED,
+		  &regs[PMENU],			pmenu_width,	FIXED,
+		  RE_END);
+
+	    re_divide(regs[TYPING_LINE_ENTER_AREA], RE_VERTICAL,
+		  &regs[TYPING_LINE],		0,		REST,
+		  &line,			1,		FIXED,
+		  &regs[HISTORY_BUTTON],	historybutton_width,	FIXED,
+		  &line,			1,		FIXED,
+		  &regs[ENTER_BUTTON],		enterbutton_width,	FIXED,
+		  RE_END);
+	    pmd = make_permanent_menu(regs[PMENU]);
+	    Mdelete_entry(pmd,MR_RAISE_OUT_WIN);
+
+	    /*  If the src and display are in the same region, remove the
+	    **  raise source option */
+	    if ( re_same_root_wn(get_display_area_region(),get_current_srcwin_region()))
+		Mdelete_entry(pmd,MR_RAISE_SRC_WIN);
+
+	    initialise_typing_line(regs[TYPING_LINE]);
+	    re_set_typing_regions(regs[TYPING_LINE],regs[TYPING_LINE_ENTER_AREA]);
+
+	    re_set_callbacks(regs[HISTORY_BUTTON], historybutton_draw,
+					historybutton_input, (re_destroy_proc_t)NULL);
+	    re_set_margins(regs[HISTORY_BUTTON], 1, 1, 1, 1);
+	    re_set_mhcaps(regs[HISTORY_BUTTON], "menu", "paste", "(menu)");
+	    re_set_cursor(regs[HISTORY_BUTTON], CU_MENU);
+	    re_set_callbacks(regs[ENTER_BUTTON], enterbutton_draw,
+					enterbutton_input, (re_destroy_proc_t)NULL);
+	    re_set_margins(regs[ENTER_BUTTON], 1, 1, 1, 1);
+	    re_set_mhcaps(regs[ENTER_BUTTON], "enter", "paste", "(menu)");
+	    re_set_cursor(regs[ENTER_BUTTON], CU_MENU);
+	    dragbox_line = NULL;
+	} else
+	{
+	    line = NULL;
+	    re_divide(root, RE_HORIZONTAL,
+		  &line,			1,		FIXED,
+		  RE_END);
+	    dragbox_line = line;
+	    init_line_region(line);
+	}
 
 	header_height = wn_get_sysfont()->ft_height + 6;
 	tbar_width = get_default("ScrollbarWidth", 20, 5);
 
-	line = NULL;
-	re_divide(root, RE_HORIZONTAL,
-		  &line,			1,		FIXED,
-		  RE_END);
-	dragbox_line = line;
-	init_line_region(line);
 
-	menu_and_dragbox_region = text_and_tb_region = NULL;
 	re_divide(root, RE_HORIZONTAL,
-		  &menu_and_dragbox_region,	header_height,	FIXED,
+		  &regs[MENU_AND_DRAGBOX],	header_height,	FIXED,
 		  &line,			1,		FIXED,
-		  &text_and_tb_region,		0,		outwin_fraction,
+		  &regs[TEXT_AND_TB],		0,		outwin_fraction,
 		  RE_END);
 
-	tbar_region = outwin_region = NULL;
-	re_divide(text_and_tb_region, RE_VERTICAL,
-		  &tbar_region,			tbar_width,	FIXED,
+	re_divide(regs[TEXT_AND_TB], RE_VERTICAL,
+		  &regs[TBAR],			tbar_width,	FIXED,
 		  &line,			1,		FIXED,
-		  &outwin_region,		0,		REST,
+		  &regs[OUTWIN],		0,		REST,
 		  RE_END);
 	
-	menu_region = dragbox_region = NULL;
-	re_divide(menu_and_dragbox_region, RE_VERTICAL,
-		  &menu_region,			0,		REST,
+	if (dragbox_line)
+	{
+	    re_divide(regs[MENU_AND_DRAGBOX], RE_VERTICAL,
+		  &regs[MENU],			0,		REST,
 		  &line,			1,		FIXED,
-		  &dragbox_region,		header_height,	FIXED,
+		  &regs[DRAGBOX],		header_height,	FIXED,
 		  RE_END);
+	} else
+	{
+		  regs[MENU] = regs[MENU_AND_DRAGBOX];
+	}
 
-	tbar_id = make_tbar(tbar_region);
-	ow = outwin_create(re_get_wn(outwin_region), tbar_id);
+	tbar_id = make_tbar(regs[TBAR]);
+	ow = outwin_create(re_get_wn(regs[OUTWIN]), tbar_id);
 	
 	tb_set_callback_and_data(tbar_id, tbar_outwin_callback, (char *)ow);
 
-	re_set_data(outwin_region, (char *)ow);
-	re_set_mhcaps(outwin_region, "", "edit", "(menu)");
-	re_set_callbacks(outwin_region, outwin_draw,
+	re_set_data(regs[OUTWIN], (char *)ow);
+	re_set_mhcaps(regs[OUTWIN], "", "edit", "(menu)");
+	re_set_cursor(regs[OUTWIN], CU_DISPLAY);
+	re_set_callbacks(regs[OUTWIN], outwin_draw,
 			 output_window_event_handler, (re_destroy_proc_t)NULL);
-	re_set_keyboard_focus_callbacks(outwin_region,
+	re_set_keyboard_focus_callbacks(regs[OUTWIN],
 					(re_accept_focus_proc_t)NULL,
 					outwin_release_focus);
 	
-	re_set_callbacks(dragbox_region, dragbox_draw,
+	if ( regs[DRAGBOX])
+	{
+	    re_set_callbacks(regs[DRAGBOX], dragbox_draw,
 					dragbox_input, (re_destroy_proc_t)NULL);
-	re_set_margins(dragbox_region, 1, 1, 1, 1);
-	re_set_data(dragbox_region, (char *)dragbox_line);
-	re_set_mhcaps(dragbox_region, "drag", "up", "down");
-	re_set_cursor(dragbox_region, CU_DRAGBOX);
+	    re_set_margins(regs[DRAGBOX], 1, 1, 1, 1);
+	    re_set_data(regs[DRAGBOX], (char *)dragbox_line);
+	    re_set_mhcaps(regs[DRAGBOX], "drag", "up", "down");
+	    re_set_cursor(regs[DRAGBOX], CU_DRAGBOX);
+	}
 
 
 	if (md == -1)
@@ -845,22 +1544,36 @@ double outwin_fraction;
 	else
 		md = Mdup(md);
 
-	initialise_menu(menu_region, md, "outwin",
+	if ( regs[TYPING_LINE] == NULL)
+	    Mdelete_entry(md,MR_OUTWIN_CLOSE);
+	else
+	    Mdelete_entry(md,MR_OUTWIN_SEARCH);
+
+	initialise_menu(regs[MENU], md, "outwin",
 			output_window_menu_func, (char *)ow);
 
 	set_current_outwin(ow);
+	if ( out_wn >= 0)
+	{
+	    re_add_rootwin(root, FALSE);
+	}
 }
 	
 static void
-create_srcwin(region, srcwin_fraction)
+create_srcwin(region, srcwin_fraction, split)
 Region *region;
 double srcwin_fraction;
+int split;
 {
 	static int md = -1;
 	extern MENU src_men;
 	enum {
-		LINE, HEADER, TEXT_AND_TB,
-		TEXT, TB, NAMEWIN, SRC_MENU,
+		LINE,
+		HEADER,
+		TEXT_AND_TB,
+		TEXT, TB,
+		NAMEWIN,
+		MENU_AREA,
 		NREGS
 	};
 	tbar_id_t tbar_id;
@@ -888,7 +1601,7 @@ double srcwin_fraction;
 	re_divide(regs[HEADER], RE_VERTICAL,
 		  &regs[NAMEWIN],	1,		FIXED,
 		  &regs[LINE],		1,		FIXED,
-		  &regs[SRC_MENU],	0,		REST,
+		  &regs[MENU_AREA],	0,		REST,
 		  RE_END);
 
 	re_divide(regs[TEXT_AND_TB], RE_VERTICAL,
@@ -899,15 +1612,17 @@ double srcwin_fraction;
 
 	re_set_minsize(regs[NAMEWIN], wn_get_sysfont()->ft_width * 30,
 					   wn_get_sysfont()->ft_height + 6);
-	re_set_minsize(regs[SRC_MENU], 0, Mstdfont()->ft_height + 6);
+	re_set_minsize(regs[MENU_AREA], 0, Mstdfont()->ft_height + 6);
 
 	tbar_id = make_tbar(regs[TB]);
 	sw = srcwin_create(re_get_wn(regs[TEXT]), tbar_id);
 
 	/*  Tie all the bits together with callbacks.
 	 */
+
 	re_set_data(regs[TEXT], (char *)sw);
 	re_set_data(regs[NAMEWIN], (char *)sw);
+	re_set_display_callback(regs[TEXT], source_window_get_display);
 
 	tb_set_callback_and_data(tbar_id, tbar_srcwin_callback, (char *)sw);
 	srcwin_set_data(sw, (char *)re_get_wn(regs[NAMEWIN]));
@@ -931,8 +1646,11 @@ double srcwin_fraction;
 	else
 		md = Mdup(md);
 
+        if ( split)
+	    Mdelete_entry(md,MR_SRCWIN_SEARCH);
 
-	initialise_menu(regs[SRC_MENU], md, "srcwin",
+
+	initialise_menu(regs[MENU_AREA], md, "srcwin",
 			source_window_menu_func, (char *)sw);
 	
 	set_current_srcwin_region(regs[TEXT]);
@@ -949,10 +1667,12 @@ int val;
 
 	switch (action) {
 	case TB_GOTO:
+		display_area_overlay_unselect_global_selection();
 		display_from(0, val);
 		retval = val;
 		break;
 	case TB_SCROLL:
+		display_area_overlay_unselect_global_selection();
 		retval = v_scroll(val);
 		break;
 	case TB_SHOW_POS:
@@ -965,6 +1685,15 @@ int val;
 	return retval;
 }
 
+static Edit_display*
+source_window_get_display(region)
+Region *region;
+{
+	Srcwin *sw;
+	sw = (Srcwin *)re_get_data(region);
+	return srcwin_get_display(sw);
+}
+
 static int
 tbar_srcwin_callback(data, action, val)
 char *data;
@@ -1043,6 +1772,7 @@ int val;
 		tb_set_total_units(tbar_id, val);
 		break;
 	case OBJ_NEW_YPOS:
+		display_area_overlay_unselect_global_selection();
 		tb_goto(tbar_id, val, FALSE);
 		break;
 	default:
@@ -1165,6 +1895,7 @@ display_area_input(region, ev)
 	set_dynamic_menu_updating_state(DMU_OFF);
       }
       wn_pushback_event(ev);
+      display_area_overlay_unselect_global_selection();
       obj = select_list(x_threshold, &threshold_exceeded);
       if (threshold_exceeded)
       {
@@ -1185,7 +1916,6 @@ display_area_input(region, ev)
 	 * but since we can't easily restore the selection after the
 	 * overlay code, call clear_selection() beforehand in all cases
          */
-
 	clear_selection();
 	wn_text_capture(1, 0, &y); /* set */
 	display_area_overlay_delete_all_text(dw);
@@ -1200,8 +1930,10 @@ display_area_input(region, ev)
 	wn_text_capture(0, 1, &y); /* reset */
 
 	do_selection(outwin_get_display(dw), &event);
-	display_area_overlay_tb_clear(dw);
-	set_obj_wn(event.ev_wn);
+	/**  RCB: This causes the selection to be imediately erased on button up.
+	** display_area_overlay_tb_clear(dw);
+	** set_obj_wn(event.ev_wn);
+	*/
       }
 		
       if (buttons != B_MIDDLE)
@@ -1332,6 +2064,14 @@ display_area_input(region, ev)
   }
 }
 
+static Edit_display*
+display_area_get_display(region)
+Region *region;
+{
+    char* handle = re_get_keyboard_focus_data(region);
+    return field_edit_get_display(handle);
+}
+
 static bool
 display_area_release_focus(region, data, force)
 Region *region;
@@ -1359,6 +2099,10 @@ int width, height;
 re_draw_reason_t draw_reason;
 {
 	if (draw_reason == RE_EXPOSED || draw_reason == RE_RESIZED) {
+		/* If the global selection is currently highlighted,
+		** unhighlight it before doing anything else
+		*/
+		display_area_overlay_unselect_global_selection();
 		set_obj_wn(wn);
 	
 		if (re_get_keyboard_focus_region() == region) {
@@ -1487,6 +2231,7 @@ int cmd;
 	case MR_OUTWIN_SEARCH_BACKWARDS:
 	   {
 		int offset = 0;
+		wn_clr_xclipboard();
 		pattern = get_typing_line_string();
 		/* Skip initial direction indicator if any */
 		if ( *pattern == '/' || *pattern == '?') 
@@ -1495,6 +2240,9 @@ int cmd;
 		free(pattern);
 		break;
 	   }
+	case MR_OUTWIN_CLOSE:
+		outwin_unmap(ow);
+		break;
 	default:
 		panic("bad cmd in owmf");
 	}
@@ -1570,7 +2318,25 @@ event_t *ev;
 	tbar_event_handler((tbar_id_t)re_get_data(region), ev);
 }
 
-void
+static int make_permanent_menu(region)
+Region* region;
+{
+    extern MENU cmd_men;
+    static int md  = -1;
+    int md2;
+
+    if (md == -1)
+    {
+	md = Minsert(&cmd_men);
+    }
+    md2 = Mdup(md);
+    init_menu_toggle_items(md2);
+    initialise_menu(region, md2, "pmenu",
+					permanent_menu_func, (char *)NULL);
+    return md2;
+}
+
+static void
 initialise_menu(region, md, menu_name, menu_func, menu_func_data)
 Region *region;
 int md;
@@ -1699,20 +2465,6 @@ bool enable;
 	re_expose(dmenu_region, FALSE);
 }
 
-
-void
-set_dynamic_cmd_menu_state(md, enable)
-int md;
-bool enable;
-{
-	char rvs[2];
-
-	rvs[0] = (enable) ?
-	  (char)MR_LOGGING_ON : (char)MR_LOGGING_OFF;
-	rvs[1] = 0;
-	Mnonsel(md, rvs, 1);
-}
-
 /*  Set the dynamic menu to md.
  */
 void
@@ -1745,7 +2497,7 @@ int width, height;
 re_draw_reason_t draw_reason;
 {
 	if (draw_reason == RE_EXPOSED || draw_reason == RE_RESIZED) {
-		wn_set_area(wn, 0, 0, width, height, WN_BG);
+		wn_set_area(wn, 0, 0, width, height, WN_BG(wn));
 		draw_mousehole(wn, 0, 0, width, height);
 	}
 }
