@@ -60,7 +60,8 @@ static void do_var_selection PROTO((Srcwin *sw, event_t *ev, size_t sel_point,
 static char *do_var_or_expr_selection PROTO((Edit_display *display,
 					     event_t *ev, size_t sel_point,
 					     size_t start, size_t lim,
-					     bool *p_done_sel_loop));
+					     bool *p_done_sel_loop,
+					     bool do_global_selection));
 static void do_expr_selection PROTO((Srcwin *sw, event_t *ev, size_t sel_point, 
                                      fil_t *fil, int lnum));
 static bool do_replacements PROTO((const char *oldstr, ebuf_t *eb));
@@ -478,6 +479,7 @@ int md, cmd;
 	case MR_SRCWIN_SEARCH_BACKWARDS:
 	   {
 		int offset = 0;
+		wn_clr_xclipboard();
 		pattern = get_typing_line_string();
 		/* Skip initial direction indicator if any */
 		if ( *pattern == '/' || *pattern == '?') 
@@ -518,7 +520,8 @@ event_t *ev;
 	buttons = ev->ev_buttons & (B_ANY | B_SHIFT_MASK | B_CONTROL_MASK);
 
 	if (buttons == B_LEFT) {
-		do_selection(outwin_get_display(ow), ev);
+	  wn_clr_xclipboard();
+	  do_selection(outwin_get_display(ow), ev);
 	}
 	else if (buttons == B_MIDDLE) {
 		outwin_set_point(ow, region, ev);
@@ -532,7 +535,7 @@ event_t *ev;
 }
 
 static char *
-do_var_or_expr_selection(display, ev, sel_point, start, lim, p_done_sel_loop)
+do_var_or_expr_selection(display, ev, sel_point, start, lim, p_done_sel_loop,do_global_selection)
 Edit_display *display;
 event_t *ev;
 size_t sel_point, start, lim;
@@ -547,7 +550,7 @@ bool *p_done_sel_loop;
 		*p_done_sel_loop = TRUE;
 
 		return selection_loop(display, ev, (Edit_propchange *)NULL,
-				      (Edit_propchange *)NULL);
+				      (Edit_propchange *)NULL, do_global_selection, FALSE);
 	}
 
 	add_propchange_pair(buffer, start, lim, 2, &startpc, &limpc);
@@ -567,7 +570,7 @@ bool *p_done_sel_loop;
 			edit_move_propchange(buffer, startpc, sel_point);
 			edit_move_propchange(buffer, limpc, sel_point);
 			*p_done_sel_loop = TRUE;
-			return selection_loop(display, ev, startpc, limpc);
+			return selection_loop(display, ev, startpc, limpc,do_global_selection, FALSE);
 		}
 	}
 
@@ -641,13 +644,11 @@ int lnum;
 	}
 
 	text = do_var_or_expr_selection(display, ev, sel_point, start, lim,
-					&done_sel_loop);
+					&done_sel_loop, TRUE);
 
-	if (text != NULL) {
-	  if (done_sel_loop) {
-	    wn_set_selection(text, (int)strlen(text));
-	  }
-	  else {
+	if (text != NULL && !done_sel_loop) {
+            cursor_t old_cursor = wn_get_window_cursor(WN_STDWIN);
+	    set_bm_cursor(WN_STDWIN, CU_WAIT);
 	    /*  f77 maps all upper case variable names to
 	     *  lower case, so follow suit.
 	     */
@@ -699,8 +700,8 @@ int lnum;
 	    }
 
 	    free(text);
+	    wn_define_cursor(WN_STDWIN, old_cursor);
 	  }
-	}
 }
 
 static bool
@@ -809,7 +810,7 @@ int lnum;
 	}
 		
 	text = do_var_or_expr_selection(display, ev, sel_point, start, lim,
-					&done_sel_loop);
+					&done_sel_loop, FALSE);
 
 	if (text != NULL) {
 		ebuf_t *eb;
@@ -1010,16 +1011,14 @@ event_t *ev;
 	buttons = ev->ev_buttons & (B_ANY | B_SHIFT_MASK | B_CONTROL_MASK | B_MULT_CLICKS);
 
 	if (buttons == B_LEFT &&  srcinfo.have_point) {
-            cursor_t old_cursor = wn_get_window_cursor(WN_STDWIN);
-	    set_bm_cursor(WN_STDWIN, CU_WAIT);
 	    srcwin_have_editblock(sw, region, ev);
 	    indicate_target_menu_search(1);
 	    target_menu_search_disabled(1, 0); /* set */
+	    wn_clr_xclipboard();
 	    do_var_selection(sw, ev, srcinfo.point,
 			     srcinfo.fil, srcinfo.fil_lnum);
 	    target_menu_search_disabled(0, 1); /* reset */
 	    indicate_target_menu_search(0);
-	    wn_define_cursor(WN_STDWIN, old_cursor);
 	}
 	if (buttons == (B_LEFT | B_SHIFT_MASK) && srcinfo.have_point) {
           cursor_t old_cursor = wn_get_window_cursor(WN_STDWIN);
