@@ -47,6 +47,8 @@ char ups_ups_c_sccsid[] = "@(#)ups.c	1.53 04 Jun 1995 (UKC)";
 #include "obj_stack.h"
 #include "obj_buildf.h"
 #include "obj_bpt.h"
+#include "obj_env.h"
+#include "obj_target.h"
 #include "cc.h"
 #include "state.h"
 #include "debug.h"
@@ -59,6 +61,7 @@ char ups_ups_c_sccsid[] = "@(#)ups.c	1.53 04 Jun 1995 (UKC)";
 #include "exec.h"
 #include "sccsdata.h"
 
+
 #define DEFAULT_TABWIDTH 8
 
 static void catch_fatal_sig PROTO((int signo));
@@ -68,8 +71,8 @@ static void get_default_paths PROTO((const char *textname,
 				     const char **p_dsrc, const char **p_dcore));
 static void usage PROTO((const char *mesg));
 static int setup_display PROTO((void));
-static int ups PROTO((target_t *xp, const char *args, bool replay,
-		      bool want_window, bool want_outwin,
+static int ups PROTO((target_t *xp, const char* textname, const char *args, bool replay,
+		      bool want_window,
 		      bool want_load_config, bool want_save_config,
 		      bool want_save_sigs,
 		      bool want_auto_start, bool use_full_path, const char *state_dir,
@@ -83,12 +86,60 @@ static bool statedir_exists PROTO((const char *state_dir));
 static void set_special_edit_chars PROTO((Edit_keymap *keymap));
 static void add_keymap_entry PROTO((Edit_keymap *keymap, unsigned modifiers,
 				    int ch, Edit_function func));
-static void quit_ups PROTO((void));
 
 unsigned long Debug_flags = 0;
 
 static bool Done_panic = FALSE;
 
+static Edit_history_item history_items[] =
+{
+   /*  "name"      address              default_size */
+   { "TypingLine", &typing_line_history, 20},
+   { "BreakPoint", &breakpoint_history,  20},
+   { "BptFunct",   &bpt_history,         20},
+   { "Environment",&env_history,         10},
+   { "ComandArgs", &cmd_history,         10},
+   { "SubScriptEdit",  &subscript_history, 5},
+   { "ValueEdit",  &value_history,       20},
+   { "ExprEdit",   &expr_history,        20},
+   { "Prompt",     &prompt_history,      10},
+   { NULL }
+};
+
+/*  This spider on a web icon is supposed to suggest something that
+**  catches bugs.
+*/
+#define ups_icon_width 48
+#define ups_icon_height 48
+static unsigned char ups_icon_bits[] = {
+  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+  0x00,0x00,0x00,0x00,0x20,0x00,0x00,0x03,0xf0,0x07,0x70,0x00,
+  0x80,0x07,0xfc,0x1f,0xd8,0x00,0xc0,0x0c,0xfe,0x3f,0x8c,0x01,
+  0x60,0x18,0xff,0x7f,0x06,0x03,0x30,0xb8,0xff,0xff,0x03,0x06,
+  0x18,0xf0,0xff,0xff,0x01,0x0e,0xc8,0xe0,0xff,0xff,0x01,0x08,
+  0xc4,0xe1,0xff,0xff,0x83,0x01,0x60,0xe3,0xff,0xff,0xc3,0x03,
+  0x60,0xf2,0xff,0xff,0x67,0x02,0x30,0xf6,0xff,0xff,0x37,0x06,
+  0x30,0xfc,0xff,0xff,0x1f,0x04,0x18,0xf8,0xff,0xff,0x0f,0x0c,
+  0x18,0xf8,0xff,0xff,0x07,0x08,0x0c,0xf8,0xff,0xff,0x87,0x18,
+  0xc4,0xf8,0xff,0xff,0xc7,0x11,0xc0,0xf9,0xff,0xff,0xe7,0x13,
+  0x60,0xfb,0xff,0xff,0xff,0x13,0x60,0xfe,0xff,0xff,0x1f,0x1e,
+  0x30,0xfc,0xff,0xff,0x0f,0x66,0x30,0xf8,0xff,0xff,0x0f,0x8c,
+  0x18,0xf0,0xff,0xff,0x0f,0x0c,0x18,0xe0,0xff,0xff,0xff,0x0c,
+  0x04,0xe3,0xff,0xff,0x7b,0x0f,0x00,0xcf,0xff,0xff,0xdd,0x38,
+  0x80,0xfd,0xff,0xff,0xcf,0xc0,0x80,0xf1,0xff,0xff,0x8b,0x01,
+  0xc0,0x20,0xff,0xff,0xff,0x01,0xe0,0x10,0xfe,0x7f,0x08,0x07,
+  0x60,0x08,0xfe,0x1f,0x08,0x1a,0x60,0x04,0xf2,0x07,0x08,0x66,
+  0x30,0x04,0xf9,0x0f,0x08,0x84,0x10,0x82,0xfc,0x1f,0x7f,0x04,
+  0x00,0x41,0xfc,0xff,0x88,0x03,0x00,0x41,0xfc,0x1f,0x08,0x0c,
+  0x00,0x21,0xfc,0x1f,0x08,0x10,0x80,0x20,0xfc,0x1f,0x08,0x20,
+  0x80,0x10,0xfc,0x2f,0x7f,0x40,0x40,0x10,0xf2,0xe7,0x88,0x81,
+  0xc0,0x09,0xe2,0x63,0x08,0x02,0x40,0x1e,0x41,0x50,0x08,0x04,
+  0x40,0xe8,0x21,0x88,0x3e,0x08,0x20,0x08,0x2f,0x04,0x49,0x10,
+  0x20,0x84,0xf0,0x84,0x89,0x10,0x20,0x84,0x10,0x47,0x0a,0x21,
+};
+static bitmap_t ups_icon =
+	wn_make_static_X11bm(ups_icon_width, ups_icon_height, ups_icon_bits);
+
 static void
 handle_panic(s)
 const char *s;
@@ -96,15 +147,6 @@ const char *s;
 	Done_panic = TRUE;
 }
 
-static void
-quit_ups()
-{
-  extern MENU tgt_stop_men;
-
-  if (Mselectable(&tgt_stop_men))
-    target_stop();
-  re_set_exit_event_loop_flag();
-}
 
 /*  This routine is called when we get a signal that causes a core dump.
  *  We want to avoid overwriting an existing core file in the current
@@ -312,6 +354,7 @@ main(argc, argv)
 int argc;
 char **argv;
 {
+	char res_buff[50];
 	static const int siglist[] = {
 #ifdef SIGQUIT
 		SIGQUIT,
@@ -360,8 +403,8 @@ char **argv;
 	int i;
 	bool got_source_path, replay, want_window, user_gave_core, done_cc_arg;
 	bool seen_startup_message_flag, want_startup_message;
-	bool want_load_config, want_save_config, want_auto_start, use_full_path, want_save_sigs;
-	bool want_outwin;
+	bool want_load_config, want_save_config;
+	bool want_auto_start, use_full_path, want_save_sigs;
 	target_t *xp;
 
 	/*  Object if main is called recursively.  This can happen on a
@@ -412,9 +455,15 @@ Usage: ups -V\n\
    or: ups -fullusage\n\
    or: ups obj [-coredir dir] [corefile|pid] [srcdir[:srcdir]...] [-a args]\n\
 	       [-noconfig] [-noloadconfig] [-nosaveconfig]  [-nosavesigs]\n\
-               [-statedir dir] [-replay path] [-record path] [-nowindow]\n\
-	       [-replayidstr str] [-replaymode [lnum:]{%s}]\n\
-               [-go] [-nostartupmesg] [-use_srcpath_only] [-fullpath] [-dbflags flags]\n\
+	       [-statedir dir] [-nodemangle]\n\
+	       [-[no]split[:num]] [-[no]splitoutwin[:num]]\n\
+	       [-[no]mousehole] [-[no]logging]\n \
+	       [-[no]raise_on_break] [-[no]lower_on_run] [-[no]iconify_on_run]\n \
+	       [-replay path] [-record path]\n\
+               [-nowindow] [-replayidstr str]\n\
+	       [-replaymode [lnum:]{%s}]\n\
+               [-go] [-nostartupmesg] [-use_srcpath_only] [-fullpath]\n\
+               [-dbflags flags]\n\
                %s\n",
 		       td_get_replay_mode_list(), driver_flags);
 
@@ -503,7 +552,6 @@ Usage: ups -V\n\
 	replay = FALSE;
 	want_auto_start = FALSE;
 	use_full_path = FALSE;
-	want_outwin = FALSE;
 
 	for (; *argv != NULL; argv++) {
 		if (strcmp(*argv, "-use_srcpath_only") == 0) {
@@ -512,8 +560,11 @@ Usage: ups -V\n\
 		else if (strcmp(*argv, "-dbflags") == 0) {
 			Debug_flags = strtol(checkarg(++argv), (char **)NULL,0);
 		}
+		else if (strcmp(*argv, "-nooutwin") == 0) {
+			wn_set_default("WantOutwin", "no");
+		}
 		else if (strcmp(*argv, "-outwin") == 0) {
-			want_outwin = TRUE;
+			wn_set_default("WantOutwin", "yes");
 		}
 		else if (strcmp(*argv, "-nowindow") == 0) {
 			want_window = FALSE;
@@ -528,6 +579,64 @@ Usage: ups -V\n\
 		else if (strcmp(*argv, "-nosaveconfig") == 0) {
 			want_save_config = FALSE;
 		}
+		else if (strcmp(*argv, "-nosplit") == 0) {
+			wn_set_default("WantSplitWindows", "no");
+		}
+		else if (strcmp(*argv, "-split") == 0) {
+			wn_set_default("WantSplitWindows", "yes");
+		}
+		else if (strncmp(*argv, "-split:", 7) == 0) {
+			strcpy(res_buff, "yes:");
+			strncat(res_buff, *argv+7,10);
+			wn_set_default("WantSplitWindows", res_buff);
+		}
+		else if (strcmp(*argv, "-nosplitoutwin") == 0) {
+			wn_set_default("WantSplitOutputWindow", "no");
+		}
+		else if (strcmp(*argv, "-splitoutwin") == 0) {
+			wn_set_default("WantSplitOutputWindow", "yes");
+		}
+		else if (strncmp(*argv, "-splitoutwin:", 13) == 0) {
+			strcpy(res_buff, "yes:");
+			strncat(res_buff, *argv+13,10);
+			wn_set_default("WantSplitOutputWindow", res_buff);
+		}
+		else if (strcmp(*argv, "-nomousehole") == 0) {
+			wn_set_default("WantMousehole", "no");
+		}
+		else if (strcmp(*argv, "-mousehole") == 0) {
+			wn_set_default("WantMousehole", "yes");
+		}
+		else if (strcmp(*argv, "-nologging") == 0) {
+			wn_set_default("WantMessageLogging", "no");
+		}
+		else if (strcmp(*argv, "-logging") == 0) {
+			wn_set_default("WantMessageLogging", "yes");
+		}
+		else if (strcmp(*argv, "-noraise_on_break") == 0) {
+			wn_set_default("WantRaiseOnBreak", "no");
+			wn_set_default("WantLowerOnRun", "no");
+			wn_set_default("WantIconifyOnRun", "no");
+		}
+		else if (strcmp(*argv, "-raise_on_break") == 0) {
+			wn_set_default("WantRaiseOnBreak", "yes");
+			wn_set_default("WantLowerOnRun", "no");
+			wn_set_default("WantIconifyOnRun", "no");
+		}
+		else if (strcmp(*argv, "-lower_on_run") == 0) {
+			wn_set_default("WantLowerOnRun", "yes");
+			wn_set_default("WantIconifyOnRun", "no");
+		}
+		else if (strcmp(*argv, "-nolower_on_run") == 0) {
+			wn_set_default("WantLowerOnRun", "no");
+			wn_set_default("WantIconifyOnRun", "no");
+		}
+		else if (strcmp(*argv, "-iconify_on_run") == 0) {
+			wn_set_default("WantIconifyOnRun", "yes");
+		}
+		else if (strcmp(*argv, "-noiconify_on_run") == 0) {
+			wn_set_default("WantIconifyOnRun", "no");
+		}
 		else if (strcmp(*argv, "-nosavesigs") == 0) {
 			want_save_sigs = FALSE;
 		}
@@ -659,16 +768,20 @@ Usage: ups -V\n\
 		td_restore_replay_fp();
 	}
 
+	/* Initialize the edit histories */
+	edit_initialize_history(state_dir, "editHistory",".upsEditHistory", history_items);
+
 	if (make_target(textname, corename, user_gave_core, &xp,
 			&args, FALSE) == -1)
 		exit(1);
 	set_current_target(xp);
 
-	if (ups(xp, args, replay, want_window, want_outwin, want_load_config,
-		want_save_config, want_save_sigs, want_auto_start, use_full_path, state_dir,
-		is_number(corename)) != 0)
+	if (ups(xp,textname, args, replay, want_window, want_load_config,
+		want_save_config, want_save_sigs, want_auto_start,
+		use_full_path, state_dir, is_number(corename)) != 0)
 		exit(1);
 
+	edit_terminate_history();
 	exit(0);
 	return 0;	/* to satisfy gcc */
 }
@@ -777,8 +890,7 @@ setup_display()
 	display_overlay_set_font(sysfont);
 
 	set_special_edit_chars(edit_get_default_keymap());
-	
-	wn_set_quit_func(quit_ups);
+
 	wn_set_cur_posn_func(get_cur_posn);
 	wn_set_overlay_insert_string_func(display_area_overlay_insert_string);
 	return 0;
@@ -906,22 +1018,28 @@ const char *s;
 }
 
 static int
-ups(xp, args, replay, want_window, want_outwin, want_load_config,
-    want_save_config, want_save_sigs, want_auto_start, use_full_path, state_dir, attach)
+ups(xp, textname, args, replay, want_window, want_load_config,
+    want_save_config, want_save_sigs, want_auto_start,
+    use_full_path, state_dir, attach)
 target_t *xp;
+const char* textname;
 const char *args;
-bool replay, want_window, want_outwin;
-bool want_load_config, want_save_config, want_save_sigs, want_auto_start, use_full_path;
+bool replay, want_window;
+bool want_load_config, want_save_config, want_save_sigs,
+  want_auto_start, use_full_path;
 const char *state_dir;
 bool attach;
 {
-	Region *root_region;
-	window_t wn;
+	Region *root_region, *root_region1;
+	window_t wn, wn1, wn2;
 	func_t *f, *f1;
 	const char *basepath, *state_path, *user_path;
 	bool file_want_auto_start;
 	int res, file_access = -1;
 	symtab_t *st;
+	bool want_split = FALSE, want_outwin;
+	int screen2 = -1;
+	const char* defstr;
 		
  	td_set_no_window_flag();
 	if ((f = xp_get_mainfunc(xp)) == NULL) {
@@ -940,8 +1058,8 @@ bool attach;
 	}
 
  	td_set_window_flag();
-	root_region = NULL;	/* to satisfy gcc */
-	wn = -1;		/* to satisfy gcc */
+	root_region = root_region1 = NULL;	/* to satisfy gcc */
+	wn = wn1 = wn2 = -1;		/* to satisfy gcc */
 
 	if (want_window) {
 		wn_npixels_hint(1);
@@ -949,6 +1067,40 @@ bool attach;
 			errf("Can't open window");
 			return -1;
 		}
+
+		/* To split the UPS windows set
+			WantSplitWindows: yes
+		   To split across screens e.g. screen 0 & 2, set
+			WantSplitScreen: 2
+		   or
+			WantSplitWindows: yes:2
+		*/
+		defstr = wn_get_default("WantSplitWindows");
+		want_split = ((defstr != NULL) && (strncmp(defstr, "yes", 3)) == 0);
+
+		if (want_split && (defstr[3] == ':'))
+			    defstr = &defstr[4];
+		else if (defstr==NULL)
+			    defstr = wn_get_default("WantSplitScreen");
+		if ((defstr != NULL) &&
+			(atoi(defstr) > 0 || *defstr == '0'))
+		{
+			want_split = TRUE;
+			screen2 = atoi(defstr);
+		}
+#if WANT_MULTI_SCREEN
+#else
+		if (want_split && screen2 != -1)
+		{
+			errf("Not built for multiple screens, will put both windows on default screen");
+			screen2 = -1;
+		}
+#endif
+		if (   (want_split && (wn1=wn_open_scr_stdwin("Src",screen2)) == -1))
+		{
+			errf("Can't open second window");
+			return -1;
+		}
 	}
 
 	do_formats(want_window);
@@ -965,15 +1117,27 @@ bool attach;
 	signal(SIGINT, SIG_IGN);
 
 	if (want_window) {
-		if (!want_outwin) {
-			const char *defstr;
+		char title[20];
+		strcpy(title, "%a ");
+		if (strchr(textname, '/'))
+		    textname = strrchr(textname,'/')+1;
+		strncpy(title+3,textname,sizeof(title)-4);
+		title[sizeof(title)-1] = 0;
+		wn_set_window_hints(wn, title,title, &ups_icon);
 
+		/* Determine it split windows are requested */
 			defstr = wn_get_default("WantOutwin");
-			if (defstr != NULL && strcmp(defstr, "yes") == 0)
-				want_outwin = TRUE;
-		}
+		want_outwin = (defstr != NULL && strcmp(defstr, "yes") == 0);
 		
-		root_region = divide_window_into_regions(wn, want_outwin);
+		if (want_split)
+		{
+		  wn_set_window_hints(wn1, "%a source","%a source", &ups_icon);
+		  root_region = divide_window_into_regions1(wn, want_outwin);
+		  root_region1 = divide_window_into_regions2(wn1, want_outwin);
+		}
+		else
+		  root_region = divide_window_into_regions(wn, want_outwin);
+
 	}
 
 	initialise_display_area(xp, args, use_full_path);
@@ -1039,11 +1203,11 @@ bool attach;
 			fprintf(stderr, "%s in replay file - %s",
 					(res == 0) ? "EOF" : "Error",
 					"switching to normal events\n");
-			re_event_loop(root_region);
+			re_event_loop();
 		}
 	}
 	else {
-		re_event_loop(root_region);
+		re_event_loop();
 		res = 0;
 	}
 
@@ -1066,3 +1230,87 @@ bool attach;
 
 	return res;
 }
+
+
+int
+want_split_output( p_screen )
+int* p_screen;
+{
+
+    /* To split the UPS windows set
+    	    WantSplitOutputWindow: yes
+       To split across screens e.g. screen 0 & 2, set
+            WantSplitOutputWindow: 2
+    */
+    const char *defstr = wn_get_default("WantSplitOutputWindow");
+    bool want_split_outwin =  ((defstr != NULL) && (strncmp(defstr, "yes", 3)) == 0);
+    int out_screen = -1;
+
+    if ( want_split_outwin)
+    {
+	if (defstr[3] == ':')
+	    defstr = &defstr[4];
+	else
+	    defstr = wn_get_default("WantSplitScreen");
+	if ((defstr != NULL) &&
+	    (atoi(defstr) > 0 || *defstr == '0'))
+	{
+	    out_screen = atoi(defstr);
+	}
+#if WANT_MULTI_SCREEN
+#else
+	if (p_screen && out_screen != -1)
+	{
+	    /* This can cause recursion if message logging is on.
+	    ** Better to silently correct the error.
+	    **
+	    ** errf("Not built for multiple screens, will put both windows on default screen");
+	    */
+	    out_screen = -1;
+	}
+#endif
+    }
+    if ( p_screen)
+	*p_screen = out_screen;
+    return want_split_outwin;
+}
+
+int
+get_split_out_win()
+{
+    int screen;
+    int wn;
+    if ( !want_split_output(&screen))
+	 return -1;
+    wn =  wn_open_scr_stdwin("Output",screen);
+    wn_set_window_hints(wn, "%a output","%a output", &ups_icon);
+    return wn;
+}
+
+int
+rational_products_running()
+{
+    static int rational_products = -1;
+
+    if (rational_products == -1)
+    {
+	/* see if Purify/Quantify/Purecov are used */
+	func_t *f, *f1;
+	int res;
+	errf_ofunc_t oldf, oldf1;
+
+	res = find_func_by_name("main", &f, &f1);
+	oldf = errf_set_ofunc(null_ofunc);
+	oldf1 = errf_set_ofunc1(null_ofunc);
+	res = find_func_by_name("purify_is_running", &f, &f1);
+	if (res != 0)
+	    res = find_func_by_name("quantify_is_running", &f, &f1);
+	if (res != 0)
+	    res = find_func_by_name("purecov_is_running", &f, &f1);
+	rational_products = res == 0;
+	errf_set_ofunc(oldf);
+	errf_set_ofunc1(oldf1);
+    }
+    return rational_products;
+}
+
