@@ -183,9 +183,24 @@ const char **p_cmdline, **p_path;
 bool *p_overwrite;
 {
 	const char *new_cmdline, *path;
+	int template = 0;
 	bool overwrite;
 
-	if ((path = strchr(cmdline, '>')) == NULL) {
+	/* Look for redirection of output with syntax
+	**
+	**   $debug:asmsrc func > file
+	**
+	** but do not get confused by template parameters in function names, 
+	** e.g.   $debug:asmsrc  funct<type>::foo 
+	*/
+
+        for (path = cmdline; *path && (*path != '>' || template) ; path++)
+	{
+		if (*path == '<') template++;
+		if (*path == '>') template--;
+	}
+	if ( *path == 0)
+	{
 		path = defpath;
 		new_cmdline = cmdline;
 		overwrite = TRUE;
