@@ -87,9 +87,11 @@ static void dec_index PROTO((Edit_display *display));
 /* RGA use elastic format ... */
 char Var_format[] = "%[-]40cv%[-]:400cV\n";
 
+Edit_history* subscript_history = NULL;
+Edit_history* value_history = NULL;
 fnamemap_t Var_fnamemap[] = {
-	{ FN_VAR_DECL,	"subscript",	TRUE,	var_quitfunc	},
-	{ FN_VAR_VALUE,	"value",	TRUE,	value_quitfunc	},
+	{ FN_VAR_DECL,	"subscript",	TRUE,	var_quitfunc, &subscript_history	},
+	{ FN_VAR_VALUE,	"value",	TRUE,	value_quitfunc, &value_history	},
 	{ 0,		NULL,		FALSE,	NULL		},
 };
 
@@ -130,7 +132,7 @@ draw_t *dets;
 	if (want_color == -1)
 	{
 	  want_color = 0;
-	  get_variable_colors(&p_bg, &p_fg);
+	  get_variable_colors(dets->dr_wn, &p_bg, &p_fg);
 	  def = (char *)wn_get_default(colorvar);
 	  if (def && strcmp(def, "off") == 0)
 	    want_color = 0;
@@ -152,7 +154,7 @@ draw_t *dets;
 	if (want_color == 1)
 	{
 	  len = strlen(dv->dv_var->va_name);
-	  get_value_colors(TRUE, &hi_fg, &hi_bg);
+	  get_value_colors(dets->dr_wn, TRUE, &hi_fg, &hi_bg);
 	  if (!(hi_fg == dets->dr_fg && hi_bg == dets->dr_bg))
 	  {/* turn off when highlighting */
 	    wn_text_capture(0, -1, &z); /* turn off */
@@ -624,7 +626,8 @@ const char **p_display_string;
 }
 
 void
-get_value_colors(val_changed, p_fg, p_bg)
+get_value_colors(wn, val_changed, p_fg, p_bg)
+int wn;
 bool val_changed;
 long *p_fg, *p_bg;
 {
@@ -632,12 +635,12 @@ long *p_fg, *p_bg;
   		{ -1, 0, 0, 0 }, 
    		{ -1, 0, 0, 0 }
 	};
-	int pixel, planes;
+	unsigned long pixel, planes;
 
 	if (wn_use_mono())
 	{
-	  colors[0].co_pixel = WN_BG;
-	  colors[1].co_pixel = WN_FG;
+	  colors[0].co_pixel = WN_BG(wn);
+	  colors[1].co_pixel = WN_FG(wn);
 	}
 	else
 	  if (colors[0].co_pixel == -1) {
@@ -646,37 +649,37 @@ long *p_fg, *p_bg;
 		colorname = wn_get_default("HighlightColor");
 		if ( colorname == NULL)
 		    colorname = "red";
-		wn_parse_color(colorname, &colors[1]);
+		wn_parse_color(wn, colorname, &colors[1]);
 
-		colors[0].co_pixel = WN_BG;
+		colors[0].co_pixel = WN_BG(wn);
 
 		if (!wn_use_alloc_color(0))
 		{
-		  wn_get_pixel_colors(colors, 1);
-		  if (wn_get_nplanes() > 1 &&
-		      wn_get_pixels_and_planes(1, 1, TRUE, &pixel,
+		  wn_get_pixel_colors(wn, colors, 1);
+		  if (wn_get_nplanes(wn) > 1 &&
+		      wn_get_pixels_and_planes(wn, 1, 1, TRUE, &pixel,
 					       &planes) == 0) {
 		    colors[0].co_pixel = pixel;
 		    colors[1].co_pixel = pixel | planes;
-		    wn_set_pixel_colors(colors, 2);
+		    wn_set_pixel_colors(wn, colors, 2);
 		  }
 		  else
 		  {
-		    wn_parse_color(colorname, &colors[1]);
-		    if (!wn_alloc_color(&colors[1], colorname))
+		    wn_parse_color(wn, colorname, &colors[1]);
+		    if (!wn_alloc_color(wn, &colors[1], colorname))
 		    {
 		      wn_use_alloc_color_for_highlight(1);
-		      colors[1].co_pixel = WN_FG;
+		      colors[1].co_pixel = WN_FG(wn);
 		    }
 		  }
 		}
 		else
 		{
-		  wn_parse_color(colorname, &colors[1]);
-		  if (!wn_alloc_color(&colors[1], colorname))
+		  wn_parse_color(wn, colorname, &colors[1]);
+		  if (!wn_alloc_color(wn, &colors[1], colorname))
 		  {
 		    wn_use_alloc_color_for_highlight(1);
-		    colors[1].co_pixel = WN_FG;
+		    colors[1].co_pixel = WN_FG(wn);
 		  }
 		}
 	}
@@ -686,24 +689,26 @@ long *p_fg, *p_bg;
 		*p_fg = colors[1].co_pixel;
 	}
 	else {
-		*p_bg = WN_BG;
-		*p_fg = WN_FG;
+		*p_bg = WN_BG(wn);
+		*p_fg = WN_FG(wn);
 	}
 }
 
 void
-var_getcolor(obj, p_fg, p_bg)
+var_getcolor(obj, wn, p_fg, p_bg)
 objid_t obj;
+int wn;
 long *p_fg, *p_bg;
 {
 	bool val_changed;
 
 	val_changed = (((dvar_t *)obj)->dv_flags & DVF_VAL_CHANGED) != 0;
-	get_value_colors(val_changed, p_fg, p_bg);
+	get_value_colors(wn, val_changed, p_fg, p_bg);
 }
 
 void
-get_variable_colors(p_bg, p_fg)
+get_variable_colors(wn, p_bg, p_fg)
+int wn;
 long *p_bg, *p_fg;
 {
 	static color_t colors[] = {
@@ -711,12 +716,12 @@ long *p_bg, *p_fg;
 		{ -1, 0, 0, 0},
 	};
 
-	int pixel, planes;
+	unsigned long pixel, planes;
 
 	if (wn_use_mono())
 	{
-	  colors[0].co_pixel = WN_BG;
-	  colors[1].co_pixel = WN_FG;
+	  colors[0].co_pixel = WN_BG(wn);
+	  colors[1].co_pixel = WN_FG(wn);
 	}
 	else
 	  if (colors[0].co_pixel == -1) {
@@ -726,36 +731,36 @@ long *p_bg, *p_fg;
 		if ( colorname == NULL)
 		  colorname = "purple";
 
-		wn_parse_color(colorname, &colors[1]);
+		wn_parse_color(wn, colorname, &colors[1]);
 
-		colors[0].co_pixel = WN_BG;
+		colors[0].co_pixel = WN_BG(wn);
 
 		if (!wn_use_alloc_color(0))
 		{
-		  wn_get_pixel_colors(colors, 1);
-		  if (wn_get_nplanes() > 1 &&
-		      wn_get_pixels_and_planes(1, 1, TRUE, &pixel, &planes) == 0) {
+		  wn_get_pixel_colors(wn, colors, 1);
+		  if (wn_get_nplanes(wn) > 1 &&
+		      wn_get_pixels_and_planes(wn, 1, 1, TRUE, &pixel, &planes) == 0) {
 		    colors[0].co_pixel = pixel;
 		    colors[1].co_pixel = pixel | planes;
-		    wn_set_pixel_colors(colors, 2);
+		    wn_set_pixel_colors(wn, colors, 2);
 		  }
 		  else
 		  {
-		    wn_parse_color(colorname, &colors[1]);
-		    if (!wn_alloc_color(&colors[1], colorname))
+		    wn_parse_color(wn, colorname, &colors[1]);
+		    if (!wn_alloc_color(wn, &colors[1], colorname))
 		    {
 		      wn_use_alloc_color_for_highlight(1);
-		      colors[1].co_pixel = WN_FG;
+		      colors[1].co_pixel = WN_FG(wn);
 		    }
 		  }
 		}
 		else
 		{
-		  wn_parse_color(colorname, &colors[1]);
-		  if (!wn_alloc_color(&colors[1], colorname))
+		  wn_parse_color(wn, colorname, &colors[1]);
+		  if (!wn_alloc_color(wn, &colors[1], colorname))
 		  {
 		    wn_use_alloc_color_for_highlight(1);
-		    colors[1].co_pixel = WN_FG;
+		    colors[1].co_pixel = WN_FG(wn);
 		  }
 
 		}
