@@ -96,8 +96,9 @@ fdef_t Expr_fdefs[] = {
 	{ '\0', NULL, NULL, NULL				}
 };
 
+Edit_history* expr_history = NULL;
 fnamemap_t Expr_fnamemap[] = {
-	{ FN_EXPR_STR,	"expression",	FALSE,	expr_quitfunc	},
+	{ FN_EXPR_STR,	"expression",	FALSE,	expr_quitfunc, &expr_history	},
 	{ 0,		NULL,		FALSE,	NULL		}
 };
 
@@ -183,11 +184,12 @@ type_t **btype;
 }
 
 void
-expr_getcolor(obj, p_fg, p_bg)
+expr_getcolor(obj, wn, p_fg, p_bg)
 objid_t obj;
+int wn;
 long *p_fg, *p_bg;
 {
-	get_value_colors(((dexpr_t *)obj)->de_val_changed, p_fg, p_bg);
+	get_value_colors(wn, ((dexpr_t *)obj)->de_val_changed, p_fg, p_bg);
 }
 
 static void
@@ -627,7 +629,6 @@ char *arg;
 		note_var_change(obj);
 		remove_object(obj, OBJ_DESCENDENTS);
 		break;
-	case MR_VAR_RWCSTRING:
 	case MR_VAR_WANT_TYPEDEFS:
 	case MR_VAR_NO_TYPEDEFS:
 		/*  Typedefs don't apply to expressions, as the declaration
@@ -1004,7 +1005,8 @@ ci_exec_result_t res;
 		enumval_to_string(buf, sizeof(buf), vl.vl_int, type);
 		break;
 	default:
-		panic("unknown type in es");
+		/* panic("unknown type in es"); */
+		return "<illegal expression type>";
 	}
 
 	return buf;
@@ -1062,7 +1064,8 @@ taddr_t *p_res;
 		Expr_type = NULL;
 		break;
 	default:
-		panic("unknown typecode in ser");
+		/* panic("unknown typecode in ser"); */
+		errf("Can't display unknown type");
 		argslots = 0;	/* to satisfy gcc */
 		break;
 	}
