@@ -32,12 +32,17 @@ char ups_va_menu_c_sccsid[] = "@(#)va_menu.c	1.34 04 Jun 1995 (UKC)";
 #include "ui.h"
 #include "obj_stack.h"
 #include "tdr.h"
+#include "st.h"
+#include "ao_syms.h"
+#include "ao_symload.h"
+#include "ci.h"
 #include "expr.h" /* RCB, for show_static_member() */
 
+taddr_t
+get_vtbl_based_addr PROTO((taddr_t addr, type_t* type, int offset));
+
 static taddr_t get_dv_addr PROTO((objid_t par, var_t *v, ilist_t *ilist,
 				  bool *p_decl_may_have_changed));
-static void null_ofunc PROTO((const char *s));
-static int display_mangled_component PROTO((char *name, char *component, objid_t *obj));
 static void do_var PROTO((objid_t obj, int cmd, char *arg));
 
 ALLOC_NEW_FREE(static,dvar_t,dvar,dv_nextfree)
@@ -558,11 +563,6 @@ typecode_t typecode;
     return DF_NONE;
 }
 
-static void
-null_ofunc(s)
-const char *s;
-{
-}
 static taddr_t
 get_dv_addr(par, v, ilist, p_decl_may_have_changed)
 objid_t par;
@@ -594,7 +594,32 @@ int *p_decl_may_have_changed;
 		break;
 	case CL_MOS:
 	case CL_MOU:
-		addr = var_or_expr_addr(par) + v->va_addr;
+		addr = var_or_expr_addr(par);
+		if ( v->va_flags & VA_VTBLBASED)
+		{
+		    /* SC5 virtual base class.  True offset
+		    ** is in the vtable.
+		    */
+		    type_t* type = 0;
+		    switch (get_object_type(par))
+		    {
+		    case OT_VAR:
+			type = deref_aggr((dvar_t*)par);
+			break;
+		    case OT_EXPR:
+			type = get_expr_type(par);
+			if ( type->ty_code == DT_PTR_TO)
+			    type = type->ty_base;
+			break;
+		    default:
+			panic("bad parent type in get_dv_addr");
+		    }
+		    addr = get_vtbl_based_addr(addr, type, v->va_addr);
+		}
+		else
+		{
+		    addr = addr + v->va_addr;
+		}
 		break;
 	case CL_EXT:
 	case CL_STAT:
@@ -918,19 +943,20 @@ vformat_t  fmt;
     switch(type->ty_code) {
     case DT_PTR_TO:
     case DT_ARRAY_OF:
-    case TY_U_STRUCT:
-    case TY_U_UNION:
-	for ( base_type = type->ty_base;
+	for ( base_type = type;
 	      base_type->ty_base;
 	      base_type = base_type->ty_base)
 	    continue;
 
-	if ( IS_BASIC_TYPE(base_type->ty_code ))
+	if (    IS_BASIC_TYPE(base_type->ty_code )
+	     || base_type->ty_code == TY_ENUM)
 	{
 	  cmd = MR_VAR_DEREF;
 	  break;
 	}
 	/* Else fall into stuct/union cases */
+    case TY_U_STRUCT:
+    case TY_U_UNION:
     case TY_STRUCT:
     case TY_UNION:
 	cmd = MR_VAR_EXPAND_COMPLETELY;
@@ -961,27 +987,6 @@ vformat_t  fmt;
    return cmd;
 }
 
-static int
-display_mangled_component(name, component, obj)
-     char *name;
-     char *component;
-     objid_t *obj;
-{
-  objid_t obj_var;
-  int found = 0, obj_exists;
-  errf_ofunc_t oldf;
-  char buf[256];
-
-  sprintf(buf, "%s%s", name, component);
-  oldf = errf_set_ofunc(null_ofunc);
-  found = match_name_start_in_obj(component, *obj, &obj_exists, &obj_var);
-  errf_set_ofunc(oldf);
-  if (!found)
-    errf("No such variable `%s'", buf);
-  *obj = obj_var;
-  return found;
-}
-
 void
 expand_statics(type)
 type_t* type;
@@ -1018,6 +1023,26 @@ type_t* type;
 	    show_static_member(type,v);
 	}
     }
+    if (!found && (ao_compiler(NULL, FALSE, CT_UNKNOWN) == CT_CLCC))
+    {				/* RGA try globals (needed for CLCC) */
+      func_t *cur_f;
+      fil_t *fil = 0;
+      taddr_t fp, pc, adjpc;
+      char buff[200];
+      const char *fi_name = 0;
+      int fi_name_len = 0;
+
+      sprintf(buff, "%s::*", type->ty_aggr_or_enum->ae_tag);
+      get_current_func(&cur_f, &fp, &pc, &adjpc);
+      if (cur_f)
+	fil = cur_f->fu_fil;
+      if (fil)
+      {
+	fi_name = fil->fi_name;
+	fi_name_len = strlen(fil->fi_name);
+      }
+      found = find_and_show_matching_globals(fil, fi_name, fi_name_len, buff);
+    }
     if ( !found)
 	errf("No static members");
 }
@@ -1074,6 +1099,72 @@ type_t* type;
     }
     return vtbl_addr;
 }
+/* Get the address of the vtable for the object at the given address.
+** Return BAD_ADDR if no vtable found or other error situations.
+** In the case of multiple inheritance, returns the first vtable
+** found, which should be the vtable for the first base class.
+*/
+taddr_t
+get_vtbl(addr, type)
+taddr_t addr;
+type_t* type;
+{
+    dvar_t* dv_vtbl = NULL;
+    taddr_t vtbl_addr = BAD_ADDR;
+    target_t* xp = get_current_target();
+
+    var_t *v;
+
+    while ( type->ty_code == DT_PTR_TO)
+    {
+	if (dread(xp, addr, (char *)&addr, sizeof(addr)) != 0)
+	    return BAD_ADDR;
+	type = type->ty_base;
+    }
+
+    if ( type->ty_code != TY_STRUCT)
+	return BAD_ADDR;
+
+    for (v = type->ty_aggr_or_enum->ae_aggr_members;
+	v != NULL;
+	v = v->va_next)
+    {
+	if ( v->va_flags & VA_VTBLPTR)
+	{
+	    if (dread(xp, addr+v->va_addr, (char *)&vtbl_addr, sizeof(vtbl_addr)) != 0)
+		return BAD_ADDR;
+	    break;
+	}
+	if ( v->va_flags & VA_BASECLASS)
+	{
+	    vtbl_addr = get_vtbl(addr+v->va_addr, v->va_type);
+	}
+    }
+    return vtbl_addr;
+}
+
+
+/* Return the address of a struct member whose offset into the class
+** is given by an entry in the vtable.  Offset is the entry in the vtable
+** to look at, and may be ( actually, is) negative.  This mechanism
+** is used by SC5 for virtual base classes.
+*/
+taddr_t
+get_vtbl_based_addr(addr, type, offset)
+taddr_t addr;
+type_t* type;
+int     offset;
+{
+    taddr_t vtbl_addr = get_vtbl(addr, type);
+    if ( vtbl_addr != BAD_ADDR)
+    {
+	  int true_offset;
+          target_t* xp = get_current_target();
+	  if ( dread(xp, vtbl_addr+offset, (char *)&true_offset, sizeof(true_offset)) == 0)
+	        return  addr+true_offset;
+    }
+    return BAD_ADDR;
+}
 
 static void dump_dv_to_outwin(dv)
 dvar_t* dv;
@@ -1170,12 +1261,10 @@ char *arg;
 	dvar_t *dv, *dv2;
 	type_t *type;
 	const char *mesg;
-	char *fval;
 	var_t *v;
 	bool hide_ptr;
 	int oldstate, is_ptr_to_func;
 	fval_t fields[FN_VAR_LAST + 1];
-	objid_t obj_var;
 
 	dv = (dvar_t *)obj;
 	v = dv->dv_var;
@@ -1209,35 +1298,6 @@ char *arg;
 		note_format_change(dv);
 		break;
 		
-	case MR_VAR_RWCSTRING:
-		obj_var = obj;	/* initialize */
-		if (display_mangled_component((char *)v->va_name, "pref_", &obj_var))
-		{
-		  int i;
-
-		  dv2 = (dvar_t *)obj_var;
-		  do_var((objid_t)dv2, MR_VAR_DEREF, (char *)NULL);
-		  
-		  fval = (char *)get_field_value((objid_t)dv2, FN_VAR_DECL);
-		  for (i = strlen(fval) - 1; i >= 0; i--)
-		    if (fval[i] == '{')
-		      break;
-		  if (i >= 0)
-		  {
-		    fval[++i] = 0;
-		    strcat(fval, "1}>");
-		    set_field_value((objid_t)dv2, FN_VAR_DECL, fval);
-		    dv2->dv_ilist->il_index = 1;
-		    update_var((objid_t)dv2, TRUE);
-		  }
-		  
-		  if (display_mangled_component((char*)v->va_name, "refs_", &obj_var))
-		  {
-		    dv2 = (dvar_t *)obj_var;
-		    do_var((objid_t)obj_var, MR_VAR_STRING, (char *)NULL);
-		  }
-		}
-		break;
 	case MR_VAR_DEREF:
 		if (hide_ptr || !ISDERIV(type->ty_code)) {
 			errf("Can't indirect through %s", v->va_name);
@@ -1600,9 +1660,9 @@ int **obj_var;
   }
 
   /* look for components of unions .... */
-  if (!found && !given_obj) /* just recurse one level */
+  if (!found) /* go one level up */
     for (v = type->ty_aggr_or_enum->ae_aggr_members; v != NULL; v = v->va_next)
-      if (v->va_type->ty_code == TY_UNION &&
+      if ( (v->va_type->ty_code == TY_UNION || v->va_type->ty_code == TY_STRUCT) &&
 	!get_member_of_aggr(obj, v->va_name, &sub_v, &addr, &btype))
     /* RGA Note return value of get_member_of_aggr() changed from 2.45.2 */
       {
