@@ -591,7 +591,7 @@ dvar_t *dv;
 
 	case TY_BOOLEAN:
 	     {
-		int val;
+		int val = 0;
 		int ret = 0;
 		switch ( type->ty_size)
 		{
