@@ -315,7 +315,7 @@ const char **p_cmdline;
 	}
 
 	if ((li = cc_parse_and_compile(get_ccstate(), xp->xp_textpath,
-				       &parse_id)) == NULL)
+				       &parse_id, NULL)) == NULL)
 		return -1;
 
 	cc_get_libinfo(&n_libfuncs, &n_libvars);
@@ -339,7 +339,7 @@ const char **p_cmdline;
 					   adjust_static_addrs, delta);
 	}
 
-	xf = (xc_fidata_t *)alloc(ma->ma_apool, sizeof(xc_fudata_t));
+	xf = (xc_fidata_t *)alloc(ma->ma_apool, sizeof(xc_fidata_t));
 	xf->xf_cf = cf;
 	xf->xf_letab = NULL;
 	xf->xf_letab_size = 0;
@@ -919,7 +919,8 @@ xp_opcode_t xp_opcode, *p_old_opcode;
 
 	if ((opcode == OC_TRAP) == (old_opcode == OC_TRAP)) {
 		if (opcode == OC_TRAP)
-			panic("duplicate breakpoint in xc_tswap");
+			/* Dibyendu : don't know why this happens - ignore */
+			errf("duplicate breakpoint in xc_tswap");
 		else
 			panic("vanished breakpoint in xc_tswap");
 	}
@@ -1115,6 +1116,12 @@ func_t **p_funcs;
 
 		filtab[i] = ci_make_fil(ap, xf->xf_cf->cf_srcpath, (char *)xf,
 					(block_t *)NULL, nextfil);
+		/* Dibyendu : added to fool ao_compiler() into returning
+		 * CT_UNKNOWN when running UPS over a CX executable.
+		 * This causes demangle_name_2() to do nothing 
+		 * (see ao_symscan.c).
+		 */
+		filtab[i]->fi_flags |= FI_FOUND_COMPILER;
 		nextfil = filtab[i];
 	}
 
@@ -1344,6 +1351,11 @@ fil_t *fil;
 
 	xltab = (xc_lnum_t *)alloc(ap, os->os_letab_count * sizeof(xc_lnum_t));
 
+	/* Dibyendu - not sure if this is correct - but seems to work.
+	 */
+	xf->xf_letab = xltab;
+	xf->xf_letab_size = os->os_letab_count;
+
 	xl = xltab;
 	fl = fil->fi_funclist;
 	for (i = 0; i < os->os_nfuncs; ++i) {
@@ -1483,6 +1495,7 @@ func_t *f;
 		ln = (lno_t *)alloc(f->fu_symtab->st_apool, sizeof(lno_t));
 		ln->ln_num = xl->xl_lnum;
 		ln->ln_addr = pack_textloc(ma, xf->xf_cf, text + xl->xl_addr);
+		ln->ln_fil = f->fu_fil;
 		last->ln_next = ln;
 		last = ln;
 	}
